#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009

from QCoordinatesInputWidget import QCoordinatesInputWidget
from PyQt4.QtCore import QObject, SIGNAL
from PyQt4.QtGui import QWidget, QLineEdit, QComboBox, QCheckBox, QSpinBox

##  @brief Basewidget for all inherited inputwidgets of BladeDesigner<br><em>Basiswidgets für alle folgenden Eingabewidgets des BladeDesigners</em>
#
#  BaseWidget creates a base for every inputwidget of BladeDesigner.
#  Therefore it defines some functions, which need to be implemented in order to
#  manage further functionality of the inherited inputwidget.
#  This class is abstract, thus does not have an own layout.
#  <br><em>BaseWidget bildet die Grundlage für alle Eingabewidget des BladeDesigners.
#  Dazu definiert es Funktionen die implementiert werden sollten um die
#  Verwaltung  des entsprechend abgeleiteten Eingabewidgets zu ermöglichen.
#  Diese Klasse ist abstrakt und bestizt deshalb kein eigenes Layout.</em>
class QBaseWidget(QObject):
	## @brief Returns the type of request of data, which are contained  in the inputwidget<br><em>Gibt die Art der Anforderung der Daten des Eingabewidgets zurück</em>
	#
	# Data is needed for Saving purpose.
	# <br><em>Daten werden zum Speichern angefordert.</em>
	#
	# @see Calc, SingleCalc
	Save = 1 
	## @brief Returns the type of request of data, which are contained  in the inputwidget<br><em>Gibt die Art der Anforderung der Daten des Eingabewidgets zurück</em>
	#
	# Data is needed for general calculation purpose.
	# <br><em>Daten werden zur allgemeinen Berechnung angefordert.</em>
	#
	# @see Save, SingleCalc
	Calc = 2
	## @brief Returns the type of request of data, which are contained  in the inputwidget<br><em>Gibt die Art der Anforderung der Daten des Eingabewidgets zurück</em>
	#
	# Data is needed for calculation/preview purpos.
	# <br><em>Daten werden zur einzelnen Berechnung/Vorschau angefordert.</em>
	#
	# @see Calc, Save
	SingleCalc = 4




	
	## @brief Constructor of BaseWidget<br><em>Konstruktor des BaseWidgets</em>
	#
	# The constructor of BaseWidget assignes general variables with managment purpose and
	# provides interfaces to hook in functions.
	# <br><em>Der Konstruktor des BaseWidgets setzt allgemeine Verwaltungstechnische Variablen
	# und stellt Schnittstellen zum anknüpfen von Funktionen bereit.</em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		#Standardkonstruktor des Basiswidgets aufrufen
		QWidget.__init__(self, pParent)
		
		## @brief Returns whether the inputwidget is a new copy<br><em>Gibt an ob das Eingabewidget eine frische Kopie ist</em>
		#
		# Is needed to determine, how far the inputed values of the widget has changed.
		# It is changed by copyFrom()-function.
		# <br><em>Wird dazu benötigt um festzustellen, inwiefern sich die Eingabewerte eines Widgets geändert haben.
		# Verändert sich, falls die copyFrom()-Funktion aufgerufen wird.</em>
		#
		# @sa copyFrom(), contentChanged()
		self._bIsCopy = False
		
		## @brief Returns whether something in the inputwidget has changed, which is of general calculation purpose<br><em>Gibt an ob sich etwas im Eingabewidget geändert hat, dass für die allgemeine Berechnung relevant wäre</em>
		#
		# Is needed to determine, how far the inputed values of the widget has changed.
		# Is changed when a control on the widget is changed (e.g. by inputing text, changing an index and so on) or a general calculation is done.
		# <br><em>Wird dazu benötigt um festzustellen, inwiefern sich die Eingabewerte eines Widgets geändert haben.
		# Verändert sich, sobald eines der Steuerelemente des Widgets sich ändert (zBsp durch Texteingabe, Indexveränderung usw) oder eine allgemeine
		# Berechnung ausgeführt wird.</em>
		#
		# @sa contentChanged(), performCalc(), isChangedCalc(), setCalculated()
		self._bIsChangedCalc = True
		
		## @brief Returns whether something in the inputwidget has changed, which is of saving purpose<br><em>Gibt an ob sich etwas im Eingabewidget geändert hat, dass für die Sicherung der Eingabedaten relevant wäre</em>
		#
		# Is needed to determine, how far the inputed values of the widget has changed.
		# Is changed when a control on the widget is changed (e.g. by inputing text, changing an index and so on) or the data of the widget is saved.
		# <br><em>Wird dazu benötigt um festzustellen, inwiefern sich die Eingabewerte eines Widgets geändert haben.
		# Verändert sich, sobald eines der Steuerelemente des Widgets sich ändert (zBsp durch Texteingabe, Indexveränderung usw) oder das Widget gespeichert wird.</em>
		#
		# @sa contentChanged(), setSaved()
		self._bIsChangedSave = True

		## @brief Returns whether something in the inputwidget has changed, which is of calculation/preview purpose<br><em>Gibt an ob sich etwas im Eingabewidget geändert hat, dass für die Berechnung/Vorschau relevant wäre</em>
		#
		# Is needed to determine, how far the inputed values of the widget has changed.
		# Is changed when a control on the widget is changed (e.g. by inputing text, changing an index and so on) or a single calculatio/preview is done.
		# <br><em>Wird dazu benötigt um festzustellen, inwiefern sich die Eingabewerte eines Widgets geändert haben.
		# Verändert sich, sobald eines der Steuerelemente des Widgets sich ändert (zBsp durch Texteingabe, Indexveränderung usw) oder eine einzelne
		# Berechnung/Vorschau ausgeführt wird.</em>
		#
		# @sa contentChanged(), performCalc(), __getOuputSingleCalc()
		self._bIsChangedSingleCalc = True

		## @brief the color assigned to the widget<br><em>Die dem Widget zugewiesene Farbe</em>
		#
		# Can be used to distinguish by color of the single inputwidget (e.g. as part of a hierarchy or list).
		# <br><em>Kann zur farblichen Unterscheidung einzelner Eingabewidgets verwendet werden (zBsp in einer Hierarchie oder Liste).</em>
		#
		# @sa color(), setColor()
		self._pColor = None

		## @brief placeholder for the outputobject<br><em>Platzhalter für das Ausgabeobjekt</em>
		#
		# In this object all collected data of the inputwidget is stored.
		# Should be filled with values or created at lastest in "getOutput".
		# <br><em>In diesem Objekt werden die gesammelten Daten des Eingabewidgets zurückgegeben.
		# Sollte spätestens in "getOutput" angelegt und mir Werten gefüllt werden.</em>
		#
		# @sa output(), getOutput(), __getOutputSingleCalc()
		self._pOutput = None

		## @brief Function, which can be implemented when an inputwidget shall copy data from an other inputwidget of the same type<br><em>Funktion, die implementiert werden kann, falls das Eingabewidget Daten von einem Eingabewidget selben Typs kopieren soll</em>
		#
		# The idea behind this function is to implement acutal functionality in the inherited widget, which then only needs to care about it certain task
		# and not about additional managment effort. Therefore the corresponding copy-function with its belonging parameters can simply be assigned.
		# <br><em>Sinn dieser Funktion ist es, dass die eigentliche Funktionalität im vererbten Widget implementiert wird und sich nur noch um das kopieren
		# der Werte kümmern muß und nicht um zusätzlichen Verwaltungsaufwand oder Steuerelementkopien. Dafür kann die entsprechende Kopierfunktion
		# mit den entsprechenden Parametern einfach zugewiesen werden.</em>
		#
		# @param pWidget - the inputwidget to copy from<br><em>  Das Eingabewidget von dem kopiert werden soll</em>
		#
		# @code
		# class Vererbt(QBaseWidget):
		#	def __init__(self, pParent):
		#		QBaseWidget.__init__(self, pParent)
		#		self._OnCopyFrom = self.CopyFunc
		#
		#	def CopyFunc(self, pWidgetToCopyFrom):
		#		...
		# @endcode
		#
		# @sa copyFrom()
		self._OnCopyFrom = None
		
		## @brief Function, which can be implemented when an inputwidget shall evalute the inputed data and make a calculation based on them<br><em>Funktion, die implementiert werden kann, falls das Eingabewidget die Daten sammeln und eine Berechnung durchführen soll</em>
		#
		# The idea behind this function is to implement acutal functionality in the inherited widget, which then only needs to care about it certain task
		# and not about additional managment effort. Therefore the corresponding calculation-function can simply be assigned.
		# <br><em>Sinn dieser Funktion ist es, dass die eigentliche Funktionalität im vererbten Widget implementiert wird und sich nur noch um die Berechnung
		# kümmern muß und nicht um zusätzlichen Verwaltungsaufwand (wie zum Beispiel, das markieren als bereits berechnet).
		# Dafür kann die entsprechende Berchnungsfunktion einfach übergeben werden.</em>
		#
		# @code
		# class Vererbt(QBaseWidget):
		#	def __init__(self, pParent):
		#		QBaseWidget.__init__(self, pParent)
		#		self._OnPerformCalc = self.CalcFunc
		#
		#	def CalcFunc(self):
		#		...
		# @endcode
		#
		# @sa performCalc()
		self._OnPerformCalc = None

		## @brief Function, which can be implemented when an inputwidget shall provide the inputed data in the @link #_pOutput outputobject @endlink<br><em>Funktion, die implementiert werden kann, falls das Eingabewidget die Daten in das @link #_pOutput Ausgabeobjekt @endlink schreiben soll</em>
		#
		# The idea behind this function is to implement acutal functionality in the inherited widget, which then only needs to care about it certain task
		# and not about additional managment effort. Therefore the corresponding reading-function with its belonging parameters can simply be assigned.
		# <br><em>Sinn dieser Funktion ist es, dass die eigentliche Funktionalität im vererbten Widget implementiert wird und sich nur noch um das Auslesen der Daten
		# kümmern muß und nicht um zusätzlichen Verwaltungsaufwand.
		# Dafür kann die entsprechende Auslesefunktion mit entsprechenden Parametern einfach übergeben werden.</em>
		#
		# @param pBase - @link #_pOutput outputobject @endlink, which should be updated or filled with the data of the inputwidget<br><em>  Ausgabestruktur, die aktualisiert bzw. mit den Daten des Eingabewidgets gefüllt werden soll</em>
		#
		# @code
		# class Vererbt(QBaseWidget):
		#	def __init__(self, pParent):
		#		QBaseWidget.__init__(self, pParent)
		#		self._OnGetOutput = self.GetOutputFunc
		#
		#	def GetOutputFunc(self, pBase):
		#		...
		# @endcode
		#
		# @sa getOutput()
		self._OnGetOutput = None

		## @brief Function, which is called when the calculation of an inputwidget is done and marked as calculated<br><em>Funktion die aufgerufen wird nachdem das Eingabewidget berechnet und als berechnet markiert wurde</em>
		#
		# The idea behind this function is to implement acutal functionality in the inherited widget, which then only needs to care about it certain task
		# and not about additional managment effort. Therefore the corresponding function can simply be assigned.
		# This function can be used for recursion.
		# <br><em>Sinn dieser Funktion ist es, dass die eigentliche Funktionalität im vererbten Widget implementiert wird und sich nur noch um das Weiterreichen der Markierung
		# kümmern muß und nicht um zusätzlichen Verwaltungsaufwand. Genutzt werden kann diese Funktion für Rekursion.
		# Dafür kann die entsprechende Funktion einfach übergeben werden.</em>
		#
		# @sa setCalculated()
		self._OnSetCalculated = None

		## @brief Function, which is called when the data of inputwidget is saved and marked as saved<br><em>Funktion die aufgerufen wird nachdem das Eingabewidget gespeichert und als gespeichert markiert wurde</em>
		#
		# The idea behind this function is to implement acutal functionality in the inherited widget, which then only needs to care about it certain task
		# and not about additional managment effort. Therefore the corresponding function can simply be assigned.
		# This function can be used for recursion.
		# <br><em>Sinn dieser Funktion ist es, dass die eigentliche Funktionalität im vererbten Widget implementiert wird und sich nur noch um da Weiterreichen der Markierung
		# kümmern muß und nicht um zusätzlichen Verwaltungsaufwand. Genutzt werden kann diese Funktion für Rekursion.
		# Dafür kann die entsprechende Funktion einfach übergeben werden.</em>
		#
		# @sa setSaved()
		self._OnSetSaved = None




	## @brief Returns the @link #_pColor color @endlink of inputwidget<br><em>Gibt die @link #_pColor Farbe @endlink des Widgets zurück</em>
	#  @return @link #_pColor color @endlink of the widget<br><em>@link #_pColor Farbe @endlink des Widgets</em>
	#  @sa setColor(), _pColor
	def color(self):
		return self._pColor

	## @brief Return the @link #_pOutput outputobject @endlink of the inputwidget<br><em>Gibt das @link #_pOutput Ausgabeobjekt @endlink des Eingabewidgets zurück</em>
	#  @return the @link #_pOutput outputobject @endlink containing the inputwidgets data<br><em>die Daten des Widgets im @link #_pOutput Ausgabeobjekt @endlink</em>
	#  @see _pOutput
	def output(self):
		return self._pOutput

	## @brief Returns whether something important to the calculation has changed<br><em>Gibt an, ob sich etwas berechnungsrelevantes geändert hat</em>
	# @return True or False<br><em>True oder False</em>
	# @see _bIsChangedCalc
	def isChangedCalc(self):
		return self._bIsChangedCalc


	## @brief Sets the @link #_pColor color @endlink of the inputwidget<br><em>Setzt die @link #_pColor Farbe @endlink des Widgets</em>
	#  @param pColor - the color of the inputwidget from type QColor<br><em>  die Farbe des Eingabewidgets vom Typ QColor</em>
	#  @see color(), _pColor
	def setColor(self, pColor):
		self._pColor = pColor

	##  @brief Marks the inputwidget as calculated<br><em>Markiert das Eingabewidget als berechnet</em>
	#
	#  After marking the function linked to #_OnSetCalculated is called.
	#  <br><em>Nach der Markierung wird die an #_OnSetCalculated gehangene Funktion aufgerufen.</em>
	#
	#  @sa _bIsChangedCalc, _OnSetCalculated
	def setCalculated(self):
		#Keine Veränderung bei der Berechnung mehr vorhanden
		self._bIsChangedCalc = False
		#Folgefunktion vorhanden?
		if (self._OnSetCalculated != None):
			self._OnSetCalculated()

	## @brief Marks the inputwidget as saved<br><em>Markiert das Eingabewidget als gespeichert</em>
	#
	#  After marking the function linked to #_OnSetSaved is called.
	#  <br><em>Nach der Markierung wird die an #_OnSetSaved gehangene Funktion aufgerufen.</em>
	#
	#  @sa _bIsChangedSave, _OnSetSaved
	def setSaved(self):
		#Keine Veränderung bei der Berechnung mehr vorhanden
		self._bIsChangedSave = False
		#Folgefunktion vorhanden?
		if (self._OnSetSaved != None):
			self._OnSetSaved()




	## @brief Copies all inputvalues of the controls from another QBaseWidget<br><em>Kopiert alle Eingabewerte der Steuerelemente des QBaseWidgets von einem Anderem</em>
	#
	#  copyFrom kopiert:<br><em>copyFrom copies:</em>
	#  	- Text of QLineEdit<br><em>Die Texte der QLineEdits</em>
	#	- Entrys of QComboBox and the selected Index<br><em>Die Einträge der QComboBoxen und den aktiven Index</em>
	#	- Status of QCheckBox<br><em>Status der QCheckBoxen</em>
	#	- Values of QSpinBox<br><em>Werte der QSpinBoxen</em>
	#	- Entrys of QCoordinatesInputWidget<br><em>Einträge der QCoordinatesInputWidget</em>
	#	- @link #_pColor Color @endlink of QBaseWidget<br><em>@link #_pColor Farbe @endlink des QBaseWidget</em>
	#	- @link #_pOutput Outputobject @endlink of QBaseWidget<br><em>@link #_pOutput Ausgabeobjekt @endlink des QBaseWidget</em>
	#
	#  Also the function linked to #_OnCopyFrom is called to copy custom data.
	#  Afterwards the layout of the widget is automatically aligned to the new data with a call to optionChanged().
	#  <br><em>Des Weiteren wird die an #_OnCopyFrom hängende Funktion aufgerufen um benutzerspezifische Daten zu kopieren.
	#  Das Widget passt sich mit einem Aufruf von optionChanged() automatisch an die neuen Daten an.</em>
	#
	#  @param pWidget - QBaseWidget of the same type, from which data is copied<br><em>das QBaseWidget selben Typs von dem kopiert werden soll</em>
	#
	#  @sa optionChanged(), _OnCopyFrom, _bIsCopy
	def copyFrom(self, pWidget):
		#Alle Werte der Vorlage übernehmen
		#LineLEEdits
		for LEEdit in self.findChildren(QLineEdit):
			if LEEdit.objectName() == 'qt_spinbox_lineedit' or LEEdit.objectName() == '':
				continue
			#Signale blocken, damit nicht OptionChanged aufgerufen wird
			LEEdit.blockSignals(True)
			#Text kopieren
			LEEdit.setText(getattr(pWidget, str(LEEdit.objectName())).text())
			#Signale wieder freigeben
			LEEdit.blockSignals(False)
		#CBComboBoxen
		for CBComboBox in self.findChildren(QComboBox):
			if CBComboBox.objectName() == '':
				continue
			#Fremde Combobox holen
			pWidgetCB = getattr(pWidget, str(CBComboBox.objectName()))
			#Signale blocken, damit die CBComboBox nicht ständig
			#OptionChanged aufruft und damit Fehler in der GUI erzeugt
			CBComboBox.blockSignals(True)
			#Eigene löschen
			CBComboBox.clear()
			#Einträge übernehmen
			for Entry in range(pWidgetCB.count()):
				CBComboBox.addItem(pWidgetCB.itemText(Entry))
			#Index übernehmen
			CBComboBox.setCurrentIndex(pWidgetCB.currentIndex())
			#Signale wieder freigeben
			CBComboBox.blockSignals(False)
		#Checkboxen
		for CBCheckBox in self.findChildren(QCheckBox):
			#Checkboxen ohne Namen überspringen
			if (CBCheckBox.objectName() == ""):
				continue
			
			#Signale blocken
			CBCheckBox.blockSignals(True)
			CBCheckBox.setChecked(getattr(pWidget, str(CBCheckBox.objectName())).isChecked())
			CBCheckBox.blockSignals(False)
		#Spinboxen
		for SPSpinBox in self.findChildren(QSpinBox):
			#Signale blocken
			SPSpinBox.blockSignals(True)
			SPSpinBox.setValue(getattr(pWidget, str(SPSpinBox.objectName())).value())
			SPSpinBox.blockSignals(False)
		#CICoordinateInputs
		for CICoordinateInput in self.findChildren(QCoordinatesInputWidget):
			#Signale blocken
			CICoordinateInput.blockSignals(True)
			CICoordinateInput.copyFrom(getattr(pWidget, str(CICoordinateInput.objectName())))
			CICoordinateInput.blockSignals(False)

		#Sonstige Kopieraktionen durchführen
		if self._OnCopyFrom:
			self._OnCopyFrom(pWidget)

		#Output übernehmen
		self._pOutput = pWidget._pOutput

		#Icon übernehmen
		self._pColor = pWidget._pColor

		#OptionsCheck
		self.optionChanged()

		#Das Widget wurde geändert
		self.contentChanged()
		
		#Das Widget ist eine Kopie
		self._bIsCopy = True





	## @brief Emits that a control of the widget has been changed<br><em>Signalisiert, dass sich ein Steuerelement des Widgets geändert hat</em>
	#
	# To this hookChangeNotify() should be called or this method should been linked manually to an according control.
	# This function sets the according flags as far as changing (#_bIsChangedCalc, #_bIsChangedSave, #_bIsChangedSingleCalc) is concerned and afterwards
	# this singnal is also emited to inform higher BaseWidgets, that something has changed. This may be important to tree structures.
	# <br><em>Dazu sollte im Vorfeld hookChangeNotify() aufgerufen wurden sein oder diese Methode "per Hand" an ein entsprechendes Steuerelement gehangen wurden sein.
	# Die Funktion setzt die Entsprechenden Flags bezüglich Veränderung (#_bIsChangedCalc, #_bIsChangedSave, #_bIsChangedSingleCalc) und feuert dann
	# seinerseits dieses Signal ab um übergeordneten BaseWidgets mitzuteilen, dass sich etwas geändert hat. Wichtig für Baumstrukturen.</em>
	#
	# @see hookChangeNotify(), _bIsChangedCalc, _bIsChangedSave, _bIsChangedSingleCalc
	def contentChanged(self):
		#Signal feuern
		self.emit(SIGNAL("contentChanged()"))

		#Etwas hat sich im Widget geändert
		self._bIsChangedCalc = True
		self._bIsChangedSave = True
		self._bIsChangedSingleCalc = True

		#Falls es eine Kopie war, dann hat sie sich jetzt verändert
		if (self._bIsCopy):
			self.emit(SIGNAL("copyChanged()"))
			#Wenn es eine Kopie war, dann ist es jetzt keine mehr
			self._bIsCopy = False





	## @brief Links the slots of certain controls to contentChanged()-function<br><em>Verbindet die Slots bestimmter Steuerelemente mit der contentChanged()-Funktion</em>
	#
	# Is used to notice changes in QBaseWidget.
	# <br><em>Dient dazu um Veränderungen im QBaseWidget zu bemerken.</em>
	#
	# This function links:<br><em>Diese Funktion verbindet:</em>
	#	- QLineEdits
	#	- QComboBoxen
	#	- QCheckBoxen
	#	- QSpinBoxen
	#	- QCoordinatesInputWidget
	#
	# @see contentChanged()
	def hookChangeNotify(self):
		#Durch alle Steuerelemente gehen und anhängen
		#LineLEEdits
		for LEEdit in self.findChildren(QLineEdit):
			if LEEdit.objectName() == 'qt_spinbox_lineedit' or LEEdit.objectName() == '':
				continue
			self.connect(LEEdit, SIGNAL("textChanged(QString)"), self.contentChanged)
		#CBComboBoxen
		for CBComboBox in self.findChildren(QComboBox):
			self.connect(CBComboBox, SIGNAL("currentIndexChanged(int)"), self.contentChanged)
		#Checkboxen
		for CBCheckBox in self.findChildren(QCheckBox):
			##Checkboxen ohne Namen überspringen
			if (CBCheckBox.objectName() == ""):
				continue
			self.connect(CBCheckBox, SIGNAL("stateChanged(int)"), self.contentChanged)
			
		##Spinboxen
		for SPSpinBox in self.findChildren(QSpinBox):
			self.connect(SPSpinBox, SIGNAL("valueChanged(int)"), self.contentChanged)

		#CICoordinateInputs	
		for CICoordinateInput in self.findChildren(QCoordinatesInputWidget):
			self.connect(CICoordinateInput, SIGNAL("contentChanged()"), self.contentChanged)
		




	## @brief Realignes the layout of the widget<br><em>Passt das Aussehen der Widgets an</em>
	#
	# This function has to be reimplemented in inherited widget in any case.
	# It is responsible, that the layout of the widget is changed according to the settings made to the widget, evaluated by this function.
	# This function can be linked to critical objects(e.g. QComboBoxes, QCheckBoxes,...) in QtDesigner to automatically attend to the layout of the widget.
	# <br><em>Diese Funktion sollte unbedingt im abgeleiteten Widget reimplementiert werden.
	# Sie sorgt dafür, dass das Aussehen des Widgets entsprechend der gewählten Einstellungen, die in der Funktion ausgewertet
	# werden müßen, angepasst wird. Diese Funktion kann dann im QtDesigner entsprechend an kritische Objekte(zBsp QComboBoxen, QCheckBoxen, ...)
	# angehangen werden und kümmert sich dann automatisch um das Aussehen des gesamten Widgets.</em>
	def optionChanged(self):
		raise NotImplementedError("QBaseWidget.optionChanged(self) muß in der vererbten Klasse implementiert werden!")





	## @brief Starts the calculation of the according inputwidget<br><em>Starte die Berechnung des entsprechenden Eingabewidgets</em>
	#
	#  This function can be linked to a "calculate"-Button in QtDesigner.
	#  If neccessary it starts single calculation/preview. For this it calls the function linked to #_OnPerformCalc.
	#  <br><em>Diese Funktion kann an einen Berechnen-Button im QtDesigner gehangen werden.
	#  Sie startet, falls nötig, eine Einzelberechnung/Vorschau. Dazu ruft sie die an #_OnPerformCalc gehangende Funktion auf.</em>
	#
	#  @see _OnPerformCalc
	def performCalc(self):
		#Hat sich nichts verändert?
		if (not self._bIsChangedSingleCalc):
			return

		#Daten holen
		self.__getOutputSingleCalc()

		#Funktion zur Weiterverarbeitung vorhanden?
		if(self._OnPerformCalc):
			self._OnPerformCalc()

		#Veränderung bezüglich der Berechnung nichtmehr vorhanden
		self._bIsChangedSingleCalc = False





	## @brief Collects the data of the inputwidget in case of single calculation/preview<br><em>Sammelt die Daten des Eingabewidgets im falle einer Einzelberechnung/Vorschau</em>
	#
	#  This function checks whether a valid @link #_pOutput outputobject @endlink is available. If it is, it is returned. Otherwise the data is new collected.
	#  <br><em>Diese Funktion überprüft ob bereits ein korrektes @link #_pOutput Ausgabeobjekt @endlink vor liegt und gibt es falls vorhanden zurück. Ansonsten werden die Daten gesammelt.</em>
	#
	#  @param pBase - @link #_pOutput outputobject @endlink, which is written or created<br><em>@link #_pOutput Ausgabeobjekt @endlink, das geschrieben bzw. erstellt werden soll</em>
	#
	#  @return @link #_pOutput outputobject @endlink<br><em>@link #_pOutput Ausgabeobjekt @endlink</em>
	#
	# @sa performCalc(), _pOutput
	def __getOutputSingleCalc(self, pBase=None):
		#Hat sich nichts geändert aber ein Output ist vorhanden
		if (not self._bIsChangedSingleCalc) and (self._pOutput != None):
			#Dann Vorhandenen Output zurückggeben
			return self._pOutput

		#Daten für die Interene Berechnung bereitgestellt
		self._bIsChangedSingleCalc = False

		#Objekt zurückgeben
		return self.getOutput(pBase)





	## @brief Collects the data of the inputwidget in case of general reply<br><em>Sammelt die Daten des Eingabewidgets zur allgemeinen Rückgabe</em>
	#
	#  This function first checks, whether a recollection of data is neccessary. If it is, it returns the result of the function linked to #_OnGetOutput, otherwise
	#  uses the data already collected.
	#  <br><em>Die Funktion prüft erst, ob eine Neusammlung der Daten nötig ist. Falls ja, gibt sie diese mittels der an #_OnGetOutput gehangenen Funktion
	#  zurück, ansonsten benutzt sie die bereits vorhandenen Daten.</em>
	#
	#  @param pBase - @link #_pOutput outputobject @endlink, which is written or created<br><em>@link #_pOutput Ausgabeobjekt @endlink dass geschrieben bzw. erstellt werden soll</em>
	#
	# @return @link #_pOutput outputobject @endlink<br><em>@link #_pOutput Ausgabeobjekt @endlink</em>
	#
	#  @see _OnGetOutput, _pOutput
	def getOutput(self, pBase = None):
		#Hat sich nichts geändert aber ein Output ist vorhanden
		if ((not self._bIsChangedCalc) or (not self._bIsChangedSave)) and (self._pOutput != None):
			#Dann Vorhandenen Output zurückggeben
			return self._pOutput

		#Funktion gebundene oder bereits durch Save in die Ausgabe geschrieben?
		if (self._OnGetOutput != None):
			self._OnGetOutput(pBase)

		#Objekt zurückgeben
		return self._pOutput





	## @brief Loads the data of an existing @link #_pOutput outputobject @endlink of the according inputwidget into the widget<br><em>Läd die Daten aus einem vorhandenen @link #_pOutput Ausgabeobjekt @endlink des entsprechenden Eingabewidgets in das Widget</em>
	#
	#  This function differs for every type of inputwidget, thus it has to be reimplemented in the inherited inputwidget.
	#  It is recommended to read all data out of the @link #_pOutput outputobject @endlink and afterwards make a call to optionChanged().
	#  <br><em>Diese Funktion ist in jedem Eingabewidget unterschiedlich und muß deshalb dort erneut implementiert werden.
	#  Es empfiehlt sich alle Daten aus dem @link #_pOutput Ausgabeobjekt @endlink zulesen und dann einen abschließenden Aufruf an optionChanged()
	#  zutätigen.</em>
	#
	#  @param pBase - The @link #_pOutput outputobject @endlink, which to read the data from<br><em>Das @link #_pOutput Ausgabeobjekt @endlink, aus dem die Daten gelesen werden sollen</em>
	#
	# @sa optionChanged(), _pOutput, getOutput(), output()
	def setFromObject(self, pBase=None):
		raise NotImplementedError("QBaseWidget.setFromObject(self, pBase) muß in der vererbten Klasse implementiert werden!")





	## @brief Renames all lables in the widget<br><em>Benennt alle Schriftzüge im Widget um</em>
	#
	#  This function differs for every type of inputwidget, thus it has to be reimplemented in the inherited inputwidget.
	#  It is used for localization purpose.<br>
	#  It is used to control all labels contained in the widget. The labels can be written to a QtLinguist readable file with a call to pylupdate4.
	#  So they can be translated in the corresponding language and afterwards be converted to Qt-Languagefiles using CreateGui.sh script or lrelease.
	#  These files will be integrated into Qt-ressourcefiles.
	#  <br><em>Diese Funktion ist in jedem Eingabewidget unterschiedlich und muß deshalb dort erneut implementiert werden.
	#  Diese Funktion wird für die Lokalisation benötigt.<br>
	#  Sie wird benutzt um alle Texte im Widget zu kontrollieren.
	#  Die Texte können mithilfe pylupdate4 in eine für QtLinguist editierbare Form ausgelesen werden. Dort können sie dann
	#  für die entsprechende Sprache übersetzt werden und mithilfe des CreateGui.sh Script oder lrelease in Qt-Sprachdateien
	#  umgewandelt werden. Diese werden dann in die Resourcedatei von Qt integriert.</em>
	#
	#  @param bPassDown - Tells, whether the underlying widgets should be also retranslated<br><em>Gibt an, ob die in der hierarchie tiefer liegenden Widgets ebenfalls übersetzt werden sollen</em>
	def retranslateStrings(self, bPassDown = True):
		raise NotImplementedError("QBaseWidget.retranslateStrings(self) muß in der vererbten Klasse implementiert werden!")
		