#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009
import sys
from numpy import zeros
from PyQt4.QtGui import QItemDelegate, QWidget, QLineEdit, QLabel, QCheckBox, QDoubleValidator, QTableWidgetItem, QApplication, QDialog, QVBoxLayout
from PyQt4.QtCore import Qt, SIGNAL, QVariant
from Ui_CoordinatesInputWidget import Ui_CoordinatesInputWidget as CoordinatesInputWidget


##  @brief Additional inputdelegate for QCoordinatesInputWidget<br><em>Zusätzliches Eingabedelegate für QCoordinatesInputWidget</em>
#
# The delegate makes sure, that only valid double values can be inputed in the table.
# This delegate was not yet implemented by Qt. Up to now only LineEdit without input control was available.
# Although this delegate is appointed to deal with double values, there are no internal double values at all. Double values are
# represented as strings all the time.
# <br><em>Das Delegate sorgt dafür, dass nur gültige Double-Werte in Tabellen eingegeben werden können.
# Dieses Delegate war so noch nicht in Qt implementiert. Bisher konnte man nur kontrollose LineEdits in Tabellen verwenden.
# Obwohl das Delegate zur Eingabe von Double-Werten bestimmt ist, wird intern nie mit Double-Werten gearbeitet. Die Doublewerte werden
# die ganze Zeit als Strings repräsentiert.
# </em>
#
# @sa QCoordinatesInputWidget
class QCoordinatesInputDelegate(QItemDelegate):
	
	##  @brief Constructor of QCoordinatesInputDelegate<br><em>Konstruktor des QCoordinatesInputDelegate</em>
	#
	# Sets the maximal and minimal value
	# <br><em> Setzt nur Maximal- und Minimalwert.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		# Konstruktor der unterklasse aufrufen
		QItemDelegate.__init__(self, pParent)
		##  @brief Lowest input value<br><em>Minimaler Eingabewert</em>
		#
		# For limitiation of input range.
		# <br><em>Für die Beschränkung des Eingabebereichs.
		# </em>
		#
		# @sa __fMaxValue, setRange()
		self.__fMinValue = float('-inf')
		##  @brief Highest input value<br><em>Maximaler Eingabewert</em>
		#
		# For limitiation of input range.
		# <br><em>Für die Beschränkung des Eingabebereichs.
		# </em>
		#
		# @sa __fMinValue, setRange()
		self.__fMaxValue = float('inf')





	##  @brief Sets the maximal and minimal input value<br><em>Setzt den maximal und minimal Eingabewert</em>
	#
	# Is used to set limits to the domain of definitions(e.g. to values between 0 and 1)
	# <br><em>Wird benötigt um den Definitionsbereich eingrenzen zukönnen (zBsp auf Werte zwischen 0 und 1).
	# </em>
	#
	# @param fMinValue - Lowest valid minimal value<br><em>Kleinster zulässiger Minimalwert</em>
	# @param fMaxValue - Highest valid maximal value<br><em>Größter zulässiger Maximalwert</em>
	def setRange(self, fMinValue=float('inf'), fMaxValue=float('-inf')):
		#Speichern
		self.__fMaxValue = fMaxValue
		self.__fMinValue = fMinValue



	

	##  @brief Creates the inputwidget<br><em>Erstellt das Eingabefeld</em>
	#
	# Here the new functionality is implemented by simply attaching a QDoubleValidator with the corresponding range to the QLineEdit.
	# This function is demanded by Qt.
	# <br><em>Hier wird die neue Funktionalität implementiert, indem einfach ein QDoubleValidator mit entsprechender Reichweite
	# an den QLineEdit gehangen wird.Is needed to feed the data into the corresponding interface.
	# Basically its just a reorder of table entrys in a list.
	# The form of numpyarray is as follows: [[X1, Y1, ...], [X2,Y2, ...], [X3,Y3, ...], ... , [Xn, Yn, ...]]
	# </em>
	#
	# @param pParent - Parentwidget the delegates belongs to<br><em>Elternwidget zu dem das Delegate gehört</em>
	# @param pOption - Option, telling how to display the delegate<br><em>Option, die angibt, wie das Delegate angezeigt werden soll</em>
	# @param pEntry - The entry of the table to show<br><em>Der Eintrag der Tabelle der angezeigt werden soll</em>
	#
	# @return The inputwidget to show<br><em>Das angezeigte Eingabefeld</em>
	#
	# @sa setEditorData(), setModelData()
	def createEditor(self, pParent, pOption, pEntry):
		#LineEdit erzeugen
		# Wichtig, dass hier parent übergeben wird, damit das LineEdit an der richtigen Stelle ist
		LELineEdit = QLineEdit(pParent)
		#Valdidator anhängen
		LELineEdit.setValidator(QDoubleValidator(self))
		#Wertebereich setzen
		#1000 ist der Standardwert
		LELineEdit.validator().setRange(self.__fMinValue, self.__fMaxValue, 1000)
		#fertig, LineEdit zurückggeben
		return LELineEdit





	##  @brief Sets the data shown in the inputwidget<br><em>Setzt die Daten, die im Eingabefeld angezeigt werden sollen</em>
	#
	# The content of a cell is konverted to a string.
	# This function is demanded by Qt.
	# <br><em>Der Inhalt der Zelle wird einfach in einen String konvertiert.
	# Diese Funktion wird von Qt verlangt.
	# </em>
	#
	# @param strEditor - The created inputwidget(in this case a QLineEdit)<br><em>Das erstellte Eingabewidget(in dem Fall QLineEdit)</em>
	# @param pEntry - The entry of the table to show<br><em>Der Eintrag der Tabelle der angezeigt werden soll</em>
	#
	# @sa createEditor(), setModelData()
	def setEditorData(self, strEditor, pEntry):
		#Wert holen
		strInput = pEntry.model().data(pEntry, Qt.EditRole).toString()
		#In das LineEdit eintragen
		strEditor.setText(strInput)





	##  @brief Writes the data of the inputwidget back to the table<br><em>Schreibt die Daten des Eingabewidgets wieder in die Tabelle zurück</em>
	#
	# In this case simply the text of the QLineEdit is saved. This function is demanded by Qt.
	# <br><em>In dem Fall wird einfach der Text des QLineEdits gespeichert. Diese Funktion wird von Qt verlangt.
	# </em>
	#
	# @param pEditor - The created inputwidget(in this case a QLineEdit)<br><em>Das erstellte Eingabewidget(in dem Fall QLineEdit)</em>
	# @param pModel - The base of the model<br><em>Die Basis des Models</em>
	# @param pEntry - The edited entry of the table<br><em>Der zu bearbeitende Eintrag der Tabelle</em>
	#
	# @sa createEditor(), setEditorData()
	def setModelData(self, pEditor, pModel, pEntry):
		#Daten in das Model schreiben
		#Daten als QVariant übergeben
		pModel.setData(pEntry, QVariant(pEditor.text()), Qt.EditRole)





	##  @brief Updates the geometrie of inputwidget<br><em>Aktualisiert die Geometrie des Eingabewidgets</em>
	#
	# The QLineEdit is aligned to the size of the table cell.
	# <br><em>Das QLineEdit wird auf die Größe der Tabellenzelle gebracht.
	# </em>
	#
	# @param pEditor - The created inputwidget(in this case a QLineEdit)<br><em>Das erstellte Eingabewidget(in dem Fall QLineEdit)</em>
	# @param pOption - Option, telling how to display the delegate<br><em>Option, die angibt, wie das Delegate angezeigt werden soll</em>
	# @param pEntry - The entry of the table to show<br><em>Der Eintrag der Tabelle der angezeigt werden soll</em>
	def updateEditorGeometry(self, pEditor, pOption, pEntry):
		#Geometrie anpassen
		pEditor.setGeometry(pOption.rect)






# TODO:	ggf Graphische Eingabe ermöglichen (so mit Punkten auf eine Stelle klicken und so)
##  @brief Class to input coordinates<br><em>Klasse zur Eingabe von Koordinaten</em>
#
# This widget shall provide a comfortable method to input coordinates(no matter what dimensions).
# Therefore the dimensions of the columns dynamically changeable.There is a lock-function, which stores the values in the input widget
# after inputing and does not deletes the input to use the same input several times.
# The layout of this widget in QtDesignerform is contained in ./UI/CoordinatesInputWidget.ui, which can be converted to UI_CoordinatesInputWidget by ./CreateGUI.sh script using pyuic4
# and loaded afterward
# <br><em>Dieses Widget soll eine bequeme Art darstellen Koordinaten (egal welcher Dimension) einzugeben.
# Dazu sind die Dimensionen und Tabellenspalten dynamisch veränderbar. Zur Eingabe von Werten die man mehrfach verwendet gibt es auch
# eine lock-Funktion, die den Wert nach Eingabe nicht aus dem Eingabefeld löscht sondern beibehält.
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/CoordinatesInputWidget.ui, welche vom ./CreateGUI.sh Script mittels pyuic4 in UI_CoordinatesInputWidget konvertiert
# und dann geladen wird.
# </em>
#
# @sa QBladeRowWidget::QBladeRowWidget, QProfile2DWidget::QProfile2DWidget
class QCoordinatesInputWidget(QWidget, CoordinatesInputWidget):
	##  @brief Constructor of QCoordinatesInputWidget<br><em>Konstruktor des QCoordinatesInputWidget</em>
	#
	# Creates some lists for dynamically managment of the dimensions.
	# <br><em>Legt die Listen zur dynamischen Verwaltung der Dimensionen an.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	#
	# @sa __lEdits, __lHeader, __lLocks
	def __init__(self, pParent=None):
		QWidget.__init__(self, pParent)
		self.setupUi(self)
		##  @brief Contains the LineEdits of the Dimensions<br><em>Enthält die LineEdits der Dimensionen</em>
		#
		# Thereby it can easily be checked, whether all inputwidgets contain valid values.
		# Also the dimensions can be changed dynamically.
		# <br><em>Dadurch lässt sich leichter überprüfen ob alle Eingabefelder mit gültigen Werten gefüllt sind.
		# Außerdem kann die Dimension so dynamisch geändert werden.
		# </em>
		#
		# @todo integration of limitation of input <br><em>Eingabebereichsbeschränkung integrieren</em>
		#
		# @sa setDimensions(), add()
		self.__lEdits = []
		##  @brief Contains the header of the dimensions<br><em>Enthält die Überschriften der Dimensionen</em>
		#
		# Contains the number of Dimensions and their names(e.g. x, y, z; p,n)
		# Also the dimensions can be changed dynamically.
		# <br><em>Enthält die Anzahl der Dimensionen sowie deren Namen(zBsp: x, y, z; p, n)
		# Außerdem kann die Dimension so dynamisch geändert werden.
		# </em>
		#
		# @sa setDimensions()
		self.__lHeader = []
		##  @brief Contains the lock-checkboxen of the dimensions<br><em>Enthält die Lock-Checkboxen der Dimensionen</em>
		#
		# The locks are needed to store the valies in QLineEdit. Otherwise these would be automatically deleted after adding.
		# Also the dimensions can be changed dynamically.
		# <br><em>Die Locks werden dafür benötigt um Werte in den QLineEdits zubehalten. Diese würden sonst nach erfolgreichen
		# hinzufügen automatisch gelöscht.
		# Außerdem kann die Dimension so dynamisch geändert werden.
		# </em>
		#
		# @sa setDimensions(), add()
		self.__lLocks = []
		#Modell erstellen
		self.TWCoordinates.setItemDelegate(QCoordinatesInputDelegate(self.TWCoordinates))
		self.TWCoordinates.horizontalHeader().setStretchLastSection(True)
		##  @brief Shows, whether the QCoordinatesInputWidget does not have a default name<br><em>Zeigt an ob das QCoordinatesInputWidget keinen Standardnamen hat</em>
		#
		# Neccessary for translation. Otherwise the translation would overwrite the user defined header.
		# <br><em>Notwendig für Übersetzung. Da sonst die Übersetzung des benutzerspezifischen Titel überschrieben wird.
		# </em>
		#
		# @sa retranslateStrings(), setTitle()
		self.__bCustomTitle = False





	##  @brief Adds a new set of coordinates<br><em>Fügt einen neuen Satz Koordinaten hinzu</em>
	#
	# Checks, whether all inputwidgets are valid. If not, the focus is set to the invalid inputwidget.
	# Otherwise fills the coordinates in the table. Inputwidgets are cleared depending on the locks and the focus is set to the
	# first invalid inputwidget.
	# This function is attached to add-button in QtDesigner.
	# <br><em>Prüft ob alle Eingabefelder gültig belegt sind. Falls nicht springt es in das Eingabefeld, was ungültig ist.
	# Ansonsten füllt es die Koordinaten in die Tabelle. Es werden die Eingabefelder abhänig vom Lock geleert
	# und der Fokus auf das erste ungültige Eingabefeld gesetzt.
	# Diese Funktion ist an den Hinzufügen-Button im QtDesigner gehangen.
	# </em>
	#
	# @sa remove(), up(), down(), __lEdits, __lLocks
	def add(self):
		#Eingabe prüfen
		#Durch alle Edits gehen
		for LECurrent in self.__lEdits:
			#Keine gültige Eingabe?
			if (LECurrent.text() == ""):
				#Ins entsprechende Fenster springen
				LECurrent.setFocus(Qt.OtherFocusReason)
				return

			#Eingabe ungültig
			if (LECurrent.hasAcceptableInput() != True):
				#Ins entsprechende Fenster springen
				LECurrent.setFocus(Qt.OtherFocusReason)
				return
				

		#Eine Zeile mehr in der Tabelle
		self.TWCoordinates.setRowCount(self.TWCoordinates.rowCount()+1)
		#Einträge hinzufügen
		#Durch alle Edits gehen und hinzufügen
		for iCurrent in range(len(self.__lEdits)):
			self.TWCoordinates.setItem(self.TWCoordinates.rowCount()-1, iCurrent, QTableWidgetItem(self.__lEdits[iCurrent].text()))

		#Eingabefelder löschen
		for iCurrent in range(len(self.__lEdits)):
			#Wenn gelockt dann nicht löschen
			if self.__lLocks[iCurrent].isChecked():
				continue
			#Ansonsten löschen
			self.__lEdits[iCurrent].setText("")

		#Eingabefeld ohne Inhalt suchen
		for LECurrent in self.__lEdits:
			#Eingabefeld ohne Inhalt?
			if (LECurrent.text() != ""):
				continue

			#Anwählen
			LECurrent.setFocus()
			#Fertig
			return
				
		
		#Ansonsten auf erstes Element setzen
		self.__lEdits[0].setFocus()





	##  @brief Removes a selected set of coordinates form the table<br><em>Entfernt den ausgewählten Satz Koordinaten aus der Tabelle</em>
	#
	# If multiple sets of coordinates are selected, only the highest in order set is removed. Currently it is not possible to remove
	# multiple sets of coordinates.
	# This function is attached to remove-button in QtDesigner.
	# <br><em>Falls mehrere Koordinatensätze ausgewählt sind, wird nur der oberste Satz entfernt. Es ist derzeit nicht nötig
	# mehrere Sätze Koordinaten zu entfernen.
	# Diese Funktion ist an den Entfernen-Button im QtDesigner gehangen.
	# </em>
	#
	# @sa add()
	def remove(self):
		#Ausgewählte Items organisieren
		iSelectedIndexes = self.TWCoordinates.selectedIndexes()

		#Keine Items ausgewählt?
		if (len(iSelectedIndexes) == 0):
			return

		#Einträge löschen
		self.TWCoordinates.removeRow(iSelectedIndexes[0].row())
		#Eintrag darüber auswählen
		if (self.TWCoordinates.rowCount() != 0):
			self.TWCoordinates.selectRow(iSelectedIndexes[0].row())





	##  @brief Pushs up an entry in the table of coordinates<br><em>Verschiebt einen Eintrag in der Koordinatentabelle nach oben</em>
	#
	# Is used to correctly reorder the sets of coordinates.
	# This function is attached to up-button in QtDesinger.
	# <br><em>Wird zur korrekten Anordnung oder Umschichtung der Koordinatensätze benötigt.
	# Diese Funktion ist an den Hoch-Button im QtDesigner gehangen
	# </em>
	#
	# @sa down(), add()
	def up(self):
		#Selectierte Items holen
		pSelectedRanges = self.TWCoordinates.selectedRanges()

		#Keine da?
		if len(pSelectedRanges) == 0:
			return

		#Zuverschiebende Zeile holen
		pRows = pSelectedRanges[0]
		
		#Schon ganz oben?
		if pRows.topRow() == 0:
			return

		#Neue Zeile unterhalb einfügen
		self.TWCoordinates.insertRow(pRows.bottomRow()+1)

		#Zeilen oberhalb nach unten kopieren
		for i in range(len(self.__lHeader)):
			self.TWCoordinates.setItem(pRows.bottomRow()+1, i, QTableWidgetItem(self.TWCoordinates.takeItem(pRows.topRow()-1, i).text()))

		#Alte Zeile entfernen
		self.TWCoordinates.removeRow(pRows.topRow()-1)





	##  @brief Pushs down an entry in the table of coordinates<br><em>Verschiebt einen Eintrag in der Koordinatentabelle nach unten</em>
	#
	# Is used to corretctly reorder the sets of coordinates.
	# This function is attached to down-button in QtDesigner.
	# <br><em>Wird zur korrekten Anordnung oder Umschichtung der Koordinatensätze benötigt.
	# Diese Funktion ist an den Runter-Button im QtDesigner gehangen
	# </em>
	#
	# @sa up(), add()
	def down(self):
		#Selectierte Items holen
		pSelectedRange = self.TWCoordinates.selectedRanges()

		#Keine vorhanden?
		if len(pSelectedRange) == 0:
			return

		#Zuverschiebende Zeile holen
		pRows = pSelectedRange[0]

		#Schon ganz unten?
		if pRows.bottomRow() == self.TWCoordinates.rowCount()-1:
			return

		#Neue Zeile oberhalb einfügen
		self.TWCoordinates.insertRow(pRows.topRow())

		#Zeilen unterhalb nach oben kopieren
		for i in range(len(self.__lHeader)):
			self.TWCoordinates.setItem(pRows.topRow(), i, QTableWidgetItem(self.TWCoordinates.takeItem(pRows.bottomRow()+2, i).text()))

		#Alte Zeile entfernen
		self.TWCoordinates.removeRow(pRows.bottomRow()+2)





	##  @brief Changes the title of QCoordinatesInputWidget<br><em>Ändert den Titel des QCoordinatesInputWidget</em>
	#
	# @param strTitle - The new title of QCoordinatesInputWidget<br><em>Der neue Titel des QCoordinatesInputWidget</em>
	#
	# @sa __bCustomTitle
	def setText(self, strTitle):
		#Name der Gruppenbox ändern
		self.GBCoordinates.setTitle(strTitle)
		#Benutzerspezifischer-Titel zugewiesen
		self.__bCustomTitle = True





	##  @brief Sets the dimensions of coordinates<br><em>Setzt die Dimensionen der Koordinaten</em>
	#
	# Therefore the number of inputwidgets, locks and columns is adjusted depending on the number of dimensions the passed list contains.
	# The names of the dimension is also taken from the list.
	# <br><em>Dazu wird entsprechend der Dimension der Einträge der übergebenen Liste die Anzahl an
	# Eingabefeldern, Lockhacken und Tabellenspalten angepasst. Die Namen der Dimensionen werden ebenfalls
	# dem übergebenen Parameter entnommen.
	# </em>
	#
	# @param lDimensions - List containing the names of the dimensions and also the number<br><em>Liste, welche die Namen der Dimensionen enthält und damit die Anzahl der Dimensionen angibt</em>
	#
	# @sa __lHeader
	def setDimensions(self, lDimensions):
		#Liste leeren
		self.TWCoordinates.clear()
		#Header speichern
		self.__lHeader = lDimensions
		#Neue Spaltenanzahl setzen
		self.TWCoordinates.setColumnCount(len(lDimensions))
		#Header setzen
		self.TWCoordinates.setHorizontalHeaderLabels(lDimensions)

		#Alle Vorhandenen LineEdits, Labels und Checkboxen entfernen
		for LECurrent in self.InputLayout.findChildren(QLineEdit):
			self.InputLayout.removeWidget(LECurrent)

		for LCurrent in self.InputLayout.findChildren(QLabel):
			self.InputLayout.removeWidget(LCurrent)

		for CBCurrent in self.InputLayout.findChildren(QCheckBox):
			self.InputLayout.removeWidget(CBCurrent)

		#Neue hinzufügen
		self.__lEdits = []
		self.__lLocks = []
		for strName in lDimensions:
			#Widget hinzufügen
			LENew = QLineEdit(self)
			#Layout anpassen
			LENew.setLayoutDirection(Qt.RightToLeft)
			#Passendes Label erstellen
			LNew = QLabel("&"+strName, self)
			LNew.setBuddy(LENew)
			#Checkbox erstellen
			CBNew = QCheckBox("&Lock", self)
			#Ins Layout einfügen
			self.InputLayout.addWidget(LENew, self.InputLayout.rowCount(), 1)
			self.InputLayout.addWidget(LNew, self.InputLayout.rowCount()-1, 0)
			self.InputLayout.addWidget(CBNew, self.InputLayout.rowCount()-1, 2)
			#Validator anhängen
			LENew.setValidator(QDoubleValidator(self))
			#Zur Liste hinzufügen
			self.__lEdits.append(LENew)
			self.__lLocks.append(CBNew)

		#Funktionalität herstellen
		for iCurrent in range(len(self.__lEdits)):
			#Letztes Element mit den Add-Button verbinden
			#if iCurrent == len(self.__lEdits)-1:
			self.__lEdits[iCurrent].connect(self.__lEdits[iCurrent], SIGNAL("returnPressed()"), self.BAdd.click)
		

			
			

	##  @brief Turns the possibility to addd sets on or off<br><em>Schaltet die Möglichkeit Sätze hinzuzufügen aus oder ein</em>
	#
	# Rhus the QCoordinatesInputWidget can also be used to manipulate sets of coordinates without the possibility to add or remove additional sets.
	# Therefore the add-button, remove-button, inputwidgets and locks are hidden.
	# <br><em>So kann das QCoordinatesInputWidget auch benutzt werden um vorgebene Koordinatensätze anzupassen ohne die Möglichkeit zubieten
	# Sätze hinzufügen zukönnen. Dazu wird nur der Hinzufügen-Button, Entfernen-Button sowie die Eingabefelder und Locks versteckt.
	# </em>
	#
	# @param bAddAvailable - Tells, whether adding is possible(true) or not(false)<br><em>Gibt an ob hinzufügen möglich ist(true) oder nicht(false)</em>
	def setAddAvailable(self, bAddAvailable):
		#bVisible = False
		##Erlauben?
		#if(bAddAvailable):
			#bVisible = True

		#Werte überenehmen
		self.BAdd.setVisible(bAddAvailable)
		self.BRemove.setVisible(bAddAvailable)
		#Labels, Checkoxen und LineEdits verstecken
		for LECurrent in self.findChildren(QLineEdit):
			LECurrent.setVisible(bAddAvailable)

		for LCurrent in self.findChildren(QLabel):
			LCurrent.setVisible(bAddAvailable)

		for CBCurrent in self.findChildren(QCheckBox):
			CBCurrent.setVisible(bAddAvailable)





	##  @brief Changes sets of coordinates to a numpy-array<br><em>Wandelt die eingegebenen Koordinatensätze in ein Numpy-Array um</em>
	#
	# Is needed to feed the data into the corresponding interface.
	# Basically its just a reorder of table entrys in a list.
	# The form of numpyarray is as follows: [[X1, Y1, ...], [X2,Y2, ...], [X3,Y3, ...], ... , [Xn, Yn, ...]]
	# <br><em>Wird benötigt um die Daten in die entsprechenden Anschlußstellen zufüttern.
	# Im Grunde ist es nur eine Umordnung der Tabelleneinträge in eine Liste.<br>
	# Das NumpyArray hat die Form: [[X1, Y1, ...], [X2,Y2, ...], [X3,Y3, ...], ... , [Xn, Yn, ...]]
	# </em>
	#
	# @return Returns the numpyarray<br><em>Gibt das Numpy-Array zurück</em>
	#
	# @sa QProfile2DWidget::QProfile2DWidget::OnGetOutput(), QBladeRowWidget::QBladeRowWidget::OnGetOutput(), setCoordinatesFromNumpy()
	def coordinatesAsNumpy(self):
		#Rücdkgabevariable erstellen
		lResult = zeros([self.TWCoordinates.rowCount(), self.TWCoordinates.columnCount()])
		
		#Durch alle Einträge gehen
		for iCurRow in range(self.TWCoordinates.rowCount()):
			#Alle Einträge einfügen;  columCount = Anzahl der Dimensionen
			for iCurCol in range(self.TWCoordinates.columnCount()):
				#Entsprechenden Wert holen
				lResult[iCurRow][iCurCol] = float(self.TWCoordinates.item(iCurRow, iCurCol).text().replace(',','.'))

		#Numpy-Array zurückgeben
		return lResult
				
			
		


	##  @brief Changes the passed numpyarray to sets of coordinates<br><em>Wandelt das übergebene NumpyArray in Koordinatensätze um</em>
	#
	# Is needed to feed the data into the corresponding interface.
	# Basically its just a reorder of table entrys in a list.
	# The form of numpyarray is as follows: [[X1, Y1, ...], [X2,Y2, ...], [X3,Y3, ...], ... , [Xn, Yn, ...]]
	# <br><em>Wird benötigt um die Daten aus der entsprechenden Anschlußstelle zulesen.
	# Im Grunde ist es nur eine Umordnung einer Liste in die Tabelleneinträge.
	# Das übergebene NumpyArray hat die Form: [[X1, Y1, ...], [X2,Y2, ...], [X3,Y3, ...], ... , [Xn, Yn, ...]]
	# </em>
	#
	# @param lCoordinates The numpyarray to read<br><em>Das einzulesende NumpyArray</em>
	#
	# @sa QProfile2DWidget::QProfile2DWidget::setFromObject(), QBladeRowWidget::QBladeRowWidget::setFromObject(), coordinatesAsNumpy()
	def setCoordinatesFromNumpy(self, lCoordinates):
		#Liste leeren
		self.TWCoordinates.clear()
		#Alte Spaltenanzahl wiederherstellen
		self.TWCoordinates.setColumnCount(len(self.__lHeader))
		#Header setzen
		self.TWCoordinates.setHorizontalHeaderLabels(self.__lHeader)
		#Zeilenanzahl setzen
		self.TWCoordinates.setRowCount(len(lCoordinates))
		#Durch die Einträge gehen
		for iRow in range(len(lCoordinates)):
			#Alle Einträge hinzufügen
			for iCol in range(len(lCoordinates[iRow])):
				#Wert übernehmen
				self.TWCoordinates.setItem(iRow, iCol, QTableWidgetItem(str(lCoordinates[iRow][iCol])))




	
	##  @brief Copys an other QCoordinatesInputWidget to this one<br><em>Kopiert die Daten eines anderen QCoordinatesInputWidget in dieses</em>
	#
	# Therefore simply the QCoordinatesInputWidget to be read is converted to a numpyarray, which afterwards is set in this widget.
	# Finally the validator and checkboxs are set.
	# <br><em>Dazu wird einfach das zulesende QCoordinatesInputWidget in ein NumpyArray konvertiert, der dann in diesem hier
	# wieder gesetzt wird. Am Ende werden die Validatoren und Checkboxen gesetzt.
	# </em>
	#
	# @param CIWidget - QCoordinatesInputWidget to copy from<br><em>QCoordinatesInputWidget das kopiert werden soll</em>
	def copyFrom(self, CIWidget):
		#Liste leeren
		self.setCoordinatesFromNumpy(CIWidget.coordinatesAsNumpy())
		#Validator übernehmen
		for iIndex in range(len(self.__lEdits)):
			#Einstellungen übernehmen
			self.__lEdits[iIndex].setValidator(CIWidget.__lEdits[iIndex].validator())
			#Checkboxenstatus übernehmen
			self.__lLocks[iIndex].setChecked(CIWidget.__lLocks[iIndex].isChecked())





	##  @brief Is always called on resize<br><em>Wird immer dann aufgerufen, wenn die Größe von QCoordinatesInputWidget angepasst wird</em>
	#
	# Is supposed to change the size of the content, but it doesn't work as expected.
	# <br><em>Sollte eigentlich die Größe des Inhaltes entsprechend anpassen, aber funktioniert irgendwie nicht so, wie es soll.
	# </em>
	#
	# @param pEvent - The calling resize event<br><em>Das ausgelösende Größenevent</em>
	#
	# @todo Set size so that all columns are equal<br><em>Größe so setzen, das alle Spalten gleich breit sind</em>
	def resizeEvent(self, pEvent):
		#Noch keine Spalten vorhanden?
		if (self.TWCoordinates.columnCount() == 0):
			return

		#70 Pixel platz lassen für den Scrollbar an der rechten Seite und den Index am Anfang
		iSize = (self.TWCoordinates.width()-30) / self.TWCoordinates.columnCount()
		for iCurColumn in range(self.TWCoordinates.columnCount()):
			#Von der breite noch 5 abziehen, damit ein Scrollbalken erscheint und die Unterteilungen passen
			self.TWCoordinates.setColumnWidth(iCurColumn, iSize)





	##  @brief Sets the domain of definition of the input<br><em>Setzt den Definitionsbereich der Eingaben</em>
	#
	# Is used to set limits to the domain of definition (e.g. between 0 and 1). Therefore the domain of definition is
	# set in inputwidgets and table cells.
	# <br><em>Wird benötigt um den Definitionsbereich einzugrenzen (zBsp zwischen 0 und 1). Dazu wird dieser
	# Definitionsbereich in den Eingabefelder und Tabellenzellen eingestellt.
	# </em>
	#
	# @param iIndex - Index of coordinate to set the domain of definition for<br><em>Index der Koordinate, für die der Definitionsbereich eingestellt werden soll</em>
	# @param fMinValue - Minimal valid value<br><em>Minimalster gültiger Wert</em>
	# @param fMaxValue - Maximal valid value<br><em>Maximalster gültiger Wert</em>
	#
	# @sa QCoordinatesInputDelegate::setRange()
	def setRange(self, iIndex, fMinValue=float('-inf'), fMaxValue=float('inf')):
		#Index gültig?
		if (iIndex < 0) or (iIndex >= len(self.__lHeader)):
			return

		#Editor Validator entsprechenden Bereich verpassen
		#1000 ist der Standardwert
		self.__lEdits[iIndex].validator().setRange(fMinValue, fMaxValue, 1000)
		#Neues Delegate
		pDelegate = QCoordinatesInputDelegate()
		#Neuen Wertebereich setzen
		pDelegate.setRange(fMinValue, fMaxValue)
		#Delegate für die ensprechende Spalte anpassen
		self.TWCoordinates.setItemDelegateForColumn(iIndex, pDelegate)



	##  @brief Fires when something changes in QCoordinatesInputWidget<br><em>Feuert falls sich etwas im QCoordinatesInputWidget verändert</em>
	#
	# Is used to determin changes as far as saving or something is concerned.
	# <br><em>Wird zur Ermittlung von Änderungen zum Speichern und ähnliches benötigt.
	# </em>
	#
	# @sa QBaseWidget::QBaseWidget::contentChanged()
	def contentChanged(self):
		self.emit(SIGNAL("contentChanged()"))


	## @brief Renames all labels as the corresponding language<br><em>Bennent alle Label in die entsprechende Sprache um</em>
	#
	#  Reimplemented from QBaseWidget::retranslateStrings(). Used for localisation purpose. The texts inputed in QtDesigner will be overwritten.
	#  <br><em>Diese Funktion ist reimplementiert aus QBaseWidget::retranslateStrings(). Wird zur Lokalisation verwendet. Die im QtDesigner eingetragenen Texte für
	#  Steuerelemente werden überschrieben.</em>
	#
	#  @param bPassDown - tells whether to pass down the retranslation to underlying Widgets<br><em>Gibt an, ob die Übersetzung an tiefer liegende Widgets übergeben werden soll.</em>
	def retranslateStrings(self, bPassDown = True):
		#Falls kein benutzerspezifischer Titel gewählt
		if (not self.__bCustomTitle):
			#Titel anpassen
			self.GBCoordinates.setTitle(self.tr("Coordinates", "Titel der Coordinatesgruppenbox"))
		
		
	



## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell entfernen
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pTestWidget = QCoordinatesInputWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pTestWidget)

	#Variabel für Testfall
	bGetTest = False
		#Dialog anzeigen
	pDlg.show()
	#Durch Weiter Testfälle gehen
	for strParam in sys.argv[2:]:
		#Daten hineinladen?
		#2 Dimensional
		if strParam == 'setTest':
			#Liste mit Testwerten
			#TODO: Listenformat stimmt nichtmehr
			lListe = [[0.0, 0.0], [1.0, 0.5], [2.0, 1.0], [3.0, 1.5]]
			#Liste hinzufügen
			pTestWidget.setCoordinatesFromNumpy(lListe)
		#Daten hinterher ausgeben?
		if strParam == 'getTest':
			#Daten hinterher ausgeben
			bGetTest = True
		#Mehr Dimensionen testen?
		#3 Dimensional
		if strParam == 'DimTest3':
			#Neuen Header setzen
			pTestWidget.setDimensions(['X','Y','Z'])
		#2 Dimensional
		if strParam == 'DimTest2':
			#Neuen Header setzen
			pTestWidget.setDimensions(['U', 'V'])
		#1 Dimensional
		if strParam == 'DimTest1':
			#Neuer Header setzen
			pTestWidget.setDimensions(['Wert'])
			pTestWidget.setRange(0, 0.0, 1.0)
		#Hinzufügen verbieten?
		if strParam == 'NoAdd':
			#Wert setzen
			pTestWidget.setAddAvailable(False)
			
	#Ausführen
	iResult = pApp.exec_()
	#Läuft ein Gettest?
	if bGetTest:
		#Eingabe ausgeben
		print pTestWidget.coordinatesAsNumpy()
	
	sys.exit(iResult)
