#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009


import sys
from PyQt4.QtGui import QDialog, QApplication, QTextCursor
from Ui_StatusDialog import Ui_StatusDialog as StatusDialog

## @brief Class for statusdialog for calculation<br><em>Klasse für den Statusdialog der Berechnung</em>
#
# The statusdialog shows the progress, debug and possible error messages.
# Therefore it hooks on the standard output and writes the output of print to its own window.
# Additionally it contains a progress bar, which can be shown. The output can also be saved to a file.<br>
# To use the progess functionality of QStatusdialog, the calculating structure must have a step functionpointer, which it calls after every important step.
# To this function pointer step() needs to be attached.
# The layout of this widget in QtDesignerform is contained in ./UI/StatusDialog.ui, which can be converted to UI_StatusDialog by ./CreateGUI.sh script using pyuic4
# and loaded afterwards.
# <br><em>Der Statusdialog zeigt den Fortschritt, Debug und eventuelle Fehlermeldungen an.
# Dazu hängt er sich an die Standardausgabe und gibt die Ausgabe von print in seinem Fenster aus.
# Zusätzlich enthält er einen Fortschrittsbalken, der den aktuellen Fortschritt anzeigen kann
# Die Ausgabe kann auch in eine Datei gespeichert werden.<br>
# Um die Fortschrittsfunktionalität von QStatusdialog nutzen zukönnen, muß die berechnende Struktur einen step-Funktionszeiger besitzen, den sie nach
# jedem wichtigen Schritt aufruft. An diesen Funktionszeiger muß manuell step() gehangen werden.
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/StatusDialog.ui, welche vom ./CreateGUI.sh Script mittels pyuic4 in UI_StatusDialog konvertiert
# und dann geladen wird.
# </em>
#
# @sa QProfile2DWidget::QProfile2DWidget::OnPerformCalc(), QBladeRowWidget::QBladeRowWidget::OnPerformCalc(), QTurboMachineWidget::QTurboMachineWidget::OnPerformCalc()
class QStatusDialog(QDialog, StatusDialog):

	## @brief Constructor of QStatusDialog<br><em>Konstruktor von QStatusDialog</em>
	#
	# Attaches the standard output to QStatusDialog.
	# <br><em>Hängt die Standardausgabe an den QStatusDialog um.
	# </em>
	#
	# @sa write(), closeEvent()
	def __init__(self, pParent = None):
		#Dialog initialisieren
		QDialog.__init__(self, pParent)
		#Ui initialisieren
		self.setupUi(self)
		#Standardoutput holen
		self.__pOldStdOut = sys.stdout
		#Formular als neuen sys.stdout setzen
		sys.stdout = self
		#Progressbar Standardmäßig nicht anzeigen
		self.PBProgress.hide()
		#Nachrichtenfenster verstecken
		self.PTEOutput.hide()





	## @brief Function, which is called when something is to prompt on standard output<br><em>Funktion die Aufgerufen wird, wenn etwas über die Standardausgabe ausgegebenen werden soll</em>
	#
	# But standard output is redirected, thus the output needs to be handled.Therefore the text is written to the text window and the application
	# processes further events, otherwise it would seem like the program got stucked.
	# <br><em>Allerdings ist die Standardausgabe ja umgeleitet, sodass man sich selbst um die Ausgabe kümmern muß.
	# Deshalb wird der Text ins Textfenster geschrieben und die Application verarbeitet alle weiteren Events, da es sonst so wirken
	# würde als hätte sich das Programm aufgehangen.
	# </em>
	#
	# @param strText - Text to print on standard output<br><em>Text der auf der Standardausgabe ausgegeben werden soll</em>
	#
	# @sa __init__(), step()
	def write(self, strText):
		#Nachrichtenfenster anzeigen
		self.PTEOutput.show()
		#In das Output-Edit schreiben
		self.PTEOutput.setPlainText(self.PTEOutput.toPlainText().append(strText))
		#Cursor ans Ende des Textes setzen
		self.PTEOutput.moveCursor(QTextCursor.End)
		self.PTEOutput.ensureCursorVisible()
		#Applikation aktualisieren
		QApplication.processEvents()





	## @brief Stepfunction for progress bar<br><em>Schrittfuntkion für den Fortschrittsbalken</em>
	#
	# Increases the progress by one and processes further events.
	# <br><em>Erhöht den Fortschritt um eins und verarbeitet dann sonstige Events.
	# </em>
	#
	# @sa write(), setProgressSteps()
	def step(self):
		#Eins mehr anzeigen
		self.PBProgress.setValue(self.PBProgress.value()+1)
		#Applikation aktualisieren
		QApplication.processEvents()





	## @brief Setzt the number of possible progresses in progress bar<br><em>Setzt die Anzahl an Fortschritt im Fortschrittsbalken</em>
	#
	# The number should be calculated previously depending on the step calls of calculating class as well as the passes needed
	# <br><em>Die Anzahl sollte im Vorfeld abhänig von den step-Aufrufen der berechnenden Klasse sowie den durchläufen berechnet werden.
	# </em>
	#
	# @param iSteps - Number of steps<br><em>Anzahl an Schritten</em>
	#
	# @sa step()
	def setProgressSteps(self, iSteps):
		#Progressbar anzeigen
		self.PBProgress.show()
		#Schritte setzen
		self.PBProgress.setMaximum(iSteps)
		#Startwert ist 0
		self.PBProgress.setValue(0)





	## @brief Saves the output to a file<br><em>Speichert die Ausgabe in eine Datei</em>
	#
	# Therefore a filedialog is opened and the text of the window is written to a file.
	# This function is attached to the "save to file"-button in QtDesigner.
	# <br><em>Dazu wird ein Dateidialog geöffnet und dann der Text des Fensters in eine
	# Datei geschrieben.
	# Diese Funktion ist an den "In Datei speichern"-Button im QtDesigner gehangen.
	# </em>
	#
	# @sa write()
	def saveToFile(self):
		#Inhalt in eine Datei speichern
		#Datei speichern Dialog öffnen
		strFileName = QFileDialog.getSaveFileName(self, self.tr("In Datei speichern"))
		#Namen ausgewählt?
		if (strFileName == ""):
			#Offenbar nicht
			return

		#Datei öffnen
		pDatei = open(strFileName, "w")
		#Inhalt in Datei schreiben
		pDatei.write(self.PTEOutput.toPlainText())
		#Datei schließen
		pDatei.close()





	## @brief Makes sure, that standard output is reset.<br><em>Sorgt dafür, dass die Standardausgabe wieder zurückgesetzt wird</em>
	#
	# This function is attached to close-button in Qt-Designer.
	# <br><em>Diese Funktion ist an den Schließen-Button im QtDesigner gehangen.
	# </em>
	#
	# @param pEvent - The close-event<br><em>Das Schließen-Event</em>
	#
	# @sa __init__(), write()
	def closeEvent(self, pEvent):
		#Standardoutput wiederherstellen
		sys.stdout = self.__pOldStdOut
		#Standardclose-Event benutzen
		QDialog.closeEvent(self, pEvent)





	## @brief Calling this function after calculation makes sure, that the dialog can be closed<br><em>Aufruf nach der Berechnung sorgt dafür, dass der Dialog geschloßen werden kann</em>
	#
	# During calculation the QStatusDialog is blocked and can not be closed. This function deblocks it, thus the
	# output can be saved and the dialog be closed. Hence it should not be forgotten to call the function after calculation
	# or if the QStatusDialog is not needed anymore.
	# <br><em>Während der Berechnung ist der QStatusDialog blockiert und kann nicht geschloßen werden. Diese Funktion
	# gibt ihn wieder frei, damit die eventuelle Ausgabe gespeichert und der Dialog geschloßen werden kann.
	# Deshalb sollte man nicht vergessen diese Funktion am Ende der Berechnung oder wenn der Statusdialog nichtmehr benötigt wird
	# aufzurufen.
	# </em>
	def outputFinished(self):
		#Schließen-Button aktivieren
		self.BOk.setEnabled(True)
		#In Datei speichern aktivieren
		self.BSaveToFile.setEnabled(True)
		#Falls keine Ausgabe erfolgt:
		if (self.PTEOutput.toPlainText() == ""):
			#Standardoutput wiederherstellen
			sys.stdout = self.__pOldStdOut
			#Dialog beenden
			self.close()
			





## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell entfernen
if(__name__ == "__main__"):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QStatusDialog()
	#Dialog anzeigen
	pDlg.show()
	#Applikation ausführen
	pApp.exec_()