#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Balint Balassa                             *
#*   balint.balassa@gmx.de                                                 *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Oliver Borm
# @author Julius Fellerhoff
# @author Sebastian G. Barthmes
# @author Balint Balassa
# @date 15.05.2009

import sys, os, shutil, math, string, nurbsToolSet
from numpy import *
from pythonnurbs import NurbsMatrix, NurbsSurface, NurbsCurve, NurbsVector, NurbsPoint
from scipy.optimize.minpack import bisection
from camberSurface import camberSurface
from CGNS import CGNS


## @class CgnsCreator
# @brief This class can be used for providing a two-dimensional CGNS file. It is called in the class SCMMesher.
class CgnsCreator:
	
	
	# @function __init__
	# @brief Init function. Initializes the CgnsCreator instance.
	# @param self Self pointer.
	# @param case Directory for output files in string format.
	def __init__(self, case):
		
		self.case = case + "/"

		#SW: Ich hab hier mal was fürs Debugging umgestellt
		self.debugLevel = 0
		if(self.debugLevel > 0):
			print self.case
			
		self.ier = 0
		self.oneBlockPerRow = False
		self.mergeBlocks = True
		self.writeExponents = False
		self.CU = False
		self.staticPressure = False

		
		self.cpFt = False
		self.a11 = 3.08792717E+00
		self.a21 = 1.24597184E-03
		self.a31 = -4.23718945E-07
		self.a41 = 6.74774789E-11
		self.a51 = -3.97076972E-15
		self.a12 = 3.56839620E+00
		self.a22 = -6.78729429E-04
		self.a32 = 1.55371476E-06
		self.a42 = -3.29937060E-12
		self.a52 = -4.66395387E-13
		# The logic behind the polynomials for your convenience:
		# cp = R * (a1x + a2x*T + a3x*T**2 + a4x*T**3 + a5x*T**4)
		# if T>1000K: x=1 and else x=2

		self.Rm = 8.314472
		self.mol = 0.028964 # for air
		self.R = 0.0


		# Parameters fo SCM. Do not alter values in list by hand:
		# Use the set_parameter method to modify the parameters!
		self.parameters = {}
		self.parameters['maxScmIterations'] = [20, False]
		self.parameters['NurbsCurveDegree'] = [3, False]
		self.parameters['pointsInbetween'] = [10, False]
		self.parameters['tolerance'] = [1.0e-5, False]
		self.parameters['gammaTolerance'] = [1.0e-10, False]
		self.parameters['approxError'] = [5, False]
		self.parameters['approxSL'] = [False, False]
		self.parameters['approxQO'] = [False, False]
		self.parameters['interpSL'] = [False, False]
		self.parameters['numberSL'] = [0, False]
		self.parameters['derivativeMode'] = [1, False]
		self.parameters['startit'] = [False, False]
		self.parameters['minDistError'] = [1.0e-6, False]
		self.parameters['minDistsizeSearch'] = [0.5, False]
		self.parameters['minDistsep'] = [9, False]
		self.parameters['minDistmaxiter'] = [100, False]
		self.parameters['minDistuMin'] = [0.0, False]
		self.parameters['minDistuMax'] = [1.0, False]
		self.parameters['relaxfac'] = [0.5, False]
		self.parameters['p_min'] = [1000.0, False]
		self.parameters['minimalVelocity'] = [0.01, False]
		self.parameters['solver_approx'] = [False, False]
		self.parameters['maxError'] = [0.00001, False]
		self.parameters['MaxIt'] = [100, False]
		self.parameters['streamlineTolerance'] = [0.005, False]
		self.parameters['MkritMin'] = [0.99, False]
		self.parameters['MkritMax'] = [1.01, False]
		self.parameters['pFaktor'] = [0.9, False]

		self.cell_dim = 2	# constant, because 2D programm
		self.phys_dim = 3	# constant, because 2D programm
		
		# Initializing all names
		self.filename = "scm0.cgns"	# convention for scm input filename
		self.baseName = "TurboMachine"
		self.blockName = "Block-"
		
		# Initializing required pointers
		self.filePointer = CGNS.intp()
		self.basePointer = CGNS.intp()
		self.zoneIndexPointer = CGNS.intp()
		
	
	# @function __call__
	# @brief Call function. Creates a CGNS file in the given directory, with some initial data.
	# @param self Self pointer.
	# @param massflow Massflow for the turbomachine.
	def __call__(self, massflow):
		
		if self.debugLevel > 0:
			print "File:", self.case + self.filename
		
		self.sizeZones = self.cell_dim * 3
		self.ier = CGNS.cg_open(self.case+self.filename, CGNS.CG_MODE_WRITE, self.filePointer)
		self.debug("cg_open")
		
		self.ier = CGNS.cg_base_write(self.filePointer.value(), self.baseName, self.cell_dim, self.phys_dim, self.basePointer)
		self.debug("cg_base_write")
		
		self.ier = CGNS.cg_goto(self.filePointer.value(), self.basePointer.value(), "end")
		self.debug("cg_goto")
		
		self.ier = CGNS.cg_state_write("Dimensional")
		self.debug("cg_state_write")
		self.ier = CGNS.cg_dataclass_write(CGNS.Dimensional)
		self.debug("cg_dataclass_write")
		self.ier = CGNS.cg_units_write(CGNS.Kilogram, CGNS.Meter, CGNS.Second, CGNS.Kelvin, CGNS.Radian)
		self.debug("cg_units_write")
		
		# Global parameters
		self.ier = CGNS.cg_descriptor_write("Massflow", str(massflow))
		self.debug("cg_descriptor_write")
		
		if self.R == 0.0:
			self.R = self.Rm / self.mol
		self.ier = CGNS.cg_descriptor_write("Gasconstant", str(self.R))
		self.debug("cg_descriptor_write")

		if self.cpFt == True:
			self.ier = CGNS.cg_descriptor_write("cp-Polynomial", self.doubles2string())
			self.debug("cg_descriptor_write")

		# Writes all specified parameters into the CGNS file.
		for key in self.parameters:
			if self.parameters[key][1] == True:
				self.ier = CGNS.cg_descriptor_write(key, str(self.parameters[key][0]))
				self.debug("cg_descriptor_write")
		
		
		# close cgns file
		self.ier = CGNS.cg_close(self.filePointer.value())
		self.debug("cg_close")


	# @function set_parameter
	# @brief Sets values as parameters and checks for correct type.
	# @param self Self pointer.
	# @param par Parameter to be set.
	# @param value Value of the parameter to be set.
	def set_parameter(self, par, value):

		if type(value) == type(self.parameters[par][0]):
			self.parameters[par][0] = value
			self.parameters[par][1] = True
		elif type(value) is int and type(self.parameters[par][0]) is float:
			self.parameters[par][0] = float(value)
			self.parameters[par][1] = True
		else:
			print str(value) + ' ' + str(type(value)) + ' is not of correct type for parameter ' + par
			print 'Use ' + str(type(self.parameters[par][0])) + ' instead.'
			sys.exit()

	
	# @function doubles2string
	# @brief Converts the Polynomial koefficients for cp calculations (float) into a string.
	# @param self Self pointer.
	def doubles2string(self):

		polynomialString = str(self.a11) + " " + str(self.a21) + " " + str(self.a31) + " " + str(self.a41) + " " + str(self.a51) + " "
		polynomialString += str(self.a12) + " " + str(self.a22) + " " + str(self.a32) + " " + str(self.a42) + " " + str(self.a52)
		return polynomialString
		
	
	# @function debug
	# @brief Helps debugging CGNS functions.
	# @param self Self pointer.
	# @param message Message that is printed if an error occurs that invokes this method.
	def debug(self, message):
		
		if self.debugLevel >= 1 and self.ier != 0:
			print message, "\tError:", self.ier
			CGNS.cg_error_exit	
	
	# @function write_block
	# @brief Writes information about a single block (zone) into the CGNS file.
	# @param self Self pointer.
	# @param blockNumber Number of the block. First is 1.
	# @param ax_cells Number of cells of the block in axial direction.
	# @param rad_cells Number of cells of the block in radial direction.
	# @param axCellsBefore Number of cells of the current row before the current block in axial direction.
	# @param scmInstance An Instance of the class SCMMesher.
	def write_block(self, blockNumber, ax_cells, rad_cells, axCellsBefore, scmInstance):
		
		if self.debugLevel > 0:
			print "File:", self.case+self.filename
		
		self.ier = CGNS.cg_open(self.case+self.filename, CGNS.CG_MODE_MODIFY, self.filePointer)
		self.debug("cg_open")
		self.ier = CGNS.cg_goto(self.filePointer.value(), self.basePointer.value(), "end")
		self.debug("cg_goto")
		
		self.zoneArrayp[0] = ax_cells + 1	# axial-coordinate
		self.zoneArrayp[1] = rad_cells + 1	# radial-coordinate
		#cell size
		self.zoneArrayp[2] = ax_cells
		self.zoneArrayp[3] = rad_cells
		#boundary vertex size (always zero for structured grids)
		self.zoneArrayp[4] = 0
		self.zoneArrayp[5] = 0
		
		numberVertices = self.zoneArrayp[0] * self.zoneArrayp[1]
		
		CoordX = CGNS.doubleArray(numberVertices)
		CoordY = CGNS.doubleArray(numberVertices)
		CoordZ = CGNS.doubleArray(numberVertices)
		betaR = CGNS.doubleArray(numberVertices)
		betaZ = CGNS.doubleArray(numberVertices)
		sigma = CGNS.doubleArray(numberVertices)
		entropy = CGNS.doubleArray(numberVertices)
		if self.staticPressure == True:
			staticPressure = CGNS.doubleArray(numberVertices)
		if blockNumber == 1:
			totalPressure = CGNS.doubleArray(numberVertices)
			totalTemperature = CGNS.doubleArray(numberVertices)
			if self.CU == True:
				c_u = CGNS.doubleArray(numberVertices)
		
		bladed = False
		# Sets bladed to True if current block is bladed
		if scmInstance.pointArray2D[axCellsBefore+2][1][4] != 1:
			bladed = True
		
		for j in range(self.zoneArrayp[1]):	# radial loop
			for k in range(self.zoneArrayp[0]):	# axial loop
				l = j * self.zoneArrayp[0] + k	# j*Length of axial + k
				axVerticesBefore = 0
				if axCellsBefore > 0:
					axVerticesBefore = axCellsBefore
				CoordX[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][0]
				CoordY[l] = 0
				CoordZ[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][1]
				if bladed == True:	# If block is bladed
					betaR[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][2]
					betaZ[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][3]
					sigma[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][4]
				else:			# If block is not bladed
					betaR[l] = math.pi/2.0
					betaZ[l] = math.pi/2.0
					sigma[l] = 1
				
				entropy[l] = scmInstance.pointArray2D[k+axVerticesBefore][j][5]
				if self.staticPressure == True:
					if self.debugLevel > 1:
						print "rowCellsBefore", self.rowCellsBefore, "axCellsBefore", axCellsBefore
					staticPressure[l] = scmInstance.staticPressure[self.rowCellsBefore+axVerticesBefore+k]
				if blockNumber == 1:
					totalTemperature[l] = scmInstance.totalTemperature[j]
					totalPressure[l] = scmInstance.totalPressure[j]
					if self.CU == True:
						c_u[l] = scmInstance.c_u[j]
					
		if self.debugLevel > 0:
			print "blockNumber", blockNumber, "ax_cells", ax_cells, "rad_cells", rad_cells, "axVerticesBefore", axVerticesBefore
		
		self.zoneName = self.blockName + str(blockNumber)
		self.ier = CGNS.cg_zone_write(self.filePointer.value(), self.basePointer.value(), self.zoneName, self.zoneArrayp, CGNS.Structured, self.zoneIndexPointer)
		self.debug("cg_zone_write block "+str(blockNumber))
		
		N = CGNS.intp()
		self.ier = CGNS.cg_nzones(self.filePointer.value(), self.basePointer.value(), N)
		self.debug("cg_nzones block "+str(blockNumber))

		C = CGNS.intp()
		self.ier = CGNS.cg_coord_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), CGNS.RealDouble, "CoordinateX", CoordX, C)	# Doesnt like [X,Z] coordinate system
		self.debug("cg_coord_write block X "+str(blockNumber))
		self.ier = CGNS.cg_coord_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), CGNS.RealDouble, "CoordinateY", CoordY, C)	# Doesnt like [X,Z] coordinate system
		self.debug("cg_coord_write block Y "+str(blockNumber))
		self.ier = CGNS.cg_coord_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), CGNS.RealDouble, "CoordinateZ", CoordZ, C)	# Doesnt like [X,Z] coordinate system
		self.debug("cg_coord_write block Z "+str(blockNumber))
		
		S = CGNS.intp()
		self.ier = CGNS.cg_sol_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), "FlowSolution", CGNS.Vertex, S)
		self.debug("cg_sol_write block "+str(blockNumber))
		self.ier = CGNS.cg_goto(self.filePointer.value(), self.basePointer.value(), "Zone_t", self.zoneIndexPointer.value(), "FlowSolution_t", S.value(), "end")
		self.debug("cg_goto block "+str(blockNumber))
		F = CGNS.intp()
		self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "Entropy", entropy, F)
		self.debug("cg_field_write "+str(blockNumber))
		
		BR = CGNS.intp()
		self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "BetaR", betaR, BR)
		self.debug("cg_field_write "+str(blockNumber))
		
		BZ = CGNS.intp()
		self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "BetaZ", betaZ, BZ)
		self.debug("cg_field_write "+str(blockNumber))
		
		Sp = CGNS.intp()
		self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "sigma", sigma, Sp)
		self.debug("cg_field_write "+str(blockNumber))
		
		if self.staticPressure == True:
			SPp = CGNS.intp()
			self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "Staticpressure", staticPressure, SPp)
			self.debug("cg_field_write "+str(blockNumber))
		
		if blockNumber == 1:
			TT = CGNS.intp()
			self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "Totaltemperature", totalTemperature, TT)
			self.debug("cg_field_write "+str(blockNumber))
			
			TP = CGNS.intp()
			self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "Totalpressure", totalPressure, TP)
			self.debug("cg_field_write "+str(blockNumber))
			
			if self.CU == True:
				CU = CGNS.intp()
				self.ier = CGNS.cg_field_write(self.filePointer.value(), self.basePointer.value(), self.zoneIndexPointer.value(), S.value(), CGNS.RealDouble, "c_u", c_u, CU)
				self.debug("cg_field_write "+str(blockNumber))
		
		
		absPath = "/" + self.baseName + "/" + self.blockName + str(blockNumber)
		self.ier = CGNS.cg_gopath(self.filePointer.value(), absPath)
		self.debug("cg_gopath")
		
		omega = "0.0"
		if self.oneBlockPerRow == True:
			omega = str(scmInstance.omega)
		else:
			if self.mergeBlocks == False:
				if (blockNumber+1) % 3 == 0:
					omega = str(scmInstance.omega)
			else:
				if blockNumber % 2 == 0:
					omega = str(scmInstance.omega)
		self.ier = CGNS.cg_descriptor_write("omega", omega)
		self.debug("cg_descriptor_write")
		
		self.ier = CGNS.cg_close(self.filePointer.value())
		self.debug("cg_close")
		
		
	# @function write_blocks
	# @brief Writes information about all blocks of a row into the CGNS file.
	# @param self Self pointer.
	# @param scmInstance An Instance of the class SCMMesher.
	def write_blocks(self, scmInstance):
		
		row = scmInstance.rowIndex
		self.rowCellsBefore = 0
		for i in range(row):
			self.rowCellsBefore += scmInstance.block1_cells_ax[i] + scmInstance.block2_cells_ax[i] + scmInstance.block3_cells_ax[i]
		vertex_rad = scmInstance.block_cells_rad + 1
		self.zoneArrayp = CGNS.intArray(self.sizeZones)
		if self.oneBlockPerRow == False:
			if self.mergeBlocks == False or scmInstance.nor == 1:
				self.write_block(1+row*3, scmInstance.block1_cells_ax[row], scmInstance.block_cells_rad, 0, scmInstance)
				self.write_block(2+row*3, scmInstance.block2_cells_ax[row], scmInstance.block_cells_rad, scmInstance.block1_cells_ax[row], scmInstance)
				self.write_block(3+row*3, scmInstance.block3_cells_ax[row], scmInstance.block_cells_rad, scmInstance.block1_cells_ax[row]+scmInstance.block2_cells_ax[row], scmInstance)
			elif self.mergeBlocks == True and scmInstance.nor > 1:
				# merge_blocks == True
				if row == 0:	# First 2 blocks
					self.write_block(1, scmInstance.block1_cells_ax[row], scmInstance.block_cells_rad, 0, scmInstance)
					self.write_block(2, scmInstance.block2_cells_ax[row], scmInstance.block_cells_rad, scmInstance.block1_cells_ax[row], scmInstance)
				else:	# All following blocks
					self.fake = fakeScm(scmInstance.block3_cells_ax[row-1], scmInstance.block1_cells_ax[row], scmInstance.block_cells_rad)
					self.fake(scmInstance.previousPointArray2D, scmInstance.pointArray2D, scmInstance.staticPressure)
					
					self.rowCellsBefore -= scmInstance.block3_cells_ax[row-1]
					fakeScmInstance = self.fake
					self.write_block(row*2+1, fakeScmInstance.cells_ax, self.fake.cells_rad, 0, fakeScmInstance)
					self.rowCellsBefore += scmInstance.block1_cells_ax[row-1] - 1
					self.write_block(row*2+2, scmInstance.block2_cells_ax[row], scmInstance.block_cells_rad, scmInstance.block1_cells_ax[row], scmInstance)
					if row == scmInstance.nor - 1:	# Last block
						self.write_block(row*2+3, scmInstance.block3_cells_ax[row], scmInstance.block_cells_rad, scmInstance.block1_cells_ax[row]+scmInstance.block2_cells_ax[row], scmInstance)
		else:
			#oneBlockPerRow:
			ax_cells = scmInstance.block1_cells_ax[row] + scmInstance.block2_cells_ax[row] + scmInstance.block3_cells_ax[row]
			self.write_block(row+1, ax_cells, scmInstance.block_cells_rad, 0, scmInstance)
		
		# if in last row, set all exponents to 0.0
		if self.writeExponents == True and row + 1 == scmInstance.nor:
			nzones = CGNS.intp()
			nsolutions = CGNS.intp()
			nfields = CGNS.intp()
			exp = CGNS.doubleArray(5)
			for i in range(5):
				exp[i] = 0.0
			dataType = CGNS.DataType_tp()
			dataFieldName = CGNS.charp()
			
			if self.debugLevel > 0:
				print "File:", self.case+self.filename
			
			self.ier = CGNS.cg_open(self.case+self.filename, CGNS.CG_MODE_READ, self.filePointer)
			self.debug("cg_open")
			self.ier = CGNS.cg_nzones(self.filePointer.value(), self.basePointer.value(), nzones)
			self.debug("cg_nzones")
			
			for Z in range(1,nzones.value()+1):
				self.ier = CGNS.cg_nsols(self.filePointer.value(), self.basePointer.value(), Z, nsolutions)
				self.debug("cg_nsols")
				for S in range(1, nsolutions.value()+1):
					self.ier = CGNS.cg_nfields(self.filePointer.value(), self.basePointer.value(), Z, S, nfields)
					self.debug("cg_nfields")
					for F in range(1, nfields.value()+1):
						self.ier = CGNS.cg_field_info(self.filePointer.value(), self.basePointer.value(), Z, S, F, dataType.cast(), dataFieldName)
						self.debug("cg_field_info")
						if CGNS.charP2String(dataFieldName) == "Entropy":
							print "Entropy"
							exp[0] = 1
							exp[1] = 2
							exp[2] = -2
							exp[3] = -1
							exp[4] = 0
						if CGNS.charP2String(dataFieldName) == "BetaR" or CGNS.charP2String(dataFieldName) == "BetaZ":
							print "BetaRBetaZ"
							for i in range(5):
								exp[i] = 0
							exp[4] = 1
						if CGNS.charP2String(dataFieldName) == "Totaltemperature":
							print "Totaltemperature"
							exp[0] = 0
							exp[1] = 0
							exp[2] = 0
							exp[3] = 1
							exp[4] = 0
						if CGNS.charP2String(dataFieldName) == "Totalpressure":
							print "Totalpressure"
							exp[0] = 1
							exp[1] = -1
							exp[2] = -2
							exp[3] = 0
							exp[4] = 0
						if CGNS.charP2String(dataFieldName) == "Staticpressure":
							print "Staticpressure"
							exp[0] = 1
							exp[1] = -1
							exp[2] = -2
							exp[3] = 0
							exp[4] = 0
						if CGNS.charP2String(dataFieldName) == "sigma":
							print "sigma"
							exp[0] = 1
							exp[1] = 2
							exp[2] = -2
							exp[3] = -1
							exp[4] = 0
						print self.filePointer.value(), self.basePointer.value(), Z, S, F
						self.ier = CGNS.cg_goto(self.filePointer.value(), self.basePointer.value(), "Zone_t", Z, "FlowSolution_t", S, "DataArray_t", F, "end")
						self.debug("cg_goto")
						self.ier = CGNS.cg_exponents_write(CGNS.RealDouble, exp)
						self.debug("cg_exponents_write")
			
			self.ier = CGNS.cg_close(self.filePointer.value())
			self.debug("cg_close")
	
	
	
## @class fakeScm
# @brief This class emulates an SCMMesher instance in order to merge blocks from different rows without storing any unnecessary information.
class fakeScm:
	
	
	# @function __init__
	# @brief Init function. Initializes the fakeScm instance.
	# @param self Self pointer.
	# @param ax1 Number of cells in axial direction of the previous row in the last block.
	# @param ax2 Number of cells in axial direction of the current row in the first block.
	# @param rad Number of cells in radial direction.
	def __init__(self, ax1, ax2, rad):
		
		self.ax1 = ax1
		self.ax2 = ax2
		self.cells_ax = self.ax1 + self.ax2
		self.cells_rad = rad
		self.pointArray2D = ones((self.cells_ax+1, self.cells_rad+1, 6), float)
		self.staticPressure = None
		self.rowCellsBefore = None
		
	
	# @function __call__
	# @brief Call function. Calculates the required values in the fakeScm instance.
	# @param self Self pointer.
	# @param previousPointArray2D PointArray2D from the previous row.
	# @param pointArray2D PointArray2D from the current row.
	# @param staticPressure List with static pressure values on the hub.
	def __call__(self, previousPointArray2D, pointArray2D, staticPressure):
		
		self.staticPressure = staticPressure
		for ax in range(self.ax1+1):
			for rad in range(self.cells_rad+1):
				for i in range(6):
					self.pointArray2D[ax][rad][i] = previousPointArray2D[-(self.ax1+1-ax)][rad][i]
		for ax in range(self.ax2+1):
			for rad in range(self.cells_rad+1):
				for i in range(6):
					self.pointArray2D[self.ax1+ax][rad][i] = pointArray2D[ax][rad][i]
		
		

## @class SCMMesher
# @brief This class can be used for providing a 2D-Mesh for the Streamline Curvature Method from a 3D Nurbs Blade.
# @brief Remember that the OpenFOAM command "blockMesh" must be available!
class SCMMesher:
	
	
	# @function __init__
	# @brief Init function. Initializes the SCMMesher instance with predefined parameters.
	# @param self Self pointer.
	# @param tmachine Turbomachione instance.
	def __init__(self, tmachine):
		
		self.machine = tmachine
		
		# Must be 2 digits, i.e. 1.5 or 2.0
		self.OFversion = 1.6
		self.FoamFileVersion = 2.0
		
		# Flag for using elliptic thickness distribution for sigma calculation instead of real sigma
		# Set fubar = True if there is trouble with camber surface and blade surface
		self.fubar = False
		self.maxThickness = 6 # [mm]
		
		#self.CGNS = False
		
		self.cancel = False
		self.CSDebug = 0
		self.camberSurfaceDegree = 3
		self.arraylist = []
		self.AxialGradingBlock1 = 1
		self.AxialGradingBlock2 = 1
		self.AxialGradingBlock3 = 1
		self.RadialGrading = 0.667	# The Method auto_grading redefines this value to approximate surface equal radii.
		self.CircGrading = 1
		self.omega = 0.0
		self.LEq2D = None
		self.TEq2D = None
		self.Hub3D = None
		self.Shroud3D = None
		self.length = None
		self.block_cells_rad = 6
		self.block1_cells_ax = []
		self.block2_cells_ax = []
		self.block3_cells_ax = []
		self.nor = tmachine.numberRows	# NumberOfRows
		for row in range(self.nor):
			self.block1_cells_ax.append(4)
			self.block2_cells_ax.append(6)
			self.block3_cells_ax.append(3)
		self.previousPointArray2D = None
		
		# setting file- and directory names
		self.filename='blockMeshDict'
		self.case = tmachine.dataStruct.workdir + tmachine.dataStruct.dir2DS2m # "/2D-S2m"
		self.geom = tmachine.dataStruct.workdir + tmachine.dataStruct.dirGeom
		
		self.convertToMeters = 0.001
		self.no_blocks = 3
		self.blockCoords = zeros([16, 3], 'd')
		self.hexCoords = zeros([self.no_blocks, 8], 'i')
		self.hexCells = zeros([self.no_blocks, 3], 'i')
		self.hexClust = zeros([self.no_blocks, 3], 'd')
		
		# Parameters for interpolations
		self.sep = 20
		self.tolerance = 1e-6
		self.maxIteration = 20
		
		# Parameters for Streamline Curvature Method
		self.totalPressure = []
		self.totalTemperature = []
		self.staticPressure = []
		self.c_u = []
		p_t = 101325
		T_t = 288.15
		c_u = 0.0
		p_s = 0.0
		for rad in range(self.block_cells_rad+1):
			self.totalPressure.append(p_t)
			self.totalTemperature.append(T_t)
			self.c_u.append(c_u)
		for row in range(self.nor):
			for ax in range(self.block1_cells_ax[row]+1):
				self.staticPressure.append(p_s)
			for ax in range(self.block2_cells_ax[row]+1):
				self.staticPressure.append(p_s)
			for ax in range(self.block3_cells_ax[row]+1):
				self.staticPressure.append(p_s)
		
		self.massflow = 12.0
		self.maxEntropy = 20
		self.minEntropy = 0
		self.ScmDigitsAfterComma = 12
		
		self.CGNS = False
		self.cgnsOutput = CgnsCreator(self.case)
		
		# Debugging level
		self.debug = 0
		
		
	# @function __call__
	# @brief Call function. Calls the calculate_SCMMesh method.
	# @param self Self pointer.
	def __call__(self):
		self.calculate_SCMMesh()
		
##############################################################################
# transfinite interpolation

	# @function set_block1
	# @brief Setting the 2D boundary nurbs curves that are required to mesh this block
	# @param self Self pointer.
	def set_block1(self):
		# find intersection point between LE and hub
		hub12u = NurbsCurve.doublep()
		hub12u.assign(0.3)
		p1 = NurbsPoint.Point2Dd(self.points[1][0], self.points[1][1])
		self.Hub2D.minDist2(p1, hub12u)
		# set new hub
		self.Hub2DBlock1 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*hub12u.value()/(self.nurbsintres-1)
			x = self.Hub2D.pointAt(u).getx()
			y = self.Hub2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Hub2DBlock1 = nurbsToolSet.interpolateNurbs2D(parray)
		
		# find intersection point between LE and shroud
		shroud12u = NurbsCurve.doublep()
		shroud12u.assign(0.3)
		p1 = NurbsPoint.Point2Dd(self.points[6][0], self.points[6][1])
		self.Shroud2D.minDist2(p1, shroud12u)
		# set new hub
		self.Shroud2DBlock1 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*shroud12u.value()/(self.nurbsintres-1)
			x = self.Shroud2D.pointAt(u).getx()
			y = self.Shroud2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Shroud2DBlock1 = nurbsToolSet.interpolateNurbs2D(parray)

		self.set_inlet_outlet()
		# set boundary curves
		self.boundaryCurves = []
		self.boundaryCurves.append(self.Hub2DBlock1)
		self.boundaryCurves.append(self.inlet)
		self.boundaryCurves.append(self.Shroud2DBlock1)
		self.boundaryCurves.append(self.LE2D)

		if self.debug > 0:
			print "\nset_block1:"
			# Checking coordinates of nurbs curves
			for i in range(self.nurbsintres):
				u = float(i)*hub12u.value()/(self.nurbsintres-1)
				x = self.Hub2D.pointAt(u).getx()
				y = self.Hub2D.pointAt(u).gety()
				print "Hub2D          u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[0].pointAt(u).getx()
				y = self.boundaryCurves[0].pointAt(u).gety()
				print "Hub2DBlock1    u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[1].pointAt(u).getx()
				y = self.boundaryCurves[1].pointAt(u).gety()
				print "inlet           u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)*shroud12u.value()/(self.nurbsintres-1)
				x = self.Shroud2D.pointAt(u).getx()
				y = self.Shroud2D.pointAt(u).gety()
				print "Shroud2D       u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[2].pointAt(u).getx()
				y = self.boundaryCurves[2].pointAt(u).gety()
				print "Shroud2DBlock1 u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[3].pointAt(u).getx()
				y = self.boundaryCurves[3].pointAt(u).gety()
				print "LE2D           u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}\n".format(u, x, y)

	# @function set_block2
	# @brief Setting the 2D boundary nurbs curves that are required to mesh this block
	# @param self Self pointer.
	def set_block2(self):
		# find intersection point between LE and hub
		hub12u = NurbsCurve.doublep()
		hub12u.assign(0.3)
		p1 = NurbsPoint.Point2Dd(self.points[1][0], self.points[1][1])
		self.Hub2D.minDist2(p1, hub12u)
		# find intersection point between TE and hub
		hub23u = NurbsCurve.doublep()
		p2 = NurbsPoint.Point2Dd(self.points[2][0], self.points[2][1])
		hub23u.assign(0.3)
		self.Hub2D.minDist2(p2, hub23u)
		# set new hub
		self.Hub2DBlock2 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*(hub23u.value()-hub12u.value())/(self.nurbsintres-1)+hub12u.value()
			x = self.Hub2D.pointAt(u).getx()
			y = self.Hub2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Hub2DBlock2 = nurbsToolSet.interpolateNurbs2D(parray)

		# find intersection point between LE and shroud
		shroud12u = NurbsCurve.doublep()
		p1 = NurbsPoint.Point2Dd(self.points[6][0], self.points[6][1])
		self.Shroud2D.minDist2(p1, shroud12u)
		# find intersection point between TE and shroud
		shroud23u = NurbsCurve.doublep()
		p2 = NurbsPoint.Point2Dd(self.points[5][0], self.points[5][1])
		self.Shroud2D.minDist2(p2, shroud23u)
		# set new shroud
		self.Shroud2DBlock2 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*(shroud23u.value()-shroud12u.value())/(self.nurbsintres-1)+shroud12u.value()
			x = self.Shroud2D.pointAt(u).getx()
			y = self.Shroud2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Shroud2DBlock2 = nurbsToolSet.interpolateNurbs2D(parray)
		# set boundary curves
		self.boundaryCurves[0] = self.Hub2DBlock2
		self.boundaryCurves[1] = self.LE2D
		self.boundaryCurves[2] = self.Shroud2DBlock2
		self.boundaryCurves[3] = self.TE2D

		if self.debug > 0:
			print "\nset_block2:"
			# Checking coordinates of nurbs curves
			for i in range(self.nurbsintres):
				u = float(i)*(hub23u.value()-hub12u.value())/(self.nurbsintres-1)+hub12u.value()
				x = self.Hub2D.pointAt(u).getx()
				y = self.Hub2D.pointAt(u).gety()
				print "Hub2D          u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[0].pointAt(u).getx()
				y = self.boundaryCurves[0].pointAt(u).gety()
				print "Hub2DBlock2    u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[1].pointAt(u).getx()
				y = self.boundaryCurves[1].pointAt(u).gety()
				print "LE2D           u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)*(shroud23u.value()-shroud12u.value())/(self.nurbsintres-1)+shroud12u.value()
				x = self.Shroud2D.pointAt(u).getx()
				y = self.Shroud2D.pointAt(u).gety()
				print "Shroud2D       u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[2].pointAt(u).getx()
				y = self.boundaryCurves[2].pointAt(u).gety()
				print "Shroud2DBlock2 u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[3].pointAt(u).getx()
				y = self.boundaryCurves[3].pointAt(u).gety()
				print "TE2D           u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}\n".format(u, x, y)

	# @function set_block3
	# @brief Setting the 2D boundary nurbs curves that are required to mesh this block
	# @param self Self pointer.
	def set_block3(self):
		# find intersection point between TE and hub
		hub23u = NurbsCurve.doublep()
		hub23u.assign(0.7)
		p1 = NurbsPoint.Point2Dd(self.points[2][0], self.points[2][1])
		self.Hub2D.minDist2(p1, hub23u)
		# set new hub
		self.Hub2DBlock3 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*(1.0-hub23u.value())/(self.nurbsintres-1)+hub23u.value()
			x = self.Hub2D.pointAt(u).getx()
			y = self.Hub2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Hub2DBlock3 = nurbsToolSet.interpolateNurbs2D(parray)

		# find intersection point between TE and shroud
		shroud23u = NurbsCurve.doublep()
		shroud23u.assign(0.7)
		p1 = NurbsPoint.Point2Dd(self.points[5][0], self.points[5][1])
		self.Shroud2D.minDist2(p1, shroud23u)
		# set new shroud
		self.Shroud2DBlock3 = NurbsCurve.NurbsCurve2Dd()
		parray = zeros((self.nurbsintres, 2))
		for i in range(self.nurbsintres):
			xy = ones([2], 'd')
			u = float(i)*(1.0-shroud23u.value())/(self.nurbsintres-1)+shroud23u.value()
			x = self.Shroud2D.pointAt(u).getx()
			y = self.Shroud2D.pointAt(u).gety()
			xy = [x, y]
			parray[i] = xy
		self.Shroud2DBlock3 = nurbsToolSet.interpolateNurbs2D(parray)
		# set boundary curves
		self.boundaryCurves[0] = self.Hub2DBlock3
		self.boundaryCurves[1] = self.TE2D
		self.boundaryCurves[2] =self.Shroud2DBlock3
		self.boundaryCurves[3] = self.outlet

		if self.debug > 0:
			print "\nset_block3:"
			# Checking coordinates of nurbs curves
			for i in range(self.nurbsintres):
				u = float(i)*(1.0-hub23u.value())/(self.nurbsintres-1)+hub23u.value()
				x = self.Hub2D.pointAt(u).getx()
				y = self.Hub2D.pointAt(u).gety()
				print "Hub2D          u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[0].pointAt(u).getx()
				y = self.boundaryCurves[0].pointAt(u).gety()
				print "Hub2DBlock3    u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[1].pointAt(u).getx()
				y = self.boundaryCurves[1].pointAt(u).gety()
				print "TE2D           u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)*(1.0-shroud23u.value())/(self.nurbsintres-1)+shroud23u.value()
				x = self.Shroud2D.pointAt(u).getx()
				y = self.Shroud2D.pointAt(u).gety()
				print "Shroud2D       u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				u = float(i)/(self.nurbsintres-1)
				x = self.boundaryCurves[2].pointAt(u).getx()
				y = self.boundaryCurves[2].pointAt(u).gety()
				print "Shroud2DBlock3 u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}".format(u, x, y)
				x = self.boundaryCurves[3].pointAt(u).getx()
				y = self.boundaryCurves[3].pointAt(u).gety()
				print "outlet         u,z,r: {0:10.6f} {1:10.6f} {2:10.6f}\n".format(u, x, y)

	# @function set_inlet_outlet
	# @brief Setting the 2D inlet/outlet nurbs curves for this blade row
	# @param self Self pointer.
	def set_inlet_outlet(self):
		iarray = zeros([self.block_cells_rad+1, 2], 'd')
		oarray = zeros([self.block_cells_rad+1, 2], 'd')
		for rad in range(self.block_cells_rad+1):
			iarray[rad][1] = self.points[0][1] + rad / float(self.block_cells_rad) * (self.points[7][1] - self.points[0][1])
			iarray[rad][0] = self.points[0][0] + rad / float(self.block_cells_rad) * (self.points[7][0] - self.points[0][0])
			oarray[rad][1] = self.points[3][1] + rad / float(self.block_cells_rad) * (self.points[4][1] - self.points[3][1])
			oarray[rad][0] = self.points[3][0] + rad / float(self.block_cells_rad) * (self.points[4][0] - self.points[3][0])
		self.inlet = nurbsToolSet.interpolateNurbs2D(iarray)
		self.outlet = nurbsToolSet.interpolateNurbs2D(oarray)

	# @function calculateUFromPoint
	# @brief Calculates u-value of the curve that is the closest to the given point
	# @param self Self pointer.
	# @param point Nurbs point
	# @param curve Nurbs curve
	# @param guessU guessed u-value to start the search from
	# @param uMin Lowest u-value that can be part of the search
	# @param uMax Highest u-value that can be part of the search
	def calculateUFromPoint(self, point, curve, guessU=0.5, uMin=0.0, uMax=1.0):
		uValue = NurbsCurve.doublep()
		uValue.assign(guessU)
		curve.minDist2(point, uValue, self.tolerance, 0.5, 100, self.maxIteration, uMin, uMax)
		return uValue.value()

	# @function computeUVBoundaryValues
	# @brief Calculates uv-values for the vertices of the boundary curves of current grid
	# @param self Self pointer.
	# @param blockNumber Number of the block
	def computeUVBoundaryValues(self, blockNumber):
		self.uList = []
		self.vList = []
		if blockNumber == 1:
			# get block edge length
			blockEdgeLength = self.Hub2DBlock1.lengthIn(0, 1)
			# create list with length of all cells in block according to the grading
			cellLength  = self.computeSpacing(self.block1_cells_ax[self.rowIndex], self.AxialGradingBlock1, blockEdgeLength)
			# create list with u-values of the vertices
			u_start = 1
			for i in range(self.block1_cells_ax[self.rowIndex]-1):
				u = self.calculateUStart(u_start, cellLength[i], self.Hub2DBlock1)
				self.uList.append(u)
				u_start = u
		elif blockNumber == 2:
			blockEdgeLength = self.Hub2DBlock2.lengthIn(0,1)
			# create list with length of all cells in block according to the grading
			cellLength  = self.computeSpacing(self.block2_cells_ax[self.rowIndex], self.AxialGradingBlock2, blockEdgeLength)
			# create list with u-values of the vertices
			u_start = 1
			for i in range(self.block2_cells_ax[self.rowIndex]-1):
				u = self.calculateUStart(u_start, cellLength[i], self.Hub2DBlock2)
				self.uList.append(u)
				u_start = u
		elif blockNumber == 3:
			blockEdgeLength = self.Hub2DBlock3.lengthIn(0, 1)
			# create list with length of all cells in block according to the grading
			cellLength  = self.computeSpacing(self.block3_cells_ax[self.rowIndex], self.AxialGradingBlock3, blockEdgeLength)
			# create list with u-values of the vertices
			u_start = 1
			for i in range(self.block3_cells_ax[self.rowIndex]-1):
				u = self.calculateUStart(u_start, cellLength[i], self.Hub2DBlock3)
				self.uList.append(u)
				u_start = u
		blockEdgeLength = self.boundaryCurves[1].length()
		cellLength = self.computeSpacing(self.block_cells_rad, self.RadialGrading, blockEdgeLength)
		v_start = 1
		for i in range(self.block_cells_rad-1):
			v = self.calculateUStart(v_start, cellLength[i], self.boundaryCurves[1])
			self.vList.append(v)
			v_start = v
		# adding first (0) and last (1) u/v-values
		self.uList.reverse()
		self.uList.append(1.0)
		self.uList.insert(0, 0.0)
		self.vList.reverse()
		self.vList.append(1.0)
		self.vList.insert(0, 0.0)
		if self.debug > 3:
			print "u: block", blockNumber, self.uList
			print "v: block", blockNumber, self.vList
		self.hubUCoordinates = self.uList
		self.inletVCoordinates = self.vList

	# @function curve
	# @brief Returns the 2D point of the given curve at the given u coordinate
	# @param self Self pointer.
	# @param number Number of the boundary curve (hub=0, inlet=1. shroud=2, outlet=3)
	# @param coordinate u coordinate of the curve, which defines the point to be returned
	def curve(self, number, coordinate):
		point = self.boundaryCurves[number].pointAt(coordinate)
		tmpPoint = zeros(2)
		## r coordinate
		tmpPoint[0] = point.gety()
		## z coordinate
		tmpPoint[1] = point.getx()
		return tmpPoint

	# @function transfinite
	# @brief Returns a transfinite interpolated 2D point
	# @param self Self pointer.
	# @param u u-coordinate of the point
	# @param v v-coordinate of the point
	def transfinite(self, u, v):
		#self.curve(0,:) -> hub
		#self.curve(1,:) -> inlet
		#self.curve(2,:) -> shroud
		#self.curve(3,:) -> outlet
		surf_ti = (1-v)*self.curve(0,u)+v*self.curve(2,u)+(1-u)*self.curve(1,v)+u*self.curve(3,v)- ((1-u)*(1-v)*self.curve(0,0.0)+v*(1-u)*self.curve(1,1.0)+u*(1-v)*self.curve(0,1.0)+u*v*self.curve(2,1.0))
		return surf_ti

	# @function set_pointArray2D
	# @brief Setting the 2D array with r,z coordinates
	# @param self Self pointer.
	def set_pointArray2D(self):
		self.nurbsintres = 10 # NURBS interpolation resolution to create hub and shroud nurbs for each block
		# Block 1
		self.set_block1()
		self.computeUVBoundaryValues(1)
		for j in range(self.block_cells_rad+1):
			for i in range(self.block1_cells_ax[self.rowIndex]+1):
				point = self.transfinite(self.hubUCoordinates[i], self.inletVCoordinates[j])
				self.pointArray2D[i][j][0] = point[0]*self.convertToMeters
				self.pointArray2D[i][j][1] = point[1]*self.convertToMeters
				if self.debug > 0:
					print "\nset_pointArray2D block1 i,j:", i, j
					print "u,v: {0:10.6f} {1:10.6f}".format(self.hubUCoordinates[i], self.inletVCoordinates[j])
					print "point {0:10.6f} {1:10.6f}".format(point[0], point[1])
		# Block 2
		self.set_block2()
		self.computeUVBoundaryValues(2)
		for j in range(self.block_cells_rad+1):
			for i in range(self.block2_cells_ax[self.rowIndex]):
				point = self.transfinite(self.hubUCoordinates[i+1], self.inletVCoordinates[j])
				self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+1][j][0] = point[0]*self.convertToMeters
				self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+1][j][1] = point[1]*self.convertToMeters
				if self.debug > 0:
					print "\nset_pointArray2D block2 i,j:", i, j
					print "u,v: {0:10.6f} {1:10.6f}".format(self.hubUCoordinates[i+1], self.inletVCoordinates[j])
					print "point {0:10.6f} {1:10.6f}".format(point[0], point[1])
		# Block 3
		self.set_block3()
		self.computeUVBoundaryValues(3)
		for j in range(self.block_cells_rad+1):
			for i in range(self.block3_cells_ax[self.rowIndex]):
				point = self.transfinite(self.hubUCoordinates[i+1], self.inletVCoordinates[j])
				self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+self.block2_cells_ax[self.rowIndex]+1][j][0] = point[0]*self.convertToMeters
				self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+self.block2_cells_ax[self.rowIndex]+1][j][1] = point[1]*self.convertToMeters
				if self.debug > 0:
					print "\nset_pointArray2D block3 i,j:", i, j
					print "u,v: {0:10.6f} {1:10.6f}".format(self.hubUCoordinates[i+1], self.inletVCoordinates[j])
					print "point {0:10.6f} {1:10.6f}".format(point[0], point[1])
# transfinite ende
############################################################################

	# @function calculate_SCMMesh
	# @brief Calculates a 2D mesh for the Streamline Curvature Method (SCM).
	# @param self Self pointer.
	def calculate_SCMMesh(self):

		if(self.debug > 0):
			print "\n\n***\n*** Starting to calculate Streamline Curvature Method Input...\n***\n"
		self.rowIndex = 0
		if self.CGNS == True: # Do CGNS output
			if not os.path.exists(self.case):
				os.mkdir(self.case)
			self.cgnsOutput(self.massflow)
		self.allQO = 0
		self.QO = []
		for row in range(self.nor):
			self.QO.append(self.block1_cells_ax[row] + self.block2_cells_ax[row] + self.block3_cells_ax[row])
			self.allQO += self.QO[row]
		# loop through all rows of the turbo machine
		for row in self.machine.listRows:
			if self.debug >0:
				print "Calculating row", self.rowIndex
			self.omega = row.omega

			# getting leading edge for pressure side/suction side distinction
			self.LE3D = row.LE3D
			
			# Calculating LE and TE in bladeRow:
			self.machine.listRows[self.rowIndex].calculate_LETE2D()

			# 2D nurbs import for transfinite interpolation
			self.LE2D = row.nurbsLE
			self.TE2D = row.nurbsTE
			self.Hub2D = row.nurbsHub
			self.Shroud2D = row.nurbsShroud
			
			self.surface = row.bladeSurface


#			      shroud
#			  _______________
#		->	|7    6     5    4|
#		->	|     |     |     |
#	flow	->	|     |Blade|     |
#		->	|     |     |     |
#		->	|0____1_____2____3|
#			       hub

			# Saves important points in a list (z,r) or (x,y)
			# find intersection point between LE and hub
			u = NurbsCurve.doublep()
			u.assign(0.3)
			p1 = NurbsPoint.Point2Dd(self.LE2D.pointAt(0).getx(), self.LE2D.pointAt(0).gety())
			self.Hub2D.minDist2(p1, u)
			hubLE = self.Hub2D.pointAt(u.value())
			# find intersection point between TE and hub
			p1 = NurbsPoint.Point2Dd(self.TE2D.pointAt(0).getx(), self.TE2D.pointAt(0).gety())
			u.assign(0.7)
			self.Hub2D.minDist2(p1, u)
			hubTE = self.Hub2D.pointAt(u.value())
			# find intersection point between LE and shroud
			p1 = NurbsPoint.Point2Dd(self.LE2D.pointAt(1).getx(), self.LE2D.pointAt(1).gety())
			u.assign(0.3)
			self.Shroud2D.minDist2(p1, u)
			shroudLE = self.Shroud2D.pointAt(u.value())
			# find intersection point between TE and shroud
			p1 = NurbsPoint.Point2Dd(self.TE2D.pointAt(1).getx(), self.TE2D.pointAt(1).gety())
			u.assign(0.7)
			self.Shroud2D.minDist2(p1, u)
			shroudTE = self.Shroud2D.pointAt(u.value())
			self.points = [
				(self.Hub2D.pointAt(0).getx(), self.Hub2D.pointAt(0).gety()),		# point 0
				(hubLE.getx(), hubLE.gety()),		# point 1
				(hubTE.getx(), hubTE.gety()),		# point 2
				(self.Hub2D.pointAt(1).getx(), self.Hub2D.pointAt(1).gety()),		# point 3
				(self.Shroud2D.pointAt(1).getx(), self.Shroud2D.pointAt(1).gety()),	# point 4
				(shroudTE.getx(), shroudTE.gety()),		# point 5
				(shroudLE.getx(), shroudLE.gety()),		# point 6
				(self.Shroud2D.pointAt(0).getx(), self.Shroud2D.pointAt(0).gety()),	# point 7
				]

			# redefines the radial grading value to approximate surface equal radii.
			if self.rowIndex == 0:
				self.auto_grading()

			# calculates the number of cells of a row in axial direction
			self.cellsAx = self.block1_cells_ax[self.rowIndex] + self.block2_cells_ax[self.rowIndex] + self.block3_cells_ax[self.rowIndex]
			
			# STRUCTURE: self.pointArray2D[axial knot][radial knot][data(radius, axial-coordinate, beta_r, beta_z, sigma, entropy)]
			self.pointArray2D = ones((self.cellsAx+1, self.block_cells_rad+1, 6), float)
			self.set_pointArray2D()
			
			# The cancel flag is set to True if the read_points method could not open the required files
			if self.cancel == False:
				self.betas(row)
				self.sigma(row)
				self.entropy()
				
				if self.CGNS == False:
					if self.rowIndex == 0:
						self.write_SCM_Var()
					self.write_SCM0(row)
					self.write_SCM1(row)
					self.write_SCM2(row)
					if (self.rowIndex == self.nor - 1) and (self.debug > 0):
						print "\nSCM Text Input written!"
				else:
					if self.nor == 1:
						self.cgnsOutput.mergeBlocks == False
					self.cgnsOutput.write_blocks(self)
					if self.cgnsOutput.mergeBlocks == True:
						self.previousPointArray2D = self.pointArray2D
					if (self.rowIndex == self.nor - 1) and (self.debug > 0):
						print "\nCGNS file written!"
					
				if self.debug > 1:
					print "\nself.pointArray2D:"
					print self.pointArray2D
				
				self.arraylist.append(self.pointArray2D)
				self.rowIndex += 1
			else:
				break
				
		if (self.cancel == False) and (self.debug > 0):
			print "\n***\n*** Calculation of Streamline Curvature Method Input complete!\n***\n"
	
	
	# @function write_SCM_Var
	# @brief Writes the required variables for the Streamline Curvature Method into a text file.
	# @param self Self pointer.
	def write_SCM_Var(self):
		
		path = self.case + '/Globale_Variablen'
		if os.path.exists(path):
			os.remove(path)
		
		f = open(path, 'w', -1)
		f.write('Einleseformat: TEXT\n')
		f.write('\nMassenstrom: '+str(self.massflow))
		f.write('\nAnzahl Stromlinien: '+str(self.block_cells_rad+1))
		f.write('\nAnzahl Gitter: '+str(self.machine.numberRows*3))
		f.write('\n\nAnzahl Quasiorthogonale:')
		for i in range(self.machine.numberRows):
			f.write('\n'+str(self.block1_cells_ax[i]+1))
			f.write('\n'+str(self.block2_cells_ax[i]+1))
			f.write('\n'+str(self.block3_cells_ax[i]+1))
		f.write('\n\nabsoluter       Totaldruck:\n')
		for rad in range(self.block_cells_rad+1):
			f.write(str(self.totalPressure[rad])+'\n')
		f.write('\nabsoluter       Totaltemperatur:\n')
		for rad in range(self.block_cells_rad+1):
			f.write(str(self.totalTemperature[rad])+'\n')
		f.close()


	# @function write_SCM0
	# @brief Writes the data of the first block required by the SCM into a text file.
	# @param self Self pointer.
	# @param row bladeRow object.
	def write_SCM0(self, row):
		
		digits = self.ScmDigitsAfterComma
		format = '%'+str(digits+3)+'.'+str(digits)+'f\t'
		path = self.case + '/Gitter_Variablen-'+str(self.rowIndex*3).zfill(2)
		f = open(path, 'w', -1)
		f.write('\nOmega: 0.0')
		f.write('\n\n')
		f.write('r \t\tz \t\tbeta_ru \t\tbeta_zu \tsigma \tentropie \n')
		c = self.convertToMeters
		
		for ax in range(self.block1_cells_ax[self.rowIndex]+1):
			f.write('\n')
			for rad in range(self.pointArray2D.shape[1]):
				dataSet = self.pointArray2D[ax][rad]
				s = format * len(dataSet)
				s = s % (dataSet[0], dataSet[1], math.pi/2.0, math.pi/2.0, 1, dataSet[5])
				s = s  + '\n'
				f.write(s)
		f.close()


	# @function write_SCM1
	# @brief Writes the data of the second block required by the SCM into a text file.
	# @param self Self pointer.
	# @param row bladeRow object.
	def write_SCM1(self, row):
		
		digits = self.ScmDigitsAfterComma
		format = '%'+str(digits+3)+'.'+str(digits)+'f\t'
		path = self.case + '/Gitter_Variablen-'+str(self.rowIndex*3+1).zfill(2)
		f = open(path, 'w', -1)
		f.write('\nOmega: '+str(row.omega))
		f.write('\n\n')
		f.write('r \t\tz \t\tbeta_ru \tbeta_zu \t\tsigma \t\tentropie \n')
		c = self.convertToMeters
		for ax in range(self.block2_cells_ax[self.rowIndex]+1):
			ax = ax + self.block1_cells_ax[self.rowIndex]
			f.write('\n')
			for rad in range(self.pointArray2D.shape[1]):
				dataSet = self.pointArray2D[ax][rad]
				s = format * len(dataSet)
				s = s % (dataSet[0], dataSet[1], dataSet[2], dataSet[3], dataSet[4], dataSet[5])
				s = s  + '\n'
				f.write(s)
		f.close()


	# @function write_SCM2
	# @brief Writes the data of the third block required by the SCM into a text file.
	# @param self Self pointer.
	# @param row bladeRow object.
	def write_SCM2(self, row):
		
		digits = self.ScmDigitsAfterComma
		format = '%'+str(digits+3)+'.'+str(digits)+'f\t'
		path = self.case + '/Gitter_Variablen-'+str(self.rowIndex*3+2).zfill(2)
		f = open(path, 'w', -1)
		f.write('\nOmega: 0.0')
		f.write('\n\n')
		f.write('r \t\tz \t\tbeta_ru \t\tbeta_zu \t\tsigma \t\tentropie \n')
		c = self.convertToMeters
		for ax in range(self.block3_cells_ax[self.rowIndex]+1):
			ax = ax + self.block1_cells_ax[self.rowIndex] + self.block2_cells_ax[self.rowIndex]
			f.write('\n')
			for rad in range(self.pointArray2D.shape[1]):
				dataSet = self.pointArray2D[ax][rad]
				s = format * len(dataSet)
				s = s % (dataSet[0], dataSet[1], math.pi/2.0, math.pi/2.0, 1, dataSet[5])
				s = s  + '\n'
				f.write(s)
		f.close()
		
	
	# @function entropy
	# @brief Calculates a linear interpolation of the entropy along the Quasi-Orthogonals between two predefined values.
	# @param self Self pointer.
	def entropy(self):

		sumOfQO = 0
		for i in range(self.rowIndex):
			sumOfQO += self.QO[i]

		for j in range(self.block_cells_rad+1):
			for i in range(self.QO[self.rowIndex]+1):
				QONorm = (i + sumOfQO) / float(self.allQO)
				deltaEntr = self.maxEntropy - self.minEntropy
				self.pointArray2D[i][j][5] = self.minEntropy + QONorm * deltaEntr
				
	
	
	# @function sigma
	# @brief Calculates the sigma value (blockage ratio) for each row.
	# @param self Self pointer.
	# @param row bladeRow object.
	def sigma(self, row):
		
		if self.debug > 0:
			print "\n\nCalculating sigma...\n"
		for j in range(self.block_cells_rad+1):
			for i in range(self.block2_cells_ax[self.rowIndex]+1):
				i = i + self.block1_cells_ax[self.rowIndex]
				r = self.pointArray2D[i][j][0] / self.convertToMeters
				z = self.pointArray2D[i][j][1] / self.convertToMeters
				camberSurfaceU = self.pointArray2D[i][j][4]
				camberSurfaceV = self.pointArray2D[i][j][5]
				
				# Flag for calculating a deltaPhi. Set fubar = True if there is trouble with camber surface and blade surface
				if self.fubar == True:
					minZ = self.pointArray2D[self.block1_cells_ax[self.rowIndex]][j][1] / self.convertToMeters
					maxZ = self.pointArray2D[self.block1_cells_ax[self.rowIndex]+self.block2_cells_ax[self.rowIndex]+1][j][1] / self.convertToMeters
					x = (z - minZ*0.999) / (maxZ - minZ*0.999)
					thickness = round(self.maxThickness/2.0 * math.sin(math.acos(2.0*x-1.0)) ,8)
					deltaPhi = math.atan(2*thickness/r)
				else:
					deltaPhi = self.delta_phi(row, r, z, camberSurfaceU, camberSurfaceV)
				sigma = 1 - (deltaPhi * row.numberBlades / (2 * math.pi))
				self.pointArray2D[i][j][4] = sigma
	
	
	# @function delta_phi
	# @brief Calculates the angular difference of 2 opposing points of the blade surface at a given axial and radial coordinate.
	# @param row bladeRow object.
	# @param r Radial coordinate of the opposing points.
	# @param z Axial coordinate of teh opposing points.
	# @param camberSurfaceU U-value of the point on the camber surface at the given axial and radial coordinates.
	# @param camberSurfaceV V-value of the point on the camber surface at the given axial and radial coordinates.
	# @return deltaPhi The angular difference between the two points on the blade surface at the given radial and axial position.
	def delta_phi(self, row, r, z, camberSurfaceU, camberSurfaceV):
		
		blade = row.bladeSurface
		c = self.convertToMeters
		
		# Initializing variables:
		newP = NurbsPoint.Point3Dd()
		uCrit = NurbsCurve.doublep()
		vCrit = NurbsCurve.doublep()
		uCrit.assign(0.5)
		vCrit.assign(0.5)
		PX1 = NurbsPoint.Point3Dd()
		PX2 = NurbsPoint.Point3Dd()
		PC = NurbsPoint.Point3Dd()
		PC = self.CS.camberSurface.pointAt(camberSurfaceU, camberSurfaceV)
		YC = PC.gety()
		dist = 1
		phi = math.atan(PC.gety()/PC.getx())
		n = 0
		
		#######################
		# Find uCrit, which seperates suction side from pressure side, with the help of the leading edge.
		#######################
		self.LE3D.minDistX(r, uCrit)
		tempz = self.LE3D.pointAt(uCrit.value()).getz()
		while (n < self.maxIteration) and (dist > self.tolerance):
			p = NurbsPoint.Point3Dd(r*math.cos(phi), r*math.sin(phi), tempz)
			dist = self.LE3D.minDist2(p, uCrit, self.tolerance, 1)
			newP = self.LE3D.pointAt(uCrit.value())
			phi = math.atan2(newP.gety(), newP.getx())
			n = n + 1
		blade.minDist2(newP, uCrit, vCrit, self.tolerance, 1.0, self.sep, self.maxIteration)
		if self.debug > 2:
			print '\nuCrit:', uCrit.value(), '\tvCrit:', vCrit.value()
		
		########################
		# Find point on one side.
		########################
		dist = 1
		n = 0
		tempdist = 0
		phi1 = math.atan(PC.gety()/PC.getx())
		u1 = NurbsCurve.doublep()
		v1 = NurbsCurve.doublep()
		v1.assign(vCrit.value()/2.0)
		u1.assign(uCrit.value())
		if self.debug > 2:
			print 'uv1', u1.value(), uCrit.value(), v1.value(), vCrit.value()/2.0
			p = blade.pointAt(u1.value(), v1.value())
			print 'P1', p.getx(),p.gety(),p.getz()
		while (n < self.maxIteration) and (dist > self.tolerance):
			PX1 = NurbsPoint.Point3Dd(r*math.cos(phi1), r*math.sin(phi1), z)
			# See NURBS++ documentation for more info on the arguments and functions.
			dist = blade.minDist2(PX1, u1, v1, self.tolerance, 0.3, self.sep, self.maxIteration, 0.0, 1.0, 0.0, vCrit.value())
			newP = blade.pointAt(u1.value(), v1.value())
			phi1 = math.atan(newP.gety()/newP.getx())
			n = n + 1
			# Breaks the while-loop if iterations have no effect.
			if dist == tempdist:
				break
			tempdist = dist
		Y1 = PX1.gety()
		
		if self.debug > 2:
			print "r,z:\t\t\t\t", r, '\t\t\t', z
			P1 = blade.pointAt(u1.value(), v1.value())
			print "dist_nmax:", n, "\t", dist,"\t",P1.getx(),P1.gety(),P1.getz(), '\tU:', u1.value(), '\tV:', v1.value()#, '\tphi1:', phi1

		##########################
		# Find point on other side.
		##########################
		dist = 1
		n = 0
		tempdist = 0
		phi2 = math.atan(PC.gety()/PC.getx())
		u2 = NurbsCurve.doublep()
		v2 = NurbsCurve.doublep()
		v2.assign(vCrit.value() + (1-vCrit.value())/2.0)
		u2.assign(uCrit.value())
		if self.debug > 2:
			print 'uv2', u2.value(), uCrit.value(), v2.value(), vCrit.value()+((1-vCrit.value())/2.0)
			p=blade.pointAt(u2.value(), v2.value())
			print 'P2', p.getx(),p.gety(),p.getz()
		while (n < self.maxIteration) and (dist > self.tolerance):
			PX2 = NurbsPoint.Point3Dd(r*math.cos(phi2), r*math.sin(phi2), z)
			# See NURBS++ documentation for more info on the arguments and functions.
			dist = blade.minDist2(PX2, u2, v2, self.tolerance, 0.3, self.sep, self.maxIteration, 0.0, 1.0, vCrit.value(), 1.0)
			newP = blade.pointAt(u2.value(), v2.value())
			phi2 = math.atan(newP.gety()/newP.getx())
			n = n + 1
			# Breaks the while-loop if iterations have no effect.
			if dist == tempdist:
				break
			tempdist = dist
		Y2 = PX2.gety()
		
		if self.debug > 2:
			P2 = blade.pointAt(u2.value(), v2.value())
			print "dist_nmax:", n, "\t", dist,"\t",P2.getx(),P2.gety(),P2.getz(), '\tU:', u2.value(), '\tV:', v2.value()#, '\tphi2:', phi2
		
		# Absolute difference between the two calculated angles.
		deltaPhi = math.fabs(phi1-phi2)
		
		# Debugging options
		if self.debug > 2 and deltaPhi > self.tolerance:
			print 'deltaPhi°:', math.degrees(deltaPhi), '\tphi1°:', math.degrees(phi1), '\tphi2°:', math.degrees(phi2)
			print 'deltaRPhi:', deltaPhi*r, '\tPhi1R:', phi1*r, '\tPhi2R:', phi2*r
		if self.debug > 2 and deltaPhi <= self.tolerance:
			print "0.0 -> not good!!! DeltaPhi < tolerance"
		if self.debug > 2 and ((Y2 < YC and Y1 < YC) or (Y2 > YC and Y1 > YC)):
			print 'Problem: '
			print "P1:", PX1.getx(), PX1.gety(), PX1.getz()
			print "PC:", PC.getx(), PC.gety(), PC.getz()
			print "P2:", PX2.getx(), PX2.gety(), PX2.getz()
				
		return deltaPhi
	
	
	# @function betas
	# @brief Calculates betaR and betaZ angles for each point of 2D mesh, and adds values to pointArray2D.
	# @param self Self pointer.
	# @param row Current bladeRow object.
	def betas(self, row):
		
		if self.debug > 0:
			print "\n\nCalculating betas..."
		# Filling array with "90°" (in radians).
		for i in range(self.pointArray2D.shape[0]):
			for j in range(self.pointArray2D.shape[1]):
				self.pointArray2D[i][j][2] = math.pi / 2
				self.pointArray2D[i][j][3] = math.pi / 2
		
		# Creating camber surface through interpolating discrete points of camberlines
		resCamber = 10
		i = 0
		j = 0
		m = self.convertToMeters
		matrix = NurbsMatrix.Matrix_Point3Dd(len(row.nurbsCamber), resCamber)
		for CamberLine in row.nurbsCamber:
			for c in range(resCamber):
				u = c / (resCamber - 1.0)
				p = CamberLine.pointAt(u)
				nurbsPoint = NurbsPoint.Point3Dd(p.getx(), p.gety(), p.getz())
				matrix.setelem(i, j, nurbsPoint)
				j += 1
			i += 1
			j = 0
		self.CS = camberSurface(self.geom, self.rowIndex)
		if self.CSDebug == True:
			self.CS.debug = 1
		self.CS.CamberSurface3D(matrix, self.camberSurfaceDegree)

		
		# filling array with beta values and help values
		for rad in range(self.block_cells_rad+1):
			for ax in range(self.block2_cells_ax[self.rowIndex]+1):
				ax = ax + self.block1_cells_ax[self.rowIndex]
				axrad = self.CS.calculate_intersection(self.pointArray2D[ax][rad][0]/m, self.pointArray2D[ax][rad][1]/m)
				betas = self.CS.calculate_betaR_betaZ(axrad[0], axrad[1])
				self.pointArray2D[ax][rad][4] = axrad[0]	# temporary help values for sigma calculation
				self.pointArray2D[ax][rad][5] = axrad[1]	# temporary help values for sigma calculation
				#self.pointArray2D[ax][rad][2] = math.degrees(betas[0])	# beta_R in degrees for debugging
				#self.pointArray2D[ax][rad][3] = math.degrees(betas[1])	# beta_Z in degrees for debugging
				self.pointArray2D[ax][rad][2] = betas[0]	# beta_R
				self.pointArray2D[ax][rad][3] = betas[1]	# beta_Z	
			
	
	# @function project_to_S2
	# @brief Projects the 3D NURBS curves to the S2 plane.
	# @param self Self pointer.
	# @param curve NURBS curve.
	# @return something... New NURBS curve on S2 plane.
	def project_to_S2(self, curve):
		
		# Prepares variables for new NURBS curve
		maxPoints = curve.ctrlPnts().rows()
		degree = curve.degree()
		lengthKnot = maxPoints+degree+1
		weights = NurbsVector.Vector_DOUBLE(maxPoints)
		nvector = NurbsVector.Vector_Point3Dd(maxPoints)
		vector = NurbsVector.Vector_Point3Dd(1)
		for n in range(maxPoints):
			weights[n] = 1.0
		
		# Projects the 3D points radially to the S2m-surface through converting to cylindrical coordinates
		# and setting the angle to 0.
		for i in range(maxPoints):
			point = curve.ctrlPnts(i)
			r = math.sqrt(point.getx()**2 + point.gety()**2)
			z = point.getz()
			vector.reset(NurbsPoint.Point3Dd(0, r, z))
			nvector.as_func(i, vector)
			
		return NurbsCurve.NurbsCurved(nvector, weights, curve.knot(), degree)
		
		
	# Print discrete points of NURBS surface (function for debugging).
	def print_surf_func(self, surface):
		
		print "\n"
		for u in [0, 0.25, 0.5, 0.75, 1]:
			print "\n"
			for v in [0, 0.25, 0.5, 0.75, 1]:
				print "DiscretePnts:", surface.pointAt(u, v).getx(), surface.pointAt(u, v).gety(), surface.pointAt(u, v).getz()
				
		
	# Prints certain discrete points and all control points of NURBS curve (function for debugging).
	def print_func(self, curve):
		
		print "\n"
		for u in [0.0, 0.5, 1.0]:
			print "DiscretePnts:", curve.pointAt(u).getx(), curve.pointAt(u).gety(), curve.pointAt(u).getz()
		print "\n"
		for i in range(curve.ctrlPnts().rows()):
			knot = curve.ctrlPnts(i)
			print "Ctrl:", knot.getx(), knot.gety(), knot.getz()

	
	# @function vertices
	# @brief Prepares data for 'blockMesh' command.
	# @param self Self pointer.
	def vertices(self):
		
		self.blockCoords[0, 0] = self.Hub3D.pointAt(0).getz()
		self.blockCoords[0, 1] = self.Hub3D.pointAt(0).getx()
		self.blockCoords[0, 2] = 0
		self.blockCoords[1, 0] = self.LEq2D.pointAt(0).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[1, 1] = self.LEq2D.pointAt(0).gety()
		self.blockCoords[1, 2] = 0
		self.blockCoords[2, 0] = self.TEq2D.pointAt(0).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[2, 1] = self.TEq2D.pointAt(0).gety()
		self.blockCoords[2, 2] = 0
		self.blockCoords[3, 0] = self.Hub3D.pointAt(1).getz()
		self.blockCoords[3, 1] = self.Hub3D.pointAt(1).getx()
		self.blockCoords[3, 2] = 0
		self.blockCoords[4, 0] = self.Shroud3D.pointAt(1).getz()
		self.blockCoords[4, 1] = self.Shroud3D.pointAt(1).getx()
		self.blockCoords[4, 2] = 0
		self.blockCoords[5, 0] = self.TEq2D.pointAt(1).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[5, 1] = self.TEq2D.pointAt(1).gety()
		self.blockCoords[5, 2] = 0
		self.blockCoords[6, 0] = self.LEq2D.pointAt(1).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[6, 1] = self.LEq2D.pointAt(1).gety()
		self.blockCoords[6, 2] = 0
		self.blockCoords[7, 0] = self.Shroud3D.pointAt(0).getz()
		self.blockCoords[7, 1] = self.Shroud3D.pointAt(0).getx()
		self.blockCoords[7, 2] = 0
		
		
		self.blockCoords[8, 0] = self.Hub3D.pointAt(0).getz()
		self.blockCoords[8, 1] = self.Hub3D.pointAt(0).getx()
		self.blockCoords[8, 2] = 1
		self.blockCoords[9, 0] = self.LEq2D.pointAt(0).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[9, 1] = self.LEq2D.pointAt(0).gety()
		self.blockCoords[9, 2] = 1
		self.blockCoords[10, 0] = self.TEq2D.pointAt(0).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[10, 1] = self.TEq2D.pointAt(0).gety()
		self.blockCoords[10, 2] = 1
		self.blockCoords[11, 0] = self.Hub3D.pointAt(1).getz()
		self.blockCoords[11, 1] = self.Hub3D.pointAt(1).getx()
		self.blockCoords[11, 2] = 1
		self.blockCoords[12, 0] = self.Shroud3D.pointAt(1).getz()
		self.blockCoords[12, 1] = self.Shroud3D.pointAt(1).getx()
		self.blockCoords[12, 2] = 1
		self.blockCoords[13, 0] = self.TEq2D.pointAt(1).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[13, 1] = self.TEq2D.pointAt(1).gety()
		self.blockCoords[13, 2] = 1
		self.blockCoords[14, 0] = self.LEq2D.pointAt(1).getz()	# z-component because of cylindrical coordinate system
		self.blockCoords[14, 1] = self.LEq2D.pointAt(1).gety()
		self.blockCoords[14, 2] = 1
		self.blockCoords[15, 0] = self.Shroud3D.pointAt(0).getz()
		self.blockCoords[15, 1] = self.Shroud3D.pointAt(0).getx()
		self.blockCoords[15, 2] = 1
	
	
	# @function auto_grading
	# @brief Sets the radial grading so equal surface radii are approximated.
	# @param self Self pointer.
	def auto_grading(self):
		
		innerCellInnerR = self.points[0][1]
		outerCellOuterR = self.points[7][1]
		outerCellInnerR = sqrt(outerCellOuterR**2 - (outerCellOuterR**2 - innerCellInnerR**2) / self.block_cells_rad)
		innerCellOuterR = sqrt(innerCellInnerR**2 + (outerCellOuterR**2 - innerCellInnerR**2) / self.block_cells_rad)
		outerCell = outerCellOuterR - outerCellInnerR
		innerCell = innerCellOuterR - innerCellInnerR
		self.RadialGrading = outerCell / innerCell
	
	
	# @function write_controlDict
	# @brief Writes the controlDict file.
	# @param self Self pointer.
	def write_controlDict(self):
		
		if self.debug > 0:
			print "\n\nWriting controlDict file..."
		self.writeFileHead2()
	
	
	# @function write_BlockMeshDict_1
	# @brief Writes the q2D BlockMeshDict file for the mesher.
	# @param self Self pointer.
	def write_BlockMeshDict_1(self):
		
		self.writeFileHead()
		self.writeScale()
		self.vertices()
		self.writeVertices()
		self.writeBlocks()
		self.writeEdges()
		self.writePatches()
		if self.debug > 0:
			print "BlockMeshDict File complete!\n"

	
	# @function OFheader
	# @brief Writes the OpenFoam header with given version numbers into a file.
	# @param self Self pointer.
	def OFheader(self):
		fOut = open(self.path, 'w', -1)
		fOut.write('/*---------------------------------------------------------------------------*\\\n')
		fOut.write('| =========                 |                                                 |\n')
		fOut.write('| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
		fOut.write('|  \\\\    /   O peration     | Version:  '+str(self.OFversion)+'                                   |\n')
		fOut.write('|   \\\\  /    A nd           | Web:      http://www.openfoam.org               |\n')
		fOut.write('|    \\\\/     M anipulation  |                                                 |\n')
		fOut.write('\*---------------------------------------------------------------------------*/\n\n')
		fOut.write('FoamFile\n')
		fOut.write('{\n')
		fOut.write('    version         '+str(self.FoamFileVersion)+';\n')
		fOut.write('    format          ascii;\n\n')
		fOut.write('    class           dictionary;\n')
		fOut.close()
		
	
	# @function writeFileHead2
	# @brief Adds required parameters to the OpenFoam file header for a controlDict file.
	# @param self Self pointer.
	def writeFileHead2(self):
		fOut = open(self.path, 'a', -1)
		self.OFheader()
		fOut.write('    object          controlDict;\n')
		fOut.write('}\n\n')
		fOut.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n\n\n')
		fOut.write('application icoFoam;\n')
		fOut.write('startFrom       startTime;\n')
		fOut.write('startTime       0;\n')
		fOut.write('stopAt          endTime;\n')
		fOut.write('endTime         0.5;\n')
		fOut.write('deltaT          0.005;\n')
		fOut.write('writeControl    timeStep;\n')
		fOut.write('writeInterval   20;\n')
		fOut.write('\n')
		fOut.close()
	
	
	# @function writeFileHead
	# @brief Adds required parameters to the OpenFoam file header for a blockMeshDict file.
	# @param self Self pointer.
	def writeFileHead(self):
		fOut = open(self.path, 'a', -1)
		self.OFheader()
		fOut.write('    object          blockMeshDict;\n')
		fOut.write('}\n\n')
		fOut.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n\n\n')
		fOut.close()

	
	# @function writeScale
	# @brief Writes the convertToMeters parameter for a blockMeshDict file.
	# @param self Self pointer.
	def writeScale(self):
		
		if self.debug > 0:
			print "\n\nWriting BlockMeshDict file:"
		fOut=open(self.path, 'a', -1)
		fOut.write("convertToMeters "+str(self.convertToMeters)+";\n\n")
		fOut.close()


	# @function writeVertices
	# @brief Writes the required vertices for a blockMeshDict file.
	# @param self Self pointer.
	def writeVertices(self):
		
		fOut=open(self.path, 'a', -1)
		fOut.write('vertices\n')
		fOut.write('(\n')
		if self.debug > 1:
			print "\tWriting Vertices..."
		for blockCoordsIndex in xrange (len(self.blockCoords)):
			fOut.write('    (')
			for xyz in xrange (3):
				fOut.write(str(self.blockCoords[blockCoordsIndex,xyz]))
				if xyz != 2:
					fOut.write(' ')
			fOut.write(')   // point ' + str(blockCoordsIndex) + '\n')
		fOut.write(');\n\n')
		fOut.close()
	
	
	# @function computeBlockData
	# @brief Computes some of the data required for creating a blockMeshDict file.
	# @param self Self pointer.
	def computeBlockData(self):
		
		# def block coordinates
		self.hexCoords[0, :] = [0,1,6,7, 8,9,14,15]
		self.hexCoords[1, :] = [1,2,5,6, 9,10,13,14]
		self.hexCoords[2, :] = [2,3,4,5, 10,11,12,13]
		
		# def cells for all blocks
		self.hexCells[0, :] = [self.block1_cells_ax[self.rowIndex], self.block_cells_rad, 1]
		self.hexCells[1, :] = [self.block2_cells_ax[self.rowIndex], self.block_cells_rad, 1]
		self.hexCells[2, :] = [self.block3_cells_ax[self.rowIndex], self.block_cells_rad, 1]
		
		# def grading for all blocks
		self.hexClust[0, :] = [self.AxialGradingBlock1, self.RadialGrading, self.CircGrading]
		self.hexClust[1, :] = [self.AxialGradingBlock2, self.RadialGrading, self.CircGrading]
		self.hexClust[2, :] = [self.AxialGradingBlock3, self.RadialGrading, self.CircGrading]
	
	
	# @function writeBlocks
	# @brief Writes the required blocks into a blockMeshDict file.
	# @param self Self pointer.
	def writeBlocks(self):
		
		fOut=open(self.path, 'a', -1)
		fOut.write('blocks\n')
		fOut.write('(\n')
		if self.debug > 1:
			print "\tWriting Blocks..."
		for hexCoordsIndex in xrange (len(self.hexCoords)):
			fOut.write('    hex (')
			for point in xrange (8):
				fOut.write(str(self.hexCoords[hexCoordsIndex, point]))
				if point != 7:
					fOut.write(' ')
			fOut.write(') (')
			for xyz in xrange (3):
				fOut.write(str(self.hexCells[hexCoordsIndex, xyz]))
				if xyz != 2:
					fOut.write(' ')
			fOut.write(') simpleGrading (')
			for xyz in xrange (3):
				fOut.write(str(self.hexClust[hexCoordsIndex, xyz]))
				if xyz != 2:
					fOut.write(' ')
			fOut.write(')\n')
		fOut.write(');\n\n')
		fOut.close()
		
		
	# @function writeEdges
	# @brief Writes the required edges into a blockMeshDict file.
	# @param self Self pointer.
	def writeEdges(self):
		
		fOut = open(self.path, 'a', -1)
		fOut.write('edges\n')
		fOut.write('(\n')
		if self.debug > 1:
			print "\tWriting Edges..."
		
		
		# === block 1 hub ===
		if self.debug > 1:
			print "\t\tCalculating Block 1 Hub Polyline Edges..."
		
		hub12u = NurbsCurve.doublep()
		p1 = NurbsPoint.Point3Dd(self.points[1][1], 0, self.points[1][0])
		self.Hub3D.minDist2(p1, hub12u)
		blockEdgeLength = self.Hub3D.lengthIn(0, hub12u.value())
		# create list with length of all cells in block
		cellLength  = self.computeSpacing(self.block1_cells_ax[self.rowIndex], self.AxialGradingBlock1, blockEdgeLength)
		u_start = hub12u.value()
		xlist = []
		ylist = []
		for i in range(self.block1_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Hub3D)
			xlist.append(str(self.Hub3D.pointAt(u).getz()))
			ylist.append(str(self.Hub3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('    polyLine 0 1 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 8 9 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')


		# === block 2 hub ===
		if self.debug > 1:
			print "\t\tCalculating Block 2 Hub Polyline Edges..."
		hub23u = NurbsCurve.doublep()
		self.Hub3D.minDistZ(self.points[2][0], hub23u)
		
		blockEdgeLength = self.Hub3D.lengthIn(hub12u.value(), hub23u.value())
		cellLength  = self.computeSpacing(self.block2_cells_ax[self.rowIndex], self.AxialGradingBlock2, blockEdgeLength)
		u_start = hub23u.value()
		xlist = []
		ylist = []
		for i in range(self.block2_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Hub3D)
			xlist.append(str(self.Hub3D.pointAt(u).getz()))
			ylist.append(str(self.Hub3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 1 2 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 9 10 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')


		# === block 3 hub ===
		if self.debug > 1:
			print "\t\tCalculating Block 3 Hub Polyline Edges..."
		
		blockEdgeLength = self.Hub3D.lengthIn(hub23u.value(), 1)
		cellLength  = self.computeSpacing(self.block3_cells_ax[self.rowIndex], self.AxialGradingBlock3, blockEdgeLength)
		
		u_start = 1
		xlist = []
		ylist = []
		for i in range(self.block3_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Hub3D)
			xlist.append(str(self.Hub3D.pointAt(u).getz()))
			ylist.append(str(self.Hub3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 2 3 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 10 11 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')


		# === block 1 shroud ===
		if self.debug > 1:
			print "\t\tCalculating Block 1 Shroud Polyline Edges..."
		
		shroud12u = NurbsCurve.doublep()
		nurbsPoint = NurbsPoint.Point3Dd(self.points[6][1], 0, self.points[6][0])
		self.Shroud3D.minDist2(nurbsPoint, shroud12u)
		blockEdgeLength = self.Shroud3D.lengthIn(0, shroud12u.value())
		
		xlist = []
		ylist = []
		u_start = shroud12u.value()
		cellLength  = self.computeSpacing(self.block1_cells_ax[self.rowIndex], self.AxialGradingBlock1, blockEdgeLength)
		for i in range(self.block1_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Shroud3D)
			xlist.append(str(self.Shroud3D.pointAt(u).getz()))
			ylist.append(str(self.Shroud3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 7 6 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 15 14 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')


		# === block 2 shroud ===
		if self.debug > 1:
			print "\t\tCalculating Block 2 shroud Polyline Edges..."
		
		shroud23u = NurbsCurve.doublep()
		self.Shroud3D.minDistZ(self.points[5][0], shroud23u)
		blockEdgeLength = self.Shroud3D.lengthIn(shroud12u.value(), shroud23u.value())
		
		xlist = []
		ylist = []
		u_start = shroud23u.value()
		cellLength  = self.computeSpacing(self.block2_cells_ax[self.rowIndex], self.AxialGradingBlock2, blockEdgeLength)
		for i in range(self.block2_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Shroud3D)
			xlist.append(str(self.Shroud3D.pointAt(u).getz()))
			ylist.append(str(self.Shroud3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 6 5 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 14 13 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')


		# === block 3 shroud ===
		if self.debug > 1:
			print "\t\tCalculating Block 3 shroud Polyline Edges..."
		
		blockEdgeLength = self.Shroud3D.lengthIn(shroud23u.value(), 1)
		cellLength  = self.computeSpacing(self.block3_cells_ax[self.rowIndex], self.AxialGradingBlock3, blockEdgeLength)
		
		u_start = 1
		xlist = []
		ylist = []
		for i in range(self.block3_cells_ax[self.rowIndex]-1):
			u = self.calculateUStart(u_start, cellLength[i], self.Shroud3D)
			xlist.append(str(self.Shroud3D.pointAt(u).getz()))
			ylist.append(str(self.Shroud3D.pointAt(u).getx()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 5 4 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 13 12 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')
		
		
		# ====== LE ======
		if self.debug > 1:
			print "\t\tCalculating Leading Edge Polyline Edges..."

		blockEdgeLength = self.LEq2D.length()
		cellLength = self.computeSpacing(self.block_cells_rad, self.RadialGrading, blockEdgeLength)

		u_start = 1
		xlist = []
		ylist = []
		for i in range(self.block_cells_rad-1):
			u = self.calculateUStart(u_start, cellLength[i], self.LEq2D)
			xlist.append(str(self.LEq2D.pointAt(u).getz()))	# z-component because of cylindrical coordinate system
			ylist.append(str(self.LEq2D.pointAt(u).gety()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 1 6 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 9 14 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')')
		
		
		# ====== TE ======
		if self.debug > 1:
			print "\t\tCalculating Trailing Edge Polyline Edges..."

		blockEdgeLength = self.TEq2D.length()
		cellLength = self.computeSpacing(self.block_cells_rad, self.RadialGrading, blockEdgeLength)

		u_start = 1
		xlist = []
		ylist = []
		for i in range(self.block_cells_rad-1):
			u = self.calculateUStart(u_start, cellLength[i], self.TEq2D)
			xlist.append(str(self.TEq2D.pointAt(u).getz()))	# z-component because of cylindrical coordinate system
			ylist.append(str(self.TEq2D.pointAt(u).gety()))
			u_start = u
		xlist.reverse()
		ylist.reverse()
		
		fOut.write('\n    polyLine 2 5 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 0) ')
		fOut.write(')\n    polyLine 10 13 (')
		for i in range(len(xlist)):
			fOut.write('(' + xlist[i] + ' ' + ylist[i] + ' 1) ')
		fOut.write(')\n);\n\n')
		fOut.close()
		
		
	# @function writePatches
	# @brief Writes the required patches parameter into the blockMeshDict file.
	# @param self Self pointer.
	def writePatches(self):
		
		fOut = open(self.path, 'a', -1)
		fOut.write('patches\n')
		fOut.write('(\n);\n')
		fOut.close()
		
	
	# @function computeSpacing
	# @brief Calculates the spacing along a block edge according to the grading.
	# @param self Self pointer.
	# @param noCells Number of Cells in the given direction.
	# @param grading Grading along the given direction.
	# @param blockEdgeLength Length of the block edge.
	# @return cellLength List with the lengths of the cells along the edge.
	def computeSpacing(self, noCells, grading, blockEdgeLength):
		cellLength = zeros([noCells], 'd')
		cellToCellExpansionsRatio = grading**(1.0 / (noCells - 1.0))
		if cellToCellExpansionsRatio != 1:
			startSpacing = (blockEdgeLength * (1 - cellToCellExpansionsRatio)) / (1 - cellToCellExpansionsRatio**noCells)
		else:
			startSpacing = blockEdgeLength / noCells
		for cellIndex in xrange(int(noCells)):
			cellLength[cellIndex] = startSpacing * cellToCellExpansionsRatio**(cellIndex)
		return cellLength
	
	
	# @function minLength
	# @brief Calculates the difference in length between the length between two given u-values and the given length.
	# @param self Self pointer.
	# @param u1 U-value of the first point on the NURBS curve.
	# @param u2 U-value of the second point on the NURBS curve.
	# @param length Given length to be subtracted from the length between the two u-values.
	# @param curve Given NURBS curve.
	# #@return lengthNow-length Difference between the calculated length, and the given length.
	def minLength(self, u1, u2, length, curve):
		
		lengthNow = curve.lengthIn(u1, u2, self.tolerance, self.maxIteration)
		return lengthNow - length
	
	
	# @function calculateUStart
	# @brief Calculates a u-value, at a certain point in space.
	# @param self Self pointer.
	# @param u2 Given u-value of the NURBS curve.
	# @param length Length of the NURBS curve.
	# @param curve NURBS curve.
	# #@return uValue Calculated u-value.
	def calculateUStart(self, u2, length, curve):
		
		uValue = bisection(self.minLength, 0, 1, args=(u2, length, curve), xtol=self.tolerance, maxiter=self.maxIteration)
		return uValue
	
	
	# @function read_points
	# @brief Reads points resulting from blockMeshDict command and saves them in the array pointArray2D.
	# @param self Self pointer.
	def read_points(self):
		
		try:
			f = open(self.case+"/row"+str(self.rowIndex)+"/constant/polyMesh/points", 'r')
		except:
			print 'Could not open '+self.case+"/row"+str(self.rowIndex)+"/constant/polyMesh/points"
			print 'Check if the directories are present, and whether the files are filled or empty!\n'
			print 'If files are not present, the \'blockMesh\' command might not be linked correctly.'
			self.cancel = True
		else:
			text = f.readlines()
			
			coordlist = []
			for line in text:
				if "(" in line and ")" in line and line[-3] == "0":
					coordlist.append((line.split()[0][1:], line.split()[1]))
			
			for j in range(self.block_cells_rad+1):
				for i in range(self.block1_cells_ax[self.rowIndex]+1):
					index = j * (self.block1_cells_ax[self.rowIndex] + 1) + i
					self.pointArray2D[i][j][0] = coordlist[index][1]
					self.pointArray2D[i][j][1] = coordlist[index][0]
			for j in range(self.block_cells_rad+1):
				for i in range(self.block2_cells_ax[self.rowIndex]):
					index = ((self.block_cells_rad + 1) * (self.block1_cells_ax[self.rowIndex] + 1)) + (j * (self.block2_cells_ax[self.rowIndex]) + i)
					self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+1][j][0] = coordlist[index][1]
					self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+1][j][1] = coordlist[index][0]
			for j in range(self.block_cells_rad+1):
				for i in range(self.block3_cells_ax[self.rowIndex]):
					index = ((self.block_cells_rad + 1) * (self.block1_cells_ax[self.rowIndex] + self.block2_cells_ax[self.rowIndex] + 1)) + (j * self.block3_cells_ax[self.rowIndex] + i)
					self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+self.block2_cells_ax[self.rowIndex]+1][j][0] = coordlist[index][1]
					self.pointArray2D[i+self.block1_cells_ax[self.rowIndex]+self.block2_cells_ax[self.rowIndex]+1][j][1] = coordlist[index][0]
		
	