#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian Barthmes
# @author Oliver Borm
# @author Julius Fellerhoff
# @date 26.09.2010

# Python Imports
import os, sys
from copy import *

# 3rd Party
from math import atan2, sqrt
import numpy
from xml.dom import minidom
from pythonnurbs import NurbsPoint, NurbsMatrix, NurbsSurface, NurbsCurve, NurbsColor, NurbsVector, NurbsHPoint, NurbsMatrixRT

# BladeDesigner
import turboMachine, nurbsToolSet
import debugPlots #for using matplotlib plots
#import profile2D_patched20100924 as profile2D
import profile2D
import camberSurface

## @brief Class which creates turbomachinery blades from given parameters and profile2D-Objects.
# @brief For detailed description of the algorithms used in this class, see "Entwicklung eines parametrischen CAD-Systems für Turbomaschinen", Sebastian Barthmes, 2010, Semester Thesis, TU Munich (in german)
class bladeRow(object):
	## @brief Constructor
	# @brief Available Arguments:
	# @brief \n without arguments
	# @brief \n with a number of profiles
	# @brief \n with a list [Row Number, Number of Profiles]
	# @brief \n with an xml filename (String)
	# @brief \n with a dataStructure-object + number of profiles
	def __init__(self, *args):
		
	#+++++ global variables and lists: +++++++
		
		##Load XML?
		self.useNurbsXml=False
		## flag for definition, =0 stacking axis + stacking point + chord length, =1 leading edge + trailing edge + pitchwise offset, =2 caskade of blades
		self.definition = 0
		## number of blades
		self.numberBlades = 0.0
		## revolutions per second
		self.omega = 0.0
		## array of 2DProfil-objects
		self.listProfiles=[]
			
	#+++++++++ cascade specific: ++++++++++++++++
	
		## distance between neighbouring blades
		self.bladeDist = 0.0
		## total length of domain
		self.domainLength = 0.0
		## stacking axis, defined by the z-coordinate
		self.stackingLine = 0.0
		## span of the blade
		self.bladeSpan = 0.0
		
	#+++++++++ turbomachine specific: +++++++++++
		
		## Trailing edge flag, ="sharp" Sharp TE (non continuous / open nurbs), ="round" Round TE (continuous), ="blunt" TE (sharp cutoff)
		self.teflag="round"
		## flag for meridional surface distribution, =0 isosurface method (default), =1 userdefined
		self.spanDistr = 0
		## array containing the span distribution, according to each profile (from 0 to 1)
		self.span = []
		## the curve of revolution of the hub, coord z-r
		self.coordinatesHub=[]
		## the curve of revolution of the shroud, coord z-r
		self.coordinatesShroud=[]
		## flag for 3D-transformation algorithm, =0 length preserving (for axial machines), =1 angle preserving (for any radial machines)
		self.angleFlag=0
		## array containing the circumferential offset of each profile (e.g for bow in blades)
		self.periphalOffset=[]
		
		#******* definition=0 (stacking axis + stacking point + chord length):
		
		## array of coordinates containing the stacking points, z-r
		self.coordinatesStackingPoints=[]
		## array of coordinates containing the stacking axis, z-r
		self.coordinatesStackingLine=[]
		## nurbs curve of stacking line, z-r
		self.nurbsStackingLine=NurbsCurve.NurbsCurve2Dd()
		
		#******* Für definition=1 (leading edge + trailing edge + pitchwise offset):
		
		## array of coordinates of the leading edge
		self.coordinatesLE=[]
		## array of coordinates of the trailing edge
		self.coordinatesTE=[]
		## array containing the distance of LE and TE in rad
		self.circumferentialDistance=[]
		## maximum Iteration for LETE Algorithm
		self.maxIterLETE=10
		## maximum Error of LETE Algorithm in radiant (Tolerance)
		self.errorLETE=1e-2
		
		

	#++++++++ coordinate-outputarrays for external access etc.++++++
	
		## 3D coordinates of upper profile
		self.coordinatesUpperProfile=[]
		## 3D coordinates of lower profile
		self.coordinatesLowerProfile=[]
		## 3D coordinates of camber profile
		self.coordinatesCamberProfile=[]
		## 3D coordinates of profile points
		self.coordinatesProfiles=[]
		
		
	#++++++++ nurbs-outputarrays for external access etc.++++++
	
		## list of rotational NURBS curves:
		self.nurbsRot=[]
		## list of NURBS-profiles (3D)
		self.nurbsProfile=[]
		## list of NURBS-camber lines (3D)
		self.nurbsCamber=[]
		## NURBS blade surface
		self.bladeSurface=NurbsSurface.NurbsSurfaced()
		## NURBS camber surface
		#self.camberSurface=NurbsSurface.NurbsSurfaced()

		#BB:
		self.LE3D = NurbsCurve.NurbsCurved()
		self.TE3D = NurbsCurve.NurbsCurved()

	#+++++++++++++ output options +++++++++++++++++++++++++++++
	
		## flag to set debug level, =0 quiet mode, =1 progress info, =2 spam all the data
		self.debug=0
		
	#++++++++++++++ Mesh init options ++++++++++++++++++++++++++
		
		## blockMeshDictGen object
		self.mesh=[]
		## Number of Layers
		self.noLayer=2
		## Topology
		self.topology = "O10H"
		## start Profile
		self.startProfile=0
		
	#++++++ expert-options and internal variables +++++++++++++
		
		## Row number
		self.rowNumber = 0
		## Expert-Parameter: degree of nurbs curves, defines the minimum number of points needed for interpolation (degree+1)
		self.degree = 3
		## Expert-Parameter: degree of nurbs curves, defines the minimum number of profiles needed for interpolation (degree+1)
		self.degreeU = 2
		## Expert-Parameter: number of profiles, should be set with the setNumberProfiles() method to properly initialize profiles
		self.numberProfiles = 0
		## dataStructure-object, contains all data about the working directories and their structure
		self.dataStruct=[]
		## OCC display handler
		self.display=None
	
	#+++++++++++++Interne Variablen+++++++++++++++++++++++++++
	
		## reference profiles for blockMeshDictGen
		self.refNurbsProfile = NurbsCurve.NurbsCurved()
		## reference profiles for blockMeshDictGen
		self.refCamberLine = NurbsCurve.NurbsCurved()
		## 2D nurbs curve of leading edge
		self.nurbsLE=NurbsCurve.NurbsCurve2Dd()
		## 2D nurbs curve of trailing edge
		self.nurbsTE=NurbsCurve.NurbsCurve2Dd()
		## the curve of revolution of the hub, nurbs z-r
		self.nurbsHub=NurbsCurve.NurbsCurve2Dd()
		## the curve of revolution of the shroud, nurbs z-r
		self.nurbsShroud=NurbsCurve.NurbsCurve2Dd()
		
		## Progress function is only called from the GUI and also assigned there
		self.step = lambda: None
		## determites whether the profile is already calculated or not
		self.isCalculated = False
		
	
		tmp = -1
		for arg in args:
			if type(arg)==int:
				tmp=arg
			elif type(arg)==list:
				self.rowNumber=arg[0]
				tmp=arg[1]
			elif type(arg)==str :
				self.readRowXml(arg)
			elif type(arg)==turboMachine.dataStructure:
				self.dataStruct=arg
				#if os.path.exists(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml'):
					#self.readRowXml(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml')
			else:
				raise Exception, "init error"
		if tmp!=-1:
			self.setNumberProfiles(tmp)
			try:
				import blockMeshDictGen
				self.mesh=blockMeshDictGen.meshGen(self)
			except:
				print "blockMeshDictGen not available. Skipping."

	# BB:
	## @brief Create CamberSurface (needed for mesh snapping, blockMeshDictGen):
	def calculate_camber_surface(self):
		resCamber = 10
		i = 0
		j = 0
		matrix = NurbsMatrix.Matrix_Point3Dd(len(self.nurbsCamber), resCamber)
		for CamberLine in self.nurbsCamber:
			for c in range(resCamber):
				u = c / (resCamber - 1.0)
				p = CamberLine.pointAt(u)
				nurbsPoint = NurbsPoint.Point3Dd(p.getx(), p.gety(), p.getz())
				matrix.setelem(i, j, nurbsPoint)
				j += 1
			i += 1
			j = 0
		self.CS = camberSurface.camberSurface()
		self.CS.CamberSurface3D(matrix, self.degree)
		self.camberSurface = self.CS.camberSurface
		
	#BB:
	## @brief Calculates the Leading and trailing edges (LE3D needed for SCMMesher!)
	def calculate_LETE3D(self):
		noProfiles = len(self.listProfiles)
		# initializing arrays of coordinates, which will serve as interpolation points:
		LEarray = numpy.zeros((noProfiles, 3))
		TEarray = numpy.zeros((noProfiles, 3))
		# loop thru all nurbs camberlines:
		for i,camber in enumerate(self.nurbsCamber):
			LEarray[i] = nurbsToolSet.getCurveCoordinates(0, camber)
			TEarray[i] = nurbsToolSet.getCurveCoordinates(1, camber)
		# interpolation thru array of points:
		self.LE3D = nurbsToolSet.interpolateNurbs3D(LEarray)
		self.TE3D = nurbsToolSet.interpolateNurbs3D(TEarray)

	#BB:
	## @brief Calculates the Leading and trailing edges that are needed for the SCMMeshers transfinite interpolation meshing.
	def calculate_LETE2D(self):
		noProfiles = len(self.listProfiles)
		# initializing arrays of coordinates, which will serve as interpolation points:
		LEarray = numpy.zeros((noProfiles, 2))
		TEarray = numpy.zeros((noProfiles, 2))
		# loop thru all nurbs camberlines:
		for i,camber in enumerate(self.nurbsCamber):
			xy = numpy.ones([2], 'd')
			
			# getting LE coordinates:
			# 3D z-coordinate becomes 2D x-coordinate:
			x = camber.pointAt(0).getz()
			# 3D r-coordinate becomes 2D y-coordinate:
			y = sqrt(camber.pointAt(0).getx()**2 + camber.pointAt(0).gety()**2)
			xy = [x, y]
			LEarray[i] = xy

			# getting TE coordinates:
			# 3D z-coordinate becomes 2D x-coordinate:
			x = camber.pointAt(1).getz()
			# 3D r-coordinate becomes 2D y-coordinate:
			y = sqrt(camber.pointAt(1).getx()**2 + camber.pointAt(1).gety()**2)
			xy = [x, y]
			TEarray[i] = xy
		# getting the first point of LE in array (intersection LE and hub):
		u = NurbsCurve.doublep()
		u.assign(0.3)
		p1 = NurbsPoint.Point2Dd(LEarray[0][0], LEarray[0][1])
		self.nurbsHub.minDist2(p1, u)
		p1 = self.nurbsHub.pointAt(u.value())
		newx = p1.getx()
		newy = p1.gety()
		xy = numpy.ones([2], 'd')
		xy = [newx, newy]
		# overwriting the first point with the intersection
		LEarray[0] = xy
		# getting the last point of LE in array (intersection LE and shroud):
		u.assign(0.3)
		p1 = NurbsPoint.Point2Dd(LEarray[-1][0], LEarray[-1][1])
		self.nurbsShroud.minDist2(p1, u)
		p1 = self.nurbsShroud.pointAt(u.value())
		newx = p1.getx()
		newy = p1.gety()
		xy = numpy.ones([2], 'd')
		xy = [newx, newy]
		# overwriting the last point with the intersection
		LEarray[-1] = xy

		# getting the first point of TE in array (intersection TE and hub):
		u = NurbsCurve.doublep()
		u.assign(0.7)
		p1 = NurbsPoint.Point2Dd(TEarray[0][0], TEarray[0][1])
		self.nurbsHub.minDist2(p1, u)
		p1 = self.nurbsHub.pointAt(u.value())
		newx = p1.getx()
		newy = p1.gety()
		xy = numpy.ones([2], 'd')
		xy = [newx, newy]
		# overwriting the first point with the intersection
		TEarray[0] = xy
		# getting the last point of TE in array (intersection TE and shroud):
		u.assign(0.7)
		p1 = NurbsPoint.Point2Dd(TEarray[-1][0], TEarray[-1][1])
		self.nurbsShroud.minDist2(p1, u)
		p1 = self.nurbsShroud.pointAt(u.value())
		newx = p1.getx()
		newy = p1.gety()
		xy = numpy.ones([2], 'd')
		xy = [newx, newy]
		# overwriting the last point with the intersection
		TEarray[-1] = xy

		# interpolation thru array of points:
		self.nurbsLE = nurbsToolSet.interpolateNurbs2D(LEarray)
		self.nurbsTE = nurbsToolSet.interpolateNurbs2D(TEarray)
	
	## @brief convenient method for angle definitions
	# @param alist a list of profile chord relative angle of attack (in degrees) with length of number of profiles
	# @param olist a list of profile chord relative outflow angles (in degrees) with length of number of profiles
	def setIOAngles(self,alist=[],olist=[]):
		if len(alist)!=len(self.listProfiles) or len(olist)!=len(self.listProfiles):
			raise Exception, "Wrong number of Angles defined"
		for a in range(len(alist)):
			self.listProfiles[a].angleOfAttack=float(alist[a])
			self.listProfiles[a].outflowAngle=float(olist[a])
	
	## @brief convenient method for angle definitions
	# @param b1list a list of absolute angle of attack (in degrees) with length of number of profiles
	# @param b2list a list of absolute outflow angles (in degrees) with length of number of profiles
	def setBetaAngles(self,b1list=[],b2list=[]):
		if len(b1list)!=len(self.listProfiles) or len(b2list)!=len(self.listProfiles):
			raise Exception, "Wrong number of Angles defined"
		for a in range(len(b1list)):
			self.listProfiles[a].beta1=float(b1list[a])
			self.listProfiles[a].beta2=float(b2list[a])
	
	
	## @brief convenient method for gamma definitions
	# @param alist a list of stagger angles (in degrees) with length of number of profiles
	def setStaggerAngles(self,alist=[]):
		if len(alist)!=len(self.listProfiles):
			raise Exception, "Wrong number of Angles defined"
		for a in range(len(alist)):
			self.listProfiles[a].gamma=numpy.radians(float(alist[a]))
	
	
	## @brief convenient method for chordLength definitions
	# @param alist a list of lengths (in mm) with length of number of profiles
	def setChordLengths(self,alist=[]):
		if len(alist)!=len(self.listProfiles):
			raise Exception, "Wrong number of Lengths defined"
		for a in range(len(alist)):
			self.listProfiles[a].chordLength=float(alist[a])
	
	
	## @brief convenient method for maxThickness definitions
	# @param alist a list of lengths (in mm) with length of number of profiles
	def setMaxThickness(self,alist=[]):
		if len(alist)!=len(self.listProfiles):
			raise Exception, "Wrong number of maximum Thickness defined"
		for a in range(len(alist)):
			self.listProfiles[a].maxThickness=float(alist[a])
	
	
	## @brief convenient method for Ca0 definitions
	# @param alist a list of Ca0 with length of number of profiles
	def setCa0(self,alist=[]):
		if len(alist)!=len(self.listProfiles):
			raise Exception, "Wrong number of Ca0 defined"
		for a in range(len(alist)):
			self.listProfiles[a].Ca0=float(alist[a])
	
	
	## @brief Calls the computeBladeRow() function. Should be called to start the computation of the blade
	def __call__(self):
		#Was the profile calculated before?
		if (self.isCalculated):
			#Debugoptionen aktiviert?
			if self.debug > 0:
				print "Eingabewerte für dieses Profil haben sich nicht geändert -> übersprungen"
			#Hier abbrechen
			return
			
		if os.path.exists(self.dataStruct.workdir+self.dataStruct.dirGeom+'/Row_'+str(self.rowNumber)+'_Nurbs.xml') and self.useNurbsXml==True:
			self.__readNurbsFromXml()
		else:
			self.computeBladeRow()
		
		#BB: zur berechnung der LE und TE für den SCMMesher
		#SB: Bitte erst da aufrufen wos gebraucht wird. Verursacht momentan noch Fehler wenn der Grad der Nurbskurven != 3 ist!!. BTW: Hat schon jemand irgendwo eine "echte" Skelettfläche aus der Nurbsfläche berechnet?
		#self.calculate_LETE3D()
		#self.calculate_LETE2D()
		
		self.isCalculated = True
		
		
	## @brief function to initialize the number of profile2D-Objects
	def setNumberProfiles(self,np):
		if np == 0:
			raise Exception, "Value > 0 expected."
		if self.numberProfiles != 0:
			raise Exception, "Variable numberProfiles already set." 
		self.numberProfiles = np
		for n in range(self.numberProfiles):
			self.listProfiles.append(profile2D.profile2D(n,self.dataStruct))
			
			
	## @brief main function where the bladerow-building process gets initiated and supervised
	def computeBladeRow(self):
		#Initialisierung von Gamma und Berechnung der Profile
		for n in range(self.numberProfiles):
			if self.definition==0 or self.definition==2:
				self.__computeGamma(n)
				if self.debug>=1:
					print "*** Computing Profile",n
				self.listProfiles[n].callfunctions()
			
			#SW: Wieder ein Schritt geschafft
			self.step()
		
		if self.debug>=1:
			print "*** ...Done!"
		
		############## DEBUG plot - check profile2d output ###########
		#import pylab as st
		#x=[]
		#y=[]
		#for n in range(self.numberProfiles):
			#x.append(self.listProfiles[n].upperProfile[:,0])
			#y.append(self.listProfiles[n].upperProfile[:,1])
			#x.append(self.listProfiles[n].lowerProfile[:,0])
			#y.append(self.listProfiles[n].lowerProfile[:,1])					
		#st.axis("equal")
		#st.plot(x,y,"bo")
		#st.show()
		
		#Initialisierung der noetingen Variablen
		if self.numberProfiles==0:
			self.numberProfiles = len(self.listProfiles)
		if self.definition==2:
			self.numberProfiles=5 #Anzahl der Schnitte
			tmp=copy(self.listProfiles[0])
			self.listProfiles=[copy(tmp),copy(tmp),copy(tmp),copy(tmp),copy(tmp)]
			del tmp
		if self.definition==0 or self.definition==2:
			self.coordinatesUpperProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].upperProfile),3))
			self.coordinatesLowerProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].lowerProfile),3))
			self.coordinatesCamberProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].camberLine),3))
		
		if self.numberProfiles<(self.degreeU+1):
			raise Exception, "Interpolation not possible, decrease degreeU or increase profile number"
		if self.degreeU<2 or self.degree<3:
			print "***\n*** WARNING: degree for interpolation to small. Problems may occur!\n***"
		
		
		#Interpolation der Geometrielinien
		if self.definition==0 or self.definition==1:
			if self.coordinatesHub!=[]:
				self.nurbsHub = nurbsToolSet.interpolateNurbs2D(self.coordinatesHub)
			if self.coordinatesShroud!=[]:
				self.nurbsShroud = nurbsToolSet.interpolateNurbs2D(self.coordinatesShroud)

		#Ausführen der Berechnungsalgorithmen
		if self.definition==0 or self.definition==1:
			self.__turbAlgorithm()
			############### DEBUG plot - check 2D algorithm ###############
			#import pylab as st
			#x=[]
			#y=[]
			#for n in range(self.numberProfiles):
				#x.append(self.listProfiles[n].upperProfile[:,0])
				#y.append(self.listProfiles[n].upperProfile[:,1])
				#x.append(self.listProfiles[n].lowerProfile[:,0])
				#y.append(self.listProfiles[n].lowerProfile[:,1])
			##x.append(self.coordinatesStackingLine[:,0])
			##y.append(numpy.zeros((len(self.coordinatesStackingLine))))
			##print x
			##print y
			##print len(x)
			##print len(y)
			#st.axis("equal")
			#st.plot(x,y,"bo")
			##st.plot(self.coordinatesStackingLine[0,1]-50,0,"rx")
			#st.show()
				
		if self.definition==2: #Kaskade
			self.__cascadeAlgorithm()
		
		#Interpolation der diskreten Punkte zur Nurbsfläche
		#self.createBladeNurbsCurves()
		self.__interpolateNurbsSurface()
		self.__writeNurbsToXml()
		
		#debugPlots.plotRotNurbs(self.nurbsRot,self.nurbsStackingLine)
		#SW: Wieder ein Schritt geschafft
		self.step()
		if self.debug>=1:
			print "*** Bladerow generation complete. :-)"
			
	
	## @brief algorithm for cascade definition
	def __cascadeAlgorithm(self):
		self.span=[0]
		if self.debug>=1:
			print "\tScaling profile..."
		self.__scale2D(0)
		self.coordinatesStackingPoints = numpy.zeros((1,2))
		self.coordinatesStackingPoints[0] = self.__computeStackingPoint(0)
		
		if self.debug>=1:
				print "\tTranslating profile to stackingpoint..."
		self.__translateToStackingPoint(0)
		
		if self.debug>=1:
			print "\tRotating profile..."
		self.__rotate2D(0)
		
		if self.debug>=1:
			print "\tTranslating profile to position..."
		self.__translateToCascade(0)
			
		for n in range(self.numberProfiles):
			if self.debug>=1:
				print "\tComputing 3D data..."
			self.__computeProfile3D(n)
		
		self.refNurbsProfile=self.getBladeSection(0.0)
		self.refCamberLine=nurbsToolSet.interpolateNurbs3D(self.coordinatesCamberProfile[0]) #TODO -> richtig?
	
	
	## @brief algorithm for rotational definition
	def __turbAlgorithm(self):
		if self.spanDistr==0:
			self.span=[]
		elif self.spanDistr==1 and len(self.span)!=self.numberProfiles:
			raise Exception, "span array not defined properly"
		
		if self.definition == 0: #Wenn durch Faedelpunkt, Faedellinie und Sehnenlaenge definiert
			#Nurbskurven fuer Faedellinie interpolieren
			self.nurbsStackingLine = nurbsToolSet.interpolateNurbs2D(self.coordinatesStackingLine,degree=self.degreeU)
			if self.debug>=2:
				print "\tinterpolated StackingLine Length: ",self.nurbsStackingLine.length()
			if self.coordinatesStackingPoints == []:
				self.coordinatesStackingPoints = numpy.zeros((self.numberProfiles,2))
			#self.showInputGeometry()
			#2D-Profile auf 3D-Meridianebene bringen und skalieren
			for n in range(self.numberProfiles):
				if self.debug>=1:
					print "*** Row:",self.rowNumber,"Profile:",n
					print "\tComputing meridian..."
				self.nurbsMeri = self.__computeMeridian(n)
				self.nurbsRot.append(nurbsToolSet.convertNurbs2Dto3D(self.nurbsMeri))
				if self.debug>=1:
					print "\tScaling profile..."
				self.__scale2D(n)
				if self.debug>=1:
					print "\tComputing stackingpoint..."
				self.coordinatesStackingPoints[n] = self.__computeStackingPoint(n)
				if self.debug>=1:
					print "\tTranslating profile to stackingpoint..."
				self.__translateToStackingPoint(n)
				if self.debug>=1:
					print "\tRotating profile..."
				self.__rotate2D(n)
				if self.debug>=1:
					print "\tTranslating profile to stackingline..."
				self.__translateToStackingLine(n)
				#Fortschritt anzeigen
				self.step()
				if self.debug>=1:
					print "\tComputing 3D data..."
				self.__computeProfile3D(n)
				
				self.step()

			#SW: Wieder ein Schritt geschafft
			if self.debug>=1:
				print "*** Computing profile done. Gathering now 3D data for output..."
			self.__cylindricalToCartesian()
			
		if self.definition == 1: #Wenn durch Vorderkante, Hinterkante und circumferentialDistance definiert
			#interpoliere Vorder und Hinterkante in der S1-Ebene
			self.nurbsLE=nurbsToolSet.interpolateNurbs2D(self.coordinatesLE)
			self.nurbsTE=nurbsToolSet.interpolateNurbs2D(self.coordinatesTE)
			#2D-Profile auf 3D-Meridianebene bringen und skalieren
			for n in range(self.numberProfiles):
				if self.debug>=1:
					print "*** Row:",self.rowNumber,"Profile:",n
					print "\tComputing meridian..."
				#Meridiane berechnen
				self.nurbsMeri = self.__computeMeridian(n)
				self.nurbsRot.append(nurbsToolSet.convertNurbs2Dto3D(self.nurbsMeri))
				if self.debug>=1:
					print "\tComputing chordlength from edges..."
				#chordLength=numpy.zeros((self.numberProfiles))
				
				self.u_meri=[0.,0.]
				self.u_edge=[0.,0.]
				self.cut1,self.u_meri[0],self.u_edge[0]=nurbsToolSet.computeCuttingPoint(self.nurbsMeri,self.nurbsLE,1)
				self.cut2,self.u_meri[1],self.u_edge[1]=nurbsToolSet.computeCuttingPoint(self.nurbsMeri,self.nurbsTE,1)
				
				if self.debug>=1:
					print "\tStarting Iteration..."

				tol=self.errorLETE
				i=0
				maxIter=self.maxIterLETE
				cd=self.circumferentialDistance[n]
				origpro=copy(self.listProfiles[n])
				j=2.0
				
				iterrange=numpy.pi/6
				while i<maxIter:
					if self.debug>=1:
						if self.debug>=1:
							print "\t*",i+1
					self.listProfiles[n]=copy(origpro)
					
					offset = self.__LETEAlgorithm(cd,n)
					krit=offset-self.circumferentialDistance[n]

					if abs(krit)<=tol:
						break
					elif krit<0:
						cd+=iterrange/j
					elif krit>0:
						cd-=iterrange/j
					j*=2
					i+=1
				
				
				#calculateProfiles
				if self.debug>=1:
					print "\tTotal error in circumferential direction:",numpy.degrees(krit),"°"
					print "\tStagger Angle:",numpy.degrees(self.listProfiles[n].gamma),"°"
					print "\tChord Length:",self.listProfiles[n].chordLength
					print "\tangleOfAttack:",self.listProfiles[n].angleOfAttack,"°"
					print "\toutflowAngle:",self.listProfiles[n].outflowAngle,"°"
				
				
				#Fortschritt
				self.step()
			if self.debug>=1:	
				print "*** Computing profile done. Gathering now 3D data for output..."
			self.__cylindricalToCartesian()
			

	## @brief iterative Part of the algorithm for definition=1 (LE / TE)
	def __LETEAlgorithm(self,offset,n):
		#Schnittpunkte von LE und TE mit der Meridiane berechnen
		#m-L nge und Sehnenl nge (aus Gamma und Phi) berechnen
		self.__computeChordLengthFromLETE(n,offset)
		#if self.debug>=1:
			#print "\tScaling profile..."
		self.__scale2D(n)
		#Profil skalieren
		#if self.debug>=1:
			#print "\tRotating profile..."
		self.__rotate2D(n)
		#Profil drehen
		#if self.debug>=1:
			#print "\tTranslating profile"
		#3D-Kante mit entsprechender Profilkante zur Deckung bringen
		self.__translateToEdge3D(n)
		#Fortschritt
		self.step()
		#if self.debug>=1:
			#print "\tComputing 3D data..."
		#3D-Punkteberechnung
		self.__computeProfile3D(n)
		#print self.coordinatesCamberProfile[n,-1,1]
		return self.coordinatesCamberProfile[n,-1,1]

	
	## @brief compute Gamma from different angle/profile definitions
	# @param n the profile number
	def __computeGamma(self,n):
		if self.listProfiles[n].angleDefinition==0: #User defined
			pass
		elif self.listProfiles[n].angleDefinition==1: #Symmetrical
			self.listProfiles[n].theta=self.listProfiles[n].beta1-self.listProfiles[n].beta2
			self.listProfiles[n].chi1=90+self.listProfiles[n].theta/2.0
			self.listProfiles[n].chi2=90-self.listProfiles[n].theta/2.0
			self.listProfiles[n].gamma=numpy.radians(self.listProfiles[n].beta1-self.listProfiles[n].chi1)
			#write internal profile2D variables
			self.listProfiles[n].angleOfAttack=self.listProfiles[n].chi1
			self.listProfiles[n].outflowAngle=self.listProfiles[n].chi2
		elif self.listProfiles[n].angleDefinition==2: #NACA
			#if self.listProfiles[n].beta2==0 and self.listProfiles[n].beta1!=0:
				#refbeta=self.listProfiles[n].beta1
			#elif self.listProfiles[n].beta1==0 and self.listProfiles[n].beta2!=0:
				#refbeta=self.listProfiles[n].beta2
			#else:
				#raise Exception, "For NACA definition select one(!) of beta1 or beta2"
			self.listProfiles[n].chi1=atan2(self.listProfiles[n].camberLine[5,1]-self.listProfiles[n].camberLine[1,1],self.listProfiles[n].camberLine[5,0]-self.listProfiles[n].camberLine[1,0])+numpy.pi/2.0
			#print self.listProfiles[n].camberLine[5,1],"-",self.listProfiles[n].camberLine[1,1],"/",self.listProfiles[n].camberLine[5,0],"-",self.listProfiles[n].camberLine[1,0],"=",self.listProfiles[n].chi1
			#print self.listProfiles[n].chi1/numpy.pi*180
			self.listProfiles[n].chi2=atan2(self.listProfiles[n].camberLine[-1,1]-self.listProfiles[n].camberLine[-2,1],self.listProfiles[n].camberLine[-1,0]-self.listProfiles[n].camberLine[-2,0])+numpy.pi/2.0
			#print self.listProfiles[n].chi2/numpy.pi*180
			self.listProfiles[n].theta=self.listProfiles[n].chi1+self.listProfiles[n].chi2
			self.listProfiles[n].gamma=((self.listProfiles[n].beta1-self.listProfiles[n].chi1)+(self.listProfiles[n].beta2-self.listProfiles[n].chi2))/2.0
			print "Computed Gamma:", self.listProfiles[n].gamma/numpy.pi*180
			#write internal profile2D variables
			self.listProfiles[n].angleOfAttack=self.listProfiles[n].chi1
			self.listProfiles[n].outflowAngle=self.listProfiles[n].chi2
		else:
			raise Exception, "angleDefinition parameter of profile not recognised"
		
		
	
	## @brief cascade-definition specific coordinate translation
	# @param n the profile number
	def __translateToCascade(self,n):
		for pts in [self.listProfiles[n].upperProfile,
					self.listProfiles[n].lowerProfile,
					self.listProfiles[n].camberLine]:
			pts[:,0]+=self.stackingLine
	
	
	## @brief transforms the 3D-profile coordinates from cylindrial to cartesian coordinates
	def __cylindricalToCartesian(self):
		for pts in [self.coordinatesUpperProfile,
					self.coordinatesLowerProfile,
					self.coordinatesCamberProfile]:
			tmpcoords = pts.copy()
			pts[:,:,1]=-numpy.sin(tmpcoords[:,:,1])*tmpcoords[:,:,0]
			pts[:,:,0]=numpy.cos(tmpcoords[:,:,1])*tmpcoords[:,:,0]
	
	
	## @brief cylindrical mapping algorithm
	# @param n the profile number
	def __cylindricalMapping(self,n):
		for pts in [[self.coordinatesUpperProfile,self.listProfiles[n].upperProfile],
					[self.coordinatesLowerProfile,self.listProfiles[n].lowerProfile],
					[self.coordinatesCamberProfile,self.listProfiles[n].camberLine]]:
			for i in range(len(pts[1])):
				u=nurbsToolSet.calculateUValueFromDistance(pts[1][i,0],0.0,self.nurbsMeri)
				point = self.nurbsMeri.pointAt(u)
				pts[0][n,i,0]=point.gety()
				pts[0][n,i,1]=pts[1][i,1]/point.gety()
				pts[0][n,i,2]=point.getx()
				
		#print max(self.coordinatesUpperProfile[n,:,1]*self.coordinatesUpperProfile[n,:,0])
	
	
	## @brief conformal mapping algorithm
	# @param n the profile number
	def __conformalMapping(self,n):
		#Schreibe alle Profillinien in ein Array
		pts2D=numpy.zeros((len(self.listProfiles[n].upperProfile)+len(self.listProfiles[n].lowerProfile)+len(self.listProfiles[n].camberLine),2))
		pts2D[:len(self.listProfiles[n].upperProfile)]=self.listProfiles[n].upperProfile
		pts2D[len(self.listProfiles[n].upperProfile):-len(self.listProfiles[n].camberLine)]=self.listProfiles[n].lowerProfile
		pts2D[-len(self.listProfiles[n].camberLine):]=self.listProfiles[n].camberLine
		
		#import scitools.std as p
		#p.hold('on')
		#p.plot(pts2D[:,0],pts2D[:,1],'bo')
		#print pts2D
		
		#Spiegle 2D
		pts2D[:,1]*=-1
		
		#finde vordersten und hintersten Punkt
		minm=min(pts2D[:,0])
		maxm=max(pts2D[:,0])
		miny=min(pts2D[:,1])
		
		#Schiebe 2D in positiven Bereich
		#pts2D[:,1]+=self.listProfiles[n].chordLength
		pts2D[:,1]-=miny
		
		maxy=max(pts2D[:,1])
		
		
		
		#speichere die umfangsverschiebung
		if self.definition==0:
			deltaphi=-miny/self.nurbsMeri.pointAt(self.mCuttingPoint).gety()
		else:
			deltaphi=0
		
		pts2D[:,0]-=minm
		
		#Definition des 2D-Netzes (äquidistant mit ds, orthogonal)
		noElemI=30
		ds=(maxm-minm)/(noElemI)
		#print(int(self.listProfiles[n].chordLength*2/ds))
		noElemJ=int(numpy.ceil(maxy/ds))
		#print noElemJ
		#Zuordnung Kurvenpunkte zu den 2D-Netzelementen
		pts=numpy.zeros((len(pts2D),4)) # i,j,fm,fy
		for i in range(len(pts2D)):
			pts[i,0]=int((pts2D[i,0])/ds) #Indizierung von 0-30 (!) -> 31 Knoten werden in der Matrix benötigt
			pts[i,1]=int((pts2D[i,1])/ds)
			pts[i,2]=(pts2D[i,0]-pts[i,0]*ds)/ds
			pts[i,3]=(pts2D[i,1]-pts[i,1]*ds)/ds
		
		#print pts[:,1]
		
		#if self.definition==1:
			#dphi,m_vec=self.approximateDeltaPhiLETE(minm,maxm,noElemI)
		#else:
		dphi,m_vec=self.approximateDeltaPhi(minm,maxm,noElemI)
		
		
		
		#3D-Netzgenerierung mit quadratischen orthogonalen Zellen:
		meshVerts3D=numpy.zeros((noElemI+1,noElemJ+1,3)) # r-phi-z Netzkoordinaten
		#mStart=0
		#phiStart=noElemJ*ds/2
		#m=mStart
		for i in range(noElemI+1):
			m=m_vec[i]
			for j in range(noElemJ+1):
				meshVerts3D[i,j,0]=self.nurbsMeri.pointAt(m/self.nurbsMeri.length()).gety()
				meshVerts3D[i,j,1]=j*dphi-deltaphi
				meshVerts3D[i,j,2]=self.nurbsMeri.pointAt(m/self.nurbsMeri.length()).getx()
		
		#print meshVerts3D[0,:,:]
		
		#Bilde 2D-Geometrie über gespeicherte Zellindizes auf 3D ab
		pts3D=numpy.zeros((len(pts2D),3))
		for i in range(len(pts3D)):
			v=meshVerts3D[pts[i,0],pts[i,1]]
			if pts[i,1]!=(noElemJ):
				vdj=meshVerts3D[pts[i,0],pts[i,1]+1]
				pts3D[i,1]=v[1]+pts[i,3]*(vdj[1]-v[1])
			else:
				pts3D[i,1]=v[1]
			if pts[i,0]!=(noElemI):
				vdi=meshVerts3D[pts[i,0]+1,pts[i,1]]
				pts3D[i,0]=v[0]+pts[i,2]*(vdi[0]-v[0])
				pts3D[i,2]=v[2]+pts[i,2]*(vdi[2]-v[2])
			else:
				pts3D[i,0]=v[0]
				pts3D[i,2]=v[2]
		
		self.coordinatesUpperProfile[n,:]=pts3D[:len(self.listProfiles[n].upperProfile)]
		self.coordinatesLowerProfile[n,:]=pts3D[len(self.listProfiles[n].upperProfile):-len(self.listProfiles[n].camberLine)]
		self.coordinatesCamberProfile[n,:]=pts3D[-len(self.listProfiles[n].camberLine):]
		
		for pts in [self.coordinatesUpperProfile,
					self.coordinatesLowerProfile,
					self.coordinatesCamberProfile]:
			pts[n,:,1]*=-1
		
		#print max(self.coordinatesUpperProfile[n,:,1]*self.coordinatesUpperProfile[n,:,0])
		#print pts3D
		#print self.coordinatesUpperProfile[n]
		#print self.coordinatesLowerProfile[n]
		
	## @brief approximates the dphi to preserve length as good as possible
	# @param mmin minimum coordinate of profile
	# @param mmax maximum coordinate of profile
	# @param noElemI the number of elements in pitchwise direction
	# @return [real,list] the delta phi, the vector containing the delta m
	def approximateDeltaPhi(self,mmin,mmax,noElemI):
		m=mmin
		dphi_start = 2*(mmax-mmin)/noElemI/self.nurbsMeri.pointAt(0).gety() #Maximal mögliches dphi! Ansonsten divergent!!!
		tolerance  = 1e-5
		i          = 2.0
		dphi=dphi_start
		while abs(mmax-m)>=tolerance:
			m_vec=[]
			m=mmin
			m_vec.append(mmin)
			for k in range(noElemI):
				loc       = 1.0 #u + du
				j         = 2.0
				r=self.nurbsMeri.pointAt(m/self.nurbsMeri.length()).gety()
				dm=r*dphi
				while abs(dm-r*dphi)>=tolerance:
					dm = self.nurbsMeri.lengthIn(m/self.nurbsMeri.length(),loc,1e-6,100)
					r  = self.nurbsMeri.pointAt(loc).gety()
					if abs(dm-r*dphi)<=tolerance:
						break
					elif (dm-r*dphi)<0:
						loc+=1.0/j
					elif (dm-r*dphi)>0:
						loc-=1.0/j
					j*=2
				m=m+dm
				m_vec.append(m)
			
			if abs(mmax-m)<=tolerance:
				break
			elif mmax-m<0:
				dphi-=dphi_start/i
			elif mmax-m>0:
				dphi+=dphi_start/i
			i*=2
		return dphi,m_vec
	
	#def approximateDeltaPhiLETE(self,mmin,mmax,noElemI):
		#### Experimentelle Methode zur Annäherung einer radial konstanten Hinterkante
	
		#ds=(mmax-mmin)/(noElemI)
		#m_orig1=self.nurbsMeri.lengthIn(self.u_meri[0],0,1e-6,100)
		#m_orig2=self.nurbsMeri.lengthIn(self.u_meri[1],0,1e-6,100)
		#m=mmin
		#dphi_start = 4*(mmax-mmin)/noElemI/self.nurbsMeri.pointAt(0).gety() #Maximal mögliches dphi! Ansonsten divergent!!!
		#tolerance  = 1e-2
		#i          = 2.0
		#dphi=dphi_start
		#while abs(mmax-m)>=tolerance:
			
			##build the m-vector according to the dphi
			#m_vec=[]
			#m=mmin
			#m_vec.append(mmin)
			#for k in range(noElemI+1):
				#loc       = 1.0 #u + du
				#j         = 2.0
				#r=self.nurbsMeri.pointAt(m/self.nurbsMeri.length()).gety()
				#dm=r*dphi
				#while abs(dm-r*dphi)>=tolerance:
					#dm = self.nurbsMeri.lengthIn(m/self.nurbsMeri.length(),loc,1e-6,100)
					#r  = self.nurbsMeri.pointAt(loc).gety()
					#if abs(dm-r*dphi)<=tolerance:
						#break
					#elif (dm-r*dphi)<0:
						#loc+=1.0/j
					#elif (dm-r*dphi)>0:
						#loc-=1.0/j
					#j*=2
				#m=m+dm
				#m_vec.append(m)
			
			
			#pts2D=numpy.zeros((2,2))
			#pts2D[0]=self.cut1
			#pts2D[1]=self.cut2
			##print pts2D
			##Zuordnung Kurvenpunkte zu den 2D-Netzelementen
			#pts=numpy.zeros((2,4)) # i,j,fm,fy
			#for n in range(len(pts2D)):
				#pts[n,0]=int((pts2D[n,0])/ds) #Indizierung von 0-30 (!) -> 31 Knoten werden in der Matrix benötigt
				#pts[n,1]=(pts2D[n,0]-pts[n,0]*ds)/ds
				
			##Bilde 2D-Geometrie über gespeicherte Zellindizes auf 3D ab
			#pts_m=numpy.zeros((len(pts)))
			#for n in range(len(pts)):
				#if pts[n,0]!=(noElemI):
					#pts_m[n]=m_vec[int(pts[n,0])]+(m_vec[int(pts[n,0]+1)]-m_vec[int(pts[n,0])])*pts[n,1]
				#else:
					#pts_m[n]=m_vec[int(pts[n,0])]
			
			##print "dm ist:",(pts_m[1]-pts_m[0])
			##print "dm soll:",(m_orig1-m_orig2)
			#krit=(pts_m[1]-pts_m[0])-(m_orig1-m_orig2)
			#print krit
			
			#if abs(krit)<=tolerance:
				#break
			#elif krit<0:
				#dphi-=dphi_start/i
			#elif krit>0:
				#dphi+=dphi_start/i
			#i*=2
		#return dphi,m_vec
	
	
	## @brief coordinates the 3D mapping according to the settings
	# @param n the profile number
	def __computeProfile3D(self,n):
		if self.definition==0 or self.definition==1:
			#print max(self.listProfiles[n].upperProfile[:,1])
			
			if self.angleFlag==0:
				#Zylinderabwicklung:
				self.__cylindricalMapping(n)
				
			if self.angleFlag==1:
				#Winkelkonforme Abbildung über Grundnetztransformation
				self.__conformalMapping(n)
			
			
			if self.definition==1:
				#setze phi=0 an LE bzw TE und berücksichtige offset:
				dphi0=self.coordinatesCamberProfile[n,0,1]
				for pts in [self.coordinatesUpperProfile,
							self.coordinatesLowerProfile,
							self.coordinatesCamberProfile]:
					pts[n,:,1]-=dphi0
			
			#Berücksichte Offset:
			if self.periphalOffset!=[]:
				for pts in [self.coordinatesUpperProfile,
							self.coordinatesLowerProfile,
							self.coordinatesCamberProfile]:
					pts[n,:,1]-=self.periphalOffset[n]
					pts[n,:,1]-=self.periphalOffset[n]
					pts[n,:,1]-=self.periphalOffset[n]
		
		#cascadeAlgorithm mapping
		if self.definition==2:
			x=self.bladeSpan/(float(self.numberProfiles)-1.0)*float(n)
			self.coordinatesUpperProfile[n,:,0]=x
			self.coordinatesUpperProfile[n,:,1]=-self.listProfiles[0].upperProfile[:,1]
			self.coordinatesUpperProfile[n,:,2]=self.listProfiles[0].upperProfile[:,0]
			self.coordinatesLowerProfile[n,:,0]=x
			self.coordinatesLowerProfile[n,:,1]=-self.listProfiles[0].lowerProfile[:,1]
			self.coordinatesLowerProfile[n,:,2]=self.listProfiles[0].lowerProfile[:,0]
			self.coordinatesCamberProfile[n,:,0]=x
			self.coordinatesCamberProfile[n,:,1]=-self.listProfiles[0].camberLine[:,1]
			self.coordinatesCamberProfile[n,:,2]=self.listProfiles[0].camberLine[:,0]
	
	
	## @brief computes the meridian curve n between hub and shroud (isosurface method or userdefined)
	# @param n the profile number
	def __computeMeridian(self,n):
		ip=50
		hub = numpy.zeros((ip,2))
		shroud = numpy.zeros((ip,2))

		for i in range(ip):
			u=nurbsToolSet.calculateUValueFromDistance(self.nurbsHub.length()/(ip-1.0)*i,0,self.nurbsHub)
			point = self.nurbsHub.pointAt(u)
			hub[i,0]=point.getx()
			hub[i,1]=point.gety()
			v=nurbsToolSet.calculateUValueFromDistance(self.nurbsShroud.length()/(ip-1.0)*i,0,self.nurbsShroud)
			point = self.nurbsShroud.pointAt(v)
			shroud[i,0]=point.getx()
			shroud[i,1]=point.gety()
		meri = numpy.zeros((ip,2))
		for i in range(ip):
			if self.spanDistr == 0:
				areaTotal = numpy.pi*(hub[i,1]+shroud[i,1])*numpy.sqrt(pow((shroud[i,0]-hub[i,0]),2)+pow((shroud[i,1]-hub[i,1]),2)) #Gesamtfl che des Quasiorthogonalen Schnittes
				cosalpha = (shroud[i,1]-hub[i,1])/numpy.sqrt(pow((shroud[i,0]-hub[i,0]),2)+pow((shroud[i,1]-hub[i,1]),2)) # ffnungswinkel/2 des Kegelstumpfs
				sinalpha = (shroud[i,0]-hub[i,0])/numpy.sqrt(pow((shroud[i,0]-hub[i,0]),2)+pow((shroud[i,1]-hub[i,1]),2))
				areaSpan = areaTotal/(self.numberProfiles-1.0)*n
				meri[i,1] = numpy.sqrt(cosalpha*areaSpan/numpy.pi+hub[i,1]**2)
				meri[i,0] = (sinalpha*areaSpan+numpy.pi*hub[i,0]*(hub[i,1]+meri[i,1]))/(numpy.pi*(hub[i,1]+meri[i,1]))
				if i==0:
					self.span.append(numpy.sqrt(pow((meri[i,0]-hub[i,0]),2)+pow((meri[i,1]-hub[i,1]),2))/numpy.sqrt(pow((shroud[i,0]-hub[i,0]),2)+pow((shroud[i,1]-hub[i,1]),2)))
			else:
				span = self.span[n]
				meri[i,0] = span*(shroud[i,0]-hub[i,0])+hub[i,0]
				meri[i,1] = span*(shroud[i,1]-hub[i,1])+hub[i,1]
		nurbsMeri = nurbsToolSet.interpolateNurbs2D(meri,kps=24,degree=self.degree)
		self.meri = meri
		return nurbsMeri
	
	
	## @brief translates the 2D profile to the z-coordinate of the  3D edge
	# @param n the profile number
	def __translateToEdge3D(self,n):
		dx = self.listProfiles[n].upperProfile[0,0]
		dm = self.u_meri[0]*self.nurbsMeri.length()
		dy = self.listProfiles[n].upperProfile[0,1]
		
		for pts in [self.listProfiles[n].lowerProfile,self.listProfiles[n].camberLine,self.listProfiles[n].upperProfile]:
			#Transformiere zur Kante
			pts[:,0] -= dx
			#Transformiere zu m-Koordinaten der Meridianlinie
			pts[:,0] += dm
			#Transformiere zu y-Koordinate
			pts[:,1] -= dy
		
	
	## @brief computes the stacking point via the geometrical center of gravity
	# @param n the profile number
	def __computeStackingPoint(self,n):
		#define list of points which are a simple polygone
		poly=self.listProfiles[n].upperProfile[::-1,:]
		poly=numpy.append(poly,self.listProfiles[n].lowerProfile[:],axis=0)
		
		A=0.0
		for i in range(len(poly)-1):
			A+=0.5*(poly[i,0]*poly[i+1,1]-poly[i+1,0]*poly[i,1])
		
		stackingPoint=numpy.zeros((2))
		for i in range(len(poly)-1):
			stackingPoint[0]+=((poly[i,0]+poly[i+1,0])*(poly[i,0]*poly[i+1,1]-poly[i+1,0]*poly[i,1]))/6./A
			stackingPoint[1]+=((poly[i,1]+poly[i+1,1])*(poly[i,0]*poly[i+1,1]-poly[i+1,0]*poly[i,1]))/6./A
		
		##### Alte Variante: #####
		##Flächenmittelpunkt bestimmen:
		#accuracy=300
		#upperNurbs = nurbsToolSet.interpolateNurbs2D(self.listProfiles[n].upperProfile)
		#lowerNurbs = nurbsToolSet.interpolateNurbs2D(self.listProfiles[n].lowerProfile)
		#upper = numpy.zeros((accuracy,2))
		#lower = numpy.zeros((accuracy,2))
		#for i in range(accuracy):
			#point = upperNurbs.pointAt(1.0/accuracy*i)
			#upper[i,0]=point.getx()
			#upper[i,1]=point.gety()
			#point = lowerNurbs.pointAt(1.0/accuracy*i)
			#lower[i,0]=point.getx()
			#lower[i,1]=point.gety()
		#A=0.0 #Gesamtfl che des Profils
		#momentumX = 0.0
		#momentumY = 0.0
		#for i in range(accuracy-1):
			##Hier wird mit Hilfe der Gau schen Trapezformel die Fl che der finiten Trapeze berechnet und zur Gesamtfl che aufsummiert
			#dA = 0.5*(abs((upper[i,1]-lower[i+1,1])*(lower[i,0]-upper[i+1,0])+(upper[i+1,1]-lower[i,1])*(upper[i,0]-lower[i+1,0])))
			#dx = (upper[i,0]+upper[i+1,0]+lower[i,0]+lower[i+1,0])/4.0
			#dy = (upper[i,1]+upper[i+1,1]+lower[i,1]+lower[i+1,1])/4.0
			#momentumX += dA*dx
			#momentumY += dA*dy
			#A += dA
		#sp = numpy.zeros((2))
		#sp[0] = momentumX/A
		#sp[1] = momentumY/A
		#print A
		#print stackingPoint, sp
		
		return stackingPoint
	
	
	## @brief translates the 2D profile to stacking point
	# @param n the profile number
	def __translateToStackingPoint(self,n):
		for pts in [self.listProfiles[n].upperProfile,
					self.listProfiles[n].lowerProfile,
					self.listProfiles[n].camberLine]:
			pts[:,0] -= self.coordinatesStackingPoints[n,0]
			pts[:,1] -= self.coordinatesStackingPoints[n,1]
		
		
		
	## @brief translates the 2D profile to m-Coordinate of Stacking Line
	# @param n the profile number
	def __translateToStackingLine(self,n):
		out=nurbsToolSet.computeCuttingPoint(self.nurbsMeri,self.nurbsStackingLine,1)
		for pts in [self.listProfiles[n].upperProfile,
					self.listProfiles[n].lowerProfile,
					self.listProfiles[n].camberLine]:
			
			pts[:,0] += out[1]*self.nurbsMeri.length()
			self.mCuttingPoint=out[1]
		
	

			
	
	## @brief compute the chord length from leading and trailing edge (via meridian curve)
	# @param n the profile number
	def __computeChordLengthFromLETE(self,n,offset):
		
		m=self.nurbsMeri.lengthIn(self.u_meri[0],self.u_meri[1],1e-6,100)
		dy=self.cut2[1]*offset
		
		#self.listProfiles[n].chordLength=m/numpy.cos(self.listProfiles[n].gamma)
		self.listProfiles[n].chordLength=numpy.sqrt(dy**2+m**2)
		
		self.listProfiles[n].gamma=atan2(dy,m)
		
		#write internal profile2D variables
		self.listProfiles[n].angleOfAttack=self.listProfiles[n].beta1-numpy.degrees(self.listProfiles[n].gamma)
		self.listProfiles[n].outflowAngle=self.listProfiles[n].beta2-numpy.degrees(self.listProfiles[n].gamma)
		
		
		self.listProfiles[n].callfunctions()
	
		if n==0:
			self.coordinatesUpperProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].upperProfile),3))
			self.coordinatesLowerProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].lowerProfile),3))
			self.coordinatesCamberProfile=numpy.zeros((self.numberProfiles,len(self.listProfiles[0].camberLine),3))
		
		
		
	
	## @brief scale the 2D profile to chord length
	# @param n the profile number
	def __scale2D(self,n):
		for pts in [self.listProfiles[n].upperProfile,
					self.listProfiles[n].lowerProfile,
					self.listProfiles[n].camberLine]:
			pts *= self.listProfiles[n].chordLength
		

	## @brief rotate the 2D profile to gamma
	# @param n the profile number
	def __rotate2D(self,n):
		angle = self.listProfiles[n].gamma
		for pts in [self.listProfiles[n].upperProfile,
					self.listProfiles[n].lowerProfile,
					self.listProfiles[n].camberLine]:
			rotPts = pts.copy()
			for i in range(len(rotPts)):
				rotPts[i,0]=pts[i,0]*numpy.cos(angle)-pts[i,1]*numpy.sin(angle)
				rotPts[i,1]=pts[i,0]*numpy.sin(angle)+pts[i,1]*numpy.cos(angle)
				pts[i,0] = rotPts[i,0]
				pts[i,1] = rotPts[i,1]
			del rotPts
	
	
	## @brief generates a "closed" point array from TE over SS over LE over PS to TE from coordinatesUpperProfile and coordinatesLowerProfile
	def __updateCoordinatesProfile(self):
		#Definitionsgemaesse Generierung einer geschlossenen Nurbskurve von hinten ueber oben nach unten nach hinten
		self.coordinatesProfile=self.coordinatesUpperProfile[:,::-1,:]
		self.coordinatesProfile=numpy.append(self.coordinatesProfile,self.coordinatesLowerProfile[:,:-1],axis=1)
		self.coordinatesProfile[:,0]=self.coordinatesProfile[:,-1]
	
	
	### @brief API method for 3D nurbs interpolation
	#def interpNurbs3D(self,pts,ntype="open"):
		#if len(pts)<=self.degree:
			#raise Exception,"number of coordinates <= degree of nurbs interpolation!"
		#if len(pts)<=24:
			#ncp=len(pts)
		#else:
			#ncp=24
			
		#if self.nurbsIntSc=="leastSqares":
			#if ntype=="open":
				#return nurbsToolSet.interpolateNurbs3D(pts,ncp)
			#if ntype=="closed":
				#return nurbsToolSet.interpolateNurbs3DClosed(pts,ncp)
		#if self.nurbsIntSc=="globalInterp":
			#if ntype=="open":
				#return nurbsToolSet.globalInterpolateNurbs3D(pts)
			#if ntype=="closed":
				#return nurbsToolSet.globalInterpolateNurbs3DClosed(pts)
	
	
	## @brief creates a List of closed NURBS-Curves which describe the blade
	# @brief In this method the discrete profiles are interpolated to NURBS curves according to the trailing edge flag (teflag).
	# @brief \n"sharp" means the interpolated NURBS curve is open at the trailing edge,
	# @brief \n"round" means the interpolated NURBS curve is closed (continuous) at the trailing edge,
	# @brief \n"blunt" is an option to create a closed profile which has a thickness > 0 at the trailing edge.
	#def createBladeNurbsCurves(self):
		#self.nurbsProfile=[]
		#self.nurbsCamber=[]
		
		#self.__updateCoordinatesProfile()
		
		##fuege zur Liste der nurbsProfile die erstellten geschlossenen nurbskurven hinzu
		
		#if self.definition==2:
			#self.refNurbsProfile=nurbsToolSet.interpolateNurbs3DClosed(self.coordinatesProfile[0])
			##if self.debug==1:
				##debugPlots.diffNurbsDiscretesCascade(self.refNurbsProfile,self.coordinatesProfile[0])
			
		#for n in range(self.numberProfiles):
			#if self.teflag=="sharp":
				#self.nurbsProfile.append(self.interpNurbs3D(self.coordinatesProfile[n]))
			
			#elif self.teflag=="round":
				#self.nurbsProfile.append(self.interpNurbs3D(self.coordinatesProfile[n],"closed"))
			
			#elif self.teflag=="blunt":
				#raise Exception, "Blunt trailing Edge isn't supported yet. See Code for an approach."
				
				#nppcurve=self.interpNurbs3D(self.coordinatesProfile[n])
				
				#pole_list=[]
				#weights_list=[]
				#for n in range(nppcurve.ctrlPnts().n()):
					#pole_list.append(
							#[
								#nppcurve.ctrlPnts(n).getx(),
								#nppcurve.ctrlPnts(n).gety(),
								#nppcurve.ctrlPnts(n).getz()
							#]
						#)
					#weights_list.append(nppcurve.ctrlPnts(n).getw())
				
				#knots_list=[]
				#for n in range(nppcurve.knot().n()):
					#knots_list.append(nppcurve.knot(n))
				
				##skaliere auf 0.5
				#print knots_list
				#sumtmp=0.5-knots_list[0]
				#print sumtmp
				#scalefac=0.5/sumtmp
				#print scalefac
				#for k in range(len(knots_list)):
					#knots_list[k]-=0.5
					#knots_list[k]*=scalefac
					#knots_list[k]+=0.5
				#print knots_list
				
				##füge 0 und 1 hinzu
				#knots_list.insert(0,0.0)
				#knots_list.append(1.0)
				
				#for k in range(self.degree):
					#pole_list.insert(0,pole_list[0])
					#pole_list.append(pole_list[len(pole_list)-1])
				
				#tepole=[0,0,0]
				#for s in range(len(tepole)):
					#tepole[s]=pole_list[-1][s]-pole_list[0][s]
				#pole_list.insert(0,tepole)
				#pole_list.append(tepole)
				
				##Knotenvektor in U-Richtung
				#ukv = NurbsVector.Vector_DOUBLE(len(knots_list)) #Regel k+p+1
				#for i in range(len(knots_list)):
					#ukv[i]=knots_list[i]
				
				##3D-Kontrollpunkte aus den NurbsKurven
				#kpts = NurbsVector.Vector_Point3Dd(len(pole_list))
				#vector = NurbsVector.Vector_Point3Dd(1)
				#for n in range(len(pole_list)):
					#vector.reset(NurbsPoint.Point3Dd(pole_list[n][0],pole_list[n][1],pole_list[n][2]))
					#kpts.as_func(n,vector)
				
				##Gewichtungsmatrix
				#wv = NurbsVector.Vector_DOUBLE(len(weights_list))
				#for i in range(len(weights_list)):
					#wv[i]=weights_list[i]
				
				#print len(pole_list)
				#print len(knots_list)
				#print len(weights_list)
				#self.nurbsProfile.append(NurbsCurve.NurbsCurved(kpts,wv,ukv,self.degree))
				
			#else:
				#raise Exception,"Wrong teflag syntax."
			
		#for n in range(self.numberProfiles):
			#self.nurbsCamber.append(self.interpNurbs3D(self.coordinatesCamberProfile[n]))
			
		##if self.debug==1:
			##debugPlots.diffNurbsDiscretes(self.nurbsProfile[0],self.coordinatesProfile[0])
	
	
	## @brief Builds the 3D NURBS surface of the blade.
	# @brief \nThe NURBS surface is constructed by using the control points of the preliminary generated NURBS curves.
	# @brief \nThe u direction is the spanwise direction, the v direction is around the blade (Beginning at TE counterclockwise).
	# @warning NURBS surfaces can only take one knot vector in u/v direction. Because of that the knot vector of the first profile of nurbsProfiles is taken as the knotsV vector.
	def __interpolateNurbsSurface(self):
		
		if self.debug>=1:
			print "*** Computing now Nurbssurface..."
		
		self.__updateCoordinatesProfile()
		if self.teflag=="sharp":
			self.bladeSurface=nurbsToolSet.interpolateNurbsSurface(self.coordinatesProfile,du=self.degreeU,dv=self.degree,kpsv=24)
		if self.teflag=="round":
			self.bladeSurface=nurbsToolSet.interpolateNurbsSurfaceClosedV(self.coordinatesProfile,du=self.degreeU,dv=self.degree,kpsv=24)
		
		#deprecated compatibility:
		self.__getNurbsProfiles()
		for n in range(self.numberProfiles):
			self.nurbsCamber.append(nurbsToolSet.interpolateNurbs3D(self.coordinatesCamberProfile[n]))


	## @brief Saves the 3D NURBS surface in a .wrl file (plotable with e.g. paraview)
	# @param filename the file path
	# @param ipu the triangulation points in u direction
	# @param ipv the triangulation points in v direction
	def writeVRML(self,filename='blade.wrl',ipu=250,ipv=250):
		path=''
		if self.dataStruct!=[]:
			self.dataStruct.addGeomRow(self.rowNumber)
			path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			#if self.omega!=0:
				#filename='rotorblade.wrl'
			#if self.omega==0:
				#filename='statorblade.wrl'			
			
		blue=NurbsColor.Color(0, 0, 255)
		red=NurbsColor.Color(255, 0 , 0)
		
		self.bladeSurface.writeVRML97(path+filename,red,ipu,ipv,1,0,1,0)
		#self.rotSurface.writeVRML97(path+"rot.wrl",blue,ipu,ipv,1,0,1,0)
		#self.camberSurface.writeVRML97(path+"camber.wrl",blue,ipu,ipv,1,0,1,0)
		
		surface = NurbsSurface.NurbsSurfaced()
		surface.makeFromRevolution(self.nurbsRot[0], NurbsPoint.Point3Dd(0.0, 0.0, 0.0), NurbsPoint.Point3Dd(0.0, 0.0, 1.0), 2*numpy.pi)
		surface.writeVRML97(path+'surface.wrl', red, 100, 100)
		
		if self.debug>=1:
			print "*** Written to" , path+filename


	## @brief gets the profile at a constant spanwise relative coordinate
	# @warning TODO This method doesn't do anything other than getIsoUCurve, which is okay for uniform bsplines which is a good approximation for normally interpolated blades. An iterative Algorithm to find the proper u value was not implemented yet.
	def getBladeSection(self,span):
		u=span #!
		nurbsCurve = self.getIsoUCurve(u)
		return nurbsCurve

	## @brief gets the profile at a constant parameter u on the nurbs surface
	def getIsoUCurve(self,u):
		nurbsCurve = NurbsCurve.NurbsCurved()
		self.bladeSurface.isoCurveU(u,nurbsCurve)
		return nurbsCurve

	## @brief compatibility method to provide the (deprecated) nurbsProfile-List with contains the nurbs interpolated blade sections
	def __getNurbsProfiles(self):
		self.nurbsProfile=[]
		for sp in self.span:
			self.nurbsProfile.append(self.getBladeSection(sp))

	## @brief compute blockMeshDict
	def computeMesh(self):
		try:
			self.mesh.noLayer=self.noLayer
			# Hier noch die richtigen Profile usw übergeben (abhängig von self.startProfile)		
			if self.debug==1:
				print "*** Generating now blockMeshDict"
			self.mesh.executeBlockMeshDictGen()
			if self.debug==1:
				print "\n*** blockMeshDict generation complete"
		except:
			print sys.exc_info()[1],"\n!!! Mesh generation failed."


	## @brief get a Geom_BSplineCurve Object for OpenCascade
	def getGeom_BSplineCurve(self):
			nurbslist=[]
			for sp in self.span:
				nurbslist.append(nurbsToolSet.nppToOcc_Curve(self.getBladeSection(sp)))
			return nurbslist
	
	
	## @brief get a Geom_BSplineCurve Object for OpenCascade
	def getGeom_BSplineRotSurfaces(self):
		try:
			import OCC.gp 
			import OCC.Geom 
			import OCC.TColgp 
			import OCC.TColStd
		except:
			raise ImportError, "!!! pythonocc not found. OpenCASCADE output not available"
		else:
			nurbslist=[]
			for nurbs in self.nurbsRot:
				occnurbs=nurbsToolSet.nppToOcc_Curve(nurbs)
				nurbslist.append(OCC.Geom.Geom_SurfaceOfRevolution(occnurbs.GetHandle(), OCC.gp.gp().OZ()))
				
			return nurbslist
	
	
	## @brief get a Geom_BSplineSurface Object for OpenCascade
	def getGeom_BSplineSurface(self):
		return nurbsToolSet.nppToOcc_Surface(self.bladeSurface)
	
	
	## @brief export bladeSurface to IGES File
	# @param filename the file path
	def exportBladeToIGES(self,filename="blade.iges"):
		try:
			import OCC.Utils.DataExchange.IGES
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			bladesurf = self.getGeom_BSplineSurface()

			bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
			bladesurf_rep.Build()
			
			exporter = OCC.Utils.DataExchange.IGES.IGESExporter(path+filename,format="5.3")
			exporter.AddShape( bladesurf_rep.Shape() )
			exporter.WriteFile()
			print "*** Successfully written to",path+filename
	
	
	## @brief export bladeSurface to IGES File
	# @param filename the file path
	def exportBladeToSTEP(self,filename="blade.stp"):
		try:
			import OCC.Utils.DataExchange.STEP
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			bladesurf = self.getGeom_BSplineSurface()

			bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
			bladesurf_rep.Build()
			
			exporter = OCC.Utils.DataExchange.STEP.STEPExporter(path+filename)
			exporter.AddShape( bladesurf_rep.Shape() )
			exporter.WriteFile()
			print "*** Successfully written to",path+filename


	def exportToTurboGrid(self):
		path=''
		inffn="bladedesigner.inf"
		if self.dataStruct!=[]:
			self.dataStruct.addGeomRow(self.rowNumber)
			path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			inffn=self.dataStruct.projectname+".inf"
		
		ftext="!====== BladeDesigner Export =====\nAxis of Rotation: Z\nNumber of Blade Sets: "+str(self.numberBlades)+"\nGeometry Units: MM\nHub Data File: hub.curve\nShroud Data File: shroud.curve\nProfile Data File: profile.curve\n"
		
		inff=open(path+inffn,"w")
		inff.write(ftext)
		inff.close()
		
		htext=""
		stext=""
		for n in range(200):
			pth=self.nurbsHub.pointAt(n/199.)
			pts=self.nurbsShroud.pointAt(n/199.)
			htext+=str(pth.gety())+" 0.0 "+str(pth.getx())+"\n"
			stext+=str(pts.gety())+" 0.0 "+str(pts.getx())+"\n"
		
		hf=open(path+"hub.curve","w")
		hf.write(htext)
		hf.close()
		
		sf=open(path+"shroud.curve","w")
		sf.write(stext)
		sf.close()
		
		ptext=""
		for i,span in enumerate(numpy.linspace(0,1,10)):
			ptext+="\n# Profile "+str(i+1)+" at "+str(span*100.)+"%\n\n"
			profile=self.getBladeSection(span)
			for n in range(500):
				pt=profile.pointAt(n/499.)
				ptext+=str(pt.getx())+" "+str(pt.gety())+" "+str(pt.getz())+"\n"
		
		pf=open(path+"profile.curve","w")
		pf.write(ptext)
		pf.close()


	## @brief export hub and shroud surfaces to STEP
	# @param hfilename the hub filepath
	# @param sfilename the shroud filepath
	def exportHubShroudToSTEP(self,hfilename="hub.stp",sfilename="shroud.stp"):
		try:
			import OCC.Utils.DataExchange.STEP
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			rotsurf = self.getGeom_BSplineRotSurfaces()[0]
			rotsurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( rotsurf.GetHandle() )
			rotsurf_rep.Build()
			
			exporter = OCC.Utils.DataExchange.STEP.STEPExporter(path+hfilename)
			exporter.AddShape( rotsurf_rep.Shape() )
			exporter.WriteFile()
			
			print "*** Successfully written to",path+hfilename
			
			rotsurf = self.getGeom_BSplineRotSurfaces()[-1]
			rotsurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( rotsurf.GetHandle() )
			rotsurf_rep.Build()
			
			exporter = OCC.Utils.DataExchange.STEP.STEPExporter(path+sfilename)
			exporter.AddShape( rotsurf_rep.Shape() )
			exporter.WriteFile()
			
			print "*** Successfully written to",path+sfilename
			

	## @brief export bladeSurface to STL File
	# @param fn The file path
	def exportBladeToSTL(self,fn="blade.stl",deflection=None,coefficient=None):
		try:
			import OCC.StlAPI
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			bladesurf = self.getGeom_BSplineSurface()

			bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
			bladesurf_rep.Build()
			
			os.system("touch "+path+fn) #workaround for stl bug
			exporter = OCC.StlAPI.StlAPI_Writer()
			
			if deflection!=None:
				exporter.SetDeflection(deflection)
			if coefficient!=None:
				exporter.SetCoefficient(coefficient)
			
			exporter.Write(bladesurf_rep.Shape(),path+fn)
			print "*** Successfully written to",fn


	## @brief export Hub and Shroud to STL File
	# @param hfn The hub file path
	# @param sfn The shroud file path
	def exportHubShroudToSTL(self,hfn="hub.stl",sfn="shroud.stl",deflection=None,coefficient=None):
		try:
			import OCC.StlAPI
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			rotsurf = self.getGeom_BSplineRotSurfaces()[0]
			rotsurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( rotsurf.GetHandle() )
			rotsurf_rep.Build()
			
			os.system("touch "+path+hfn) #workaround for stl bug
			exporter = OCC.StlAPI.StlAPI_Writer()
			
			if deflection!=None:
				exporter.SetDeflection(deflection)
			if coefficient!=None:
				exporter.SetCoefficient(coefficient)
			
			exporter.Write(rotsurf_rep.Shape(),path+hfn)
			print "*** Successfully written to",hfn
			
			
			rotsurf = self.getGeom_BSplineRotSurfaces()[-1]
			rotsurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( rotsurf.GetHandle() )
			rotsurf_rep.Build()
			
			os.system("touch "+path+sfn) #workaround for stl bug
			exporter = OCC.StlAPI.StlAPI_Writer()
			
			if deflection!=None:
				exporter.SetDeflection(deflection)
			if coefficient!=None:
				exporter.SetCoefficient(coefficient)
			
			exporter.Write(rotsurf_rep.Shape(),path+sfn)
			print "*** Successfully written to",sfn



	## @brief write boundaries as vtp files - needed for transfinite interpolation
	# @param numPts The number of points used for interpolation
	def writeVtkBoundaries(self,numPts):
		#In Metern!!!
		scale=0.001
		
		try:
			import vtk
		except:
			print "writeVtkBoundaries() is expert only. Method needs further adjustment in code"
		else:
			path=''
			if self.dataStruct!=[]:
				self.dataStruct.addGeomRow(self.rowNumber)
				path=self.dataStruct.workdir+self.dataStruct.dirGeom+self.dataStruct.geomRow+str(self.rowNumber)+'/'
			
			hub=self.nurbsRot[0]
			shroud=self.nurbsRot[-1]
			
			hubpts=numpy.zeros((numPts,3))
			shroudpts=numpy.zeros((numPts,3))
			inletpts=numpy.zeros((numPts,3))
			outletpts=numpy.zeros((numPts,3))
			
			for i in range(numPts):
				hubpts[i]=[hub.pointAt(i/(numPts+1.)).getx()*scale,hub.pointAt(i/(numPts+1.)).gety()*scale,hub.pointAt(i/(numPts+1.)).getz()*scale]
				shroudpts[i]=[shroud.pointAt(i/(numPts+1.)).getx()*scale,shroud.pointAt(i/(numPts+1.)).gety()*scale,shroud.pointAt(i/(numPts+1.)).getz()*scale]
			
			for i in range(numPts):
				inletpts[i]=[i/(numPts+1.)*(shroudpts[0,0]-hubpts[0,0])+hubpts[0,0],i/(numPts+1.)*(shroudpts[0,1]-hubpts[0,1])+hubpts[0,1],i/(numPts+1.)*(shroudpts[0,2]-hubpts[0,2])+hubpts[0,2]]
				outletpts[i]=[i/(numPts+1.)*(shroudpts[-1,0]-hubpts[-1,0])+hubpts[-1,0],i/(numPts+1.)*(shroudpts[-1,1]-hubpts[-1,1])+hubpts[-1,1],i/(numPts+1.)*(shroudpts[-1,2]-hubpts[-1,2])+hubpts[-1,2]]
			
			print hub.pointAt(15/(numPts+1.)).getx()
			print hubpts
			print shroudpts
			print inletpts
			print outletpts
			
			for ptsarr in [[hubpts,"hub"],[shroudpts,"shroud"],[inletpts,"inlet"],[outletpts,"outlet"]]:
				points = vtk.vtkPoints()
				polydata=vtk.vtkPolyData()
				for n in range(numPts):
					points.InsertNextPoint(ptsarr[0][n,0],ptsarr[0][n,1],ptsarr[0][n,2])
				polydata.SetPoints(points)
				writer = vtk.vtkXMLPolyDataWriter()				
				writer.SetFileName(path+ptsarr[1]+".vtp")
				writer.SetInput(polydata)
				writer.Write()
	
	
	## @brief Writes Nurbs to xml file
	def __writeNurbsToXml(self):
		self.dataStruct.cncDir(self.dataStruct.workdir)
		self.dataStruct.cncDir(self.dataStruct.workdir+self.dataStruct.dirGeom)
		filename = self.dataStruct.workdir+self.dataStruct.dirGeom+'/Row_'+str(self.rowNumber)+'_Nurbs.xml'
		
		xml_doc = """<?xml version="1.0" encoding="UTF-8"?>
		<NurbsRow/>""".encode("utf-8")
		
		dom_object = minidom.parseString(xml_doc)
		
		elementRow = dom_object.getElementsByTagName("NurbsRow")[0]
		
		newElement = dom_object.createElement("NurbsSurface")
		newElement.setAttribute("i",str(0))
		elementRow.appendChild(newElement)
		
		elementProfiles = elementRow.getElementsByTagName("NurbsSurface")[0]
		
		nppsurf=self.bladeSurface
		
		pole_list=numpy.zeros((nppsurf.ctrlPnts().rows(),nppsurf.ctrlPnts().cols(),3))
		weights_list=numpy.zeros((nppsurf.ctrlPnts().rows(),nppsurf.ctrlPnts().cols()))
		for i in range(nppsurf.ctrlPnts().rows()):
			for j in range(nppsurf.ctrlPnts().cols()):
				pole_list[i,j]=[
						nppsurf.ctrlPnts(i,j).getx(),
						nppsurf.ctrlPnts(i,j).gety(),
						nppsurf.ctrlPnts(i,j).getz()
						]
				weights_list[i,j]=nppsurf.ctrlPnts(i,j).getw()
		
		knotsu_list=numpy.zeros((nppsurf.knotU().n()))
		for n in range(nppsurf.knotU().n()):
			knotsu_list[n]=nppsurf.knotU(n)

		knotsv_list=numpy.zeros((nppsurf.knotV().n()))
		for n in range(nppsurf.knotV().n()):
			knotsv_list[n]=nppsurf.knotV(n)
		
		newElement = dom_object.createElement("element")
		newElement.setAttribute('pole_list',str(numpy.array2string(pole_list, separator=',').replace('\n','')))
		elementProfiles.appendChild(newElement)
		
		newElement = dom_object.createElement("element")
		newElement.setAttribute('knotsu_list',str(numpy.array2string(knotsu_list, separator=',').replace('\n','')))
		elementProfiles.appendChild(newElement)
		
		newElement = dom_object.createElement("element")
		newElement.setAttribute('knotsv_list',str(numpy.array2string(knotsv_list, separator=',').replace('\n','')))
		elementProfiles.appendChild(newElement)
		
		newElement = dom_object.createElement("element")
		newElement.setAttribute('weights_list',str(numpy.array2string(weights_list, separator=',').replace('\n','')))
		elementProfiles.appendChild(newElement)
		
		
		#Schreibe Rotationskurven TODO (veraltert)
		newElement = dom_object.createElement("NurbsRots")
		newElement.setAttribute("i",str(0))
		elementRow.appendChild(newElement)
		
		elementRots = elementRow.getElementsByTagName("NurbsRots")[0]
			
		for m in range(len(self.nurbsRot)):
			ptarr=numpy.zeros((self.nurbsRot[m].ctrlPnts().n(),3))
			for n in range(self.nurbsRot[m].ctrlPnts().n()):
				ptarr[n]=[self.nurbsRot[m].ctrlPnts(n).getx(),self.nurbsRot[m].ctrlPnts(n).gety(),self.nurbsRot[m].ctrlPnts(n).getz()]
			
			ktarr=numpy.zeros((self.nurbsRot[m].knot().n()))
			for n in range(self.nurbsRot[m].knot().n()):
				ktarr[n]=self.nurbsRot[m].knot(n)
			
			weight=numpy.zeros((self.nurbsRot[m].ctrlPnts().n()))
			for n in range(self.nurbsRot[m].ctrlPnts().n()):
				weight[n]=self.nurbsRot[m].ctrlPnts(n).getw()
				
			
			newElement = dom_object.createElement("nurbsRot")
			newElement.setAttribute("i",str(m))
			elementRots.appendChild(newElement)
			
			elementRot = elementRots.getElementsByTagName("nurbsRot")[m]
	
			newElement = dom_object.createElement("element")
			newElement.setAttribute('ptarr',str(numpy.array2string(ptarr, separator=',').replace('\n','')))
			elementRot.appendChild(newElement)
			
			newElement = dom_object.createElement("element")
			newElement.setAttribute('ktarr',str(numpy.array2string(ktarr, separator=',').replace('\n','')))
			elementRot.appendChild(newElement)
			
			newElement = dom_object.createElement("element")
			newElement.setAttribute('weight',str(numpy.array2string(weight, separator=',').replace('\n','')))
			elementRot.appendChild(newElement)	
		
		
		#Schreibe Skelettlinienkurven TODO (veraltert)
		newElement = dom_object.createElement("NurbsCambers")
		newElement.setAttribute("i",str(0))
		elementRow.appendChild(newElement)
		
		elementCambers = elementRow.getElementsByTagName("NurbsCambers")[0]
			
		for m in range(len(self.nurbsCamber)):
			ptarr=numpy.zeros((self.nurbsCamber[m].ctrlPnts().n(),3))
			for n in range(self.nurbsCamber[m].ctrlPnts().n()):
				ptarr[n]=[self.nurbsCamber[m].ctrlPnts(n).getx(),self.nurbsCamber[m].ctrlPnts(n).gety(),self.nurbsCamber[m].ctrlPnts(n).getz()]
			
			ktarr=numpy.zeros((self.nurbsCamber[m].knot().n()))
			for n in range(self.nurbsCamber[m].knot().n()):
				ktarr[n]=self.nurbsCamber[m].knot(n)
			
			weight=numpy.zeros((self.nurbsCamber[m].ctrlPnts().n()))
			for n in range(self.nurbsCamber[m].ctrlPnts().n()):
				weight[n]=self.nurbsCamber[m].ctrlPnts(n).getw()
				
			
			newElement = dom_object.createElement("nurbsCamber")
			newElement.setAttribute("i",str(m))
			elementCambers.appendChild(newElement)
			
			elementCamber = elementCambers.getElementsByTagName("nurbsCamber")[m]
	
			newElement = dom_object.createElement("element")
			newElement.setAttribute('ptarr',str(numpy.array2string(ptarr, separator=',').replace('\n','')))
			elementCamber.appendChild(newElement)
			
			newElement = dom_object.createElement("element")
			newElement.setAttribute('ktarr',str(numpy.array2string(ktarr, separator=',').replace('\n','')))
			elementCamber.appendChild(newElement)
			
			newElement = dom_object.createElement("element")
			newElement.setAttribute('weight',str(numpy.array2string(weight, separator=',').replace('\n','')))
			elementCamber.appendChild(newElement)	
		
		
		
		if self.debug==1:
			print '*** Writing', self.dataStruct.workdir+self.dataStruct.dirGeom+'/Row_'+str(self.rowNumber)+'_Nurbs.xml'
		
		file = open(filename, "w")
		
		dom_object.writexml(file, "", "\t", "\n","utf-8")
		
		file.close()
		
	## @brief Reads Nurbs from xml file
	def __readNurbsFromXml(self):
		self.nurbsProfile=[]
		self.nurbsCamber=[]
		
		filename = self.dataStruct.workdir+self.dataStruct.dirGeom+'/Row_'+str(self.rowNumber)+'_Nurbs.xml'
		
		if self.debug==1:
			print '*** Reading', self.dataStruct.workdir+self.dataStruct.dirGeom+'/Row_'+str(self.rowNumber)+'_Nurbs.xml'
			
		dom_object = minidom.parse(filename)
		
		xml_doc = dom_object.toxml("utf-8")
		
		xml_doc=xml_doc.replace('\n','')
		xml_doc=xml_doc.replace('\t','')
		
		dom_object = minidom.parseString(xml_doc)
		
		elementRow = dom_object.getElementsByTagName("NurbsRow")[0]
	
		elementSurface = elementRow.getElementsByTagName("NurbsSurface")[0]
		
		elemente = elementSurface.getElementsByTagName("element")
			
		for element in elemente:
			if element.attributes.item(0).name == 'pole_list':
				k = numpy.array(eval(element.attributes.item(0).value))
			elif element.attributes.item(0).name == 'knotsu_list':
				u = numpy.array(eval(element.attributes.item(0).value))
			elif element.attributes.item(0).name == 'knotsv_list':
				v = numpy.array(eval(element.attributes.item(0).value))
			elif element.attributes.item(0).name == 'weights_list':
				w = numpy.array(eval(element.attributes.item(0).value))
		
		
		
		#Knotenvektor in U-Richtung
		ukv = NurbsVector.Vector_DOUBLE(len(u))
		for i in range(len(u)):
			ukv[i]=u[i]
		
		#Knotenvektor in V-Richtung
		vkv = NurbsVector.Vector_DOUBLE(len(v))
		for i in range(len(v)):
			vkv[i]=v[i]
		
		#Matrix mit 3D-Kontrollpunkten
		mpts = NurbsMatrix.Matrix_Point3Dd(len(k),len(k[0]))
		for i in range(len(k)):
			for j in range(len(k[0])):
				mpts.setelem(i,j,NurbsPoint.Point3Dd(k[i][j][0],k[i][j][1],k[i][j][2]))
		
		#Gewichtungsmatrix
		mw = NurbsMatrix.Matrix_double(len(w),len(w[0]))
		for i in range(len(w)):
			for j in range(len(w[0])):
				mw.setelem(i,j,w[i][j])
		
		#Erzeuge Nurbs-Flaeche
		self.bladeSurface = NurbsSurface.NurbsSurfaced(self.degreeU,self.degree,ukv,vkv,mpts,mw)
		#deprecated compatibility:
		self.__getNurbsProfiles()
		
		
		#Lese Rotationskurven aus TODO (veraltert)
		elementRots = elementRow.getElementsByTagName("NurbsRots")[0]
		
		for n in range(len(elementRots.getElementsByTagName("nurbsRot"))):
			elementRot = elementRots.getElementsByTagName("nurbsRot")[n]
			
			elemente = elementRot.getElementsByTagName("element")
			
			for element in elemente:
				if element.attributes.item(0).name == 'ptarr':
					k = self.listProfiles[0].convertListToCoordArray(eval(element.attributes.item(0).value))
				elif element.attributes.item(0).name == 'ktarr':
					u = numpy.array(eval(element.attributes.item(0).value))
				elif element.attributes.item(0).name == 'weight':
					w = numpy.array(eval(element.attributes.item(0).value))
			
			#Knotenvektor in U-Richtung
			ukv = NurbsVector.Vector_DOUBLE(len(u)) #Regel k+p+1
			for i in range(len(u)):
				ukv[i]=u[i]
			
			#3D-Kontrollpunkte aus den NurbsKurven
			kpts = NurbsVector.Vector_Point3Dd(len(k))
			vector = NurbsVector.Vector_Point3Dd(1)
			for n in range(len(k)):
				vector.reset(NurbsPoint.Point3Dd(k[n,0],k[n,1],k[n,2]))
				kpts.as_func(n,vector)
			
			#Gewichtungsmatrix
			wv = NurbsVector.Vector_DOUBLE(len(w))
			for i in range(len(w)):
				wv[i]=w[i]
				
			self.nurbsRot.append(NurbsCurve.NurbsCurved(kpts,wv,ukv,self.degree))
			
			
		#Lese Skelettlinienkurven aus TODO (veraltert)
		elementCambers = elementRow.getElementsByTagName("NurbsCambers")[0]
		
		for n in range(len(elementRots.getElementsByTagName("nurbsCamber"))):
			elementCamber = elementCambers.getElementsByTagName("nurbsCamber")[n]
			
			elemente = elementCamber.getElementsByTagName("element")
			
			for element in elemente:
				if element.attributes.item(0).name == 'ptarr':
					k = self.listProfiles[0].convertListToCoordArray(eval(element.attributes.item(0).value))
				elif element.attributes.item(0).name == 'ktarr':
					u = numpy.array(eval(element.attributes.item(0).value))
				elif element.attributes.item(0).name == 'weight':
					w = numpy.array(eval(element.attributes.item(0).value))
			
			#Knotenvektor in U-Richtung
			ukv = NurbsVector.Vector_DOUBLE(len(u)) #Regel k+p+1
			for i in range(len(u)):
				ukv[i]=u[i]
			
			#3D-Kontrollpunkte aus den NurbsKurven
			kpts = NurbsVector.Vector_Point3Dd(len(k))
			vector = NurbsVector.Vector_Point3Dd(1)
			for n in range(len(k)):
				vector.reset(NurbsPoint.Point3Dd(k[n,0],k[n,1],k[n,2]))
				kpts.as_func(n,vector)
			
			#Gewichtungsmatrix
			wv = NurbsVector.Vector_DOUBLE(len(w))
			for i in range(len(w)):
				wv[i]=w[i]
				
			self.nurbsCamber.append(NurbsCurve.NurbsCurved(kpts,wv,ukv,self.degree))


	
	## @brief saves the parameters in an xml file
	## @param dom_object the dom object
	## @param elementRow the blade row element
	def writeRowXmlFile(self,dom_object,elementRow):
		xmlList = ['definition']
		
		if self.angleFlag != 0.0:
			xmlList.append('angleFlag')
		
		if self.bladeSpan != 0.0:
			xmlList.append('bladeSpan')
				
		if self.stackingLine != 0.0:
			xmlList.append('stackingLine')
		
		if self.domainLength != 0.0:
			xmlList.append('domainLength')
		
		if self.bladeDist != 0.0:
			xmlList.append('bladeDist')
		
		if self.omega != 0.0:
			xmlList.append('omega')
			
		if self.numberBlades != 0.0:
			xmlList.append('numberBlades')

		if numpy.any(self.coordinatesHub):
			xmlList.append('coordinatesHub')
			
		if numpy.any(self.coordinatesShroud):
			xmlList.append('coordinatesShroud')
			
		if numpy.any(self.coordinatesStackingLine):
			xmlList.append('coordinatesStackingLine')

		if numpy.any(self.coordinatesStackingPoints):
			xmlList.append('coordinatesStackingPoints')

		#if len(self.span) != 0:
		#SW: Wofür war nochmal spanDistr?
		#SW: Das kann man hier genauso benutzen
		if self.spanDistr == 1:
			xmlList.append('span')
		
		for i in range(len(xmlList)):
			if type(getattr(self,xmlList[i])) == numpy.ndarray:
				setattr(self,xmlList[i],numpy.array2string(getattr(self,xmlList[i]), separator=',').replace('\n',''))
			newElement = dom_object.createElement("rowElement")
			newElement.setAttribute(xmlList[i],str(getattr(self,xmlList[i])))
			elementRow.appendChild(newElement)
		
		for n in range(len(self.listProfiles)):
			self.listProfiles[n].writeXmlFile(dom_object,elementRow,n)
	

	## @brief initializes the parameters from an xml file
	## @param dom_object the dom object
	## @param elementRow the blade row element
	def readRowXmlFile(self,dom_object,elementRow):

		elemente = elementRow.getElementsByTagName("rowElement")
		
		for element in elemente:
			if (element.attributes.item(0).name in ['coordinatesHub', 'coordinatesShroud', 'coordinatesStackingLine', 'polyThickRBs', 'coordinatesStackingPoints']):
				setattr(self,element.attributes.item(0).name, self.listProfiles[0].convertListToCoordArray(eval(element.attributes.item(0).value)))
				
			elif element.attributes.item(0).name == 'span':
				setattr(self,element.attributes.item(0).name, eval(element.attributes.item(0).value))
				if len(self.span) != 0:
					self.spanDistr = 1
				
			else:
				setattr(self,element.attributes.item(0).name, float(element.attributes.item(0).value))

		elementProfmax = elementRow.lastChild
		self.ProfileNum = int(elementProfmax.getAttribute("i")) + 1

		for n in range(self.ProfileNum):
			elementProf = elementRow.getElementsByTagName("profile")[n]
			self.listProfiles[n].readXmlFile(dom_object,elementProf)
	
	## @brief compute blockMeshDict
	def initMesh(self):
		self.mesh.noLayer  = self.noLayer
		self.mesh.topology = self.topology
		self.mesh.setTopologyData()
		self.mesh.setArrays()
	
	## @brief draws the blade surface for OpenCASCADE
	## @param display takes a pythonocc display handler
	def drawBladeSurface(self,display=None, material=None):
		if display==None:
			if self.display==None:
				raise Exception, "Draw Method has no display."
			else:
				display=self.display
		
		display.EraseAll()
		
		import OCC.BRepBuilderAPI
		import OCC.gp
		from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
		from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB
		from OCC.AIS import AIS_Shape
		
		nurbslist = []
		
		
		if self.definition==0 or self.definition==1:
			self.nurbsHub = nurbsToolSet.interpolateNurbs2D(self.coordinatesHub,4)
			self.nurbsShroud = nurbsToolSet.interpolateNurbs2D(self.coordinatesShroud,4)
			nurbslist.append(nurbsToolSet.nppToOcc_Curve(nurbsToolSet.convertNurbs2Dto3D(self.nurbsHub)))
			nurbslist.append(nurbsToolSet.nppToOcc_Curve(nurbsToolSet.convertNurbs2Dto3D(self.nurbsShroud)))
			
		if self.definition==1:
			self.nurbsLE=nurbsToolSet.interpolateNurbs2D(self.coordinatesLE)
			self.nurbsTE=nurbsToolSet.interpolateNurbs2D(self.coordinatesTE)
			nurbslist.append(nurbsToolSet.nppToOcc_Curve(nurbsToolSet.convertNurbs2Dto3D(self.nurbsLE)))
			nurbslist.append(nurbsToolSet.nppToOcc_Curve(nurbsToolSet.convertNurbs2Dto3D(self.nurbsTE)))
		if self.definition==0:
			self.nurbsStackingLine = nurbsToolSet.interpolateNurbs2D(self.coordinatesStackingLine,degree=self.degreeU)
			nurbslist.append(nurbsToolSet.nppToOcc_Curve(nurbsToolSet.convertNurbs2Dto3D(self.nurbsStackingLine)))
		
		pcolor = Quantity_Color(1, 1, 1, Quantity_TOC_RGB)
		for pro in [self.coordinatesHub,self.coordinatesShroud,self.coordinatesStackingLine,self.coordinatesLE,self.coordinatesTE]:
			for p in pro:
				occp=OCC.gp.gp_Pnt(p[1],0,p[0])
				shape_to_display = AIS_Shape(OCC.BRepBuilderAPI.BRepBuilderAPI_MakeVertex(occp).Shape()).GetHandle()
				display.Context.SetColor(shape_to_display, pcolor , 0)
				display.Context.Display(shape_to_display, True)
		
			

		for index, nurbs in enumerate(nurbslist):
			nurbs_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeEdge( nurbs.GetHandle() )
			nurbs_rep.Build()

			display.DisplayShape( nurbs_rep.Shape() )
		
		
		
		
		
		bladesurf = self.getGeom_BSplineSurface()
		
		#for i in range(bladesurf.NbVPoles()):
			#for j in range(bladesurf.NbUPoles()):
				#p=bladesurf.Pole(j+1,i+1)
				##print p.X(),p.Y(),p.Z()
				#display.DisplayShape(OCC.BRepBuilderAPI.BRepBuilderAPI_MakeVertex(p).Shape())
				
				
		bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
		bladesurf_rep.Build()


		if(material):
			display.DisplayShape( bladesurf_rep.Shape(), material)
		else:
			display.DisplayShape( bladesurf_rep.Shape() )


		from OCC.AIS import AIS_Shape
		
		nurbslist = self.getGeom_BSplineCurve()
		
		for index, nurbs in enumerate(nurbslist):
			nurbs_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeEdge( nurbs.GetHandle() )
			nurbs_rep.Build()

			display.DisplayShape( nurbs_rep.Shape() )

		display.FitAll()
		display.View_Iso()
		
	
	## @brief draw NURBS curves for OpenCASCADE
	## @param display takes a pythonocc display handler
	## @param colors Sets the colors the nurbs are displayed with
	def drawNurbsCurves(self,display=None, colors=None):
		if display==None:
			if self.display==None:
				raise Exception, "Draw Method has no display."
			else:
				display=self.display
		
		display.EraseAll()
		
		import OCC.BRepBuilderAPI
		import OCC.gp
		from OCC.AIS import AIS_Shape
		
		
		nurbslist = self.getGeom_BSplineCurve()
		
		#for nurbs in nurbslist:
			#for i in range(nurbs.NbPoles()):
				#p=nurbs.Pole(i+1)
				##print p.X(),p.Y(),p.Z()
				#display.DisplayShape(OCC.BRepBuilderAPI.BRepBuilderAPI_MakeVertex(p).Shape())
		
		#Genug Farben vorhanden?
		useColors = False

		#Farbe vorhanden und auch für jede Nurbs ein anderes?
		if(colors and (len(colors) == len(nurbslist))):
			useColors = True
			

		for index, nurbs in enumerate(nurbslist):
			nurbs_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeEdge( nurbs.GetHandle() )
			nurbs_rep.Build()

			#SW: Ich hab das mal um Farben erweitert
			if(useColors):
				shape_to_display = AIS_Shape(nurbs_rep.Shape()).GetHandle()
				display.Context.SetColor(shape_to_display, colors[index], 0)
				display.Context.Display(shape_to_display, True)
			else:
				display.DisplayShape( nurbs_rep.Shape() )
		
		display.FitAll()
		display.View_Iso()
				
		
	## @brief draws a visualisation of the blade row with OpenCASCADE
	## @param display takes a pythonocc display handler
	## @param erase Sets whether the display should be erased or not
	## @param material Sets the material the Row is displayed with
	def drawRow(self,display=None,erase=True, material=None):
		if display==None:
			if self.display==None:
				raise Exception, "Draw Method has no display."
			else:
				display=self.display
		
		if erase==True:
			display.EraseAll()
		
		import OCC.BRepBuilderAPI
		import OCC.gp
		
		bladesurf = self.getGeom_BSplineSurface()
		
		bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
		
		for n in range(self.numberBlades):
			transform=OCC.gp.gp_Trsf()
			transform.SetRotation(OCC.gp.gp().OZ(),n*2*numpy.pi/self.numberBlades)

			#SW: Ich hab das mal um ein Material erweitert
			if(material):
				display.DisplayShape(OCC.BRepBuilderAPI.BRepBuilderAPI_Transform( bladesurf_rep.Shape(), transform, 1 ).Shape(), material)
			else:
				display.DisplayShape(OCC.BRepBuilderAPI.BRepBuilderAPI_Transform( bladesurf_rep.Shape(), transform, 1 ).Shape())
		
		bladesurf_rep.Build()

		#SW: Ich hab das mal auskommentiert, da eh alle verfügbaren Blades gezeichnet werden
		#display.DisplayShape( bladesurf_rep.Shape() )
		
		rotsurf = self.getGeom_BSplineRotSurfaces()[0]
		
		rotsurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( rotsurf.GetHandle() )
		
		rotsurf_rep.Build()

		#SW: Ich hab das mal um ein Material erweitert
		if(material):
			display.DisplayShape( rotsurf_rep.Shape(), material)
		else:
			display.DisplayShape( rotsurf_rep.Shape() )
		
		display.FitAll()
		display.View_Iso()
			
			
			
	## @brief Display NURBS curves with OpenCASCADE
	def displayNurbsCurves(self):
		try:
			
			import OCC.Display.SimpleGui
			
		except:
			print "!!! pythonocc (OpenCASCADE for python) not found. Display not possible."
		else:
			OCC.Display.SimpleGui.set_backend('qt')
			self.display, start_display, add_menu, add_function_to_menu = OCC.Display.SimpleGui.init_display()
			
			
			
			self.drawNurbsCurves()
			
			start_display()
	
	def qtviewer(self):
		from OCC.Display.qtDisplay import qtViewer3d
		
		from PyQt4.QtGui import QWidget, QLineEdit, QDoubleValidator, QIntValidator, QApplication, QDialog, QVBoxLayout, QPushButton
		from PyQt4.QtCore import SIGNAL,  QVariant, Qt
		from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
		from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB
		
		
		#Qt-Application erstellen
		pApp = QApplication([])
		#Dummy Dialog zur Anzeige des Widgets erstellen
		pDlg = QDialog()
		#pWidget = qtViewer3d(pDlg)
		#Layout für den Dialog erstellen
		pDlg.setLayout(QVBoxLayout())
		#pDlg.layout().addWidget(pWidget)
		
		#Widget ins Layout einfügen
		Plot = qtViewer3d()
		Plot.setObjectName("Plot")
		pDlg.resize(800,600)
		pDlg.layout().addWidget(Plot)
		#Dialog anzeigen
		pDlg.show()

		#Driver initialisieren
		Plot.InitDriver()
		#Ansicht auf Isometrisch setzen
		display=Plot._display
		
		Plot.autoFillBackground=True
		Plot.setBackgroundColor=(Qt.white)
		
		#Plot._display.View_Iso()
		##Passform
		#Plot._display.FitAll()
		#Testmodus wechseln
		#Plot._display.Test()
		ais_context = display.GetContext().GetObject()
		#for elem in dir(display.GetView().GetObject()):
			#print elem
		dc = ais_context.DeviationCoefficient()
		dc_hlr = ais_context.HLRDeviationCoefficient()
		da = ais_context.DeviationAngle()
		da_hlr = ais_context.HLRAngle()
		
		#ais_context.SetDeviationCoefficient(dc/10)
		#ais_context.SetDeviationAngle(da/10)
		ais_context.SetHLRDeviationCoefficient(dc_hlr/10)
		ais_context.SetHLRAngle(da_hlr/10)
		
		bgcolor=Quantity_Color(1, 1, 1, Quantity_TOC_RGB)
		display.GetView().GetObject().SetBackgroundColor(bgcolor)
		
		#Basismaterial erstellen
		Material = Graphic3d_MaterialAspect(Graphic3d_NOM_SILVER)
		#Diffusestärke setzen
		Material.SetDiffuse(0.8)

		#Farben erstellen
		color = Quantity_Color(0.3, 0.3, 0.3, Quantity_TOC_RGB)
		#Farbe setzen
		Material.SetDiffuseColor(color)
		self.drawBladeSurface(Plot._display, Material)
		


		#pWidget.InitDriver()
		#pBladeRowWidget.OGLPlot.InitDriver()
		#pDlg.resize(pDlg.sizeHint())
		#Ausführen
		sys.exit(pApp.exec_())
