#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Andreas Kührmann                           *
#*   andreas.kuehrmann@gmail.com                                           *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
#  @author Andreas Kührmann
#  @date 2009

# Python Imports
# üöä

import bladeRow
import nurbsToolSet as nts
import blockMeshDictGenToolSet as bts
from pythonnurbs import NurbsCurve, NurbsVector, NurbsPoint
from numpy import ones, delete, pi

class meshGen(object):
	
	def __init__(self, *args):
		
		# bool
		self.cyclicGgi     = False
		self.plot          = False
		self.symCascade    = False
		self.simpleGrading = True
		self.writeVRML     = False
		# int
		self.bladeDist       =  20
		self.debug           =   0
		self.curveToPolt     =   0
		self.maxIteration    =  75
		self.noBlades        =  23
		self.noCellsOGrid    =   7
		self.noPointsOnCurve = 200
		self.rowNumber       =   0
		# float
		self.convertToMeters =  1.0
		self.domainLength    = 70.0
		self.tolerance       = 1e-9
		self.pitch           = 20.0
		self.version         =  1.6
		# string
		self.dimension   = '2D'
		self.fileName    = 'blockMeshDict'
		self.gridType    = 'fS'
		self.path        = 'projekt/OpenFOAM/constant/polyMesh/'
		self.surfacePath = 'projekt/geometry/'
		# list
		# A
		self.axGrading = []
		# B
		self.blockCoordinates = []
		self.blockGrading     = []
		# C
		self.camberLine    = []
		self.cellsPerBlock = []
		self.circGrading   = []
		self.coordsToPolt  = []
		self.curve         = []
		# I
		self.inletOffset = []
		#
		self.lOnCurve = []
		# N
		self.noCellsAx   = []
		self.noCellsCirc = []
		self.noCellsRad  = []
		# O
		self.oGridGrading = []
		self.outletOffset = []
		# P
		self.profile = []
		# R
		self.radGrading        = []
		self.revolutionCurve   = []
		self.revolutionSurface = []
		self.row               = []
		# S
		self.scaledProfile      = []
		self.simpleAxGrading    = []
		self.simpleCircGrading  = []
		self.simpleOGridGrading = []
		self.simpleRadGrading   = []
		# U
		self.u        = []
		self.uOnCurve = []
		# V
		self.vertexCoordinates = []
		
		for arg in args:
			if type(arg) == tuple:
				self.topology = arg[0]
				self.noLayer  = arg[1]
				self.setTopologyData()
				self.setArrays()
				
			elif type(arg) == bladeRow.bladeRow:
				self.row      = arg
				self.topology = self.row.topology
				self.noLayer  = self.row.noLayer
				
			else:
				raise Exception, "init error"
	
	
	def setTopologyData(self):
		
		if self.topology == "O10H":
			# int
			self.centerStart              = 10
			self.inletStart               =  0
			self.noBlocksBetweenTwoLayers = 16
			self.noBlocksCenter           =  4
			self.noBlocksInlet            =  3
			self.noBlocksOutlet           =  3
			self.noBlockPerPassageSide    =  4
			self.noBlocksAroundProfile    =  6
			self.noControlPoints          =  8
			self.noEdgesAxGrading         = 16
			self.noEdgesCircGrading       = 16
			self.noInletParameter         =  2
			self.noOGridCurves            =  4
			self.noOutletParameter        =  2
			self.outletStart              = 22
			self.uValuesPerLayer          = 13
			self.uValuesPerProfile        =  4
			self.uValuesPerScaledProfile  =  4
			self.vertexPerLayer           = 26             # Anzahl von Netzpunkten pro Layer
			
			# array
			self.passageSide = [[0, 4, 10, 16, 22], [3, 9, 15, 21, 25]]
			self.smeVertex  = [[19, 13, 7, 6, 12, 18, 19], [20, 14, 8, 5, 11, 17, 20], [5, 4], [5, 1], [8, 2], [8, 9], [20, 21], [20, 24], [17, 23], [17, 16], [6, 5], [7, 8], [19, 20], [18, 17]]
			
			
		else:
			raise ValueError('*** Check topology value! {0} don\'t exist'.format(self.topology))
	
	
	def setArrays(self):
		
		# A
		self.absControlPoint = ones([self.noLayer, self.noControlPoints, 3], 'd')
		self.axGrading       = ones([self.noLayer, self.noEdgesAxGrading],   'd')
		# B
		self.bladePatches     = ones([self.noBlocksAroundProfile*(self.noLayer-1), 4],     'i')
		self.blockCoordinates = ones([self.noBlocksBetweenTwoLayers*(self.noLayer-1), 8],  'i')
		self.blockGrading     = ones([self.noBlocksBetweenTwoLayers*(self.noLayer-1), 12], 'd')
		# C
		self.cellsPerBlock     = ones([self.noBlocksBetweenTwoLayers*(self.noLayer-1), 3], 'i')
		self.centerCoordinates = ones([self.noLayer, self.noBlocksCenter, 3],              'd')
		self.circGrading       = ones([self.noLayer, self.noEdgesCircGrading],             'd')
		# H
		self.hubPatches = ones([self.noBlocksBetweenTwoLayers, 4], 'i')
		# I
		self.inletCoordinates = ones([self.noLayer, self.noBlocksInlet, 3],    'd')
		self.inletOffset      = ones([self.noLayer, self.noInletParameter],    'd')
		self.inletPatches     = ones([self.noBlocksInlet*(self.noLayer-1), 4], 'i')
		# N
		self.noCellsAx   = ones([self.noBlockPerPassageSide], 'i')
		self.noCellsCirc = ones([self.noBlocksInlet],         'i')
		self.noCellsRad  = ones([self.noLayer-1],             'i')
		# O
		self.oGridGrading      = ones([self.noLayer, self.noBlocksAroundProfile], 'd')
		self.oGridWidth        = ones([self.noLayer],                             'd')
		self.outletCoordinates = ones([self.noLayer, self.noBlocksOutlet, 3],     'd')
		self.outletOffset      = ones([self.noLayer, self.noOutletParameter],     'd')
		self.outletPatches     = ones([self.noBlocksOutlet*(self.noLayer-1), 4],  'i')
		# P
		self.passageSidePatches = ones([self.noBlockPerPassageSide*2*(self.noLayer-1), 4], 'i')
		# R
		self.radGrading      = ones([self.vertexPerLayer*(self.noLayer-1)],  'd')
		self.relControlPoint = ones([self.noLayer, self.noControlPoints, 2], 'd')
		# Sif self.plot == True:
		self.shroudPatches      = ones([self.noBlocksBetweenTwoLayers, 4],         'i')
		self.simpleAxGrading    = ones([self.noLayer, self.noBlockPerPassageSide], 'd')
		self.simpleCircGrading  = ones([self.noLayer, self.noBlocksInlet],         'd')
		self.simpleOGridGrading = ones([self.noLayer],                             'd')
		self.simpleRadGrading   = ones([self.noLayer-1],                           'd')
		self.span               = ones([self.noLayer],                             'd')
		# U
		self.u                     = ones([self.noLayer, self.uValuesPerLayer],         'd')
		self.uValueOnProfile       = ones([self.noLayer, self.uValuesPerProfile],       'd')
		self.uValueOnScaledProfile = ones([self.noLayer, self.uValuesPerScaledProfile], 'd')
		# V
		self.vertexCoordinates = ones([self.noLayer, self.vertexPerLayer, 3], 'd')
		
		# list
		for layerIndex in xrange(self.noLayer):
			self.camberLine.append([])
			self.curve.append([])
			self.profile.append([])
			self.revolutionCurve.append([])
			self.revolutionSurface.append([])
			self.scaledProfile.append([])
			self.uOnCurve.append([])
			
	
	
	def executeBlockMeshDictGen(self):
		
		if self.debug == 1: print '\n*** Computing blockMesh data'
		self.computeBlockMeshData()
		
		if self.debug == 1: print '*** Writing blockMeshDict file'
		self.writeBlockMeshDictFile()
	
	
	def computeBlockMeshData(self):
		
		if self.debug == 1: print '*** Checking data source'
		self.checkDataSource()
		
		if self.debug ==1: print '*** Computing vertex coordinates'
		self.computeVertexCoordinates()
		
		if self.debug == 1: print '*** Computing block data'
		self.computeBlockData()
		
		if self.debug == 1: print '*** Computing edge data'
		self.computeEdgeData()
		
		if self.plot == True:
			if self.debug == 1: print '*** Ploting topology'
			self.layerPlot()
			
		
		if self.debug == 1: print '*** Computing control curves'
		self.computeContolCurve()
		
		if self.debug == 1: print '*** Computing patch data'
		self.computePatchData()
	
	
	def checkDataSource(self):
		
		if type(self.row) == bladeRow.bladeRow:
			if self.gridType == 'fS':
				if self.dimension == '3D':
					self.row.dataStruct.add3DcComputation("row"+str(self.row.rowNumber),True)
					self.path = self.row.dataStruct.workdir+self.row.dataStruct.dir3Dc+"/row"+str(self.row.rowNumber)+"/constant/polyMesh/"
					
				elif self.dimension == '2D':
					self.row.dataStruct.add2DComputation("row"+str(self.row.rowNumber),True)
					self.path = self.row.dataStruct.workdir+self.row.dataStruct.dir2D+"/row"+str(self.row.rowNumber)+"/constant/polyMesh/"
					
				else:
					raise ValueError('*** Check dimension value! {0} don\'t exist'.format(self.dimension))
						
				if self.dimension == '3D' and self.symCascade == False:
					centerLayer = len(self.row.nurbsCamber)/2
					
					self.camberLine[1] = self.row.nurbsCamber[centerLayer]
					self.profile[1]    = self.row.nurbsProfile[centerLayer]
					
					lastLayer = 2
					
				else: lastLayer = 1
				
				self.bladeDist             = self.row.bladeDist
				self.camberLine[0]         = self.row.nurbsCamber[0]
				self.camberLine[lastLayer] = self.row.nurbsCamber[-1]
				self.domainLength          = self.row.domainLength
				self.profile[0]            = self.row.nurbsProfile[0]
				self.profile[lastLayer]    = self.row.nurbsProfile[-1]
				self.rowNumber             = self.row.rowNumber
				
			elif self.gridType == 'rS':
				if self.dimension == '3D':
					self.row.dataStruct.add3DComputation("row"+str(self.row.rowNumber)+"_span"+str(int(self.span[0]*100))+"-"+str(int(self.span[-1]*100)), True)
					self.path = self.row.dataStruct.workdir+self.row.dataStruct.dir3D+"/row"+str(self.row.rowNumber)+"_span"+str(int(self.span[0]*100))+"-"+str(int(self.span[-1]*100))+"/constant/polyMesh/"
					
				elif self.dimension == '2D':
					self.row.dataStruct.add2DS1Computation("row"+str(self.row.rowNumber)+"_span"+str(int(self.span[0]*100))+"-"+str(int(self.span[-1]*100)), True)
					self.path = self.row.dataStruct.workdir+self.row.dataStruct.dir2DS1+"/row"+str(self.row.rowNumber)+"_span"+str(int(self.span[0]*100))+"-"+str(int(self.span[-1]*100))+"/constant/polyMesh/"
					
				else:
					raise ValueError('*** Check dimension value! {0} don\'t exist'.format(self.dimension))
						
				for layerIndex in xrange(self.noLayer):
					self.camberLine[layerIndex]      = self.row.nurbsCamber[layerIndex+self.row.startProfile]
					self.profile[layerIndex]         = self.row.nurbsProfile[layerIndex+self.row.startProfile]
					self.revolutionCurve[layerIndex] = self.row.nurbsRot[layerIndex+self.row.startProfile]
					#self.camberLine[layerIndex]      = self.row.neueFunktion(self.span[layerIndex])
					#self.profile[layerIndex]         = self.row.neueFunktion(self.span[layerIndex])
					#self.revolutionCurve[layerIndex] = self.row.neueFunktion(self.span[layerIndex])
				
				self.noBlades    = self.row.numberBlades
				self.rowNumber   = self.row.rowNumber
				self.surfacePath = self.row.dataStruct.workdir+"/geometry/bladeRow_"+str(self.row.rowNumber)+"/"
				
			else: 
				raise ValueError('*** Check gridType value! {0} don\'t exist'.format(self.gridType))
	
	
	def computeVertexCoordinates(self):
		
		if self.topology == "O10H":
			for layerIndex in xrange(self.noLayer):
				if self.gridType == 'rS':
					self.pitch = 2*pi/self.noBlades
					
					origin  = NurbsPoint.Point3Dd(0.0, 0.0, 0.0)
					rVector = NurbsPoint.Point3Dd(0.0, 0.0, 1.0)
					tVector = [0, 0, -1]
					theta   = 2*pi
					
					self.revolutionSurface[layerIndex] = nts.computeRevolutionSurface(self.revolutionCurve[layerIndex], origin, rVector, theta, tVector, self.writeVRML, self.span[layerIndex], self.surfacePath)
					
				elif self.gridType == 'fS': self.pitch = self.bladeDist
				
				self.scaledProfile[layerIndex] = nts.computeScaledCurve(self.profile[layerIndex], self.revolutionSurface[layerIndex], self.oGridWidth[layerIndex], self.gridType, self.noPointsOnCurve, self.tolerance, self.maxIteration)
				
				self.uValuesO10H(layerIndex)
				self.computeVertexO10H(layerIndex)
	
	
	def uValuesO10H(self, layerIndex):
		
		self.u[layerIndex, 0]     = nts.calculateUStart(1, self.camberLine[layerIndex].length()/2.0, self.camberLine[layerIndex], self.tolerance, self.maxIteration)
		
		uValueAtHalfProfile       = nts.calculateUStart(1, self.profile[layerIndex].length()/2.0, self.profile[layerIndex], self.tolerance, self.maxIteration)
		uValueAtHalfScaledProfile = nts.calculateUStart(1, self.scaledProfile[layerIndex].length()/2.0, self.scaledProfile[layerIndex], self.tolerance, self.maxIteration)
		
		# profile
		# set
		self.u[layerIndex,  1] = self.uValueOnProfile[layerIndex, 0]
		self.u[layerIndex,  3] = self.uValueOnProfile[layerIndex, 1]
		self.u[layerIndex,  4] = self.uValueOnProfile[layerIndex, 2]
		self.u[layerIndex,  6] = self.uValueOnProfile[layerIndex, 3]
		
		self.u[layerIndex,  7] = self.uValueOnScaledProfile[layerIndex, 0]
		self.u[layerIndex,  9] = self.uValueOnScaledProfile[layerIndex, 1]
		self.u[layerIndex, 10] = self.uValueOnScaledProfile[layerIndex, 2]
		self.u[layerIndex, 12] = self.uValueOnScaledProfile[layerIndex, 3]
		
		if self.gridType == 'fS':
			
			z = self.camberLine[layerIndex].pointAt(self.u[layerIndex, 0]).getz()
			
			# profile
			self.u[layerIndex,  2] = nts.calculateUFromZ(z, self.profile[layerIndex], -1, 0, 0, uValueAtHalfProfile, self.tolerance, self.maxIteration)
			self.u[layerIndex,  5] = nts.calculateUFromZ(z, self.profile[layerIndex], -1, uValueAtHalfProfile, uValueAtHalfProfile, 1, self.tolerance, self.maxIteration)
			
			# scaledProfile
			self.u[layerIndex,  8] = nts.calculateUFromZ(z, self.scaledProfile[layerIndex], -1, 0, 0, uValueAtHalfScaledProfile, self.tolerance, self.maxIteration)
			self.u[layerIndex, 11] = nts.calculateUFromZ(z, self.scaledProfile[layerIndex], -1, uValueAtHalfScaledProfile, uValueAtHalfScaledProfile, 1, self.tolerance, self.maxIteration)
			
		elif self.gridType == 'rS':
			
			m = nts.getCurveCoordinates(self.u[layerIndex, 0], self.camberLine[layerIndex], "m")
			
			# profile
			self.u[layerIndex,  2] = nts.calculateUFromM(m, -1.0, 0.0, uValueAtHalfProfile, self.profile[layerIndex], self.tolerance, self.maxIteration)
			self.u[layerIndex,  5] = nts.calculateUFromM(m, -1.0, uValueAtHalfProfile, 1.0, self.profile[layerIndex], self.tolerance, self.maxIteration)
			
			# scaledProfile
			self.u[layerIndex,  8] = nts.calculateUFromM(m, -1.0, 0.0, uValueAtHalfScaledProfile, self.scaledProfile[layerIndex], self.tolerance, self.maxIteration)
			self.u[layerIndex, 11] = nts.calculateUFromM(m, -1.0, uValueAtHalfScaledProfile, 1.0, self.scaledProfile[layerIndex], self.tolerance, self.maxIteration)
	
	
	def computeVertexO10H(self, layerIndex):
		
		## def points 6, 7, 12, 13, 18 and 19 --> profile
		self.vertexCoordinates[layerIndex,  6] = nts.getCurveCoordinates(self.u[layerIndex,  4], self.profile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex,  7] = nts.getCurveCoordinates(self.u[layerIndex,  3], self.profile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 12] = nts.getCurveCoordinates(self.u[layerIndex,  5], self.profile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 13] = nts.getCurveCoordinates(self.u[layerIndex,  2], self.profile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 18] = nts.getCurveCoordinates(self.u[layerIndex,  6], self.profile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 19] = nts.getCurveCoordinates(self.u[layerIndex,  1], self.profile[layerIndex], "xyz")
		## def points 5, 8, 11, 14, 17 and 20 --> oGrid
		self.vertexCoordinates[layerIndex,  5] = nts.getCurveCoordinates(self.u[layerIndex, 10], self.scaledProfile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex,  8] = nts.getCurveCoordinates(self.u[layerIndex,  9], self.scaledProfile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 11] = nts.getCurveCoordinates(self.u[layerIndex, 11], self.scaledProfile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 14] = nts.getCurveCoordinates(self.u[layerIndex,  8], self.scaledProfile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 17] = nts.getCurveCoordinates(self.u[layerIndex, 12], self.scaledProfile[layerIndex], "xyz")
		self.vertexCoordinates[layerIndex, 20] = nts.getCurveCoordinates(self.u[layerIndex,  7], self.scaledProfile[layerIndex], "xyz")
		
		if self.gridType == 'fS':
			
			inputCS      = "xyz"
			outputCS     = "xyz"
			
			domainLength = self.domainLength
			
		elif self.gridType == 'rS':
			
			inputCS      = "rpz"
			outputCS     = "rpz"
			
			domainLength = self.revolutionCurve[layerIndex]
			
		vertexCoordinatesInTmp  = bts.getDomainCoordinates(0.000000001, self.camberLine[layerIndex], domainLength, self.gridType, outputCS, self.tolerance, self.maxIteration)
		vertexCoordinatesOutTmp = bts.getDomainCoordinates(0.999999999, self.camberLine[layerIndex], domainLength, self.gridType, outputCS, self.tolerance, self.maxIteration)
		
		for i in xrange(4):
			
			xTwoOffsetIn   = self.pitch*(-i/3.0+0.5+self.inletOffset[layerIndex, 0])
			xTwoOffsetOut  = self.pitch*(-i/3.0+0.5+self.outletOffset[layerIndex, 0])
			
			## def points 0, 1, 2, 3 --> inlet
			self.vertexCoordinates[layerIndex, i]    = nts.movePoint(vertexCoordinatesInTmp,  0,  xTwoOffsetIn,  0, inputCS, "xyz")
			
			## def points 22, 23, 24, 25 --> outlet
			self.vertexCoordinates[layerIndex, i+22] = nts.movePoint(vertexCoordinatesOutTmp, 0,  xTwoOffsetOut, 0, inputCS, "xyz")
		
		self.vertexCoordinates[layerIndex, 10] = nts.moveCurvePoint(self.u[layerIndex, 0], self.camberLine[layerIndex], 0,  self.pitch/2.0, 0, inputCS, "xyz")
		self.vertexCoordinates[layerIndex, 15] = nts.moveCurvePoint(self.u[layerIndex, 0], self.camberLine[layerIndex], 0, -self.pitch/2.0, 0, inputCS, "xyz")
		
		vertexCoordinatesTmp = bts.getDomainCoordinates(self.inletOffset[layerIndex, 1], self.camberLine[layerIndex], domainLength, self.gridType, outputCS, self.tolerance, self.maxIteration)
		
		lEdge = bts.computeL(0, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
		
		pitchOffset = ((0.0-self.inletOffset[layerIndex, 0])/(lEdge)*self.inletOffset[layerIndex, 1]+self.inletOffset[layerIndex, 0])
		
		self.vertexCoordinates[layerIndex,  4] = nts.movePoint(vertexCoordinatesTmp, 0,  self.pitch*(0.5+pitchOffset), 0, inputCS, "xyz")
		self.vertexCoordinates[layerIndex,  9] = nts.movePoint(vertexCoordinatesTmp, 0, -self.pitch*(0.5-pitchOffset), 0, inputCS, "xyz")
		
		vertexCoordinatesTmp = bts.getDomainCoordinates(self.outletOffset[layerIndex, 1], self.camberLine[layerIndex], domainLength, self.gridType, outputCS, self.tolerance, self.maxIteration)
		
		tEdge = bts.computeL(1, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
		
		grad = (self.outletOffset[layerIndex, 0])/(1.0-tEdge)
		
		pitchOffset = (grad*self.outletOffset[layerIndex, 1]-(grad*tEdge))
		
		self.vertexCoordinates[layerIndex, 16] = nts.movePoint(vertexCoordinatesTmp, 0,  self.pitch*(0.5+pitchOffset), 0, inputCS, "xyz")
		self.vertexCoordinates[layerIndex, 21] = nts.movePoint(vertexCoordinatesTmp, 0, -self.pitch*(0.5-pitchOffset), 0, inputCS, "xyz")
	
	
	def computeBlockData(self):
		
		if self.topology == "O10H": self.computeBlockDataO10H()
	
	
	def computeBlockDataO10H(self):
		
		for radBlockIndex in xrange(self.noLayer-1):
			a = len(self.blockCoordinates)/(self.noLayer-1)*radBlockIndex
			b = self.vertexPerLayer*radBlockIndex
			
			self.blockCoordinates[ 0+a] = [ 0+b,  4+b,  5+b,  1+b, 26+b, 30+b, 31+b, 27+b]
			self.blockCoordinates[ 1+a] = [ 1+b,  5+b,  8+b,  2+b, 27+b, 31+b, 34+b, 28+b]
			self.blockCoordinates[ 2+a] = [ 2+b,  8+b,  9+b,  3+b, 28+b, 34+b, 35+b, 29+b]
			self.blockCoordinates[ 3+a] = [ 4+b, 10+b, 11+b,  5+b, 30+b, 36+b, 37+b, 31+b]
			self.blockCoordinates[ 4+a] = [ 5+b, 11+b, 12+b,  6+b, 31+b, 37+b, 38+b, 32+b]
			self.blockCoordinates[ 5+a] = [ 5+b,  6+b,  7+b,  8+b, 31+b, 32+b, 33+b, 34+b]
			self.blockCoordinates[ 6+a] = [ 7+b, 13+b, 14+b,  8+b, 33+b, 39+b, 40+b, 34+b]
			self.blockCoordinates[ 7+a] = [ 8+b, 14+b, 15+b,  9+b, 34+b, 40+b, 41+b, 35+b]
			self.blockCoordinates[ 8+a] = [10+b, 16+b, 17+b, 11+b, 36+b, 42+b, 43+b, 37+b]
			self.blockCoordinates[ 9+a] = [11+b, 17+b, 18+b, 12+b, 37+b, 43+b, 44+b, 38+b]
			self.blockCoordinates[10+a] = [18+b, 17+b, 20+b, 19+b, 44+b, 43+b, 46+b, 45+b]
			self.blockCoordinates[11+a] = [13+b, 19+b, 20+b, 14+b, 39+b, 45+b, 46+b, 40+b]
			self.blockCoordinates[12+a] = [14+b, 20+b, 21+b, 15+b, 40+b, 46+b, 47+b, 41+b]
			self.blockCoordinates[13+a] = [16+b, 22+b, 23+b, 17+b, 42+b, 48+b, 49+b, 43+b]
			self.blockCoordinates[14+a] = [17+b, 23+b, 24+b, 20+b, 43+b, 49+b, 50+b, 46+b]
			self.blockCoordinates[15+a] = [20+b, 24+b, 25+b, 21+b, 46+b, 50+b, 51+b, 47+b]
			# def celles for all blocks
			self.cellsPerBlock[ 0+a] = [self.noCellsAx[0], self.noCellsCirc[0], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 1+a] = [self.noCellsAx[0], self.noCellsCirc[1], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 2+a] = [self.noCellsAx[0], self.noCellsCirc[2], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 3+a] = [self.noCellsAx[1], self.noCellsCirc[0], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 4+a] = [self.noCellsAx[1], self.noCellsOGrid,   self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 5+a] = [self.noCellsOGrid, self.noCellsCirc[1], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 6+a] = [self.noCellsAx[1], self.noCellsOGrid,   self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 7+a] = [self.noCellsAx[1], self.noCellsCirc[2], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 8+a] = [self.noCellsAx[2], self.noCellsCirc[0], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[ 9+a] = [self.noCellsAx[2], self.noCellsOGrid,   self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[10+a] = [self.noCellsOGrid, self.noCellsCirc[1], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[11+a] = [self.noCellsAx[2], self.noCellsOGrid,   self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[12+a] = [self.noCellsAx[2], self.noCellsCirc[2], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[13+a] = [self.noCellsAx[3], self.noCellsCirc[0], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[14+a] = [self.noCellsAx[3], self.noCellsCirc[1], self.noCellsRad[radBlockIndex]]
			self.cellsPerBlock[15+a] = [self.noCellsAx[3], self.noCellsCirc[2], self.noCellsRad[radBlockIndex]]
			
			if self.simpleGrading == True:
				for radBlockIndex in xrange(self.noLayer-1):
					b = self.vertexPerLayer*radBlockIndex
					for edgeIndex in xrange(self.vertexPerLayer):
						self.radGrading[edgeIndex+b] = self.simpleRadGrading[radBlockIndex]
						
				for layerIndex in xrange(self.noLayer):
					for edgeIndex in xrange(6):
						self.oGridGrading[layerIndex, edgeIndex] = self.simpleOGridGrading[layerIndex]
						
					axIndex = 0
					for axLine in [0, 3, 8, 13]:
						if axLine != 0 and axLine != 13: noEdge = 5
						
						else: noEdge = 3
						
						for edgeIndex in xrange(noEdge):
							self.axGrading[layerIndex, axLine+edgeIndex] = self.simpleAxGrading[layerIndex, axIndex]
							
						axIndex = axIndex + 1
						
					circIndex = 0
					for circLine in [0, 5, 11]:
						if circLine != 5: noEdge = 5
						
						else: noEdge = 6
						
						for edgeIndex in xrange(noEdge):
							self.circGrading[layerIndex, circLine+edgeIndex] = self.simpleCircGrading[layerIndex, circIndex]
							
						circIndex = circIndex + 1
						
			for radBlockIndex in xrange(self.noLayer-1):
				a = len(self.blockGrading)*radBlockIndex/(self.noLayer-1)
				b = self.vertexPerLayer*radBlockIndex
				
				self.blockGrading[ 0+a] = [self.axGrading[radBlockIndex+1,  0], self.axGrading[radBlockIndex+1,  1], self.axGrading[radBlockIndex,  1], self.axGrading[radBlockIndex,  0], self.circGrading[radBlockIndex+1,  0], self.circGrading[radBlockIndex+1,  1], self.circGrading[radBlockIndex,  1], self.circGrading[radBlockIndex,  0], self.radGrading[ 0+b], self.radGrading[ 4+b], self.radGrading[ 5+b], self.radGrading[ 1+b]]
				self.blockGrading[ 1+a] = [self.axGrading[radBlockIndex+1,  1], self.axGrading[radBlockIndex+1,  2], self.axGrading[radBlockIndex,  2], self.axGrading[radBlockIndex,  1], self.circGrading[radBlockIndex+1,  5], self.circGrading[radBlockIndex+1,  6], self.circGrading[radBlockIndex,  6], self.circGrading[radBlockIndex,  5], self.radGrading[ 1+b], self.radGrading[ 5+b], self.radGrading[ 8+b], self.radGrading[ 2+b]]
				self.blockGrading[ 2+a] = [self.axGrading[radBlockIndex+1,  2], self.axGrading[radBlockIndex+1,  0], self.axGrading[radBlockIndex,  0], self.axGrading[radBlockIndex,  2], self.circGrading[radBlockIndex+1, 11], self.circGrading[radBlockIndex+1, 12], self.circGrading[radBlockIndex, 12], self.circGrading[radBlockIndex, 11], self.radGrading[ 2+b], self.radGrading[ 8+b], self.radGrading[ 9+b], self.radGrading[ 3+b]]
				self.blockGrading[ 3+a] = [self.axGrading[radBlockIndex+1,  3], self.axGrading[radBlockIndex+1,  4], self.axGrading[radBlockIndex,  4], self.axGrading[radBlockIndex,  3], self.circGrading[radBlockIndex+1,  1], self.circGrading[radBlockIndex+1,  2], self.circGrading[radBlockIndex,  2], self.circGrading[radBlockIndex,  1], self.radGrading[ 4+b], self.radGrading[10+b], self.radGrading[11+b], self.radGrading[ 5+b]]
				self.blockGrading[ 4+a] = [self.axGrading[radBlockIndex+1,  4], self.axGrading[radBlockIndex+1,  5], self.axGrading[radBlockIndex,  5], self.axGrading[radBlockIndex,  4], self.oGridGrading[radBlockIndex+1, 0], self.oGridGrading[radBlockIndex+1, 1], self.oGridGrading[radBlockIndex, 1], self.oGridGrading[radBlockIndex, 0], self.radGrading[ 5+b], self.radGrading[11+b], self.radGrading[12+b], self.radGrading[ 6+b]]
				self.blockGrading[ 5+a] = [self.oGridGrading[radBlockIndex+1, 0], self.oGridGrading[radBlockIndex+1, 5], self.oGridGrading[radBlockIndex, 5], self.oGridGrading[radBlockIndex, 0], self.circGrading[radBlockIndex+1, 8], self.circGrading[radBlockIndex+1, 9], self.circGrading[radBlockIndex, 7], self.circGrading[radBlockIndex, 5], self.radGrading[ 5+b], self.radGrading[ 6+b], self.radGrading[ 7+b], self.radGrading[ 8+b]]
				self.blockGrading[ 6+a] = [self.axGrading[radBlockIndex+1,  6], self.axGrading[radBlockIndex+1,  7], self.axGrading[radBlockIndex,  7], self.axGrading[radBlockIndex,  6], 1/self.oGridGrading[radBlockIndex+1, 5], 1/self.oGridGrading[radBlockIndex+1, 4], 1/self.oGridGrading[radBlockIndex, 4], 1/self.oGridGrading[radBlockIndex, 5], self.radGrading[7+b], self.radGrading[13+b], self.radGrading[14+b], self.radGrading[ 8+b]]
				self.blockGrading[ 7+a] = [self.axGrading[radBlockIndex+1,  7], self.axGrading[radBlockIndex+1,  3], self.axGrading[radBlockIndex,  3], self.axGrading[radBlockIndex,  7], self.circGrading[radBlockIndex+1, 12], self.circGrading[radBlockIndex+1, 13], self.circGrading[radBlockIndex, 13], self.circGrading[radBlockIndex, 12], self.radGrading[ 8+b], self.radGrading[14+b], self.radGrading[15+b], self.radGrading[ 9+b]]
				self.blockGrading[ 8+a] = [self.axGrading[radBlockIndex+1,  8], self.axGrading[radBlockIndex+1,  9], self.axGrading[radBlockIndex,  9], self.axGrading[radBlockIndex,  8], self.circGrading[radBlockIndex+1,  2], self.circGrading[radBlockIndex+1,  3], self.circGrading[radBlockIndex,  3], self.circGrading[radBlockIndex,  2], self.radGrading[10+b], self.radGrading[16+b], self.radGrading[17+b], self.radGrading[11+b]]
				self.blockGrading[ 9+a] = [self.axGrading[radBlockIndex+1,  9], self.axGrading[radBlockIndex+1, 10], self.axGrading[radBlockIndex, 10], self.axGrading[radBlockIndex,  9], self.oGridGrading[radBlockIndex+1, 1], self.oGridGrading[radBlockIndex+1, 2], self.oGridGrading[radBlockIndex, 2], self.oGridGrading[radBlockIndex, 1], self.radGrading[11+b], self.radGrading[17+b], self.radGrading[18+b], self.radGrading[12+b]]
				self.blockGrading[10+a] = [1/self.oGridGrading[radBlockIndex+1, 2], 1/self.oGridGrading[radBlockIndex+1, 3], 1/self.oGridGrading[radBlockIndex, 3], 1/self.oGridGrading[radBlockIndex, 2], self.circGrading[radBlockIndex, 8], self.circGrading[radBlockIndex+1, 9], self.circGrading[radBlockIndex, 9], self.circGrading[radBlockIndex, 8], self.radGrading[18+b], self.radGrading[17+b], self.radGrading[20+b], self.radGrading[19+b]]
				self.blockGrading[11+a] = [self.axGrading[radBlockIndex+1, 11], self.axGrading[radBlockIndex+1, 12], self.axGrading[radBlockIndex, 12], self.axGrading[radBlockIndex, 11], 1/self.oGridGrading[radBlockIndex+1, 4], 1/self.oGridGrading[radBlockIndex+1, 3], 1/self.oGridGrading[radBlockIndex, 3], 1/self.oGridGrading[radBlockIndex, 4], self.radGrading[13+b], self.radGrading[19+b], self.radGrading[20+b], self.radGrading[14+b]]
				self.blockGrading[12+a] = [self.axGrading[radBlockIndex+1, 12], self.axGrading[radBlockIndex+1,  8], self.axGrading[radBlockIndex,  8], self.axGrading[radBlockIndex, 12], self.circGrading[radBlockIndex+1, 13], self.circGrading[radBlockIndex+1, 14], self.circGrading[radBlockIndex, 14], self.circGrading[radBlockIndex, 13], self.radGrading[14+b], self.radGrading[20+b], self.radGrading[21+b], self.radGrading[15+b]]
				self.blockGrading[13+a] = [self.axGrading[radBlockIndex+1, 13], self.axGrading[radBlockIndex+1, 14], self.axGrading[radBlockIndex, 14], self.axGrading[radBlockIndex, 13], self.circGrading[radBlockIndex+1,  3], self.circGrading[radBlockIndex+1,  4], self.circGrading[radBlockIndex,  4], self.circGrading[radBlockIndex,  3], self.radGrading[16+b], self.radGrading[21+b], self.radGrading[23+b], self.radGrading[17+b]]
				self.blockGrading[14+a] = [self.axGrading[radBlockIndex+1, 14], self.axGrading[radBlockIndex+1, 15], self.axGrading[radBlockIndex, 15], self.axGrading[radBlockIndex, 14], self.circGrading[radBlockIndex+1,  9], self.circGrading[radBlockIndex+1, 10], self.circGrading[radBlockIndex, 10], self.circGrading[radBlockIndex,  9], self.radGrading[17+b], self.radGrading[23+b], self.radGrading[24+b], self.radGrading[20+b]]
				self.blockGrading[15+a] = [self.axGrading[radBlockIndex+1, 15], self.axGrading[radBlockIndex+1, 13], self.axGrading[radBlockIndex, 13], self.axGrading[radBlockIndex, 15], self.circGrading[radBlockIndex+1, 14], self.circGrading[radBlockIndex+1, 15], self.circGrading[radBlockIndex, 15], self.circGrading[radBlockIndex, 14], self.radGrading[20+b], self.radGrading[24+b], self.radGrading[25+b], self.radGrading[21+b]]
	
	def computeEdgeData(self):
		
		if self.topology == "O10H":
			for layerIndex in xrange(self.noLayer):
				self.computeEdgeDataO10H(layerIndex)
	
	
	def computeEdgeDataO10H(self, layerIndex):
		
		self.curve[layerIndex].append(self.profile[layerIndex])
		self.curve[layerIndex].append(self.scaledProfile[layerIndex])
		
		fixPointsOnCurve  = [[], []]
		grading           = [[], []]
		cellLength        = [[], []]
		
		for pointIndex in xrange(6):
			
			fixPointsOnCurve[0].append(self.u[layerIndex, pointIndex+1])
			fixPointsOnCurve[1].append(self.u[layerIndex, pointIndex+7])
			
		noCells    = [self.noCellsAx[2], self.noCellsAx[1], self.noCellsCirc[1], self.noCellsAx[1], self.noCellsAx[2], self.noCellsCirc[1]]
		
		grading[0] = [1.0/self.axGrading[layerIndex, 11], 1.0/self.axGrading[layerIndex, 6], 1.0/self.circGrading[layerIndex, 7], self.axGrading[layerIndex, 5], self.axGrading[layerIndex, 10], self.circGrading[layerIndex, 8]]
		grading[1] = [1.0/self.axGrading[layerIndex, 12], 1.0/self.axGrading[layerIndex, 7], 1.0/self.circGrading[layerIndex, 6], self.axGrading[layerIndex, 4], self.axGrading[layerIndex,  9], self.circGrading[layerIndex, 9]]
		
		for curveIndex in xrange(2):
			
			blockEdgeLength = nts.computePartLengthCurve(fixPointsOnCurve[curveIndex], self.curve[layerIndex][curveIndex], True, self.tolerance, self.maxIteration)
			
			for partIndex in xrange(len(noCells)):
				
				cellLength[curveIndex].append(bts.computeAbsCellLength(noCells[partIndex], grading[curveIndex][partIndex], blockEdgeLength[partIndex]))
				
			self.uOnCurve[layerIndex].append(nts.transformCurveIntoUValues(fixPointsOnCurve[curveIndex], cellLength[curveIndex], self.curve[layerIndex][curveIndex], self.tolerance, self.maxIteration))
		
		if self.gridType == 'fS':
			
			domainLength = self.domainLength
			
		elif self.gridType == 'rS':
			
			self.computeZConstCoordinatesO10H(layerIndex)
			
			domainLength = self.revolutionCurve[layerIndex]
		
		lCenter = bts.computeL(self.u[layerIndex, 0], self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
		
		fixPointsOnCurve  = []
		grading           = []
		cellLength        = []
		
		fixPointsOnCurve = [0.000000001, self.inletOffset[layerIndex, 1], lCenter, self.outletOffset[layerIndex, 1], 0.999999]
		
		noCells          = [self.noCellsAx[0], self.noCellsAx[1], self.noCellsAx[2], self.noCellsAx[3]]
		
		grading          = [self.axGrading[layerIndex, 0], self.axGrading[layerIndex, 3], self.axGrading[layerIndex, 8], self.axGrading[layerIndex, 13]]
		
		blockEdgeLength = bts.computePartLengthDomain(fixPointsOnCurve, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
		
		for partIndex in xrange(len(noCells)):
				
				cellLength.append(bts.computeAbsCellLength(noCells[partIndex], grading[partIndex], blockEdgeLength[partIndex]))
				
		self.lOnCurve.append(bts.transformDomainIntoLValues(fixPointsOnCurve, cellLength, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration))
		
	def layerPlot(self):
		
		if self.topology == 'O10H':
			
			self.plotO10H()
			
		#raise Exception('The plot modus is only for preview! To geneate a blockMeshDict file the polt value musst be False!')
			
	def computeContolCurve(self):
		
		for layerIndex in xrange(self.noLayer):
			
			if self.topology == 'O10H':
				
				self.computeControlCurveO10H(layerIndex)
	
	
	def computeZConstCoordinatesO10H(self, layerIndex):
		
		for i in xrange(self.noBlocksInlet):
			tmpVertexCoordinatesRPZ1 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i],   "rpz")
			tmpVertexCoordinatesRPZ2 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i+1], "rpz")
			self.inletCoordinates[layerIndex, i] = nts.transformCoordinates((tmpVertexCoordinatesRPZ1+tmpVertexCoordinatesRPZ2)/2.0, "xyz")
			
		for i in xrange(self.noBlocksOutlet):
			tmpVertexCoordinatesRPZ1 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i+22], "rpz")
			tmpVertexCoordinatesRPZ2 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i+23], "rpz")
			self.outletCoordinates[layerIndex, i] = nts.transformCoordinates((tmpVertexCoordinatesRPZ1+tmpVertexCoordinatesRPZ2)/2.0, "xyz")
			
		for i in xrange(self.noBlocksCenter):
			tmpVertexCoordinatesRPZ1 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i+10], "rpz")
			tmpVertexCoordinatesRPZ2 = nts.transformCoordinates(self.vertexCoordinates[layerIndex, i+11], "rpz")
			self.centerCoordinates[layerIndex, i] = nts.transformCoordinates((tmpVertexCoordinatesRPZ1+tmpVertexCoordinatesRPZ2)/2.0, "xyz")
			
	
	
	def computeControlCurveO10H(self, layerIndex):
		
		pointOne   = []
		pointTwo   = []
		pointThree = []
		
		cellLength = []
		
		if self.gridType == 'fS':
			domainLength = self.domainLength
			
		elif self.gridType == 'rS':
			domainLength = self.revolutionCurve[layerIndex]
			
		for vertex in (5, 5, 8, 8, 20, 20, 17, 17):
			pointOne.append(self.vertexCoordinates[layerIndex, vertex])
			
		for vertex in (4, 1, 2, 9, 21, 24, 23, 16):
			pointThree.append(self.vertexCoordinates[layerIndex, vertex])
		
		for controlPointIndex in xrange(self.noControlPoints):
			pointTwo.append(bts.computeAbsControlPoint(self.relControlPoint[layerIndex][controlPointIndex] , self.camberLine[layerIndex], domainLength, self.pitch, self.gridType, self.inletOffset[layerIndex, 0], self.outletOffset[layerIndex, 0], self.tolerance, self.maxIteration))
			
		for curveIndex in xrange(self.noControlPoints):
			cellLength.append([])
			pointArray = [pointOne[curveIndex], pointTwo[curveIndex], pointThree[curveIndex]]
			self.curve[layerIndex].append(self.computeNurbs(pointArray, 2, layerIndex))
		
		fixPointsOnCurve = [0, 1]
		noCells          = [self.noCellsCirc[0], self.noCellsAx[0], self.noCellsAx[0], self.noCellsCirc[2], self.noCellsCirc[2], self.noCellsAx[3], self.noCellsAx[3], self.noCellsCirc[0]]
		grading          = [self.circGrading[layerIndex, 1], self.axGrading[layerIndex, 1], self.axGrading[layerIndex, 2], self.circGrading[layerIndex, 12], self.circGrading[layerIndex, 14], self.axGrading[layerIndex, 15], self.axGrading[layerIndex, 14], self.circGrading[layerIndex, 3]]
		
		for curveIndex in xrange(2, self.noControlPoints+2):
			
			blockEdgeLength = nts.computePartLengthCurve(fixPointsOnCurve, self.curve[layerIndex][curveIndex], False, self.tolerance, self.maxIteration)
			
			cellLength[curveIndex-2].append(bts.computeAbsCellLength(noCells[curveIndex-2], grading[curveIndex-2], blockEdgeLength[0]))
			
			self.uOnCurve[layerIndex].append(nts.transformCurveIntoUValues(fixPointsOnCurve, cellLength[curveIndex-2], self.curve[layerIndex][curveIndex], self.tolerance, self.maxIteration))
			
		if self.gridType == 'rS':
			
			pointOne   = []
			pointTwo   = []
			
			cellLength = []
			
			for vertex in (6, 7, 19, 18):
				pointOne.append(self.vertexCoordinates[layerIndex, vertex])
				
			for vertex in (5, 8, 20, 17):
				pointTwo.append(self.vertexCoordinates[layerIndex, vertex])
				
			for curveIndex in xrange(self.noOGridCurves):
				cellLength.append([])
				pointArray = [pointOne[curveIndex], pointTwo[curveIndex]]
				self.curve[layerIndex].append(self.computeNurbs(pointArray, 1, layerIndex))
				
			fixPointsOnCurve = [0, 1]
			noCells          = [self.noCellsOGrid]
			grading          = [self.oGridGrading[layerIndex, 1], self.oGridGrading[layerIndex, 2], self.oGridGrading[layerIndex, 4], self.oGridGrading[layerIndex, 5]]
			
			for curveIndex in xrange(10, self.noOGridCurves+10):
				blockEdgeLength = nts.computePartLengthCurve(fixPointsOnCurve, self.curve[layerIndex][curveIndex], False, self.tolerance, self.maxIteration)
				cellLength[curveIndex-10].append(bts.computeAbsCellLength(noCells[0], grading[curveIndex-10], blockEdgeLength[0]))
				self.uOnCurve[layerIndex].append(nts.transformCurveIntoUValues(fixPointsOnCurve,  cellLength[curveIndex-10], self.curve[layerIndex][curveIndex], self.tolerance, self.maxIteration))
				
	def computeNurbs(self, pointArray, degree, layerIndex):
		
		curve    = NurbsCurve.NurbsCurved()
		vector   = NurbsVector.Vector_Point3Dd(1)
		vectorSR = NurbsVector.Vector_Point3Dd(1)
		points   = NurbsVector.Vector_Point3Dd(len(pointArray))
		pointsSR = NurbsVector.Vector_Point3Dd(self.noPointsOnCurve)
		uValue   = NurbsCurve.doublep()
		vValue   = NurbsCurve.doublep()
		
		nurbsCurve = ones([self.noPointsOnCurve, 3], 'd')
		
		for pointIndex in xrange(len(pointArray)):
			vector.reset(NurbsPoint.Point3Dd(pointArray[pointIndex][0], pointArray[pointIndex][1], pointArray[pointIndex][2]))
			points.as_func(pointIndex,vector)
			curve.globalInterp(points, degree)
		
		if self.gridType == 'rS':
			for pointIndex in xrange(self.noPointsOnCurve):
				u = 1.0/(self.noPointsOnCurve-1)*pointIndex
				tmpPoint = NurbsPoint.Point3Dd(curve.pointAt(u).getx(), curve.pointAt(u).gety(), curve.pointAt(u).getz())
				self.revolutionSurface[layerIndex].projectOn(tmpPoint, uValue, vValue, self.maxIteration)
				
				nurbsCurve[pointIndex, 0] = self.revolutionSurface[layerIndex].pointAt(uValue.value(), vValue.value()).getx()
				nurbsCurve[pointIndex, 1] = self.revolutionSurface[layerIndex].pointAt(uValue.value(), vValue.value()).gety()
				nurbsCurve[pointIndex, 2] = self.revolutionSurface[layerIndex].pointAt(uValue.value(), vValue.value()).getz()
				
				vectorSR.reset(NurbsPoint.Point3Dd(nurbsCurve[pointIndex, 0], nurbsCurve[pointIndex, 1], nurbsCurve[pointIndex, 2]))
				pointsSR.as_func(pointIndex,vectorSR)
				
			curve.globalInterp(pointsSR, 3)
			
		return curve
	
	
	def computePatchData(self):
		
		if self.topology == "O10H":
			for radBlockIndex in xrange(self.noLayer-1):
				self.computePatchDataO10H(radBlockIndex)
	
	
	def computePatchDataO10H(self, radBlockIndex):
		
		a = len(self.inletPatches)/(self.noLayer-1)*radBlockIndex
		b = self.vertexPerLayer*radBlockIndex
		c = self.vertexPerLayer*(radBlockIndex+1)
		
		for i in xrange(3):
			self.inletPatches [i+a] = [i+b, i+c, i+1+c, i+1+b]
			self.outletPatches[i+a] = [self.vertexPerLayer-1-i+b, self.vertexPerLayer-1-i+c, self.vertexPerLayer-2-i+c, self.vertexPerLayer-2-i+b]
			
		passageSidesLS = [0, 4, 10, 16, 22]
		passageSidesUS = [3, 9, 15, 21, 25]
		
		a = len(self.passageSidePatches)/2/(self.noLayer-1)*radBlockIndex
		
		for i in xrange (len(passageSidesLS)-1):
			self.passageSidePatches[i+a] = [passageSidesLS[i]+b, passageSidesLS[i+1]+b, passageSidesLS[i+1]+c, passageSidesLS[i]+c]
			self.passageSidePatches[i+a+(len(passageSidesLS)-1)*(self.noLayer-1)] = [passageSidesUS[i]+b, passageSidesUS[i]+c, passageSidesUS[i+1]+c, passageSidesUS[i+1]+b]
			
		self.hubPatches[ 0] = [ 0,  1,  5,  4]
		self.hubPatches[ 1] = [ 1,  2,  8,  5]
		self.hubPatches[ 2] = [ 2,  3,  9,  8]
		self.hubPatches[ 3] = [ 4,  5, 11, 10]
		self.hubPatches[ 4] = [ 5,  6, 12, 11]
		self.hubPatches[ 5] = [ 5,  8,  7,  6]
		self.hubPatches[ 6] = [ 7,  8, 14, 13]
		self.hubPatches[ 7] = [ 8,  9, 15, 14]
		self.hubPatches[ 8] = [10, 11, 17, 16]
		self.hubPatches[ 9] = [11, 12, 18, 17]
		self.hubPatches[10] = [18, 19, 20, 17]
		self.hubPatches[11] = [13, 14, 20, 19]
		self.hubPatches[12] = [14, 15, 21, 20]
		self.hubPatches[13] = [16, 17, 23, 22]
		self.hubPatches[14] = [17, 20, 24, 23]
		self.hubPatches[15] = [20, 21, 25, 24]
		
		for i in xrange(len(self.hubPatches)):
			self.shroudPatches[i, 0] = self.hubPatches[i, 0]+self.vertexPerLayer*(self.noLayer-1)
			self.shroudPatches[i, 1] = self.hubPatches[i, 3]+self.vertexPerLayer*(self.noLayer-1)
			self.shroudPatches[i, 2] = self.hubPatches[i, 2]+self.vertexPerLayer*(self.noLayer-1)
			self.shroudPatches[i, 3] = self.hubPatches[i, 1]+self.vertexPerLayer*(self.noLayer-1)
			
		blade = [6, 7, 13, 19, 18, 12, 6]
		
		a = (len(self.bladePatches))/(self.noLayer-1)*radBlockIndex
		
		for i in xrange (len(blade)-1):
			self.bladePatches[i+a] = [blade[i]+b, blade[i]+c, blade[i+1]+c, blade[i+1]+b]
	
	
	def plotO10H(self):
		
		import matplotlib.pylab as plt
		
		camberLineCoordinates      = ones([self.noPointsOnCurve, 3], 'd')
		profileCoordinates         = ones([self.noPointsOnCurve, 3], 'd')
		revolutionCurveCoordinates = ones([self.noPointsOnCurve, 3], 'd')
		scaledProfileCoordinates   = ones([self.noPointsOnCurve, 3], 'd')
		
		fig = plt.figure()
		
		if self.dimension == '2D':
			ZY  = fig.add_subplot(111)
			ZY.invert_yaxis()
			ZY.set_xlabel('Z')
			ZY.set_ylabel('Y')
			ZY.grid()
			ZY.axis('equal')
			
		elif self.dimension == '3D':
			ZY  = fig.add_subplot(221)
			ZY.invert_yaxis()
			ZY.set_xlabel('Z')
			ZY.set_ylabel('Y')
			ZY.grid()
			ZY.axis('equal')
			
			XY  = fig.add_subplot(222)
			XY.set_xlabel('Y')
			XY.set_ylabel('X')
			XY.grid()
			XY.axis('equal')
			
			XZ  = fig.add_subplot(223)
			XZ.set_xlabel('Z')
			XZ.set_ylabel('X')
			XZ.grid()
			XZ.axis('equal')	
			
		for point in xrange(self.noPointsOnCurve):
			u = point*1.0/self.noPointsOnCurve
			camberLineCoordinates[point]      = nts.getCurveCoordinates(u, self.camberLine[self.curveToPolt],    "xyz")
			profileCoordinates[point]         = nts.getCurveCoordinates(u, self.profile[self.curveToPolt],       "xyz")
			scaledProfileCoordinates[point]   = nts.getCurveCoordinates(u, self.scaledProfile[self.curveToPolt], "xyz")
			
			if self.gridType == 'fS':
				revolutionCurveCoordinates[point, 2]   = self.domainLength*u
				revolutionCurveCoordinates[point, 0:2] = 0
				
			elif self.gridType == 'rS':
				revolutionCurveCoordinates[point] = nts.getCurveCoordinates(u, self.revolutionCurve[self.curveToPolt], "xyz")
				
		ZY.plot(profileCoordinates[:, 2],         profileCoordinates[:, 1],         'k-')
		ZY.plot(scaledProfileCoordinates[:, 2],   scaledProfileCoordinates[:, 1],   'k-')
		ZY.plot(camberLineCoordinates[:, 2],      camberLineCoordinates[:, 1],      'k--')
		ZY.plot(revolutionCurveCoordinates[:, 2], revolutionCurveCoordinates[:, 1], 'k--')
		
		if self.dimension == '3D':
			XY.plot(profileCoordinates[:, 1],         profileCoordinates[:, 0],         'k-')
			XY.plot(scaledProfileCoordinates[:, 1],   scaledProfileCoordinates[:, 0],   'k-')
			XY.plot(camberLineCoordinates[:, 1],      camberLineCoordinates[:, 0],      'k--')
			XY.plot(revolutionCurveCoordinates[:, 1], revolutionCurveCoordinates[:, 0], 'k--')
			
			XZ.plot(profileCoordinates[:, 2],         profileCoordinates[:, 0],         'k-')
			XZ.plot(scaledProfileCoordinates[:, 2],   scaledProfileCoordinates[:, 0],   'k-')
			XZ.plot(camberLineCoordinates[:, 2],      camberLineCoordinates[:, 0],      'k--')
			XZ.plot(revolutionCurveCoordinates[:, 2], revolutionCurveCoordinates[:, 0], 'k--')
			
		if self.gridType == 'fS':
			domainLength = self.domainLength
			
		elif self.gridType == 'rS':
			domainLength = self.revolutionCurve[self.curveToPolt]
			
		for coordsIndex in self.coordsToPolt:
			if coordsIndex == 0:
				ZY.plot(self.vertexCoordinates[self.curveToPolt, :, 2], self.vertexCoordinates[self.curveToPolt, :, 1], 'o', color='#000000')
				
				if self.dimension == '3D':
					XY.plot(self.vertexCoordinates[self.curveToPolt, :, 1], self.vertexCoordinates[self.curveToPolt, :, 0], 'o', color='#000000')
					XZ.plot(self.vertexCoordinates[self.curveToPolt, :, 2], self.vertexCoordinates[self.curveToPolt, :, 0], 'o', color='#000000')
					
			elif coordsIndex == 1:
				
				profileCoordinatesUValue = ones([len(self.uOnCurve[self.curveToPolt][0]), 3], 'd')
				
				delrow = []
				
				for point in xrange(len(self.uOnCurve[self.curveToPolt][0])):
					
					if self.uOnCurve[self.curveToPolt][0][point] != 's':
						
						profileCoordinatesUValue[point] = nts.getCurveCoordinates(self.uOnCurve[self.curveToPolt][0][point],  self.profile[self.curveToPolt], "xyz")
					
					else: delrow.append(point)
					
				profileCoordinatesUValueShort = delete(profileCoordinatesUValue, delrow, 0)
				
				ZY.plot(profileCoordinatesUValueShort[:, 2], profileCoordinatesUValueShort[:, 1], 'o', color='#00C8FF')
				
				if self.dimension == '3D':
					
					XY.plot(profileCoordinatesUValueShort[:, 1], profileCoordinatesUValueShort[:, 0], 'o', color='#00C8FF')
					XZ.plot(profileCoordinatesUValueShort[:, 2], profileCoordinatesUValueShort[:, 0], 'o', color='#00C8FF')
					
			elif coordsIndex == 2:
				
				scaledProfileCoordinatesUValue = ones([len(self.uOnCurve[self.curveToPolt][1]), 3], 'd')
				
				delrow = []
				
				for point in xrange(len(self.uOnCurve[self.curveToPolt][1])):
					
					if self.uOnCurve[self.curveToPolt][1][point] != 's':
						
						scaledProfileCoordinatesUValue[point] = nts.getCurveCoordinates(self.uOnCurve[self.curveToPolt][1][point], self.scaledProfile[self.curveToPolt], "xyz")
						
					else: delrow.append(point)
					
				scaledProfileCoordinatesUValueShort = delete(scaledProfileCoordinatesUValue, delrow, 0)
				
				ZY.plot(scaledProfileCoordinatesUValueShort[:, 2], scaledProfileCoordinatesUValueShort[:, 1], 'o', color='#FF8C00')
				
				if self.dimension == '3D':
					
					XY.plot(scaledProfileCoordinatesUValueShort[:, 1], scaledProfileCoordinatesUValueShort[:, 0], 'o', color='#FF8C00')
					XZ.plot(scaledProfileCoordinatesUValueShort[:, 2], scaledProfileCoordinatesUValueShort[:, 0], 'o', color='#FF8C00')
					
			elif coordsIndex == 3:
				
				passageSideCoordinatesLValue = ones([len(self.lOnCurve[self.curveToPolt]), 3], 'd')
				
				delrow = []
				
				for point in xrange(len(self.lOnCurve[self.curveToPolt])):
					
					if self.lOnCurve[self.curveToPolt][point] != 's':
						
						passageSideCoordinatesLValue[point] = bts.getDomainCoordinates(self.lOnCurve[self.curveToPolt][point], self.camberLine[self.curveToPolt], domainLength, self.gridType, 'xyz', self.tolerance, self.maxIteration)
						
					else: delrow.append(point)
				passageSideCoordinatesLValueShort = delete(passageSideCoordinatesLValue, delrow, 0)
				
				ZY.plot(passageSideCoordinatesLValueShort[:, 2], passageSideCoordinatesLValueShort[:, 1], 'o', color='#B36200')
				
				if self.dimension == '3D':
					
					XY.plot(passageSideCoordinatesLValueShort[:, 1], passageSideCoordinatesLValueShort[:, 0], 'o', color='#B36200')
					XZ.plot(passageSideCoordinatesLValueShort[:, 2], passageSideCoordinatesLValueShort[:, 0], 'o', color='#B36200')
					
		plt.show()
	
	
	def writeBlockMeshDictFile(self):
		
		import writeBlockMeshDictFile
		
		write = writeBlockMeshDictFile.write()
		
		# bool
		write.cyclicGgi  = self.cyclicGgi
		write.symCascade = self.symCascade
		# int
		write.centerStart    = self.centerStart
		write.debug          = self.debug
		write.inletStart     = self.inletStart
		write.maxIteration   = self.maxIteration
		write.noBlocks       = self.noBlocksBetweenTwoLayers
		write.noBlocksInlet  = self.noBlocksInlet
		write.noBlocksCenter = self.noBlocksCenter
		write.noBlocksOutlet = self.noBlocksOutlet
		write.noLayer        = self.noLayer
		write.outletStart    = self.outletStart
		write.rowNumber      = self.rowNumber
		write.vertexPerLayer = self.vertexPerLayer
		# float
		write.convertToMeters = self.convertToMeters
		write.domainLength    = self.domainLength
		write.pitch           = self.pitch
		write.tolerance       = self.tolerance
		write.version         = self.version
		# string
		write.dimension = self.dimension
		write.fileName  = self.fileName
		write.path      = self.path
		write.zoneName  = self.zoneName
		write.gridType   = self.gridType
		# array
		write.bladePatches       = self.bladePatches
		write.blockCoordinates   = self.blockCoordinates
		write.blockGrading       = self.blockGrading
		write.cellsPerBlock      = self.cellsPerBlock
		write.centerCoordinates  = self.centerCoordinates
		write.curve              = self.curve
		write.hubPatches         = self.hubPatches
		write.inletCoordinates   = self.inletCoordinates
		write.inletOffset        = self.inletOffset
		write.inletPatches       = self.inletPatches
		write.camberLine         = self.camberLine
		write.lOnCurve           = self.lOnCurve
		write.outletCoordinates  = self.outletCoordinates
		write.outletOffset       = self.outletOffset
		write.outletPatches      = self.outletPatches
		write.passageSide        = self.passageSide
		write.passageSidePatches = self.passageSidePatches
		write.revolutionCurve    = self.revolutionCurve
		write.shroudPatches      = self.shroudPatches
		write.smeVertex          = self.smeVertex
		write.uOnCurve           = self.uOnCurve
		write.vertexCoordinates  = self.vertexCoordinates
		
		write.blockMeshDictFile()
	
