#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2009-2010 by Oliver Borm                                *
#*   oli.borm@web.de                                                       *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

# Author: Oliver Borm
# Date: July 2009

import numpy
import scipy.optimize.minpack
from pythonnurbs import NurbsPoint, NurbsHPoint, NurbsCurve, NurbsSurface, NurbsVector, NurbsColor

## @class camberSurface
# @brief Class for the computation of camber curves and surfaces
#
#
#	Ein verwölbtes Profil wird im Allgemeinen dadurch erzeugt, dass eine Dickenverteilung \f$ f(x) \f$ eines symmetrischen Tropfens über eine Wölbungsverteilung \f$ g(x) \f$ überlagert wird. Dazu ordnet man einander korrespondierende Punkte derselben  <a href="http://de.wikipedia.org/wiki/Abszisse">Abszisse</a> von Skelettlinie und Dickenverteilung zu. Dabei wird die Tropfendicke \f$ f(x) \f$ vom gewählten Skelettlinienpunkt aus auf beiden Seiten abgetragen. Analytisch gelten also nach (Jacobs et al. und Bausch) für die Koordinaten der Oberseite \f$ \vec{p}_o \f$ und Unterseite \f$ \vec{p}_u \f$ die Beziehungen
#
#	@anchor gl_1
#	@anchor gl_2
#	@f{eqnarray}{
#	\label{gl_1}
#	\vec{p}_{o,u} = \left( \begin{array}{l}
#	x_{o,u} \\ y_{o,u}
#	\end{array} \right)
#	& = & \left( \begin{array}{l}
#	x \mp f(x) \frac{g^{\prime} (x)}{\sqrt{1+g^{\prime 2} (x)}} \\
#	g(x) \pm f(x) \frac{1}{\sqrt{1+g^{\prime 2} (x)}}
#	\end{array} \right)_{x,y} \\
#	\label{gl_2}
#	& = & \left( \begin{array}{l}
#	x \mp f(x) \sin \theta \\
#	g(x) \pm f(x) \cos \theta
#	\end{array} \right)_{x,y}
#	@f}
#
#	@image html Profil_Naca-460.png "Profildefinition"
#
#	Der Steigungswinkel der Skelettlinie ist dabei folgendermaßen definiert:
#
#	@anchor gl_3
#	@f{equation}{
#	\label{gl_3}
#	\tan \theta = \frac{d\: g(x)}{d\: x} = g^{\prime} (x) = \vec{\mathcal{T}}_{sk}
#	@f}
#
#	<a href="http://en.wikipedia.org/wiki/List_of_trigonometric_identities#Related_identities">Folgende Zusammenhänge</a> gelten dann für andere trigonometrische Funktionen:
#
#	@anchor gl_4
#	@anchor gl_5
#	@f{eqnarray}{
#	\label{gl_4}
#	\sin \theta & = & \frac{g^{\prime}}{ \sqrt{ 1 + g^{\prime 2} } }\\
#	\label{gl_5}
#	\cos \theta & = & \frac{1}{ \sqrt{ 1 + g^{\prime 2} } }
#	@f}
#
#	@svg{../Bilder/Profile_Ausschnitt}
#
#	Im Folgenden werden die Bedingungen dargestellt, damit man eine Skelettlinie rückwärts aus den Profilkoordinaten bestimmen kann.
#	Wie Vinnemeier bereits erläuterte, ergeben sich die Tangenten an die Profiloberseite und Profilunterseite durch Differentiation von @glref{gl_2}
#
#	@anchor gl_7
#	@anchor gl_8
#	@f{eqnarray}{
#	\label{gl_7}
#	\frac{\mathrm{d}\: y_o}{\mathrm{d}\: x} = \vec{\mathcal{T}}_{o} & = & \frac{\mathrm{d}\: g(x)}{\mathrm{d}\: x} + \frac{\mathrm{d}\: f(x)}{\mathrm{d}\: x} \cos \theta + f(x) (- \sin \theta) \frac{\mathrm{d}\: \theta}{\mathrm{d}\: x} \\ 
#	\label{gl_8}
#	\frac{\mathrm{d}\: y_u}{\mathrm{d}\: x} = \vec{\mathcal{T}}_{u} & = & \frac{\mathrm{d}\: g(x)}{\mathrm{d}\: x} - \frac{\mathrm{d}\: f(x)}{\mathrm{d}\: x} \cos \theta - f(x) (- \sin \theta) \frac{\mathrm{d}\: \theta}{\mathrm{d}\: x}
#	@f}
#
#	Durch Aufsummation von @glref{gl_7} und @glref{gl_8} erhält man dann einen allgemeinen Zusammenhang zwischen Kontur- und Skelettliniensteigung:
#
#	@anchor gl_9
#	@f{equation}{
#	\label{gl_9}
#	\frac{\mathrm{d}\: g(x)}{\mathrm{d}\: x} = \vec{\mathcal{T}}_{sk} = \frac{1}{2} \left( \vec{\mathcal{T}}_{o} + \vec{\mathcal{T}}_{u} \right)
#	@f}
#
#	@svg{../Bilder/Profile_gesamt}
#
#	Wird das Profil durch eine ,,geschlossene'' parametrische Kurve beschrieben, bei der der Parameter von der Hinterkante mit \f$ v=0 \f$ startet über die Vorderkante wieder bis zur Hinterkante mit \f$ v=1 \f$ endet, dann ergibt sich die Tangente der Skelettlinie aus den Ableitungen des Profils entlang dem Parameter wie folgt:
#
#	@anchor gl_10
#	@f{equation}{
#	\label{gl_10}
#	\vec{\mathcal{T}}_{sk} = \frac{1}{2} \left( \frac{\mathrm{d}\: \vec{p}_{o} }{\mathrm{d}\: v} - \frac{\mathrm{d}\: \vec{p}_{u}}{\mathrm{d}\: v} \right) = \frac{1}{2} \left( \vec{\mathcal{T}}_{o}^v - \vec{\mathcal{T}}_{u}^v \right)
#	@f}
#
#	Die Verbindungsgerade zwischen den Punkten \f$ \vec{p}_u \f$ und \f$ \vec{p}_o \f$ ist genau die Differenz der Koordinaten der Punkte von Ober- und Unterseite des Profils und schreibt sich somit als:
#
#	@anchor gl_6
#	@f{equation}{
#	\label{gl_6}
#	\Delta \vec{p}_{ou} = \vec{p}_o - \vec{p}_u
#	@f}
#
#	Die Verbindungsgerade zwischen den Punkten \f$ \vec{p}_u \f$ und \f$ \vec{p}_o \f$ steht per Definition normal zur Skelettlinie und damit auch normal zur Tangente der Skelettlinie \f$ \vec{\mathcal{T}}_{sk} \f$. Somit muss sich das Skalarprodukt aus den Randbedingungen in @glref{gl_10} und @glref{gl_6} wie folgt zu Null ergeben:
#
#	@anchor gl_11
#	@f{eqnarray}{
#	\label{gl_11}
#	H(v) =  \left( \vec{p}_o - \vec{p}_u \right) \bullet \frac{1}{2} \left( \vec{\mathcal{T}}_{o}^v - \vec{\mathcal{T}}_{u}^v \right) & = & \nonumber \\
#	\left( \vec{p} (v_o) - \vec{p} (v_u) \right) \bullet \frac{1}{2} \left( \frac{\mathrm{d}\: \vec{p} (v_{o}) }{\mathrm{d}\: v} - \frac{\mathrm{d}\: \vec{p} (v_{u}) }{\mathrm{d}\: v} \right) & \stackrel{!}{=} & 0
#	@f}
#
#	Bei einem festen Parameter für den Punkt auf der Profilunterseite \f$ v_u \f$ wird der Parameter des Punktes der Profiloberseite \f$ v_o \f$ von einem geeigeneten Startwert so lange mit einem Bisektionsverfahren iteriert bis @glref{gl_11} erfüllt ist.
#
#	<b>Literatur:</b>
#	@li <a href="http://ntrs.nasa.gov">Eastman N. Jacobs; Kenneth E. Ward, Robert M. Pinkerton; NACA Report 460: The Characteristics Of 78 Related Airfoil Sections From Tests In The Variable-Density Wind Tunnel, 1933</a>
#	@li <a href="http://dispatch.opac.ddb.de/DB=1.1/SET=2/TTL=6/CMD?ACT=SRCHA&IKT=8509&SRT=LST_ty&TRM=Zentrale+f%FCr+Wissenschaftliches+Berichtswesen+der+Luftfahrtforschung">Bausch, K.: Strenges Verfahren der Skelettlinienbestimmung für Profile unbekannter Verwölbung; Technische Berichte / Zentrale für Wissenschaftliches Berichtswesen der Luftfahrtforschung (ZWB). - Berlin-Adlershof, Bd. 11 (1944), S. 11-16</a> <a href="http://hdl.handle.net/2060/19760015069">(englische Übersetzung der NASA)</a>
#	@li Vinnemeier, Franz Maria; Analyse der Strömung in einem mehrstufigen Axialverdichter, 1991, S.35f
#
#

#	<b>Inhalt der Funktion:</b><br>
#	@li <a href="http://books.google.de/books?id=pHo53mTXvHAC&lpg=PP1&dq=Geometrie%20Nitschke&pg=PA132#v=onepage&q=&f=false">Nitschke, Martin; Geometrie: Anwendungsbezogene Grundlagen und Beispiele, 2005, S.132ff</a>
#
class camberSurface(object):
	## @brief Constructor
	def __init__(self,*args):
		## number of profiles
		self.numberProfiles = 1
		## number of points of each closed profile
		self.numberPoints = 4
		## closed nurbs curves of profiles
		self.nurbsProfile = []
		## closed pointlist of profiles
		self.profileCoordinates = []
		## camberlines as nurbscurves
		self.nurbsCamber = []
		## camber surface as NURBS surface
		self.camberSurface = NurbsSurface.NurbsSurfaced()
		## coordinates of camber points
		self.camberCoordinates = []
		## parametric U values for closed profile
		self.profileUValues = []
		## parametric U values for upper profile side
		self.profileULowerSide = []
		## parametric U values for lower profile side
		self.profileUUpperSide = []
		## discrete points of upper profile
		self.pointsUpperSide = []
		## discrete points of lower profile
		self.pointsLowerSide = []
		## discrete points of the thickness distribution
		self.thicknessCoordinates = []
		## discrete points of the camber function
		self.camberFunction = []
		## chord length of each profile
		self.chordLength = []
		## maximum thickness and point of maximum thickness
		self.maxThickness = []
		## maximum camber and point of maximum camber
		self.maxCamber = []
		## approximated leading edge radius
		self.leRadius = []
		
		
		## 
		self.minimumLength = 1
		## additional distance for start point uTemp
		self.uTempAdd = 0.01
		## global tolerance
		self.tolerance = 1e-9 ## Vorrausgesetzt die Daten sind alle in mm!; bei 1e-15 läufts über die VK hinaus
		## maximum iterations
		self.maxIteration = 100
		## debug mode
		self.debug = 0
		
		self.path = ''
		
		self.rowIndex = 0
		
		#self.degree = 3
		
		self.maxError = 1e-6 ##0.000001
	
	## @brief Computes the camber profiles
	# @param self
	# @return void
	def computeCamberProfiles(self):
		self.Update()
		self.calculateProfileUValues()

		self.computeUValues()
		self.checkUValues()
		
		self.computeLeadingEdge()
		
		self.computePoints()
	
	## @brief Updates some values
	# @param self
	# @return void
	def Update(self):
		self.numberPoints = len(self.profileCoordinates[0])
		self.profileUValues = numpy.zeros((self.numberProfiles,self.numberPoints))
		self.profileULowerSide = range(self.numberProfiles)
		self.profileUUpperSide = range(self.numberProfiles)
		
		self.pointsUpperSide = range(self.numberProfiles)
		self.pointsLowerSide = range(self.numberProfiles)
		self.camberCoordinates = range(self.numberProfiles)
		self.thicknessCoordinates = range(self.numberProfiles)
		self.camberFunction = range(self.numberProfiles)
		self.chordLength = range(self.numberProfiles)
		self.maxThickness = range(self.numberProfiles)
		self.maxCamber = range(self.numberProfiles)
		self.leRadius = range(self.numberProfiles)
	
	## @brief computes the length of the curve between two parametric values
	# @param u1 the U-value at start of the length
	# @param u2 the U-value at end of the length
	# @param the length the curve should be
	# @param curve the NURBS - curve
	# @return the difference between the given length and the length of the curve between the values u1 and u2
	def minLength(self, u1, u2, length, curve):
		
		lengthNow = curve.lengthIn(u1, u2, self.tolerance, self.maxIteration)
		
		return lengthNow-length
	
	## @brief computes parametric U value from length and an ending point
	# @param u2 the U-value at end of the length
	# @param the length the curve should be
	# @param curve the NURBS - curve
	# @return the uVale of the curve where the given length between this uValue and the given end U-Value exists
	def calculateUStart(self,u2,length,curve):
		
		uValue = scipy.optimize.minpack.bisection(self.minLength, 0.0, 1.0, args=(u2, length, curve), xtol = self.tolerance, maxiter = self.maxIteration)
		
		return uValue
	
	## @brief computes parametric U value from length and an ending point
	# @param point the given input NURBS point
	# @param curve the NURBS - curve
	# @param uGuess starting U-value
	# @param uMin minimal parametric search values
	# @param uMax maximal parametric search values
	# @return the uValue of the curve which is closest to the given point
	def calculateUFromPoint(self, point, curve, guessU, uMin, uMax):
		
		uValue = NurbsCurve.doublep()
		#uValue.assign(guessU)
		curve.minDist2(point, uValue, self.tolerance, 0.5, 100, self.maxIteration, uMin, uMax)
		
		return uValue.value()
	
	## @brief return the coordinates from a given point as numpy array
	# @param point the given input NURBS point
	# @return the coordinates from a given point as numpy array
	def getPoint(self,point):
		return numpy.array([point.getx(),point.gety(),point.getz()])
	
	## @brief returns a NurbsPoint3D from given coordinates in an array or list
	# @param point the given input point as array or list
	# @return NURBS 3D point of given coordinates
	def setPoint(self,point):
		return NurbsPoint.Point3Dd(point[0], point[1], point[2])
	
	## @brief computes the magnitide of NurbsPoint Coordinates
	# @param point the given input NURBS point
	# @return the magnitide of NurbsPoint Coordinates
	def getMagnitude(self,point):
		return self.norm(self.getPoint(point))
	
	## @brief computes the distance from two given coordinates as array or list
	# @param point the given input point as array or list
	# @return the distance from two given coordinates as array or list
	def norm(self,point):
		return numpy.sqrt(numpy.square(point).sum())
	
	## @brief Caclulates the scalar product of 2 point vectors
	# @param self self pointer
	# @param P0 First nurbspoint
	# @param P1 Second nurbspoint
	# @return NoVariable Returns the scalar product of the point vectors
	def dotS(self, P0, P1):
		return P0.getx()*P1.getx() + P0.gety()*P1.gety() + P0.getz()*P1.getz()
	
	## @brief Calculates the angle between 2 point vectors
	# @param self self pointer
	# @param P0 First nurbspoint
	# @param P1 Second nurbspoint
	# @return NoVariable Returns the angle between the point vectors
	def angle(self, P0, P1):
		
		if P0.norm()*P1.norm() == 0:
			return 0.0
		else:
			return numpy.arccos(self.dotS(P0, P1) / (P0.norm()*P1.norm()))
	
	## @brief computes the distances from discrete points and corresponding points at the nurbs curve
	# @param self
	# @return void
	def getDistances(self):
		for i in xrange(self.numberProfiles):
			for j in xrange(len(self.profileCoordinates[i])):
				
				pointA = self.getPoint(self.nurbsProfile[i].pointAt(self.profileUValues[i][j]))
				pointB = self.profileCoordinates[i][j]
				
				print str(self.norm(pointA-pointB)) + " " + str(self.profileUValues[i][j])
	
	## @brief computes the first derivation of a given NURBS curve at one parametric value
	# @param u parametric value at which the derivation should take place
	# @param curve the NURBS - curve
	# @return the normalized first derivation of the curve at value u
	def calculateDerivative(self, u, curve):
		
		derivationVector = NurbsVector.Vector_Point3Dd()
		curve.deriveAt(u,1,derivationVector)
		
		return self.getPoint(derivationVector[1])/self.getMagnitude(derivationVector[1])
	
	## @brief computes the scalar product of the distance of the points, which are described by two parametric values (uStar, u), on a nurbs curve with distance of the first derivations at this points on the curve.
	# @param uStar parametric value of the searched point
	# @param u parametric value of the given point
	# @param curve the NURBS - curve
	# @return the scalar product of the distance of the points on a nurbs curve, with distance of the first derivations at this points on the curve.
	def getNormalDistance(self, uStar, u, curve):
		
		xPStar = self.getPoint(curve.pointAt(uStar))
		xP = self.getPoint(curve.pointAt(u))
		
		return numpy.dot((xPStar - xP),(self.calculateDerivative(uStar,curve) - self.calculateDerivative(u,curve)))
	
	def getNormalDistanceNegative(self, uStar, u, curve):
		
		xPStar = self.getPoint(curve.pointAt(uStar))
		xP = self.getPoint(curve.pointAt(u))
		
		return numpy.dot((xPStar - xP),(self.calculateDerivative(uStar,curve) + self.calculateDerivative(u,curve)))
	
	## @brief computes the parametric values of discrete points at the nurbs profile
	# @param self
	# @return void
	def calculateProfileUValues(self):
		
		for i in xrange(self.numberProfiles):
			
			self.profileUValues[i][0] = 0.0
			#self.calculateUFromPoint(self.setPoint(self.profileCoordinates[i][0]),self.nurbsProfile[i],0.1,0.0,1.0)
			
			for j in xrange(1,self.numberPoints):
				uAdd = self.uTempAdd*(self.profileUValues[i][j-1]+1.0)
				uStart = self.profileUValues[i][j-1]
				uStartU = max(self.profileUValues[i][j-1]-uAdd,0.0)
				
				self.profileUValues[i][j] = self.calculateUFromPoint(self.setPoint(self.profileCoordinates[i][j]),self.nurbsProfile[i],uStart,uStartU,1.0)
			
			if self.debug > 3:
				
				for j in xrange(self.numberPoints):
					print "uValue " + str(self.profileUValues[i][j])
	
	## @brief checks the U-values for monotony
	# @param self
	# @return void
	def checkUValues(self):
		
		pointLength = range(self.numberProfiles)
		uMonotony = range(self.numberProfiles)
		
		for i in xrange(self.numberProfiles):
			
			pointLength[i] = len(self.profileULowerSide[i])
	
			for j in xrange(1,self.numberPoints):
				
				if self.profileUValues[i][j-1] >= self.profileUValues[i][j]:
					if self.debug > 2: print str(i) + " " + str(j) + " " + str(self.profileUValues[i][j])
					uMonotony[i] = j
					break
			
			if pointLength[i] > uMonotony[i]:
				print "WARNING: pointLength[" + str(i) + "] > uMonotony["+str(i)+"]"
				print str(pointLength[i]) + " " + str(uMonotony[i])
		
		self.minimumLength = min(pointLength)
		
	
	## @brief calculates an U-coordinate at a given curve with the bisection method and self.getNormalDistance
	# @param uStarOld parametric value of the latest searched point
	# @param u parametric value of the given point
	# @param curve the NURBS - curve
	# @return uTemp parametric value of the searched point
	def calculateUValueBisection(self,uStarOld,uValue,curve):
		
		#bisection(self.getNormalDistance, uStart, uValue+delta, args=(uValue, self.nurbsProfile[i]), xtol = self.tolerance, maxiter = self.maxIteration)
		uPos = uValue # + 0.5*(uStarOld-uValue)
		#uNeg = max(uStarOld+0.01*abs(uValue+uStarOld),1.0)
		
		uNeg = uStarOld
		
		#uTemp = uStarOld - 0.01*(uNeg+uPos) ## löst bis 182 sehr gut auf!
		#uTemp = uStarOld - 0.001*(uNeg+uPos) ## löst bis 207 gut auf (Überschwinger)!
		#uTemp = uStarOld - 0.005*(uNeg+uPos) ## löst bis 188 gut auf (Unterschwinger)!
		#uTemp = uStarOld - 0.00125*(uNeg+uPos) ## löst bis 208 auf (Überschwinger)!
		
		uTemp = uStarOld - self.uTempAdd*(uNeg+uPos)
		
		start1 = self.getNormalDistance(uPos+0.001, uValue, curve)
		start2 = self.getNormalDistance(uNeg, uValue, curve)
		
		#if self.debug > 2:
			#print str(uPos)+" "+str(uNeg)+" "+str(uTemp)
			#print str(start1)+" "+str(start2)
		
		negative = False
		
		#if start2 < 0.0:
			#print "negative"
			#negative = True
		
		if start1*start2 >= 0.0:
			if self.debug > 1: print "start1*start2 >= 0.0: " + str(start1)+ " " +str(start2)

			if start2 < 0.0:
				if self.debug > 2: print "negative"
				negative = True
				
			if abs(start1) <= abs(start2):
				start1 = abs(start2)*cmp(-start2,0.0)
				#if self.debug > 2: print "Start1: " + str(start1)
				#uNeg = max(uStarOld+0.1*abs(uValue+uStarOld),1.0)
			else:
				start2 = abs(start1)*cmp(-start1,0.0)
				#if self.debug > 2: print "Start2: " + str(start2)
		
		for n in range(self.maxIteration):
			
			#if negative:
				#cdist = self.getNormalDistanceNegative(uTemp, uValue, curve)
			#else:
			cdist = self.getNormalDistance(uTemp, uValue, curve)
			
			if self.debug > 3:
				print str(uPos)+" "+str(uNeg)+" "+str(uTemp)
				print str(cdist)
			
			if abs(cdist)<=self.tolerance:
				if self.debug > 2:
					print "cdist: " + str(cdist)
					print "uTemp: " + str(uTemp)
				return uTemp
			
			#if negative:
				#if (cdist)>0:
					#uPos = uTemp
					#if self.debug > 3: print "uPos: " + str(uPos)
				
				#else:
					#uNeg = uTemp
					#if self.debug > 3: print "uNeg: " + str(uNeg)
			
			#else:
			if (cdist)<0:
				uPos = uTemp
				if self.debug > 3: print "uPos: " + str(uPos)
			
			else:
				uNeg = uTemp
				if self.debug > 3: print "uNeg: " + str(uNeg)
			
			uTemp = 0.5*(uPos+uNeg)
			
		if self.debug > 1: print "maxIterations: "+str(uTemp)

		return uTemp
		
	
	## @brief computes the corresponding U values for upper and lower side with self.calculateUValueBisection
	# @param self
	# @return void
	def computeUValues(self):
		
		for i in xrange(self.numberProfiles):
			
			self.profileUUpperSide[i] = [0.0]
			self.profileULowerSide[i] = [1.0]
			
			#self.profileUValues[i][0]
			
			for j in xrange(1,len(self.profileUValues[i])):
			#for j in xrange(200):
			
				if self.debug > 1: print "j: "+str(j)
				
				#uStart = self.profileUUpperSide[i][-1]
				#uEnd = self.profileULowerSide[i][-1]
				
				#uStartU = uEnd-0.1*(uEnd+uStart)
				
				#point3Dd = NurbsPoint.Point3Dd(self.profileCoordinates[i][j][0], self.profileCoordinates[i][j][1], self.profileCoordinates[i][j][2])
				
				#self.getPoint(self.profileCoordinates[i][j])
				
				#uValue = self.calculateUFromPoint(self.setPoint(self.profileCoordinates[i][j]),self.nurbsProfile[i],uStartU,uStart,1.0)
				
				#uValue = self.calculateUFromPoint(self.setPoint(self.profileCoordinates[i][j]),self.nurbsProfile[i],1.0,uStart,1.0)
				
				#print str(uStart) + " " + str(uEnd) + " " + str(uStartU) + " " + str(uValue)
				
				#print str(self.profileUUpperSide[i][-1]) + " " + str(uValue) + " " + str(self.profileCoordinates[i][j])
				
				if abs(self.profileUUpperSide[i][-1]-self.profileUValues[i][j]) > self.tolerance:
					
					#delta = 1e-7
					
					#start1 = self.getNormalDistance(uValue+delta, uValue, self.nurbsProfile[i])
					
					#start2 = self.getNormalDistance(uStart, uValue, self.nurbsProfile[i])
					
					#if (start1*start2) == 0.0:
						#print "Info: start1*start2 >= 0"
						#print str(start1) + " "+ str(start2)
						#test = True
						#break # Springt aus der ganzen Methode raus
					#else:
						#uValueStar = scipy.optimize.minpack.bisection(self.getNormalDistance, uStart, uValue+delta, args=(uValue, self.nurbsProfile[i]), xtol = self.tolerance, maxiter = self.maxIteration)
						
						#if self.verbose > 2: print str(uValue) + " " + str(uValueStar)
						
						#self.profileULowerSide[i].append(uValueStar)
						#self.profileUUpperSide[i].append(uValue)
					
					
					uValueStar = self.calculateUValueBisection(self.profileULowerSide[i][-1],self.profileUValues[i][j],self.nurbsProfile[i])
					
					if uValueStar < self.profileUValues[i][j]:
						if self.debug > 1: print "uValueStar < uValue: "+str(uValueStar)+" "+str(self.profileUValues[i][j])
						break
					
					self.profileULowerSide[i].append(uValueStar)
					self.profileUUpperSide[i].append(self.profileUValues[i][j])
			
			#print self.profileULowerSide[i]
			#print self.profileUUpperSide[i]
	
	## @brief computes the discrete points from the U values list on the profile curve
	# @param self
	# @return void
	def computePoints(self):
		
		for i in xrange(self.numberProfiles):
			
			#print str(len(self.profileUUpperSide[i]))
			#print str(len(self.profileULowerSide[i]))
			
			self.pointsUpperSide[i] = numpy.zeros((self.minimumLength,3))
			self.pointsLowerSide[i] = numpy.zeros((self.minimumLength,3))
			self.camberCoordinates[i] = numpy.zeros((self.minimumLength,3))
			self.thicknessCoordinates[i] = numpy.zeros((self.minimumLength,2))
			
			for j in xrange(self.minimumLength):
				self.pointsUpperSide[i][j] = self.getPoint(self.nurbsProfile[i].pointAt(self.profileUUpperSide[i][j]))
				
				self.pointsLowerSide[i][j] = self.getPoint(self.nurbsProfile[i].pointAt(self.profileULowerSide[i][j]))
				
				self.camberCoordinates[i][j] = 0.5*(self.pointsLowerSide[i][j] + self.pointsUpperSide[i][j])
			
			self.chordLength[i] = self.norm(self.camberCoordinates[i][-1] - self.camberCoordinates[i][0])
			
			for j in xrange(self.minimumLength):
				self.thicknessCoordinates[i][j][1] = self.norm(self.pointsLowerSide[i][self.minimumLength-j-1] - self.pointsUpperSide[i][self.minimumLength-j-1])*0.5/self.chordLength[i]
		
		self.computeCamber()
	
	## @brief computes the camber (Wölbung) of the points
	# @param self
	# @return void
	def computeCamber(self):
		
		for i in xrange(self.numberProfiles):
			
			self.camberFunction[i] = numpy.zeros((self.minimumLength,2))
			
			newCoordinates = numpy.zeros((self.minimumLength,3))
			
			## translate all points in such a way that the first point is in the center (0,0,0)
			for j in xrange(self.minimumLength):
				newCoordinates[j] = self.camberCoordinates[i][self.minimumLength-j-1] - self.camberCoordinates[i][-1]
			
			### rotate the new coordinates about the z-axis in such a way, that the last point is in the z-y-plane
			alpha = - numpy.arctan2(newCoordinates[-1][0],newCoordinates[-1][1])
			
			#print newCoordinates[0], newCoordinates[-1], alpha
			
			for j in xrange(self.minimumLength):
				x1 = newCoordinates[j][0]
				y1 = newCoordinates[j][1]
				
				newCoordinates[j][0] =  x1*numpy.cos(alpha)+y1*numpy.sin(alpha)
				newCoordinates[j][1] = -x1*numpy.sin(alpha)+y1*numpy.cos(alpha)
			
			### rotate the new coordinates about the x-axis in such a way, that the last point is in the x-z-plane
			beta = - numpy.arctan2(newCoordinates[-1][1],newCoordinates[-1][2])
			
			#print newCoordinates[0], newCoordinates[-1], beta
			
			for j in xrange(self.minimumLength):
				y1 = newCoordinates[j][1]
				z1 = newCoordinates[j][2]
				
				newCoordinates[j][1] =  y1*numpy.cos(beta)+z1*numpy.sin(beta)
				newCoordinates[j][2] = -y1*numpy.sin(beta)+z1*numpy.cos(beta)
				
				self.camberFunction[i][j][1] = numpy.hypot(newCoordinates[j][0],newCoordinates[j][1])/self.chordLength[i]
				
				self.camberFunction[i][j][0] = newCoordinates[j][2]/self.chordLength[i]
				self.thicknessCoordinates[i][j][0] = newCoordinates[j][2]/self.chordLength[i]
			
			## computing some maximum values for each profile
			self.maxThickness[i] = [max(self.thicknessCoordinates[i][:,1]), self.thicknessCoordinates[i][numpy.argmax(self.thicknessCoordinates[i][:,1])][0] ]
			self.maxCamber[i] = [max(self.camberFunction[i][:,1]),self.camberFunction[i][numpy.argmax(self.camberFunction[i][:,1])][0] ]
			self.leRadius[i] = self.norm(self.camberCoordinates[i][-2] - self.camberCoordinates[i][-1])
	
	## @brief Computes the U values for the leading edge
	# @param self
	# @return void
	def computeLeadingEdge(self):
		
		
		#for i in xrange(self.numberProfiles):
			
			#for j in range(self.minimumLength,len(self.profileULowerSide[i])):
				#print str(i) + " " + str(j) + str(len(self.profileULowerSide[i]))
				#self.profileULowerSide[i].remove(j)
				#self.profileUUpperSide[i].remove(j)
			
		
		for i in xrange(self.numberProfiles):
			
			#self.averageULeadingEdge = 0.5*(self.profileULowerSide[i][-2]+self.profileUUpperSide[i][-2])
			#print str(self.averageULeadingEdge)
			
			#lenghtLeadingEdge = self.nurbsProfile[i].lengthIn(self.profileUUpperSide[i][-2],self.profileULowerSide[i][-2], self.tolerance, self.maxIteration)
			
			#lengthAverageUValue = self.calculateUStart(self.profileULowerSide[i][-2],lenghtLeadingEdge*0.5,self.nurbsProfile[i])
			
			lenghtLeadingEdge = self.nurbsProfile[i].lengthIn(self.profileUUpperSide[i][self.minimumLength-2],self.profileULowerSide[i][self.minimumLength-2], self.tolerance, self.maxIteration)
			
			lengthAverageUValue = self.calculateUStart(self.profileULowerSide[i][self.minimumLength-2],lenghtLeadingEdge*0.5,self.nurbsProfile[i])
			
			self.profileUUpperSide[i][self.minimumLength-1] = lengthAverageUValue
			self.profileULowerSide[i][self.minimumLength-1] = lengthAverageUValue
			#print str(lengthAverageUValue)
			
	
	## @brief Creates a nurbs surface from a nurbs point matrix
	# @param self self pointer
	# @param camberPointsMatrix Nurbs Matrix object
	# @param deg Degree of the nurbs surface
	def CamberSurface3D(self, camberPointsMatrix, deg):# workdirInput):
		#self.degree = deg
		self.camberSurface.globalInterp(camberPointsMatrix, deg, deg)
		#self.createCamberSurface3D(camberPointsMatrix)
		#self.workdir = workdirInput
		if self.debug > 0:
			self.writeVRML()
			
	### @function createCamberSurface3D
	#def createCamberSurface3D(self, camberPointsMatrix):
		
		#self.camberSurface.globalInterp(camberPointsMatrix, self.degree, self.degree)
	
	## @brief Calculates the betaR and betaZ angles
	# @param self self pointer
	# @param uSF U-value on the camber surface for which the calculation is applied
	# @param vSF V-value on the camber surface for which the calculation is applied
	# @return betaR betaR
	# @return betaZ betaZ
	def calculate_betaR_betaZ(self, uSF, vSF):
		
		normalVec = NurbsPoint.Point3Dd()
		unitVecZ = NurbsPoint.Point3Dd()
		unitVecR = NurbsPoint.Point3Dd()
		normalVec = self.camberSurface.normal(uSF, vSF)
		
		unitVecZ = NurbsPoint.Point3Dd(0.0, 0.0, 1.0)
		unitVecR = NurbsPoint.Point3Dd(self.intersection.getx(), self.intersection.gety(), 0.0)
		
		beta0 = self.angle(unitVecR, normalVec)
		beta1 = self.angle(unitVecZ, normalVec)
		
		return beta0, beta1
	
	## @brief Calculates the u- and v-values on the camber surface at the given radius and axial coordinate
	# @param self self pointer
	# @param radius Radial coordinate
	# @param axialPosition Axial coordinate
	# @return uSF.value() u-value of the point on the surface at the given radius and axial position
	# @return vSF.value() v-value of the point on the surface at the given radius and axial position
	def calculate_intersection(self, radius, axialPosition):
		
		uSF = NurbsCurve.doublep()
		uSF.assign(0.5)
		vSF = NurbsCurve.doublep()
		vSF.assign(0.5)
		phi = 0.0
		n = 0
		error = 1.0
		newIntersection = NurbsPoint.Point3Dd()
		
		#print "intersection:",n, self.camberSurface.pointAt(uSF.value(), vSF.value()).getx(), self.camberSurface.pointAt(uSF.value(), vSF.value()).gety(), self.camberSurface.pointAt(uSF.value(), vSF.value()).getz()
		
		while (n < self.maxIteration) and (error > self.maxError):
			#self.intersection = NurbsPoint.Point3Dd(radius* numpy.sin(phi), radius* numpy.cos(phi), axialPosition)
			self.intersection = NurbsPoint.Point3Dd(radius*numpy.cos(phi), radius*numpy.sin(phi), axialPosition)
			error = self.camberSurface.minDist2(self.intersection, uSF, vSF, self.maxError)
			
			newIntersection = self.camberSurface.pointAt(uSF.value(), vSF.value())
			phi = numpy.arctan2(newIntersection.gety(), newIntersection.getx())
			#phi =  numpy.atan(newIntersection.getx()/newIntersection.gety())
			n = n + 1
			
		#print "\n", radius, axialPosition
		#print "intersection:",n, self.camberSurface.pointAt(uSF.value(), vSF.value()).getx(), self.camberSurface.pointAt(uSF.value(), vSF.value()).gety(), self.camberSurface.pointAt(uSF.value(), vSF.value()).getz()
		self.intersection = newIntersection
		
		return uSF.value(), vSF.value()
	
	## @brief Saves the 3D NURBS camber surface in a .wrl file (plotable with e.g. paraview)
	# @param self self pointer
	# @param filename Name of the output file
	def writeVRML(self,filename='camber',ipu=250,ipv=250):
		
		filename = filename+str(self.rowIndex)+'.wrl'
		blue=NurbsColor.Color(0, 255, 0)
		self.camberSurface.writeVRML97(self.path+filename,blue,ipu,ipv,1,0,1,0)
		print "*** Written to" , path+filename
	
	## @brief Plots camber and profile curves and discrete points
	# @param curve the given NURBS - curve
	# @param camberCurve the given (controlling) NURBS - curve of the camber curve
	# @param camberPoints the given (controlling) points of the camber curve
	# @return void
	#def diffNurbsDiscretesCascade(self,curve,camberCurve,camberPoints):
	def diffNurbsDiscretesCascade(self,curve,**args):
		import pylab as p
		
		disk = 1000.0
		
		#### Plot zum Vergleich von Nurbskurve und diskreten Punkten
		arrr=numpy.zeros((disk,2))
		for i in xrange(int(disk)):
			arrr[i,0]=curve.pointAt(i/(disk-1)).getx()
			arrr[i,1]=curve.pointAt(i/(disk-1)).gety()
		
		if args.has_key("camberCurve"):
			camberCurve = args.get("camberCurve")
			barrr=numpy.zeros((disk,2))
			for i in xrange(int(disk)):
				barrr[i,0]=camberCurve.pointAt(i/(disk-1)).getx()
				barrr[i,1]=camberCurve.pointAt(i/(disk-1)).gety()
		
		i = 0
		if self.debug > 1: print self.profileULowerSide[i]
		if self.debug > 1: print self.profileUUpperSide[i]
		
		#print self.pointsLowerSide[i][0]
		#print self.pointsUpperSide[i][0]
		#print self.camberCoordinates[i][0]
		
		#print arrr[i]
		#print barrr[-1]
		#print self.getPoint(camberCurve.pointAt(1.0))
		
		#leadingEadgeA = numpy.zeros((1,3))
		#leadingEadgeB = numpy.zeros((1,3))
		
		#leadingEadgeA[0] = self.getPoint(self.nurbsProfile[i].pointAt(self.averageULeadingEdge))
		#leadingEadgeB[0] = self.getPoint(self.nurbsProfile[i].pointAt(self.lengthAverageUValue))
		
		#print str(leadingEadgeA)
		#print str(leadingEadgeB)
		
			
		p.plot(arrr[:,0],arrr[:,1])
		
		if args.has_key("camberCurve"):
			p.plot(barrr[:,0],barrr[:,1])
		
		p.plot(self.pointsLowerSide[i][:,2],self.pointsLowerSide[i][:,1],'bo')
		p.plot(self.pointsUpperSide[i][:,2],self.pointsUpperSide[i][:,1],'rx')
		p.plot(self.camberCoordinates[i][:,2],self.camberCoordinates[i][:,1],'gx')
		
		if args.has_key("camberPoints"):
			camberPoints = args.get("camberPoints")
			p.plot(camberPoints[:,0],camberPoints[:,1],'bx')
		
		#p.plot(leadingEadgeA[:,0],leadingEadgeA[:,1],'ro')
		#p.plot(leadingEadgeB[:,0],leadingEadgeB[:,1],'go')
		p.axis('equal')
		p.show()
	
	## @brief 3D-Plot of the computed Row - experimental and not for cascades!	
	def plotBlade3D(self,curve,**args):
		import pylab as p
		import matplotlib.axes3d as p3
		
		disk = 1000.0
		if args.has_key("index"):
			index = args.get("camberPoints")
		
		#### Plot zum Vergleich von Nurbskurve und diskreten Punkten
		arrr=numpy.zeros((disk,3))
		for i in xrange(int(disk)):
			arrr[i,0]=curve.pointAt(i/(disk-1)).getx()
			arrr[i,1]=curve.pointAt(i/(disk-1)).gety()
			arrr[i,2]=curve.pointAt(i/(disk-1)).getz()

		plotpts=p.figure()
		ax=p3.Axes3D(plotpts)
		
		if args.has_key("index"):
			i = index
			ax.scatter3D(self.pointsLowerSide[i][:,0],self.pointsLowerSide[i][:,1],self.pointsLowerSide[i][:,2])
			ax.scatter3D(self.pointsUpperSide[i][:,0],self.pointsUpperSide[i][:,1],self.pointsUpperSide[i][:,2])
			ax.scatter3D(self.camberCoordinates[i][:,0],self.camberCoordinates[i][:,1],self.camberCoordinates[i][:,2])
			ax.scatter3D(arrr[:,0],arrr[:,1],arrr[:,2])
		else:
				
			for i in range(self.numberProfiles):
				ax.scatter3D(self.pointsLowerSide[i][:,0],self.pointsLowerSide[i][:,1],self.pointsLowerSide[i][:,2])
				ax.scatter3D(self.pointsUpperSide[i][:,0],self.pointsUpperSide[i][:,1],self.pointsUpperSide[i][:,2])
				ax.scatter3D(self.camberCoordinates[i][:,0],self.camberCoordinates[i][:,1],self.camberCoordinates[i][:,2])
				ax.scatter3D(arrr[:,0],arrr[:,1],arrr[:,2])
			
		p.show()
