#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Andreas Kührmann, Sebastian Barthmes       *
#*   sebastian.barthmes@gmail.com                                          *
#*   andreas.kuehrmann@gmail.com                                           *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
#  @author Sebastian Barthmes
#  @author Andreas Kührmann
#  @date 26.09.2010

# Python Imports
# üöä
from numpy import ones, sqrt, vdot, arctan2, cos, sin, zeros
from pythonnurbs import NurbsCurve, NurbsPoint, NurbsMatrixRT, NurbsSurface, NurbsColor, NurbsVector, NurbsMatrix
from scipy.optimize.minpack import bisection


def computeNormedNomalVector(u, curve, normalS):
	
	normalVector = ones([3], 'd')
	
	normalVector[0]  = curve.normal(u, normalS).getx()
	normalVector[1]  = curve.normal(u, normalS).gety()
	normalVector[2]  = curve.normal(u, normalS).getz()
	
	normedNormalVector = normalVector/sqrt(vdot(normalVector,normalVector))
	
	return normedNormalVector


def calculateUStart(uEnd, length, curve, tolerance=1e-9, maxIteration=100):
	
	uStart = bisection(diffLength, 0.0, 1.0, args=(uEnd, length, curve, tolerance, maxIteration),  xtol = tolerance, maxiter = maxIteration)
	
	return uStart


def diffLength(uStart, uEnd, length, curve, tolerance=1e-9, maxIteration=100):
	
	diff = curve.lengthIn(uStart, uEnd, tolerance, maxIteration)-length
	
	return diff


def getCurveCoordinates(u, curve, outputCS='xyz'):
	
	xyz = ones([3], 'd')
	
	x = curve.pointAt(u).getx()
	y = curve.pointAt(u).gety()
	z = curve.pointAt(u).getz()
	
	xyz = [x, y, z]
	
	if outputCS == 'xyz':
	
		return xyz
	
	else:
		
		tCoordinates = transformCoordinates(xyz, outputCS)
			
		return tCoordinates


def calculateUFromM(m, direction, uStart, uEnd, curve, tolerance=1e-9, maxIteration=100):
	
	u = bisection(diffM, uStart, uEnd, args=(m, curve, direction), xtol = tolerance, maxiter = maxIteration)
	
	return u


def diffM(u, m, curve, direction):
	
	diff = getCurveCoordinates(u, curve, 'm')-m
	
	return direction*diff


def moveCurvePoint(u, curve, offset0=0, offset1=0, offset2=0, inputCS='xyz', outputCS='xyz'):
	
	oldCoordinates = ones([3], 'd')
	newCoordinates = ones([3], 'd')
	
	oldCoordinates = getCurveCoordinates(u, curve, inputCS)
	
	newCoordinates = movePoint(oldCoordinates, offset0, offset1, offset2, inputCS, outputCS)
	
	return newCoordinates


def calculateUFromZ(z, curve, s, guessU, uMin=0, uMax=1, tolerance=1e-9, maxIteration=100):
	
	u = NurbsCurve.doublep()
	u.assign = guessU
	curve.minDistZ(z, u, tolerance, -1*s, maxIteration*2, maxIteration, uMin, uMax)
	
	return u.value()


def transformCurveIntoUValues(fixPointsOnCurve, partLength, curve, tolerance=1e-9, maxIteration=100):
	
	jump = 0
	
	u       = []
	uReturn = []
	
	fixPointsOnCurve.reverse()
	partLength.reverse()
	
	for fixPointIndex in xrange(len(fixPointsOnCurve)):
		
		if fixPointsOnCurve[fixPointIndex] == 0: fixPointsOnCurve[fixPointIndex] = 0+tolerance
		if fixPointsOnCurve[fixPointIndex] == 1: fixPointsOnCurve[fixPointIndex] = 1-tolerance
	
	if len(fixPointsOnCurve) == len(partLength):
		
		jump = 1
		
		u.append([])
		
		length = curve.lengthIn(fixPointsOnCurve[0], 1, tolerance, maxIteration)+curve.lengthIn(0, fixPointsOnCurve[-1], tolerance, maxIteration)
		
		tmpLength = 0
		
		lastU = fixPointsOnCurve[-1]
		
		partLength[0] = partLength[0][::-1]
		
		for partLengthIndex in xrange(len(partLength[0])):
			
			tmpLength = tmpLength+partLength[0][partLengthIndex]
			
			if  tmpLength*(1.0+tolerance) < length:
				
				if curve.lengthIn(0, lastU, tolerance, maxIteration) < partLength[0][partLengthIndex]:
					
					partLength[0][partLengthIndex] = partLength[0][partLengthIndex]-curve.lengthIn(0, lastU, tolerance, maxIteration)
					
					lastU = 1
					
				u[0].append(calculateUStart(lastU, partLength[0][partLengthIndex], curve, tolerance, maxIteration))
				
				lastU = u[0][-1]
			
		u[0].reverse()
		
	for pointIndex in xrange(len(fixPointsOnCurve)-1):
		
		u.append([])
		
		length = curve.lengthIn(fixPointsOnCurve[pointIndex+1], fixPointsOnCurve[pointIndex], tolerance, maxIteration)
		
		tmpLength = 0
		
		lastU = fixPointsOnCurve[pointIndex]
		
		partLength[pointIndex+jump] = partLength[pointIndex+jump][::-1]
		
		for partLengthIndex in xrange(len(partLength[pointIndex+jump])):
			
			tmpLength = tmpLength+partLength[pointIndex+jump][partLengthIndex]
			
			if  tmpLength*(1.0+tolerance) < length:
				
				u[-1].append(calculateUStart(lastU, partLength[pointIndex+jump][partLengthIndex], curve, tolerance, maxIteration))
				
				lastU = u[-1][-1]
				
		u[-1].reverse()
		
	for listIndex in xrange(len(u)-1, -1, -1):
		
		for uIndex in xrange(len(u[listIndex])):
			
			uReturn.append(u[listIndex][uIndex])
			
		uReturn.append('s')	
		
	fixPointsOnCurve.reverse()
	
	return uReturn


def computeScaledCurve(curve, surface, offset, gridType, noPointsOnCurve=200, tolerance=1e-9, maxIteration=100):
	
	scaledCurve             = NurbsCurve.NurbsCurved()
	uValue                  = NurbsCurve.doublep()
	vValue                  = NurbsCurve.doublep()
	scaledProfilePointArray = ones([noPointsOnCurve, 3], 'd')
	
	for pointIndex in xrange (noPointsOnCurve):
		
		u = 1.0/(noPointsOnCurve-1)*pointIndex
		
		if gridType == 'fS':
			
			normalS = NurbsPoint.Point3Dd(1, 0, 0)
			
		elif gridType == 'rS':
			
			tmpPoint = NurbsPoint.Point3Dd(curve.pointAt(u).getx(), curve.pointAt(u).gety(), curve.pointAt(u).getz())
			
			surface.projectOn(tmpPoint, uValue, vValue, maxIteration)
			
			normalS = surface.normal(uValue.value(), vValue.value())
			
		normedNomalVector = computeNormedNomalVector(u, curve, normalS)
		
		scaledProfilePointArray[pointIndex] = moveCurvePoint(u, curve, normedNomalVector[0]*offset, normedNomalVector[1]*offset, normedNomalVector[2]*offset, "xyz", "xyz")
		
	scaledCurve = interpolateNurbs3D(scaledProfilePointArray)
	
	if gridType == 'rS':
		
		for pointIndex in xrange(noPointsOnCurve):
			
			u = 1.0/(noPointsOnCurve-1)*pointIndex
			
			tmpPoint = NurbsPoint.Point3Dd(scaledCurve.pointAt(u).getx(), scaledCurve.pointAt(u).gety(), scaledCurve.pointAt(u).getz())
			
			surface.projectOn(tmpPoint, uValue, vValue, maxIteration)
			
			scaledProfilePointArray[pointIndex, 0] = surface.pointAt(uValue.value(), vValue.value()).getx()
			scaledProfilePointArray[pointIndex, 1] = surface.pointAt(uValue.value(), vValue.value()).gety()
			scaledProfilePointArray[pointIndex, 2] = surface.pointAt(uValue.value(), vValue.value()).getz()
			
		scaledCurve = interpolateNurbs3D(scaledProfilePointArray)
		
	return scaledCurve


def computeRevolutionSurface(curve, origin, rVector, theta, tVector=[0, 0, 0], writeVRML=False, surfaceName='0', storageLocation='./'):
	
	mRT        = NurbsMatrixRT.MatrixRTd()
	mRTreverse = NurbsMatrixRT.MatrixRTd()
	
	surface = NurbsSurface.NurbsSurfaced()
	
	mRT.rotate(tVector[0], tVector[1], tVector[2])
	mRTreverse.rotate(-tVector[0], -tVector[1], -tVector[2])
	
	curve.transform(mRT)
	
	surface.makeFromRevolution(curve, origin, rVector, theta)
	
	curve.transform(mRTreverse)
	
	red = NurbsColor.Color(255, 0, 0)
	
	if writeVRML == True:
		
		surface.writeVRML97(str(storageLocation)+"rotSurface_"+str(surfaceName)+".wrl", red, 200, 200)
		
	return surface
	


def computePartLengthCurve(fixPointsOnCurve, curve, closedCurve=False, tolerance=1e-9, maxIteration=100):
	
	partLength = []
	
	for pointIndex in xrange(len(fixPointsOnCurve)-1):
		partLength.append(curve.lengthIn(fixPointsOnCurve[pointIndex], fixPointsOnCurve[pointIndex+1], tolerance, maxIteration))
		
	if closedCurve == True:
		partLength.append(curve.lengthIn(fixPointsOnCurve[-1], 1, tolerance, maxIteration)+curve.lengthIn(0, fixPointsOnCurve[0], tolerance, maxIteration))
		
	return partLength


def transformCoordinates(coordinates, outputCS):
	
	tCoordinates = ones([3], 'd')
	
	if outputCS == 'xyz':
		
		tCoordinates[0] = coordinates[0]*cos(coordinates[1])
		tCoordinates[1] = coordinates[0]*sin(coordinates[1])
		tCoordinates[2] = coordinates[2]
		
		return tCoordinates
		
	elif outputCS == 'rpz':
		
		tCoordinates[0] = sqrt(coordinates[0]**2.0+coordinates[1]**2.0)
		tCoordinates[1] = arctan2(coordinates[1], coordinates[0])
		tCoordinates[2] = coordinates[2]
		
		return tCoordinates
		
	elif outputCS == 'm':
		
		m = sqrt(coordinates[0]**2.0+coordinates[1]**2.0+coordinates[2]**2.0)
		
		return m
		
	else:
		
		raise ValueError('*** Wrong outputCS! Current outputCS! = {0}'.format(outputCS))


def movePoint(oldCoordinates, offset0=0, offset1=0, offset2=0, inputCS='xyz', outputCS='xyz'):
	
	newCoordinates = ones([3], 'd')
	
	newCoordinates = [oldCoordinates[0]+offset0, oldCoordinates[1]+offset1, oldCoordinates[2]+offset2]
	
	if inputCS == outputCS:
		
		return newCoordinates
		
	else:
		
		tCoordinates = transformCoordinates(newCoordinates, outputCS)
		
		return tCoordinates


## @brief least squares interpolation of 2D Points with a NURBS curve
# @param parray The list of 2D Points
# @param kps number of control points
# @param degree the degree of the nurbs curve
# @return the nurbs curve
def interpolateNurbs2D(parray,kps=0,degree=3):
	if kps==0:
		pts=len(parray)
	else:
		pts=kps
		
	## o.b.: TODO:! implementierung nach pythonNurbs-0.2
	ptsVector = NurbsVector.Vector_Point2Dd(len(parray))
	vector = NurbsVector.Vector_Point2Dd(1)
	for n in range(len(parray)):
		vector.reset(NurbsPoint.Point2Dd(parray[n,0],parray[n,1]))
		ptsVector.as_func(n,vector)
	nurbsCurve = NurbsCurve.NurbsCurve2Dd()
	nurbsCurve.leastSquares(ptsVector,degree, pts)

	return nurbsCurve
	
## @brief least squares interpolation of 3D Points with a NURBS curve
# @param parray The list of 3D Points
# @param kps number of control points
# @param kv (optional) a knot vector
# @param degree the degree of the nurbs curve
# @return the nurbs curve
def interpolateNurbs3D(parray,kps=0,kv=None,degree=3):
	if kps==0:
		pts=len(parray)
	else:
		pts=kps
	## o.b.: TODO:! implementierung nach pythonNurbs-0.2
	ptsVector = NurbsVector.Vector_Point3Dd(len(parray))
	vector = NurbsVector.Vector_Point3Dd(1)
	for n in range(len(parray)):
		vector.reset(NurbsPoint.Point3Dd(parray[n,0],parray[n,1],parray[n,2]))
		ptsVector.as_func(n,vector)
	nurbsCurve = NurbsCurve.NurbsCurved()
	nurbsCurve.leastSquares(ptsVector,degree,pts)
	
	return nurbsCurve

## @brief least squares interpolation of 3D Points with a closed NURBS curve
# @param parray The list of 3D Points
# @param kps number of control points
# @param kv (optional) a knot vector
# @param degree the degree of the nurbs curve
# @return the nurbs curve
def interpolateNurbs3DClosed(parray,kps=0,kv=None,degree=3):
	if kps==0:
		pts=len(parray)
	else:
		pts=kps
	
	ptsVector = NurbsVector.Vector_Point3Dd(len(parray))
	vector = NurbsVector.Vector_Point3Dd(1)
	for n in range(len(parray)):
		vector.reset(NurbsPoint.Point3Dd(parray[n,0],parray[n,1],parray[n,2]))
		ptsVector.as_func(n,vector)

	nurbsCurve = NurbsCurve.NurbsCurved()
	
	nurbsCurve.leastSquaresClosed(ptsVector,degree,pts)
	
	return nurbsCurve

## @brief least squares interpolation of 3D Points matrix with surface
# @param pmatrix The Matrix of 3D Points (U=Rows, V=Columns)
# @param du degree in u direction
# @param dv degree in v direction
# @param kpsu number of control points in u
# @param kpsu number of control points in v
# @return the nurbs surface
def interpolateNurbsSurface(pmatrix,du=3,dv=3,kpsu=0,kpsv=0):
	if len(pmatrix)<(du+1):
		raise Exception, "Number of Points in u-Direction to few for NURBS-Interpolation"
	if len(pmatrix[0])<(dv+1):
		raise Exception, "Number of Points in v-Direction to few for NURBS-Interpolation"
	
	if kpsu==0:
		kpsu=len(pmatrix)
	if kpsv==0:
		kpsv=len(pmatrix[0])
	
	#Matrix mit 3D-Punkten der Profile
	mpts = NurbsMatrix.Matrix_Point3Dd(len(pmatrix),len(pmatrix[0]))
	for i,pro in enumerate(pmatrix):
		for j,p in enumerate(pro):
			mpts.setelem(i,j,NurbsPoint.Point3Dd(p[0],p[1],p[2]))

	#Erzeuge Nurbs-Flaeche
	surface = NurbsSurface.NurbsSurfaced()
	surface.leastSquares(mpts,du,dv,kpsu,kpsv)

	return surface
	
## @brief least squares interpolation of 3D Points matrix with a closed NURBS surface in U-Direction
# @param pmatrix The Matrix of 3D Points (U=Rows, V=Columns)
# @param du degree in u direction
# @param dv degree in v direction
# @param kpsu number of control points in u
# @param kpsu number of control points in v
# @return the nurbs surface
def interpolateNurbsSurfaceClosedU(pmatrix,du=3,dv=3,kpsu=0,kpsv=0):
	if len(pmatrix)<(du+1):
		raise Exception, "Number of Points in u-Direction to few for NURBS-Interpolation"
	if len(pmatrix[0])<(dv+1):
		raise Exception, "Number of Points in v-Direction to few for NURBS-Interpolation"
	
	if kpsu==0:
		kpsu=len(pmatrix)
	if kpsv==0:
		kpsv=len(pmatrix[0])
	
	#Matrix mit 3D-Punkten der Profile
	mpts = NurbsMatrix.Matrix_Point3Dd(len(pmatrix),len(pmatrix[0]))
	for i,pro in enumerate(pmatrix):
		for j,p in enumerate(pro):
			mpts.setelem(i,j,NurbsPoint.Point3Dd(p[0],p[1],p[2]))

	#Erzeuge Nurbs-Flaeche
	surface = NurbsSurface.NurbsSurfaced()
	surface.leastSquaresClosedU(mpts,du,dv,kpsu,kpsv)

	return surface
	
## @brief least squares interpolation of 3D Points matrix with a closed NURBS surface in V-Direction
# @param pmatrix The Matrix of 3D Points (U=Rows, V=Columns)
# @param du degree in u direction
# @param dv degree in v direction
# @param kpsu number of control points in u
# @param kpsu number of control points in v
# @return the nurbs surface
def interpolateNurbsSurfaceClosedV(pmatrix,du=3,dv=3,kpsu=0,kpsv=0):
	if len(pmatrix)<(du+1):
		raise Exception, "Number of Points in u-Direction to few for NURBS-Interpolation"
	if len(pmatrix[0])<(dv+1):
		raise Exception, "Number of Points in v-Direction to few for NURBS-Interpolation"
	
	if kpsu==0:
		kpsu=len(pmatrix)
	if kpsv==0:
		kpsv=len(pmatrix[0])
	
	#Matrix mit 3D-Punkten der Profile (transponiert)
	mpts = NurbsMatrix.Matrix_Point3Dd(len(pmatrix[0]),len(pmatrix))
	for i,pro in enumerate(pmatrix):
		for j,p in enumerate(pro):
			mpts.setelem(j,i,NurbsPoint.Point3Dd(p[0],p[1],p[2]))

	#Erzeuge Nurbs-Flaeche (transponiert)
	tmps = NurbsSurface.NurbsSurfaced()
	tmps.leastSquaresClosedU(mpts,dv,du,kpsv,kpsu)
	#surface=tmps.transpose()
	
	##Transponiere Nurbs-Flaeche
	surface = NurbsSurface.NurbsSurfaced()
	surface = NurbsSurface.NurbsSurfaced(du,dv,tmps.knotV(),tmps.knotU(),tmps.ctrlPnts().transpose())

	return surface
	

## @brief global nurbs interpolation of 3D Points
def globalInterpolateNurbs3D(parray,degree=3):
	ptsVector = NurbsVector.Vector_Point3Dd(len(parray))

	for n in range(len(parray)):
		ptsVector[n] = NurbsPoint.Point3Dd(parray[n,0],parray[n,1],parray[n,2])
		
	nurbsCurve = NurbsCurve.NurbsCurved()
	nurbsCurve.globalInterp(ptsVector,degree)
	
	return nurbsCurve
	
## @brief global closed nurbs interpolation of 3D Points
def globalInterpolateNurbs3DClosed(parray,degree=3):
	ptsVector = NurbsVector.Vector_Point3Dd(len(parray))

	for n in range(len(parray)):
		ptsVector[n] = NurbsPoint.Point3Dd(parray[n,0],parray[n,1],parray[n,2])
		
	nurbsCurve = NurbsCurve.NurbsCurved()
	nurbsCurve.globalInterpClosed(ptsVector,degree)
	
	return nurbsCurve

	
	
## @brief gets an u-coordinate from a NURBS curve which matches to a given x-coordinate
def calculateUValueFromX(x,curve):
	tolerance=0.0001
	check=False
	loc=1.0
	i=2.0
	if curve.pointAt(0.0).getx()>x:
		return curve.pointAt(0.0).getx()
	while check != True:
		cx = curve.pointAt(loc).getx()
		if abs(cx-x)<=tolerance:
			return loc
		if (cx-x)<0:
			loc+=1.0/i
		if (cx-x)>0:
			loc-=1.0/i
		i*=2

## @brief calculates an u-coordinate which is located in a given real positive distance from another u
def calculateUValueFromDistance(dist,u,curve):
	tolerance=0.00001
	check=False
	loc=1.0
	i=2.0
	for n in range(30):
		cdist = curve.lengthIn(u,loc,1e-6,100)
		if abs(cdist-dist)<=tolerance:
			return loc
		if (cdist-dist)<0:
			loc+=1.0/i
		if (cdist-dist)>0:	
			loc-=1.0/i
		i*=2
	raise Exception, "Bisection algorithm didn't converge while computing local distance."

			
## @brief finds an u-coordinate on a curve, which is located on the vector m of a point pt
def findBinormalPoint(m,pt,curve): #Gradient an dem Punkt,Punkt, Nurbskurve auf der gesucht wird
	tolerance=0.0001
	check=False
	loc=1.0
	i=1.0
	a=-atan2(m[1],m[0])
	n=0
	while check != True:
		i*=2
		n+=1
		tmp = curve.pointAt(loc)
		d=[tmp.getx(),tmp.gety()]
		d[0]-=pt[0]
		d[1]-=pt[1]
		dtmp=copy(d)
		dtmp[0]=d[0]*cos(a)-d[1]*sin(a)
		dtmp[1]=d[0]*sin(a)+d[1]*cos(a)
		d=dtmp
		if abs(d[0])<=tolerance:
			return loc
		if self.debug>=1:
			if n==100:
				raise Exception, "No binormal point found. Check the code and/or input" 
		elif n==100:
			exit()
		if d[0]>0:
			loc-=1.0/i
		if d[0]<0:
			loc+=1.0/i


## @brief computes the cutting point of two curves
def computeCuttingPoint(curve1,curve2,flag=0): #Vorsicht, diese Funktion beinhaltet Toleranzen!
	ip=200
	dx= -1.0
	tolerance=(curve1.length()+curve2.length())/1.5/ip
	for i in range(ip):
		for j in range(ip):
			point1 = curve1.pointAt(1.0/ip*i)
			point2 = curve2.pointAt(1.0/ip*j)
			if abs(point1.getx()-point2.getx())<tolerance and abs(point1.gety()-point2.gety())<tolerance:
				dx=(point1.getx()+point2.getx())/2
				dy=(point1.gety()+point2.gety())/2
				u1=1.0/ip*i
				u2=1.0/ip*j
				break
	if dx==-1.0:
		raise Exception, "No intersection found. Please check your input data and mind a potential error."
	cut=[dx,dy]
	if flag==0:
		return cut
	if flag==1:
		return cut,u1,u2
			
			
## @brief convert nurbs++ curve to occ bspline
# @param nppcurve the nurbs++ curve object
# @return Geom_BSplineCurve the pytonocc curve object
def nppToOcc_Curve(nppcurve):
	try:
		import OCC.gp 
		import OCC.Geom 
		import OCC.TColgp 
		import OCC.TColStd
	except:
		raise ImportError, "!!! pythonocc not found. OpenCASCADE output not available"
	else:
		
		degree=nppcurve.knot().n()-1-nppcurve.ctrlPnts().n()
		
		pole_list=[]
		weights_list=[]
		for n in range(nppcurve.ctrlPnts().n()):
			pole_list.append(
					OCC.gp.gp_Pnt(
						nppcurve.ctrlPnts(n).getx(),
						nppcurve.ctrlPnts(n).gety(),
						nppcurve.ctrlPnts(n).getz()
					)
				)
			weights_list.append(nppcurve.ctrlPnts(n).getw())
		
		knots_list=[]
		mult_list=[]
		for n in range(nppcurve.knot().n()):
			if knots_list!=[]:
				if nppcurve.knot(n)==knots_list[-1]:
					mult_list[-1]+=1
				else:
					knots_list.append(nppcurve.knot(n))
					mult_list.append(1)
			else:
				knots_list.append(nppcurve.knot(n))
				mult_list.append(1)
		
		
		
		if min(knots_list)<0.0 or max(knots_list)>1.0:
			#skaliere auf 0.5
			sumtmp=0.5-knots_list[0]
			scalefac=0.5/sumtmp
			for k in range(len(knots_list)):
				knots_list[k]-=0.5
				knots_list[k]*=scalefac
				knots_list[k]+=0.5
			
			pole_list=pole_list[:-(degree)]
			weights_list=weights_list[:-(degree)]
			knots_list=knots_list[:-(2*degree)]
			mult_list=mult_list[:-(2*degree)]
			periodic=1
		else:
			periodic=0
			
		#for pole in pole_list:
			#print pole.X(),pole.Y(),pole.Z()
		
		pole_array = OCC.TColgp.TColgp_Array1OfPnt(0, len(pole_list)-1)
		for n,i in enumerate(pole_list):
			pole_array.SetValue(n,i)
		
		knots_array = OCC.TColStd.TColStd_Array1OfReal(0, len(knots_list)-1)
		for n,i in enumerate(knots_list):
			knots_array.SetValue(n,i)
		
		weights_array = OCC.TColStd.TColStd_Array1OfReal(0, len(weights_list)-1)
		for n,i in enumerate(weights_list):
			weights_array.SetValue(n,i)
		
		mult_array = OCC.TColStd.TColStd_Array1OfInteger(0, len(mult_list)-1)
		for n,i in enumerate(mult_list):
			mult_array.SetValue(n,int(i))
		
		

		return OCC.Geom.Geom_BSplineCurve(
			pole_array,
			weights_array,
			knots_array,
			mult_array,
			degree,
			periodic
		)
	

## @brief convert nurbs++ surface to occ bspline surface
# @param nppsurf the nurbs++ surface object
# @return Geom_BSplineSurface the pytonocc surface object
def nppToOcc_Surface(nppsurf):
	try:
		import OCC.gp
		import OCC.Geom
		import OCC.TColgp
		import OCC.TColStd
	except:
		raise ImportError, "!!! pythonocc not found. OpenCASCADE output not available"
	else:
		degreeu=nppsurf.knotU().n()-1-nppsurf.ctrlPnts().rows()
		degreev=nppsurf.knotV().n()-1-nppsurf.ctrlPnts().cols()
		
		
		pole_list=[]
		weights_list=[]
		for i in range(nppsurf.ctrlPnts().rows()):
			pole_list.append([])
			weights_list.append([])
			for j in range(nppsurf.ctrlPnts().cols()):
				pole_list[i].append(
						OCC.gp.gp_Pnt(
						nppsurf.ctrlPnts(i,j).getx(),
						nppsurf.ctrlPnts(i,j).gety(),
						nppsurf.ctrlPnts(i,j).getz()
						)
					)
				weights_list[i].append(nppsurf.ctrlPnts(i,j).getw())
		
		
		knotsu_list=[]
		multu_list=[]
		for n in range(nppsurf.knotU().n()):
			if knotsu_list!=[]:
				if nppsurf.knotU(n)==knotsu_list[-1]:
					multu_list[-1]+=1
				else:
					knotsu_list.append(nppsurf.knotU(n))
					multu_list.append(1)
			else:
				knotsu_list.append(nppsurf.knotU(n))
				multu_list.append(1)
		
		knotsv_list=[]
		multv_list=[]
		for n in range(nppsurf.knotV().n()):
			if knotsv_list!=[]:
				if nppsurf.knotV(n)==knotsv_list[-1]:
					multv_list[-1]+=1
				else:
					knotsv_list.append(nppsurf.knotV(n))
					multv_list.append(1)
			else:
				knotsv_list.append(nppsurf.knotV(n))
				multv_list.append(1)
		
		if min(knotsv_list)<0.0 or max(knotsv_list)>1.0:
			#skaliere auf 0.5
			sumtmp=0.5-knotsv_list[0]
			scalefac=0.5/sumtmp
			for k in range(len(knotsv_list)):
				knotsv_list[k]-=0.5
				knotsv_list[k]*=scalefac
				knotsv_list[k]+=0.5
			
			for i in range(len(pole_list)):
				pole_list[i]=pole_list[i][:-(degreev)]
				weights_list[i]=weights_list[i][:-(degreev)]
			knotsv_list=knotsv_list[:-(2*degreev)]
			multv_list=multv_list[:-(2*degreev)]
			periodicv=1
		else:
			periodicv=0		
		
		
		pole_array = OCC.TColgp.TColgp_Array2OfPnt(1, len(pole_list),1,len(pole_list[0]))
		for i in range(len(pole_list)):
			for j in range(len(pole_list[0])):
				pole_array.SetValue(i+1,j+1,pole_list[i][j])

		weights_array = OCC.TColStd.TColStd_Array2OfReal(1, len(weights_list),1,len(weights_list[0]))
		for i in range(len(weights_list)):
			for j in range(len(weights_list[0])):
				weights_array.SetValue(i+1,j+1,weights_list[i][j])
				
		knotsu_array = OCC.TColStd.TColStd_Array1OfReal(0, len(knotsu_list)-1)
		for n,i in enumerate(knotsu_list):
			knotsu_array.SetValue(n,i)
		
		multu_array = OCC.TColStd.TColStd_Array1OfInteger(0, len(multu_list)-1)
		for n,i in enumerate(multu_list):
			multu_array.SetValue(n,int(i))
		
		knotsv_array = OCC.TColStd.TColStd_Array1OfReal(0, len(knotsv_list)-1)
		for n,i in enumerate(knotsv_list):
			knotsv_array.SetValue(n,i)
		
		multv_array = OCC.TColStd.TColStd_Array1OfInteger(0, len(multv_list)-1)
		for n,i in enumerate(multv_list):
			multv_array.SetValue(n,int(i))

		periodicu=0
		
		occsurf = OCC.Geom.Geom_BSplineSurface(
									pole_array,
									weights_array,
									knotsu_array,
									knotsv_array,
									multu_array,
									multv_array,
									degreeu,
									degreev,
									int(periodicu),
									int(periodicv)
									)
		
		return occsurf	


## @brief converts r-z NURBS to r-phi-z NURBS with phi=0 (second coordinate=0)!
# @param nurbs2D the two dimensional NURBS
def convertNurbs2Dto3D(nurbs2D):
	ptarr=zeros((nurbs2D.ctrlPnts().n(),2))
	for n in range(nurbs2D.ctrlPnts().n()):
		ptarr[n]=[nurbs2D.ctrlPnts(n).getx(),nurbs2D.ctrlPnts(n).gety()]
		
	#3D-Kontrollpunkte aus den NurbsKurven
	k=zeros((nurbs2D.ctrlPnts().n(),3))
	kpts = NurbsVector.Vector_Point3Dd(len(k))
	vector = NurbsVector.Vector_Point3Dd(1)
	for n in range(len(k)):
		vector.reset(NurbsPoint.Point3Dd(ptarr[n,1],0.0,ptarr[n,0]))
		kpts.as_func(n,vector)
	
	#Gewichtungsmatrix
	wv = NurbsVector.Vector_DOUBLE(nurbs2D.ctrlPnts().n())
	for i in range(nurbs2D.ctrlPnts().n()):
		wv[i]=nurbs2D.ctrlPnts(i).getw()
	
	degree=nurbs2D.knot().n()-1-nurbs2D.ctrlPnts().n()
	
	curve3D=NurbsCurve.NurbsCurved(kpts,wv,nurbs2D.knot(),degree)
	return curve3D