#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Julius Fellerhoff                          *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Oliver Borm
# @author Julius Fellerhoff
# @date 24.02.2010

# Python Imports

import os
import sys

import math
import numpy

import turboMachine

from xml.dom import minidom
from pythonnurbs import NurbsCurve, NurbsPoint, NurbsVector

## @class profile2D
# @brief creates 2D profiles
class profile2D(object):
	## @brief Initializes all public attributes
	def __init__(self,*args):
		## dataStructure-object, contains all data about the working directories and their structure
		self.dataStruct = []
		## profile number
		self.profileNumber=0
		## flag for the use of methods approximating the coefficients for a polynomial thickness distribution, =1 methods in use
		self.approxPolyCoeff=0
		## flag for use of xml-file, =1 uses xml-file to import the variables
		self.readXml=0
		## resolution of circular arc camber line
		self.camberResolution=0
		## resolution of any profile
		self.profileResolution=0
		
		## flag for thickness distribution type, =1 polynomial, =2 NURBS, =3 double circular arc profile
		self.flagThickDistr=0
		## flag for camber line definition, =0 circular arc, =1 polynomial, =2 NURBS, =3 NACA 65 camber line
		self.flagCamberLine=0
		
		## for circular arc or polynomial camber line methods
		self.angleOfAttack=0
		## for circular arc or polynomial camber line methods
		self.outflowAngle=0
		
		## angle definition for bladeRow use
		self.angleDefinition=0.0 #0: User defined chi1/2, Gamma 1: Symmetrical profile, beta1/2 2: NACA, beta1 or! beta2
		## angle for bladeRow use
		self.chi1=0
		## angle for bladeRow use
		self.chi2=0
		## angle for bladeRow use
		self.beta1=0
		## angle for bladeRow use
		self.beta2=0
		## angle for bladeRow use
		self.theta=0
		
		## only for double circular arc profiles, \f$ r_{leading Edge} = r_{trailing Edge} \f$, max 10% of chord length
		self.leadingEdgeRadius=0
		## chord length of profile
		self.chordLength=0.0
		## stagger angle
		self.gamma = 0.0
		## definition for double circular arc profiles, naca assumed, polynomial and elliptic
		self.maxThickness=0
		
		## weight function boundary value for polynomial point distribution, =0 parabolic, =1 cubic, =2 tangens, =3 arcsin, =4 hyperbolic, =5 polygon
		self.weightCondition=0
		## value for weight function combinations, =0 curvature of polynomial, =1 curvature of reduced polynomial part, =2 div weight funct, =3 combination of polynomial & other weight function like parabolic, cubic etc.
		self.weightCombination=0
		## value for general polynomial thickness distribution, =0 NACA assumed, =1 polynomial, =2 elliptic, used when flagThickDistr=1
		self.thickMeth=0
		
		## value for degree of polynomial camber line, degree is 4, 3 or 2
		self.polyCamberLineDegree=0
		## array of coefficients for the polynomial thickness distribution function
		self.polyThickRBs=0
		## x-coordinate of the maximum camber position for higher order polynomial camber lines
		self.maxCambPosition=0
		
		## array of NURBS - describer points for the camberline
		self.controlPointsCamb=0
		## array of NURBS - describer points for the thickness distribution
		self.controlPointsThick=0
		## approximate leading and trailing edge radius for a NURBS thickness distribution
		self.approxNurbsNoseRad=0
		## \f$ C_{a0} \f$ for NACA 65 camber line
		self.Ca0=0
		
		## NURBS-curve camber line, when flagCamberLine=2
		self.curveCamber=0
		## the single thickness distribution array
		self.thickDistr=0
		
		## converted numpy-array of the camberline
		self.camberLine=0
		## converted numpy-array of the pressure side
		self.lowerProfile=0
		## converted numpy-array of the suction side
		self.upperProfile=0

		## parameter for minDistX-NURBS-function, maximum number of iterations
		self.maxIter=1000
		## parameter for minDistX-NURBS-function, error-value
		self.error=1e-10
		
		## array of the coordinates of camber line, suction side & pressure side
		self.gesarray=[]
		## debug level
		self.debug=0
		## determites whether the profile is already calculated or not
		self.isCalculated = False
		
		##parameter for writePS-function in debug-level 2; flag indicating if the control points should be drawn, 0 = no and 1 = yes
		self.cp = 1
		##parameter for writePS-function in debug-level 2; magnification factor for the control points
		self.magFact = 2000
		##parameter for writePS-function in debug-level 2; the size of the dash line polygone between the control points
		self.dash = 5
		
		for arg in args:
			#print "profile arg",type(arg)
			if type(arg)==str:
				self.readXmlFile(arg)
			elif type(arg)==int:
				self.profileNumber=arg
			elif type(arg)==turboMachine.dataStructure:
				self.dataStruct=arg
				#if os.path.exists(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml'):
					#self.readXmlFile(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml')
			else:
				raise Exception, "init error"
	
	## @brief Main control function of the program, which calls all necessary methods according to the flag camber line
	# @brief If the flag camber line is of value 0, a circular arc camber line will be generated, in other cases the general polynomial camber line function is called. For value 1, the coefficients for the polynomial camber line are calculated first.
	def callfunctions(self):
		#Wurde das Profil bereits berechnet?
		if (self.isCalculated):
			#Debugoptionen aktiviert?
			if self.debug > 0:
				print "Aleady calculated... Skipping"
			#Hier abbrechen
			return
			
		if self.debug > 0:
			print "Profilauflösung:",self.profileResolution
			print "flag Thickness Distribution:",self.flagThickDistr
			print "flag Camberline:",self.flagCamberLine
			print "Sehnenlänge:",self.chordLength
			print "Staffelungswinkel:",self.gamma
			
		if self.flagCamberLine!=3:
			sumChi= self.angleOfAttack + self.outflowAngle
			
			if self.debug > 0:
				print "Zuströmwinkel:",self.angleOfAttack
				print "Abströmwinkel:",self.outflowAngle
			
			if self.angleOfAttack < 0.0 or self.angleOfAttack > 180.0:
				raise Exception, "Wert für angleOfAttack ungültig"
			
			if self.outflowAngle < 0.0 or self.outflowAngle > 180.0:
				raise Exception, "Wert für outflowAngle ungültig"
			
			if sumChi < 90.0 or sumChi > 270.0:
				raise Exception, "Werte für angleOfAttack und outflowAngle ungültig"
		
		if self.flagCamberLine==0:
			self.computeCircularArcCamberLine()
		
		elif self.flagCamberLine==1:
			## array of coefficients for the polynomial camber line
			self.polyCambRBs = self.computePolynomialCoefficients()
			self.computePolynomialCamberline()
		
		elif self.flagCamberLine==2 or self.flagCamberLine==3:
			self.computePolynomialCamberline()
		
		else:
			raise Exception, "Wert für flagCamberLine ungültig"
		
		if self.flagCamberLine==0 and self.flagThickDistr==3:
			self.computeDoubleCircularArcProfile()
		
		if self.flagThickDistr==0:
			print "Warnung: keine Dickenverteilung ausgewählt!"
		
		else:
			self.camberLine = self.convertListToCoordArray(self.gesarray[0:2])
			self.lowerProfile = self.convertListToCoordArray(self.gesarray[2:4])
			self.upperProfile = self.convertListToCoordArray(self.gesarray[4:6])
			
			self.camberLine = self.convertArrayToArray(self.camberLine)
			self.lowerProfile = self.convertArrayToArray(self.lowerProfile)
			self.upperProfile = self.convertArrayToArray(self.upperProfile)

		#Profil ist nun berechnet worden
		self.isCalculated = True
	
	## @brief interpolates a 2D coord array to a new coord array with length npts
	def scaleNumberOfPoints2D(self,pts,npts):
		import nurbsToolSet
		
		curve = nurbsToolSet.interpolateNurbs2D(pts)		
		
		arrr=numpy.zeros((npts,2))
		for i in range(npts):
			arrr[i,0]=curve.pointAt(i/(npts-1.)).getx()
			arrr[i,1]=curve.pointAt(i/(npts-1.)).gety()
		
		return arrr
	
	## @brief set profile points manually without profile2d calculation
	# @param 
	def setProfilePts(self,upperp,lowerp,npts=50):
		
		upperp=self.scaleNumberOfPoints2D(upperp,npts)
		lowerp=self.scaleNumberOfPoints2D(lowerp,npts)
		
		self.upperProfile=self.convertListToCoordArray(upperp)
		self.lowerProfile=self.convertListToCoordArray(lowerp)
		self.camberLine=numpy.zeros((len(upperp),2))
		
		self.isCalculated=True
	
	## @brief Converts rows to columns and columns to rows of a numpy array
	def convertArrayToArray(self,arrayIn):
		Array = numpy.zeros([len(arrayIn[0]),len(arrayIn)],'d')
		
		for i in range(len(arrayIn)):
			Array[:,i] = arrayIn[i,:]
		
		return Array
		
	## @brief Converts the a list into a numpy array
	def convertListToCoordArray(self,liste):
		numArray = numpy.zeros([len(liste),len(liste[0])],'d')
		
		for i in range(len(liste)):
			numArray[i,:] = liste[i][:]
		
		return numArray

	## @brief Approximates the coefficients for polynomial thickness distribution
	# @brief the coefficients can be approximated with the use of four different methods.
	# @brief For the general thickness function \f[D(x)= p_0 (x^{p_1} - x^{p_2})^{\frac{1}{p_3}} + a_4 x^4 + a_3 x^3 + a_2 x^2 + a_1 x + a_0 \f] the coefficients \f$ p_0, \dots, p_3, a_0, \dots, a_4 \f$ are calculated. \n Geometrical boundary conditions are: \f[D(0)=0, \hspace{0,5cm} D(1)=0 \f] Maximum Thickness: \f[D'(x_{max,D}) = 0,  \hspace{0,5cm} D(x_{max,D}) = \frac{D_{max}}{2} \f] Radius of curvature at the point of maximum thickness \f[D''(x_{max,D}) = \kappa = \frac{1}{R_{Kr}} \f] For a unique solution more simplifications are necessary.
	# @brief The coefficients \f$ p_1 = 1, p_2 = 2 \f$ are steady and a value for \f$ p_3 \f$ needs to be declared. By diversify the values for \f$ a_3, a_4, p_0 \f$ an explicit solution can be generated.
	# @param RandB flag for the method selection
	# @param b2 \f$	x_{max,D} \f$ - location of maximum thickness
	# @param radiusOfCurvature \f$ \kappa \f$ - radius of curvature at the point of maximum thickness
	# @param p0,p1,p2,p3 partially given coefficients for the polynomial thickness distribution, depending on selected method
	def computePolynomialThicknessCoefficients(self,RandB,b2,radiusOfCurvature,p0,p1,p2,p3):
		
		b0 = 0.0
		b1 = 0.0
		
		b3 = self.maxThickness / 2.0
		b4 = -1.0/(abs(radiusOfCurvature))
		
		if p3 <= 1.0:
			raise Exception, "Polynom Koeffizienten: Wert für Parameter p3 ist ungültig(kleiner als 1)"

		if RandB == 1 :
			
			ordnung = 4
			A = numpy.mat(numpy.ones([ordnung,ordnung], 'd'))
			b = numpy.mat(numpy.ones([1,ordnung], 'd')).transpose()
			
			#1. Reihe
			A[0,0] = 1.0
			A[0,1] = 0.0
			A[0,2] = 0.0
			A[0,3] = 0.0
			
			#2. Reihe
			A[1,0] = 1.0
			A[1,1] = 1.0
			A[1,2] = 1.0
			A[1,3] = 1.0
			
			#3. Reihe
			A[2,0] = 0.0
			A[2,1] = 1.0
			A[2,2] = 2.0*b2
			A[2,3] = 3.0*b2**2.0
			
			#4. Reihe
			A[3,0] = 1.0
			A[3,1] = b2
			A[3,2] = b2**2.0
			A[3,3] = b2**3.0
			
			b[0] = b0
			b[1] = b1
			b[2] = -p0/p3*(1.0-2.0*b2)*(b2-b2**2.0)**(1.0/p3-1.0)
			b[3] = b3-p0*(b2-b2**2.0)**(1.0/p3)

			z = numpy.linalg.inv(A)*b

			a0 = float(z[0])
			a1 = float(z[1])
			a2 = float(z[2])
			a3 = float(z[3])
			a4 = 0.0
			
		elif RandB == 2 :
		
			ordnung = 4
			A = numpy.mat(numpy.ones([ordnung,ordnung], 'd'))
			b = numpy.mat(numpy.ones([1,ordnung], 'd')).transpose()
			
			#1. Reihe
			A[0,0] = 0.0
			A[0,1] = 1.0
			A[0,2] = 0.0
			A[0,3] = 0.0
			
			#2. Reihe
			A[1,0] = 0.0
			A[1,1] = 1.0
			A[1,2] = 1.0
			A[1,3] = 1.0
			
			#3. Reihe
			A[2,0] = 1.0/p3*(1.0-2.0*b2)*(b2-b2**2.0)**(1.0/p3-1.0)
			A[2,1] = 0.0
			A[2,2] = 1.0
			A[2,3] = 2.0*b2
			
			#4. Reihe
			A[3,0] = (b2-b2**2.0)**(1.0/p3)
			A[3,1] = 1.0
			A[3,2] = b2
			A[3,3] = b2**2.0
		
			b[0] = b0
			b[1] = b1
			b[2] = 0.0
			b[3] = b3
			
			z = numpy.linalg.inv(A)*b
			
			p0 = float(z[0])
			a0 = float(z[1])
			a1 = float(z[2])
			a2 = float(z[3])
			a3 = 0.0
			a4 = 0.0
			
		elif RandB == 3 :
		
			ordnung = 5
			A = numpy.mat(numpy.ones([ordnung,ordnung], 'd'))
			b = numpy.mat(numpy.ones([1,ordnung], 'd')).transpose()
			
			#1. Reihe
			A[0,0] = 0.0
			A[0,1] = 1.0
			A[0,2] = 0.0
			A[0,3] = 0.0
			A[0,4] = 0.0
			
			#2. Reihe
			A[1,0] = 0.0
			A[1,1] = 1.0
			A[1,2] = 1.0
			A[1,3] = 1.0
			A[1,4] = 1.0
			
			#3. Reihe
			A[2,0] = 1.0/p3*(1.0-2.0*b2)*(b2-b2**2.0)**(1.0/p3-1.0)
			A[2,1] = 0.0
			A[2,2] = 1.0
			A[2,3] = 2.0*b2
			A[2,4] = 3.0*b2**2.0
			
			#4. Reihe
			A[3,0] = (b2-b2**2.0)**(1.0/p3)
			A[3,1] = 1.0
			A[3,2] = b2
			A[3,3] = b2**2.0
			A[3,4] = b2**3.0
			
			#5. Reihe
			A[4,0] = ((b2-b2**2.0)**(1.0/p3-2.0))/p3 * ((1.0/p3 - 1.0) * (1.0-2.0*b2)**2.0 - 2.0*(b2-b2**2.0))
			A[4,1] = 0.0
			A[4,2] = 0.0
			A[4,3] = 2.0
			A[4,4] = 6.0*b2
			
			b[0] = b0
			b[1] = b1
			b[2] = 0.0
			b[3] = b3
			b[4] = b4
			
			z = numpy.linalg.inv(A)*b
			
			p0 = float(z[0])
			a0 = float(z[1])
			a1 = float(z[2])
			a2 = float(z[3])
			a3 = float(z[4])
			a4 = 0.0
			
		elif RandB == 4 :
			
			ordnung = 5
			A = numpy.mat(numpy.ones([ordnung,ordnung], 'd'))
			b = numpy.mat(numpy.ones([1,ordnung], 'd')).transpose()
			
			#1. Reihe
			A[0,0] = 1.0
			A[0,1] = 0.0
			A[0,2] = 0.0
			A[0,3] = 0.0
			A[0,4] = 0.0
			
			#2. Reihe
			A[1,0] = 1.0
			A[1,1] = 1.0
			A[1,2] = 1.0
			A[1,3] = 1.0
			A[1,4] = 1.0
			
			#3. Reihe
			A[2,0] = 0.0
			A[2,1] = 1.0
			A[2,2] = 2.0*b2
			A[2,3] = 3.0*b2**2.0
			A[2,4] = 4.0*b2**3.0
			
			#4. Reihe
			A[3,0] = 1.0
			A[3,1] = b2
			A[3,2] = b2**2.0
			A[3,3] = b2**3.0
			A[3,4] = b2**4.0
			
			#5. Reihe
			A[4,0] = 0.0
			A[4,1] = 0.0
			A[4,2] = 2.0
			A[4,3] = 6.0*b2
			A[4,4] = 12.0*b2**2.0
			
			b[0] = b0
			b[1] = b1
			b[2] = -p0/p3*(1.0-2.0*b2)*(b2-b2**2.0)**(1.0/p3-1.0)
			b[3] = b3-p0*(b2-b2**2.0)**(1.0/p3)
			b[4] = b4-p0/p3*(b2-b2**2.0)**(1.0/p3-2.0) * ((1.0/p3 - 1.0)*(1.0-2.0*b2)**2.0 - 2.0*(b2-b2**2.0))
			
			z = numpy.linalg.inv(A)*b
			
			a0 = float(z[0])
			a1 = float(z[1])
			a2 = float(z[2])
			a3 = float(z[3])
			a4 = float(z[4])
			
		else:
			raise Exception, "Polynom Koeffizienten: Wert für Bound ungültig"

		if self.debug > 0:
			print "Koeffizienten für Polynom Parameterbestimmung, Randbedingung, b0-b4, p0-p3:",RandB,b0,b1,b2,b3,b4,p0,p1,p2,p3
			print "Ergebnisparameter für Polynom Dickenverteilung, a0-a4, p0-p3:",a0,a1,a2,a3,a4,p0,p1,p2,p3
		
		CoeffArray = numpy.zeros((9))
		
		CoeffArray[0]= a0
		CoeffArray[1]= a1
		CoeffArray[2]= a2
		CoeffArray[3]= a3
		CoeffArray[4]= a4
		CoeffArray[5]= p0
		CoeffArray[6]= p1
		CoeffArray[7]= p2
		CoeffArray[8]= p3
		
		return CoeffArray

	## @brief Computes the coefficients for a polynomial camber line
	# @brief With the angle of attack \f$ \chi_1 \f$ and the outflow angle \f$ \chi_2 \f$
	# @brief \f[\alpha=\chi_1 - \frac{\pi}{2} , \hspace{0,5cm} \beta=\frac{\pi}{2} - \chi_2 \f]
	# @brief a camber line of degree 4:
	# @brief \f[f(x)=a_4 x^{4} + a_3 x^{3} + a_2 x^{2} + a_1 x + a_0  \f]
	# @brief is calculated with the boundary conditions: \f[f(0)=0, \hspace{0,5cm} f(1)=0, \hspace{0,5cm} f'(0)=tan(\alpha), \hspace{0,5cm} f'(1)=-tan(\beta) \f]
	# @brief \n and \f$ f'(x_{max,f})=0 \f$ with \f$ x_{max,f} \f$ as maximum camber position.
	# @brief \n The program shows inflection points and their discriminants, if existing.
	# @brief \n\n A camber line of degree 3:
	# @brief \f[f(x)=a_3 x^{3} + a_2 x^{2} + a_1 x + a_0 \f]
	# @brief is calculated with \f[f(0)=0, \hspace{0,5cm} f(1)=0, \hspace{0,5cm} f'(0)=tan(\alpha), \hspace{0,5cm} f'(1)=-tan(\beta) \f]
	# @brief \n The program shows inflection points and their discriminants, if existing.
	# @brief \n\n A camber line of degree 2:
	# @brief \f[f(x) = a_2 x^2 + a_1 x + a_0 \f]
	# @brief is calculated with \f[f(0)=0, \hspace{0,5cm} f(1)=0, \hspace{0,5cm} f'(0)=tan(\alpha), \hspace{0,5cm} \alpha=\beta \f]
	# @brief \n In this case the maximum camber position is always 0.5.
	# @param alpha acute angle of attack in profile coordinates
	# @param beta acute outflow angle in profile coordinates
	# @return coefficients for the polynomial camber line
	def computePolynomialCoefficients(self):
		#profilbezogene winkel!!!
		alpha=numpy.radians(self.angleOfAttack-90.0)
		beta=numpy.radians(90.0-self.outflowAngle)
		
		if self.polyCamberLineDegree == 4:
			b = self.maxCambPosition
			if b < 0.0 or b > 1.0:
				raise Exception, "Polynom Skelettlinie: Wert für maxCambPosition ungültig"
			#polynom grad 4, def über an-/abströmwinkel und wölbungsrücklage, kontrolle: wendepunkte
			a4 = (numpy.tan(alpha)*(-3.0*b**2.0+4.0*b-1.0)+numpy.tan(beta)*(3.0*b**2.0-2.0*b))/(4.0*b**3.0-6.0*b**2.0+2.0*b)
			a3 = numpy.tan(alpha) - numpy.tan(beta) - 2.0*a4
			a2 = -numpy.tan(alpha)-a3-a4
			a1 = numpy.tan(alpha)
			a0 = 0.0
			
			#wendepunkte, 2.ableitung = 0
			b1 = -a3/(4.0*a4) + numpy.sqrt((3.0*a3**2.0-8.0*a2*a4)/(48.0*a4**2.0))
			b2 = -a3/(4.0*a4) - numpy.sqrt((3.0*a3**2.0-8.0*a2*a4)/(48.0*a4**2.0))
			#diskriminante der wurzel
			c = 3.0*a3**2.0-8.0*a2*a4
			
			if self.debug > 0:
				print "Wölbungsrücklage:", b
				if c >= 0.0:
					print "Wendepunkte der Skelettlinie:", b1," ", b2, " Diskriminante:", c
		
		elif self.polyCamberLineDegree == 3:
			#polynom grad 3, def über an-/abströmwinkel, kontrolle: wölbungsrücklage
			a4 = 0.0
			a3 = numpy.tan(alpha)-numpy.tan(beta)
			a2 = -a3-numpy.tan(alpha)
			a1 = numpy.tan(alpha)
			a0 = 0.0
			
			#wölbungsrücklage, 1.ableitung = 0
			b1 = -a2/(3.0*a3) + numpy.sqrt((a2**2.0-3.0*a1*a3)/((3.0*a3)**2.0))
			b2 = -a2/(3.0*a3) - numpy.sqrt((a2**2.0-3.0*a1*a3)/((3.0*a3)**2.0))
			
			#f''(b)< 0 => maximum
			if 6.0*a3*b1+2.0*a2 < 0.0:
				b=b1
			
			if 6.0*a3*b2+2.0*a2 < 0.0:
				b=b2
			
			#wölbungsrücklage, wenn a3 = 0
			if a3==0.0 and a2 < 0.0:
				b = -a1/(2.0*a2)
			
			#wendepunkte, 2.ableitung = 0
			w = -a2/(3.0*a3)
			
			if self.debug > 0:
				print "Wölbungsrücklage:", b
				if w > 0.0 and w < 1.0:
					print "Wendepunkt der Skelettlinie:",w
		
		elif self.polyCamberLineDegree == 2:
			#polynom grad 2, def über anströmwinkel
			a4 = 0.0
			a3 = 0.0
			a2 = -numpy.tan(alpha)
			a1 = numpy.tan(alpha)
			a0 = 0.0
			
		else :
			raise Exception, "Wert für polyCamberLineDegree ungültig"
		
		if self.debug > 0:
			print "Grad der Skelettlinie:", self.polyCamberLineDegree
			if self.debug > 1:
				print "Koeffizienten der Skelettlinie, a0-a4:",a0,a1,a2,a3,a4
		
		return a0,a1,a2,a3,a4
	
	## @brief Calls circular arc camber line distribution and calculation methods, calls the polynomial thickness function, if required.
	# @param camberarray array for circular arc camber line points
	# @param pvCamber equal distributed x - coordinates for circular arc camber line
	# @param pointOnCamberLine x - and y - coordinates of a point on the camber line
	# @return list of points on the circular arc camberline or the whole profile
	def computeCircularArcCamberLine(self):
		if self.flagThickDistr==0 or self.flagThickDistr==3:
			camberarray=numpy.zeros([2,self.camberResolution],'d')
			
			pvCamber=self.distributeCircularArcCamberLine()
			
			if self.debug > 0:
				print "Auflösung der Kreisbogen - Skelettlinie:",self.camberResolution
			
			for i in range(int(self.camberResolution)):
				pointOnCamberLine=self.coordinatesCircularArcCamberLine(pvCamber[i])
				camberarray[0][i]=pointOnCamberLine[0]
				camberarray[1][i]=pointOnCamberLine[1]
			
			if self.debug > 1:
				print "Kreisbogen - Skelettlinie:",camberarray
			
			self.gesarray.append(camberarray[0])
			self.gesarray.append(camberarray[1])
			
		elif self.flagThickDistr==1 or self.flagThickDistr==2:
			self.computePolynomialCamberline()
		
		else:
			raise Exception, "Wert für flagThicknessDistribution ungültig"
		
	## @brief Calls double circular arc thickness distribution methods
	# @param pointDistribution distributed x - coordinates for the pressure side, suction side and leading/trailing edge
	# @param presssidearray array of pressure side points
	# @param pointOnPressure x - and y - coordinates of one point on the pressure side
	# @param suctsidearray array of suction side points
	# @param pointOnSuction x - and y - coordinates of one point on the suction side
	# @return lists of points on the suction and pressure side of a double circular arc profile
	def computeDoubleCircularArcProfile(self):
		if self.debug > 0:
			print "Vorderkantenradius = Hinterkantenradius:",self.leadingEdgeRadius
			print "maximale Dicke:", self.maxThickness
			
		if self.leadingEdgeRadius > 0.1 or self.leadingEdgeRadius <= 0.0:
			raise Exception, "Wert für leadingEdgeRadius ungültig"
		
		if self.maxThickness < 0.0 or self.maxThickness > 0.2:
			raise Exception, "Wert für maxThickness ungültig"
		
		
		pointDistribution=self.distributeDoubleCircularArc()
		
		presssidearray=numpy.zeros([2,len(pointDistribution[0])], 'd')
		for i in range(len(pointDistribution[0])):
			pointOnPressure=self.calculateDoubleCircularArcProfile(pointDistribution[0][i],1,self.gamma_r0_ratio_press,self.totalAnglePress,self.thetaPress2,self.R_press)
			presssidearray[0][i]=pointOnPressure[0]
			presssidearray[1][i]=pointOnPressure[1]
		
		suctsidearray=numpy.zeros([2,len(pointDistribution[1])], 'd')
		for i in range(len(pointDistribution[1])):
			pointOnSuction=self.calculateDoubleCircularArcProfile(pointDistribution[1][i],0,self.gamma_r0_ratio_suction,self.totalAngleSuction,self.thetaSuction2,self.R_suction)
			suctsidearray[0][i]=pointOnSuction[0]
			suctsidearray[1][i]=pointOnSuction[1]
		
		if self.debug > 1:
			print "Doppelkreisbogen, Druckseite:",presssidearray
			print "Saugseite:",suctsidearray
		
		self.gesarray.append(presssidearray[0])
		self.gesarray.append(presssidearray[1])
		
		self.gesarray.append(suctsidearray[0])
		self.gesarray.append(suctsidearray[1])
		
	## @brief Computes the equal point distribution of circular arc camber lines on the x-axis, which is used for calculating the points on the profile
	# @param camberDistr equal point distribution of the x - coordinate array
	# @return point distribution array of the camberline
	def distributeCircularArcCamberLine(self):
		camberDistr=[]
		for i in range(int(self.camberResolution)):
			camberDistr.append(i/float(self.camberResolution-1))
		return camberDistr
	
	
	## @brief Calculates the relevant parameters for a double circular arc profile and the number of points on the pressure/suction side
	# @brief At first all relevant parameters for the calculation of a double circular arc profile are assigned. \n From the angular correlations \f[\Delta x_{pressure} = 0,5 - r_0 \cos\left(\frac{\theta}{2}\right) \hspace{1,5cm} \Delta y_{pressure} = R_{pressure} + \frac{D_{max}}{2} - y_0 + r_0 \sin\left(\frac{\theta}{2}\right) \f] and \f[\Delta x_{pressure}^2 + \Delta y_{pressure}^2 = ( R_{pressure} + r_0)^2\f] summing-up known parameter: \f[d_{pressure} :=  - \frac{D_{max}}{2} + y_0 - r_0 \sin\left(\frac{\theta}{2}\right)\f] the Radius of the pressure side arc is calculated: \f[R_{pressure} = 	\frac{ d_{pressure}^2 - r_0^2 + \left(0,5 - r_0 \cos\left(\frac{\theta}{2}\right)\right)^2 }{ 2 (d_{pressure} + r_0)}\f] Furthermore the opening angle of the pressure side arc is computed with:	\f[\tan\left(\frac{\theta_{pressure,arc}}{2}\right) = \frac{\Delta x_{pressure}}{\Delta y_{pressure}} \f] and \f[\frac{\theta_{pressure,arc}}{2} = \arctan\left(\frac{ 0,5 - r_0 \cos\left(\frac{\theta}{2}\right) }{ R_{pressure} + \frac{D_{max}}{2} - y_0 + r_0 \sin\left(\frac{\theta}{2}\right) }\right)\f] At last the opening angle part of the pressure side at the nose radius is evaluated \f[\theta_{r_0,pressure} = \frac{\pi}{2} + \frac{\theta_{pressure,arc}}{2}  - \frac{\theta}{2}\f]
	# @brief \n\n\n From the angular correlations \f[\Delta x_{suction} = 0,5 - r_0 \cos\left(\frac{\theta}{2}\right) \hspace{1,5cm} \Delta y_{suction} = R_{suction} - \frac{D_{max}}{2} - y_0 + r_0 \sin\left(\frac{\theta}{2}\right)\f] and \f[\Delta x_{suction}^2 + \Delta y_{suction}^2 = ( R_{suction} - r_0)^2\f] summing-up known parameter: \f[d_{suction} := \frac{D_{max}}{2} + y_0 - r_0 \sin\left(\frac{\theta}{2}\right)\f] the Radius of the suction side arc is calculated:	\f[R_{suction} = \frac{ d_{suction}^2 - r_0^2 + \left(0,5 - r_0 \cos\left(\frac{\theta}{2}\right)\right)^2 }{ 2 (d_{suction} - r_0)}\f] Furthermore the opening angle of the suction side arc is computed with: \f[\tan\left(\frac{\theta_{suction,arc}}{2}\right) = \frac{\Delta x_{suction}}{\Delta y_{suction}}\f] and		\f[\frac{\theta_{suction,arc}}{2} = \arctan\left(\frac{ 0,5 - r_0 \cos\left(\frac{\theta}{2}\right) }{ R_{suction} - \frac{D_{max}}{2} - y_0 + r_0 \sin\left(\frac{\theta}{2}\right) }\right)\f] At last the opening angle part of the suction side at the nose radius is evaluated \f[\theta_{r0,suction} = \frac{\pi}{2} - \frac{\theta_{suction,arc}}{2} + \frac{\theta}{2}\f]
	# @brief the opening angles of each side are accumulated \f[\theta_{suction} = \theta_{suction,arc} + 2 \theta_{r0,suction} \hspace{2cm} \theta_{pressure} = \theta_{pressure,arc} + 2 \theta_{r0,pressure} \f] and after that the relative ratio of the point distribution is calculated: \f[	n_{suction} = \frac{\theta_{suction}}{\theta_{suction}+\theta_{pressure}} n_{profil} = \frac{n_{profil}}{1+ \frac{\theta_{pressure}}{\theta_{suction}}} \hspace{1cm} n_{pressure} = \frac{\theta_{pressure}}{\theta_{suction}+\theta_{pressure}} n_{profil} = \frac{n_{profil}}{1+ \frac{\theta_{suction}}{\theta_{pressure}}}\f]
	def distributeDoubleCircularArc(self):
		points_press=[]
		points_suct=[]
		
		theta2=abs(numpy.radians(self.angleOfAttack-self.outflowAngle)/2)
		
		r_0=self.leadingEdgeRadius
		maxdicke2=self.maxThickness/2
		
		sl=self.coordinatesCircularArcCamberLine(0.5)
		## max camber of a circular arc camberline
		self.y0=abs(sl[1])
		
		d_press = self.y0 - maxdicke2 - r_0*numpy.sin(theta2)
		
		## pressure side radius of a double circular arc thickness distribution
		self.R_press = (d_press**2 - r_0**2 + (1.0/2 - r_0*numpy.cos(theta2))**2) / (2*(d_press + r_0))
		
		## opening angle of the pressure side arc
		self.thetaPress2 = numpy.arctan((1.0/2 - r_0*numpy.cos(theta2)) / (self.R_press - self.y0 + maxdicke2 + r_0*numpy.sin(theta2)))
		
		## pressure side part of noseradius
		self.gamma_r0_ratio_press = self.thetaPress2 + numpy.pi/2 - theta2
		
		d_suction = self.y0 + maxdicke2 - r_0*numpy.sin(theta2)
		
		## suction side radius of a double circular arc thickness distribution
		self.R_suction = (d_suction**2 - r_0**2 + (1.0/2 - r_0*numpy.cos(theta2))**2) / ( 2*(d_suction - r_0))
		
		## opening angle of the suction side arc
		self.thetaSuction2 = numpy.arctan((1.0/2 - r_0*numpy.cos(theta2))/(self.R_suction - self.y0 - maxdicke2 + r_0*numpy.sin(theta2)))
		
		## suction side part of noseradius
		self.gamma_r0_ratio_suction = numpy.pi/2 - self.thetaSuction2 + theta2
		
		## total opening angle of the pressure side
		self.totalAnglePress = 2*(self.gamma_r0_ratio_press + self.thetaPress2)
		
		## total opening angle of the suction side
		self.totalAngleSuction = 2*(self.gamma_r0_ratio_suction + self.thetaSuction2)
		
		num_press=int(round(self.profileResolution/(1+(self.totalAngleSuction/self.totalAnglePress)) , 0))
		for i in range(num_press):
			points_press.append(i/float(num_press-1))

		num_suct=int(round(self.profileResolution/(1+(self.totalAnglePress/self.totalAngleSuction)) ,0))
		for i in range(num_suct):
			points_suct.append(i/float(num_suct-1))
			
		return points_press,points_suct
	
	## @brief Calculates a point on the circular arc camber line
	# @brief The opening angle of the circular arc camber line is calculated by the use of the angle of attack and the outflow angle: \f[\theta = \chi_1 - \chi_2\f]
	# @brief For a circular arc camberline the following applies:
	# @brief \f[x=R_{camber}\cdot sin(x_0\cdot\theta-\frac{\theta}{2}) + \frac{1}{2} \hspace{1cm} y=R_{camber}\cdot cos(x_0\cdot\theta-\frac{\theta}{2}) + y_{Center} \f]
	# @brief The origin lies halfway on the chord.
	# @param x0 \f$ x_0 \f$ - Normalized value, for which the point on the camber line is being calculated.
	# @return x-coordinate of the point on the camber line
	# @return y-coordinate of the point on the camber line
	def coordinatesCircularArcCamberLine(self,x0):
		theta2=numpy.radians(self.angleOfAttack-self.outflowAngle)/2
		curvatureRadius=1.0/(2*numpy.sin(theta2))
		
		ycenter=-curvatureRadius*numpy.cos(theta2)
		xcamberline=curvatureRadius*numpy.sin(x0*theta2*2-theta2)+1.0/2
		ycamberline=curvatureRadius*numpy.cos(x0*theta2*2-theta2)+ycenter
		return xcamberline, ycamberline
	
	
	## @brief Calculates the coordinates of a point on the pressure/suction side of the profile
	# @brief The center points of the leading edge radius (LE) and the trailing edge radius (TE) are calculated with: \f[x_{m,r_0,LE} = r_0 \cos\left(\frac{\theta}{2}\right) \hspace{2cm} y_{m,r_0,LE} = r_0 \sin\left(\frac{\theta}{2}\right)\f] \f[x_{m,r_0,TE} = 1 - x_{m,r_0,LE} \hspace{2cm} y_{m,r_0,TE} = y_{m,r_0,LE}\f]
	# @brief The normalized value \f$ x_0\f$ of a point on the arc is dimensioned with the opening angle of pressure or suction side: \f[\alpha_{ctrl,0} = x_0  \theta_{suction/pressure}\f]
	# @brief A point on a circular arc of the pressure or suction side is calculated with: \f[x = 0,5 - R_{suction/pressure} \sin\alpha_{arc} \hspace{2cm} y = y_{Center} + R_{suction/pressure} \cos\alpha_{arc} \f] with \f[\alpha_{arc} = \frac{\theta_{suction,arc/pressure,arc}}{2} - (\alpha_{ctrl,0} - \theta_{r0,suction/pressure}) \f]  \f[y_{Center,pressure} = y_0 - \frac{D_{max}}{2} - R_{pressure} \f] \f[y_{Center,suction} = y_0 + \frac{D_{max}}{2} - R_{suction}	\f] Before a point on a nose radius is evaluated, a usefull coordinate transformation is applied: \f[\alpha_{ctrl,LE,PS} = \frac{\theta}{2} + \alpha_{ctrl,0} \f]\f[\alpha_{ctrl,LE,SS} = \frac{\theta}{2}  - \alpha_{ctrl,0}	\f]\f[\alpha_{ctrl,TE,PS} = \alpha_{ctrl,0} - \theta_{pressure} + \pi - \frac{\theta}{2} \f]\f[	\alpha_{ctrl,TE,SS} = \frac{\theta}{2} - \alpha_{ctrl,0} \f]	Subsequently the point on a nose radius is calculated by calling the function calculateCircle.
	# @param ctrl_var \f$ \alpha_{ctrl,0}\f$ - the control variable to compute a point on the double circular arc profile with
	# @param ctrl_var1 \f$ \alpha_{ctrl,LE,PS}, \alpha_{ctrl,LE,SS}\f$ - the control variable for the leading edge (PS - pressure side,SS - suction side)
	# @param ctrl_var2 \f$ \alpha_{ctrl,TE,PS}, \alpha_{ctrl,TE,SS}\f$ - the control variable for the trailing edge (PS - pressure side,SS - suction side)
	# @param x0 \f$ x_0 \f$ - normalized value of a point on the arc 
	# @param pressindex flag for calculation of pressure or suction side
	# @param r0_ratioAngle \f$\theta_{r0,suction/pressure}\f$ - opening angle part of the pressure or suction side at the nose radius
	# @param totalAngle total \f$\theta_{suction/pressure}\f$ - opening angle of the pressure or suction side
	# @param thetaProfile2 \f$\frac{\theta_{suction,arc/pressure,arc}}{2}\f$ - opening angle of the pressure or suction side arc
	# @param r \f$R_{suction/pressure}\f$ - Radius of the pressure or suction side arc
	# @return x-value of the point on the profile
	# @return y-value of the point on the profile
	def calculateDoubleCircularArcProfile(self,x0,pressindex,r0_ratioAngle,totalAngle,thetaProfile2,r):
		theta2=abs(numpy.radians(self.angleOfAttack-self.outflowAngle)/2)
		sign=(numpy.radians(self.angleOfAttack-self.outflowAngle)/2)/theta2

		r_0=self.leadingEdgeRadius
		maxdicke2=self.maxThickness/2
		
		xmvkr=r_0*numpy.cos(theta2)
		ymvkr=r_0*numpy.sin(theta2)

		xmhkr=1.0-r_0*numpy.cos(theta2)
		ymhkr=ymvkr
		
		ctrl_var = x0*totalAngle
		
		if ctrl_var >= r0_ratioAngle and ctrl_var <= (totalAngle - r0_ratioAngle):
			
			if pressindex == 1:
				maxdicke2 = -maxdicke2
			
			phi = thetaProfile2 - (ctrl_var - r0_ratioAngle)
			
			x_1 = 1.0/2 - r*numpy.sin(phi)
			
			y_center = self.y0 + maxdicke2 - r
			
			y_1 = y_center + r*numpy.cos(phi)
			
			return x_1 , sign * y_1
		
		elif ctrl_var < r0_ratioAngle:
			
			if pressindex == 1:
			
				ctrl_var1 = theta2 + ctrl_var
			
			else:
				ctrl_var1 = theta2 - ctrl_var
		
			leadingEdgeCircle = self.calculateCircle(ctrl_var1, r_0, xmvkr, ymvkr)
			
			return leadingEdgeCircle[0], sign*leadingEdgeCircle[1]
		
		elif ctrl_var > (totalAngle - r0_ratioAngle):
			
			if pressindex == 1:
				ctrl_var2 = numpy.pi - theta2 - totalAngle + ctrl_var
				
			else:
				ctrl_var2 = theta2 - ctrl_var
				
			trailingEdgeCircle = self.calculateCircle(ctrl_var2 , r_0 , xmhkr , ymhkr)
			
			return trailingEdgeCircle[0], sign*trailingEdgeCircle[1]	

	## @brief Computes a weighted point distribution and calculates the camber line at these points; calls the function to superpose the thickness distribution with the camber line
	# @brief To distribute the discret points on the camber line a weight function is necessary.
	# @brief \n An iterating Tridiagonal Matrix Algorithm (TDMA) is solving this Finite Difference problem.
	# @brief Each equation contains only the variables at its own node and its immediate left and right neighbours,\n for \f$ i = 0 \f$ . . . \f$ n-1 \f$ :
	# @brief \f[A_W^i x_{i-1} + A_P^i x_i + A_E^i x_{i+1} = Q_i \f]
	# @brief with: \f[x_i = 0 \hspace{0,5cm} Q_i = 0 \hspace{0,5cm} A_W^i = 1 \hspace{0,5cm} A_P^i = -2 \hspace{0,5cm} A_E^i = 1 \f]
	# @brief To begin with an equal point distribution, the boundary conditions are:
	# @brief \f[x_{n-1} = 1 \hspace{0,5cm} Q_1 = -A_W^1 x_0 \hspace{0,5cm} Q_{n-2} = -A_E^{n-2} x_{n-1} \f] and after that: \f[A_W^1 = 0 \hspace{0,5cm} A_E^{n-2} = 0\f]
	# @brief for \f$ i = 1 \f$ . . . \f$ n-2 \f$ : \f[x_i = \frac{1}{n-1} + x_{i-1} \f]
	# @brief results in an equal distributed array with \f$ x_0 = 0 \f$ and \f$ x_{n-1} = 1 \f$ .
	# @brief \n The following equation contains the weight function:
	# @brief \f[x_{i+1} - 2x_i + x_{i-1} - \frac{1}{4w(x_i)}(x_{i+1} - x_{i-1})(w(x_{i+1}) - w(x_{i-1})) = 0 \f]
	# @brief So the new values are: \f[A_W = 1 - \frac{w(x_{i+1}) - w(x_{i-1})}{4 w(x_i)} \hspace{0,5cm} A_E = 1 + \frac{w(x_{i+1}) - w(x_{i-1})}{4 w(x_i)} \f]
	# @brief With the boundary conditions from above: \f[x_{n-1} = 1 \hspace{0,5cm} Q_1 = -A_W^1 x_0 \hspace{0,5cm} Q_{n-2} = -A_E^{n-2} x_{n-1} \f] and after that: \f[A_W^1 = 0 \hspace{0,5cm} A_E^{n-2} = 0\f]
	# @brief we can start with the solver part.
	# @brief \n The Tridiagonal Matrix Algorithm solves the equation, so the forward elimination modifies, \n for \f$ i = 1 \f$ . . . \f$ n-1 \f$ :
	# @brief \f[A_P^{i*} = A_P^i - \frac{A_W^{i*} A_E^{i-1*}}{A_P^{i-1*}} \f]
	# @brief To avoid zero - division errors, we define
	# @brief \f[B_P^i = (A_P^{i*})^{-1} = \frac{1}{A_P^i - A_W^{i*} A_E^{i-1*} B_P^{i-1*}} \f]
	# @brief The forcing term becomes: \f[R_i = Q_i^* = Q_i - \frac{A_W^{i*} Q_{i-1}^*}{A_P^{i-1*}} = Q_i - A_W^{i*} Q_{i-1}^* B_P^{i-1} \f]
	# @brief The x - Array is backward eliminated, for \f$ i = n-2 \f$ . . . \f$ 1 \f$:
	# @brief \f[x_i = \frac{Q_i^* - A_E^{i*} x_{i+1}}{A_P^{i*}} = (Q_i^* - A_E^{i*} x_{i+1}) B_P^i \f]
	# @brief With the new x - vector we can appropriate new \f$ A_W \f$'s and \f$ A_E \f$'s using the weight function, followed by forward and backward elimination again.
	# @brief \n So these equations are solved iterative.
	# @brief \n To delimitate this calculation, we define an abort criterion.
	# @brief \n A residual should be: \f[|\vec{Res}| = \sqrt{ \sum_{i=0}^{n-1}(x_{old}^i - x_{new}^i)^2 } \f]
	# @brief \n The residual in the first iteration step is used as reference.
	# @brief \n Every new residual is compared with it. \n The solver will quit, if the variation is less then 0,1% of the reference or the iteration number rises above 50 steps (avoidance of infinite loops).
	# @brief \n\n The camber line is calculated by using the weighted x - array and the function "computeProfile" superposes the thickness distribution.
	# @brief \n Camber line methods are: Circular arc, polynomial, NURBS and NACA 65.
	# @brief \n The circular arc camber line is generated by calling the function "coordinatesCircularArcCamberLine".
	# @brief \n The polynomial camber line function is \f[	f(x) = a_4 x^4 + a_3 x^3 + a_2 x^2 + a_1 x + a_0 \f]
	# @brief \n The NACA-camber line is assumed by using the following function: \f[f(x) = - \frac{C_{A,0}}{4\pi} [(1 - x) \ln(1 - x) + x \ln(x) ] \f]
	# @brief \n A NURBS-camber line is calculated by calling the NURBS-method computeNurbsCurve using the array of NURBS-control points. After that the angle of attack and the outflow angle are impressed by setting the tangent to the leading and trailing edges.
	# @param aw coefficient for TDMA
	# @param ap coefficient for TDMA
	# @param ae coefficient for TDMA
	# @param q coefficient for TDMA
	# @param x point array to distribute
	# @param bp is new Ap
	# @param r is new Q
	# @param w weight function of polynom
	# @param v weight function of polynomial part
	# @param g other weight function
	# @param res residuum, abort criterion
	# @param y is new X
	# @param profil y-coordinate of the profile camberline
	# @param pointOnCamberLine x - and y - coordinates of a point on the circular arc camber line
	# @param curveCamber NURBS - curve camber line
	# @param point point on the NURBS - curve close to the given x - coordinate
	# @param coordinatesCamberline assembled camber line coordinates
	# @return array with all points on the camber line, pressure side and suction side
	def computePolynomialCamberline(self):
		
		if self.flagThickDistr < 0 or self.flagThickDistr > 2:
			raise Exception,"Wert für flagThicknessDistribution ungültig"
		
		ordnung=int(self.profileResolution)
		
		aw= numpy.ones([ordnung], 'd')
		ap= -2.0 * numpy.ones([ordnung], 'd')
		ae= numpy.ones([ordnung], 'd')
		
		q= numpy.zeros([ordnung], 'd')
		
		x= numpy.zeros([ordnung], 'd')
		
		bp= numpy.zeros([ordnung], 'd')
		r= numpy.zeros([ordnung], 'd')
		w= numpy.zeros([ordnung], 'd')
		v= numpy.zeros([ordnung], 'd')
		g= numpy.zeros([ordnung], 'd')
		
		res= numpy.zeros([ordnung-2], 'd')
		y= numpy.zeros([ordnung], 'd')
		
		profil= numpy.zeros([ordnung],'d')
		
		## list of residuals for debugging use
		self.residuenlist = []
		
		#Boundary values x
		x[ordnung-1]=1.0
		
		#inner points
		#ap[:]=-2.0
		#ae[:]=1.0
		#aw[:]=1.0

		# left bound
		#i=1
		#q[i]=-aw[i]*x[i-1]
		#aw[i]=0.0
		
		## right bound
		#i=ordnung-2
		#q[i]=-ae[i]*x[i+1]
		#ae[i]=0.0
		
		i=1
		while i < ordnung-1:
			x[i]=1.0/(ordnung-1)+x[i-1]
			i+=1
		
		#end equal point distribution
		
		# iteration counter
		j=0
		resb=1
		resb1=0
		
		if self.debug > 0:
			print "Kombination von Gewichtungsfunktionen:",self.weightCombination
			if self.weightCombination==2:
				print "Koeffizient für sonstige Gewichtungsfunktionen:",self.weightCondition
			
		
		while resb > 0.001*resb1:
			# global maximum of w
			wb=0.0
			
			for i in range(ordnung):
				if x[i] != 0.0 and x[i] != 1.0:
				
					if self.weightCombination==0 or self.weightCombination==3:
						w[i]=self.give_sym_w(x[i])	#Gewichtung
						
					if self.weightCombination!=0:
						g[i]=self.give_sym_g(x[i])
						if self.weightCombination!=2:
							v[i]=self.give_sym_v(x[i])
			
			for i in range(6):
				g[i+1] *= 4.0
				g[-2-i] *= 4.0
			
			wb = numpy.max(w)
			gb = numpy.max(g)
			vb = numpy.min(v)
			
			w[0]=3*w[1]
			w[ordnung-1]=3*w[ordnung-2]
			
			v[0]=v[1]
			v[ordnung-1]=v[ordnung-2]
			
			g[0]=4*g[1]
			g[ordnung-1]=4*g[ordnung-2]
			
			if self.weightCombination==0:
				w[:]/= 3.0 * wb	#normalized: 0<w<1
				w[:]+=0.02
			elif self.weightCombination==1:
				v[:]-=vb
				v[:]+=0.05
				w[:]=v[:] / 3.0
			elif self.weightCombination==2:
				w[:]=g[:]/(4.0 * gb)
			elif self.weightCombination==3:
				w[:]/= 3.0 * wb	#normalized: 0<w<1
				w[:]+=0.02
				
				g[:]/= 4.0 * gb 
				
				w[:]=w[:]+g[:]
				w[:]/=2.0
			else:
				raise Exception,"Wert für weightCombination ungültig"
			
			i=0
			while i < ordnung-1:
				aw[i] = 1.0 - (w[i+1] - w[i-1])/(4.0*w[i])	#neues Aw mit Gewichtung
				ae[i] = 1.0 + (w[i+1] - w[i-1])/(4.0*w[i])	#neues Ae mit Gewichtung
				
				i+=1
			
			# left bound
			i=1
			q[i]=-aw[i]*x[i-1]
			aw[i]=0.0
		
			# right bound
			i=ordnung-2
			q[i]=-ae[i]*x[i+1]
			ae[i]=0.0
			
		#Solver Part
		#forward elimination
			i=1
			y[:] = x[:]	#for residuum
			while i < ordnung:
				bp[i]= 1.0/(ap[i]-aw[i]*ae[i-1]*bp[i-1])
				r[i]=q[i]-aw[i]*r[i-1]*bp[i-1]
				i+=1
			
		#backward  elimination
			i=ordnung-2
			while i > 0:
				x[i]=(r[i]-ae[i]*x[i+1])*bp[i]
				i-=1
			
			i=1
			resb=0
			while i < ordnung-1:	#abbruch krit
				res[i-1]=y[i]-x[i]
				resb+=res[i-1]**2
				i+=1
			resb=numpy.sqrt(resb)		#resb=sqrt( (x_alt[0]-x_neu[0])**2 + (x_alt[1]-x_neu[1])**2 ...etc.)
			
			if j==0:
				resb1=resb	#1st residuum for abort criterion
			
			self.residuenlist.append(resb)
			
			j+=1
			
			#abbruch von endlosschleifen
			if j>=50:
				if self.debug > 0:
					print "Warnung: nicht auskonvergiert!"
				break
			
		## weightfunction for debugging use
		self.polyweight = w
		
		if self.debug > 0:
				print "Iterationsschritte:" ,j
			
		# Erzeugen des Profil Objektes
		if self.flagCamberLine==0:		#circular arc Camberline
			for n in range(ordnung):
				pointOnCamberLine=self.coordinatesCircularArcCamberLine(x[n])
				
				x[n]=pointOnCamberLine[0]
				profil[n]=pointOnCamberLine[1]
		
		elif self.flagCamberLine==1:		#polynomial Camberline
			a0=self.polyCambRBs[0]
			a1=self.polyCambRBs[1]
			a2=self.polyCambRBs[2]
			a3=self.polyCambRBs[3]
			a4=self.polyCambRBs[4]
			
			profil[:] = a4*x[:]**4.0+a3*x[:]**3.0+a2*x[:]**2.0+a1*x[:]+a0
			
			for i in range(ordnung):
				profil[i]=round(profil[i],8)
			
		elif self.flagCamberLine==2:		#nurbs Camberline
			if self.debug > 1:
				print "Kontrollpunkte der NURBS - Skelettlinie:", self.controlPointsCamb
			
			self.curveCamber=self.computeNurbsCurve(self.controlPointsCamb)
			
			alpha = numpy.radians(self.angleOfAttack-90.0)
			beta = numpy.radians(90.0-self.outflowAngle)
			
			self.curveCamber.setTangentAtEnd(NurbsPoint.Point2Dd(1.0,numpy.tan(alpha)),NurbsPoint.Point2Dd(1.0,-numpy.tan(beta)))
			
			for n in range(ordnung):
				if x[n] != 0 and x[n] != 1:
					point = self.curveCamber.pointAt(self.calculateUvalue(x[n],self.curveCamber))
					
					profil[n]=point.gety()
					
					if self.flagCamberLine!=2:
						if profil[n] == profil[n-1]:
							raise Exception,"Abbruchwerte fuer MinDistX - Funktion zu gering"
			
			x[0]==0
			x[-1]==1
			profil[0] = 0.0
			profil[-1] = 0.0
				
			if self.debug > 0:
				print "post - script file NURBS - Skelettlinie: nurbsCamb.ps"
				self.curveCamber.writePS("nurbsCamb.ps",self.cp,self.magFact,self.dash)
		
		elif self.flagCamberLine==3:
			profil[:] = - self.Ca0/(4*numpy.pi)*((1-x[:])*numpy.log(1-x[:]) + x[:]*numpy.log(x[:]))
			
			x[0] = 0.0
			x[-1] = 1.0
			profil[0] = 0.0
			profil[-1] = 0.0
			
			for i in range(ordnung):
				profil[i]=round(profil[i],8)
		
		if self.debug > 1:
			print "Gewichtungsfunktion:",w
			print "Skelettlinie Punkteverteilung:",x
			print "Skelettlinie Wölbungsverteilung:",profil
		
		if self.flagThickDistr==0:
			self.gesarray.append(x)
			self.gesarray.append(profil)
		
		elif self.flagThickDistr==1 or self.flagThickDistr==2:
			coordinatesCamberline= numpy.zeros([2,self.profileResolution], 'd')
			coordinatesCamberline[0,:]=x[:]
			coordinatesCamberline[1,:]=profil[:]
			
			koordinatenProfil=self.computeProfile(coordinatesCamberline)
			
			self.gesarray.append(x)
			self.gesarray.append(profil)
			self.gesarray.append(koordinatenProfil[0][0])
			self.gesarray.append(koordinatenProfil[0][1])
			self.gesarray.append(koordinatenProfil[1][0])
			self.gesarray.append(koordinatenProfil[1][1])

	## @brief Calculates the u - value at a x - position
	# @brief The function minDistX iterates to find the closest point to the x - coordinate laying on the curve
	# @param	xValue		x - value
	# @param	Curve		the NURBS curve
	# @return	u parameter
	def calculateUvalue(self,xValue,Curve):
		uValue = NurbsCurve.doublep()
		Curve.minDistX(xValue,uValue,self.error,1,self.maxIter)
		return uValue.value()

	## @brief Computes a NURBS curve from given describer points, knot- and weight vector
	# @param maxPoints number of controlpoints
	# @param lengthKnot length of the knot vector is number of control points + degree of the curve + 1
	# @param weight array with equal weight 1
	# @param knots array with knots from type: (0,0,0,0,...0.5...,1,1,1,1)
	# @return the NURBS - curve
	def computeNurbsCurve(self,controlPoints):
		
		maxPoints=controlPoints.shape[0]
		
		deg=3
		lengthKnot= maxPoints+deg+1
		
		weight = NurbsVector.Vector_DOUBLE(maxPoints)
		knots = NurbsVector.Vector_DOUBLE(lengthKnot)
		
		for n in range(maxPoints):
			weight[n] = 1.0
		
		for n in range(lengthKnot):
			if n < 3:
				knot = 0.0
			
			elif n > (lengthKnot-4):
				knot = 1.0
			
			else:
				knot = (n-3)*1.0/(maxPoints-3)
			
			knots[n] = knot
			
			if self.debug > 1:
				print "NURBS - Knotenvektor:",knots[n]
			
		pointsB = NurbsVector.Vector_Point2Dd(maxPoints)
		vector = NurbsVector.Vector_Point2Dd(1)
		
		for n in range(maxPoints):
			vector.reset(NurbsPoint.Point2Dd(controlPoints[n][0],controlPoints[n][1]))
			pointsB.as_func(n,vector)
		
		if self.debug > 1:
			print "NURBS - Gewichtungsvektor:",weight[0]
			print "Grad der NURBS - Kurve:",deg
		
		return NurbsCurve.NurbsCurve2Dd(pointsB,weight,knots,deg)

	## @brief Calculates the gradient angle of a NURBS - curve at a given u - value
	# @brief The NURBS - function "deriveAt" figures out the derivation vector on the NURBS - curve for a given u - value.
	# @brief \n The gradient angle is calculates with: \f[\theta_{gradient} = \arctan{(\frac{y}{x}}) \f]
	# @brief with \f$ x,y \f$ components of the derivation vector.
	# @param U the U - value
	# @param curve the NURBS - curve
	# @return the gradient angle \f$ \theta_{gradient} \f$
	def deriveNurbs(self,u,curve):
		derivationVector = NurbsVector.Vector_Point2Dd()
		curve.deriveAt(u,1,derivationVector)
		
		firstDerivation = derivationVector[1]
		
		return math.atan2(firstDerivation.gety(), firstDerivation.getx())

	## @brief Calculates local curvature of the polynomial thickness distribution
	# @brief With the polynomial: \f[D(x)= p_0 (x^{p_1} - x^{p_2})^{\frac{1}{p_3}} + a_4 x^4 + a_3 x^3 + a_2 x^2 + a_1 x + a_0 \f]
	# @brief and curvature: \f[w(x) = \kappa(x)= \frac{D''(x)}{ (1 + (D'(x))^2)^{\frac{3}{2}}} \f]
	# @brief only the absolute value is from interest
	# @param dx local x-coordinate
	# @param a0 coefficient for polynomial weight function
	# @param a1 coefficient for polynomial weight function
	# @param a2 coefficient for polynomial weight function
	# @param a3 coefficient for polynomial weight function
	# @param a4 coefficient for polynomial weight function
	# @param p0 coefficient for polynomial weight function
	# @param p1 coefficient for polynomial weight function
	# @param p2 coefficient for polynomial weight function
	# @param p3 coefficient for polynomial weight function
	# @return local curvate
	def give_sym_w(self,dx):
		try:
			a0=self.polyThickRBs[0]
			a1=self.polyThickRBs[1]
			a2=self.polyThickRBs[2]
			a3=self.polyThickRBs[3]
			a4=self.polyThickRBs[4]
			p0=self.polyThickRBs[5]
			p1=self.polyThickRBs[6]
			p2=self.polyThickRBs[7]
			p3=self.polyThickRBs[8]
		except:
			#use old definition:
			a0=self.polyThickRBs[0][0]
			a1=self.polyThickRBs[0][1]
			a2=self.polyThickRBs[0][2]
			a3=self.polyThickRBs[0][3]
			a4=self.polyThickRBs[0][4]
			p0=self.polyThickRBs[0][5]
			p1=self.polyThickRBs[0][6]
			p2=self.polyThickRBs[0][7]
			p3=self.polyThickRBs[0][8]
		
		
		dx=numpy.fabs(dx)
		
		w_local=(p0/p3*(dx**p1-dx**p2)**(1.0/p3-2.0)*  ((1.0/p3-1.0)*  ((p1*dx**(p1-1.0)-p2*dx**(p2-1.0))**2.0)+ (dx**p1-dx**p2)**(1.0/p3-1.0)* (p1*(p1-1.0)*dx**(p1-2.0)- p2*(p2-1.0)*dx**(p2-2.0)))  +12.0*a4*dx**2.0+6.0*a3*dx+2.0*a2)

		z=(1.0+(p0/p3*(dx**p1-dx**p2)**(1.0/p3-1.0)*(p1*dx**(p1-1.0)-p2*dx**(p2-1.0))+4.0*a4*dx**3+3.0*a3*dx**2+2.0*a2*dx+a1)**2)**(3/2)

		w_local/=z
		
		return numpy.fabs(w_local) #betrag von w

	## @brief Calculates local curvature only of the polynomial part of the equation
	# @brief with polynomial part: \f[D(x)= a_4 x^4 + a_3 x^3 + a_2 x^2 + a_1 x + a_0 \f]
	# @brief and curvature: \f[w(x) = \kappa(x)= \frac{D''(x)}{ (1 + (D'(x))^2)^{\frac{3}{2}}} \f]
	# @brief only the absolute value is from interest
	# @param dx local x-coordinate
	# @param a0 coefficient for polynomial weight function
	# @param a1 coefficient for polynomial weight function
	# @param a2 coefficient for polynomial weight function
	# @param a3 coefficient for polynomial weight function
	# @param a4 coefficient for polynomial weight function
	# @param p0 coefficient for polynomial weight function
	# @param p1 coefficient for polynomial weight function
	# @param p2 coefficient for polynomial weight function
	# @param p3 coefficient for polynomial weight function
	# @return local curvate of polynomial part
	def give_sym_v(self,dx):
		try:
			a0=self.polyThickRBs[0]
			a1=self.polyThickRBs[1]
			a2=self.polyThickRBs[2]
			a3=self.polyThickRBs[3]
			a4=self.polyThickRBs[4]
			p0=self.polyThickRBs[5]
			p1=self.polyThickRBs[6]
			p2=self.polyThickRBs[7]
			p3=self.polyThickRBs[8]
		except:
			#use old definition:
			a0=self.polyThickRBs[0][0]
			a1=self.polyThickRBs[0][1]
			a2=self.polyThickRBs[0][2]
			a3=self.polyThickRBs[0][3]
			a4=self.polyThickRBs[0][4]
			p0=self.polyThickRBs[0][5]
			p1=self.polyThickRBs[0][6]
			p2=self.polyThickRBs[0][7]
			p3=self.polyThickRBs[0][8]
		
		w_local=(12*a4*dx**2+6*a3*dx+2*a2)/((1+(4*a4*dx**3+3*a3*dx**2+2*a2*dx+a1)**2)**(3/2))
		
		return w_local
	
	## @brief Calculates local value of several weight functions
	# @brief The boundary conditions should be: \f[w(x) > 0 \hspace{0,5cm} \forall \hspace{0,1cm} x \f]
	# @brief \n a) parabolic: \f[w(x)= 4(x- \frac{1}{2})^2 \f]
	# @brief b) cubic: \f[w(x)= \left| 8\left(x- \frac{1}{2}\right)^3 \right|\f]	
	# @brief c) tangens: \f[ w(x)= \left|\tan\left(b\pi\left(x-\frac{1}{2}\right)\right)\right| \hspace{1,0cm} b \approx 0,999 \f]
	# @brief d) arcsine: \f[ w(x)=\left|\arcsin\left(2\left(x-\frac{1}{2}\right)\right)\right|\f]
	# @brief e) hyperbolic: \f[w(x)= \frac{1}{x} \hspace{0,5cm} 0<x<0,5 \f]
	# @brief \f[w(x)= \frac{1}{1-x} \hspace{0,5cm} 0,5 \leq x<1 \f]
	# @brief f) line combination with high and low weight \f[w_1(x)= m_1 x + 1 \hspace{1,5cm} 0\leq x< x_{break} \f]
	# @brief \f[w_2(x)= m_2 (x - x_{break}) + y_{break} \hspace{1,5cm} x_{break}<x<0,5 \f]
	# @brief \f[w_3(x)= -m_2 (x - 0,5) + 0,01 \hspace{1,5cm}	0,5 \leq x< 1-x_{break} \f]
	# @brief \f[w_4(x)= -m_1 (x + x_{break} - 1) + y_{break} \hspace{1,5cm} 1-x_{break}<x\leq 1 \f]
	# @brief with \f$x_{break}, y_{break}\f$ for example: \f[x_{break} = 0,02 \hspace{0,5cm} y_{break} = 0,08 \f]
	# @brief \f[m_1 = \frac{y_{break} - 1}{x_{break}} \f]
	# @brief \f[m_2 = \frac{0,01 - y_{break}}{0,5-x_{break}} \f]
	# @param weightCondition flag value for function type
	# @param x_search local x-coordinate
	# @return local value of weight function
	def give_sym_g(self,x_search):
		
		b_tan = 0.999
		b_hyp = 1.0 - b_tan
		
		w_local=1.0
		
		if self.weightCondition == 0:	#parabolic
			w_local=4.0*(x_search-0.5)**2.0 + 0.02
		
		elif self.weightCondition == 1:	#cubic
			w_local=8.0*(x_search-0.5)**3.0
			
			w_local=abs(w_local)
			
			w_local+=0.02
		
		elif self.weightCondition == 2: #tangens
			w_local=numpy.tan((b_tan*numpy.pi)*(x_search-0.5))
			w_local=abs(w_local)
			
			w_local+=0.01
		
		elif self.weightCondition == 3: #arcsine
			w_local=numpy.arcsin(2.0*(x_search-0.5))
			w_local=abs(w_local)
			
			w_local+=0.01

		elif self.weightCondition == 4: #hyperbolic
			if x_search > 0.0 and x_search < 0.5:
				w_local = 1.0 / (x_search + b_hyp)
				
			if x_search >= 0.5 and x_search < 1.0:
				w_local = 1.0 / (1.0 + b_hyp - x_search)
			
		elif self.weightCondition == 5: #line combination
			
			x_break = 0.02
			y_break = 0.08
			
			m_1 = (y_break - 1.0)/x_break
			
			p_2 = 0.01
			
			m_2 = (p_2 - y_break)/(0.5 - x_break)
		
			if x_search >= 0.0 and x_search < x_break:
				w_local= m_1*x_search + 1.0
				
			elif x_search >= x_break and x_search < 0.5:
				w_local= m_2*(x_search-x_break) + y_break
				
			elif x_search >= 0.5 and x_search < (1.0 - x_break):
				w_local= -m_2*(x_search-0.5) + p_2
				
			elif x_search >= (1.0 - x_break) and x_search <= 1.0:
				w_local= -m_1*(x_search+x_break-1.0) + y_break
			
		else:
			raise Exception,"Wert für weightCondition ungültig"
		
		return w_local

	## @brief Computes the NACA-65 thickness distribution
	# @brief The NACA-65 thickness distribution is assumed with: \f[D(x) = D_{max} (1-x) \frac{ - 2,8385 x^3  + 2,4478 x^2 - 0,2758 x + 1,0675 \sqrt{x} }{ 1 - 0,176 x }\f]
	# @param x local x-coordinate
	# @param maxThickness \f$ D_{max}\f$ - maximum thickness
	# @return array with thickness distribution at discrete points
	def computeNACA65Thick(self,x):
		thickDistr = numpy.zeros([self.profileResolution], 'd')
		
		thickDistr[:]=self.maxThickness*(1-x[:])*(1.0675*numpy.sqrt(x[:])-0.2758*x[:]+2.4478*x[:]**2-2.8385*x[:]**3)/(1-0.176*x[:])
		
		return thickDistr

	## @brief Computes polynomial thickness distribution
	# @brief with polynom: \f[D(x)= p_0 (x^{p_1} - x^{p_2})^{\frac{1}{p_3}} + a_4 x^4 + a_3 x^3 + a_2 x^2 + a_1 x + a_0 \f]
	# @param x local x-coordinate
	# @param a0, a1, a2, a3, a4, p0, p1, p2, p3 coefficients for polynomial weight function
	# @return array with thickness distribution at discrete points
	def computePolyThick(self,x):
		try:
			a0=self.polyThickRBs[0]
			a1=self.polyThickRBs[1]
			a2=self.polyThickRBs[2]
			a3=self.polyThickRBs[3]
			a4=self.polyThickRBs[4]
			p0=self.polyThickRBs[5]
			p1=self.polyThickRBs[6]
			p2=self.polyThickRBs[7]
			p3=self.polyThickRBs[8]
		except:
			#use old definition:
			a0=self.polyThickRBs[0][0]
			a1=self.polyThickRBs[0][1]
			a2=self.polyThickRBs[0][2]
			a3=self.polyThickRBs[0][3]
			a4=self.polyThickRBs[0][4]
			p0=self.polyThickRBs[0][5]
			p1=self.polyThickRBs[0][6]
			p2=self.polyThickRBs[0][7]
			p3=self.polyThickRBs[0][8]
		
		thickDistr=numpy.zeros([self.profileResolution], 'd')
		
		thickDistr[:] = p0*(x[:]**p1-x[:]**p2)**(1.0/p3)+a4*x[:]**4.0+a3*x[:]**3.0+a2*x[:]**2.0+a1*x[:]+a0
		
		for n in range(int(self.profileResolution)):
			thickDistr[n] = round(thickDistr[n] ,8)
		
		return thickDistr
	
	## @brief Computes elliptic thickness distribution
	# @brief Elliptic function: \f[D(x)= \frac{D_{max}}{2} \sin(\arccos(2x-1)) \f]
	# @param x array with x-coordinates
	# @param maxThickness \f$ D_{max}\f$ - maximum thickness
	# @return array with thickness distribution at discrete points
	def computeEllipticThick(self,x):
		thickDistr=numpy.zeros([self.profileResolution], 'd')
		
		thickDistr[:] = self.maxThickness/2.0 * numpy.sin(numpy.arccos(2.0*x[:]-1.0))
		
		for n in range(int(self.profileResolution)):
			thickDistr[n]=round(thickDistr[n] ,8)
		
		return thickDistr

	## @brief Computes thickness distribution and superposes with camber line after coordinate transformation
	# @brief The profile results from superposition of the camber line coordinates and the thickness distribution with use of the camber line's gradient angle.
	# @brief \n The local circular arc gradient angle is calculated with: \f[\sin{(\theta_{gradient}(x))} = \frac{x - 0,5}{R_{camber}} \f] on the other hand the opening angle of the circular arc camber line: \f[\sin{ \left(\frac{\theta}{2} \right)} = \frac{1}{2R_{camber}} \f]
	# @brief consequential: \f[ \theta_{gradient}(x) = \arcsin\left( 2(x - 0,5)  \sin\left(\frac{ \theta}{2} \right) \right)\f]
	# @brief \n\n The polynomial camberline takes it's first derivation: \f[f'(x)= 4 a_4 x^{3} + 3 a_3 x^{2} + 2 a_2 x + a_1 \f] and \f[\theta = \arctan{ (f'(x) )} \f]
	# @brief The deriveNurbs - function solves the gradient angle of the NURBS - curve on given x - coordinates.
	# @brief \n\n The NACA camberline also takes the first derivation of the approximated function: \f[f' (x) = -\frac{C_{A0}}{4 \pi} (\ln x - ln(1-x)) \f]
	# @brief \n The profile coordinates are computed from the leading edge to the trailing edge.
	# @brief \n The superposition is calculated with: \f[x_{suction} = x_{camber} - D(x_{camber}) \sin{ (\theta_{gradient} )}  \f]
	# @brief \f[y_{suction} = y_{camber} + D(x_{camber}) \cos{ (\theta_{gradient} )} \f]
	# @brief \f[x_{pressure} = x_{camber} + D(x_{camber}) \sin{ (\theta_{gradient} )} \f]
	# @brief \f[y_{pressure} = y_{camber} - D(x_{camber}) \cos{ (\theta_{gradient} )} \f]
	# @brief Thickness distribution methods are: Polynomial and NURBS.
	# @param cambCoord array with coordinates of discrete camber line points
	# @param upperProfile array with coordinates of upper profil
	# @param lowerProfile array with coordinates of lower profil
	# @param thickDistr array with thickness distribution at discrete points
	# @param profileGradient array with local gradient of a polynomial camber line
	# @param theta \f$ \theta_{gradient}\f$ - array with local gradient angles
	# @param curveThick NURBS - curve thickness distribution
	# @param point point on the NURBS - curve close to the given x - coordinate
	# @param thetaMax \f$ \theta \f$ opening angle of a circular arc camber line
	# @return array with coordinates of lower profil
	# @return array with coordinates of upper profil
	def computeProfile(self,cambCoord):
		upperProfile = numpy.zeros([2,self.profileResolution], 'd')
		lowerProfile = numpy.zeros([2,self.profileResolution], 'd')
		
		self.thickDistr = numpy.zeros([self.profileResolution], 'd')
		
		profileGradient=numpy.zeros([self.profileResolution], 'd')
		theta=numpy.zeros([self.profileResolution], 'd')
		
		if self.debug > 0:
			print "maximale Dicke:", self.maxThickness
			print "polynomiale Dickenmethode:",self.thickMeth
		
		if self.maxThickness < 0.0 or self.maxThickness > 0.5:
			raise Exception, "Wert für maxThickness ungültig"
		
		if self.flagThickDistr == 1:
			if self.thickMeth == 0:
				self.thickDistr = self.computeNACA65Thick(cambCoord[0])
			
			elif self.thickMeth == 1:
				self.thickDistr = self.computePolyThick(cambCoord[0])
			
			elif self.thickMeth == 2:
				self.thickDistr = self.computeEllipticThick(cambCoord[0])
			
			else:
				raise Exception,"Wert fuer polyThickMeth ungueltig"
		
		elif self.flagThickDistr == 2:
			if self.debug > 1:
				print "Kontrollpunkte der NURBS - Dickenverteilung:", self.controlPointsThick
				
			curveThick=self.computeNurbsCurve(self.controlPointsThick)
			
			if self.approxNurbsNoseRad!=0:
				curveThick.setTangentAtEnd(NurbsPoint.Point2Dd(0.0,self.approxNurbsNoseRad),NurbsPoint.Point2Dd(0.0,-self.approxNurbsNoseRad))
			
			for n in range(int(self.profileResolution)):
					
				if cambCoord[0][n] != 0 and cambCoord[0][n] != 1:
					point = curveThick.pointAt(self.calculateUvalue(cambCoord[0][n],curveThick))
					self.thickDistr[n]=point.gety()
					
					if self.flagThickDistr!=2:
						if self.thickDistr[n] == self.thickDistr[n-1]:
							raise Exception,"Abbruchwerte fuer MinDistX - Funktion zu gering"
			
			self.thickDistr[0]=0.0
			self.thickDistr[-1]=0.0
			
			if self.debug > 0:
				print "post - script file NURBS - Dickenverteilung: nurbsThick.ps"
				curveThick.writePS("nurbsThick.ps",self.cp,self.magFact,self.dash)
		
		if self.flagCamberLine == 0:
			thetaMax = self.angleOfAttack - self.outflowAngle
			
			theta[:] = -1.0*(numpy.arcsin(2*numpy.sin(numpy.radians(thetaMax)/2.0)*(cambCoord[0,:]-1.0/2.0)))
		
		elif self.flagCamberLine == 1:
			a0=self.polyCambRBs[0]
			a1=self.polyCambRBs[1]
			a2=self.polyCambRBs[2]
			a3=self.polyCambRBs[3]
			a4=self.polyCambRBs[4]
	
			profileGradient[:] = 4.0*a4*cambCoord[0][:]**3.0+3.0*a3*cambCoord[0][:]**2.0+2.0*a2*cambCoord[0][:]+a1
			
			theta[:] = numpy.arctan(profileGradient[:])
		
		elif self.flagCamberLine == 2:
			for n in range(int(self.profileResolution)):
				theta[n] = self.deriveNurbs(self.calculateUvalue(cambCoord[0,n],self.curveCamber),self.curveCamber)
		
		elif self.flagCamberLine == 3:
			cambCoord[0][0]=0.00000001
			cambCoord[0][-1]=0.99999999
			
			profileGradient[:] = -self.Ca0/(4*numpy.pi)*(numpy.log(cambCoord[0][:]) - numpy.log(1.0 - cambCoord[0][:]))
			
			theta[:] = numpy.arctan(profileGradient[:])
		
		upperProfile[0,:] = cambCoord[0,:] - numpy.sin(theta[:])*self.thickDistr
		upperProfile[1,:] = cambCoord[1,:] + numpy.cos(theta[:])*self.thickDistr
		lowerProfile[0,:] = cambCoord[0,:] + numpy.sin(theta[:])*self.thickDistr
		lowerProfile[1,:] = cambCoord[1,:] - numpy.cos(theta[:])*self.thickDistr
		
		for i in range(int(self.profileResolution)):
			upperProfile[0,i] = round(upperProfile[0,i],7)
			upperProfile[1,i] = round(upperProfile[1,i],8)
			lowerProfile[0,i] = round(lowerProfile[0,i],7)
			lowerProfile[1,i] = round(lowerProfile[1,i],8)
		
		if self.debug > 1:
			print "Dickenverteilung:",self.thickDistr
			print "Steigungswinkel Theta:",theta
			print "Saugseite:",upperProfile
			print "Druckseite:",lowerProfile
		
		return lowerProfile, upperProfile

	## @brief Calculates the leading edge and trailing edge circles for double circular arc thickness distributions
	# @brief \f[	x = x_m - r_0 \cos \alpha \hspace{2cm} y = y_m - r_0 \sin \alpha \f]
	# @param alpha local angle
	# @param r0 radius of leading/trailing edge
	# @param xm x-coordinate at the centre of the circle
	# @param ym y-coordinate at the centre of the circle
	# @return x/y-coordinates of a point on the leading/trailing edge circular arc
	def calculateCircle(self,alpha,radius,xmittelpunkt,ymittelpunkt):
		r = radius
		xm = xmittelpunkt
		ym = ymittelpunkt
		return xm - r*numpy.cos(alpha), ym - r*numpy.sin(alpha)
	
	## @brief Writes a xml file with all public attributes, used to describe the current profile
	# @brief First the xml - object is created by parsing a string or a file, according to the value of the 'WriteMode' - parameter.
	# @brief The function generates a new child element - profile - with increasing index.
	# @brief The profile - element itself achieves several child elements, which are allocated to the profile parameters.
	# @brief In addition the child elements take over the attributes of the public parameters they represent.
	# @brief At last the extended xml - object is saved in a xml - file.
	# @param WriteMode for value 0: overwrites the current file or creates a new one with 'fileName', if it is not available, even if WriteMode = 1; for value 1: appends a new profile element to the existing file
	#def writeXmlFile(self,filename,WriteMode):
	def writeXmlFile(self,dom_object,elementRow,ProfileNum):
		
		newElement = dom_object.createElement("profile")
		newElement.setAttribute("i",str(ProfileNum))
		elementRow.appendChild(newElement)
		
		element1 = elementRow.getElementsByTagName("profile")[ProfileNum]
		
		xmlList = ['flagCamberLine', 'flagThickDistr', 'profileResolution', 'chordLength', 'gamma','angleOfAttack','outflowAngle']

		#SW:Maximale Iterationen schreiben?
		if self.maxIter > 0:
			xmlList.append('maxIter')

		#SW:Error schreiben?
		if self.error > 0:
			xmlList.append('error')
		
		if self.camberResolution:
			xmlList.append('camberResolution')
		
		if self.leadingEdgeRadius:
			xmlList.append('leadingEdgeRadius')
		
		if self.maxThickness:
			xmlList.append('maxThickness')
		
		if self.weightCondition:
			xmlList.append('weightCondition')
			xmlList.append('weightCombination')
		
		if self.thickMeth:
			xmlList.append('thickMeth')
		
		if self.polyCamberLineDegree:
			xmlList.append('polyCamberLineDegree')
		
		if self.maxCambPosition:
			xmlList.append('maxCambPosition')
			
		if self.approxNurbsNoseRad:
			xmlList.append('approxNurbsNoseRad')
		
		if self.flagCamberLine == 2:
			xmlList.append('controlPointsCamb')
			
		if self.flagThickDistr == 2:
			xmlList.append('controlPointsThick')
			
		if self.flagCamberLine == 3:
			xmlList.append('Ca0')

		#SW: Ich hab das mal angepasst, dass es wieder ordentlich schreibt
		if len(self.polyThickRBs) != 0:
			xmlList.append('polyThickRBs')
		
		for i in range(len(xmlList)):
			if type(getattr(self,xmlList[i])) == numpy.ndarray:
				newElement = dom_object.createElement("profileElement")
				newElement.setAttribute(xmlList[i],str(numpy.array2string(getattr(self,xmlList[i]), separator=',').replace('\n','')))
				element1.appendChild(newElement)
			
			else:
				newElement = dom_object.createElement("profileElement")
				newElement.setAttribute(xmlList[i],str(getattr(self,xmlList[i])))
				element1.appendChild(newElement)

	## @brief Reads a xml file with all public attributes, which are nedded to describe the current profile
	# @brief First the xml - object is created by parsing the xml - file.
	# @brief The function selects the current profile - element with all child elements corresponding to it.
	# @brief All public parameters take over their adapting attributes from the xml - object.
	def readXmlFile(self,dom_object,elementProf):
		
		elemente = elementProf.getElementsByTagName("profileElement")
			
		for element in elemente:
			if (element.attributes.item(0).name in ['controlPointsCamb', 'controlPointsThick']):
				setattr(self,element.attributes.item(0).name, self.convertListToCoordArray(eval(element.attributes.item(0).value)))
			
			elif (element.attributes.item(0).name in ['polyThickRBs']):
				setattr(self,element.attributes.item(0).name, eval(element.attributes.item(0).value))
				
			elif (element.attributes.item(0).name in ['flagCamberLine', 'flagThickDistr']):
				setattr(self,element.attributes.item(0).name, int(element.attributes.item(0).value))
			else:
				setattr(self,element.attributes.item(0).name, float(element.attributes.item(0).value))
