#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian G. Barthmes
# @date 28.04.2009


# Python Imports
# üöä
import sys, os
sys.path.insert(0 ,"../../src")

import turboMachine
from numpy import *
from copy import *

# Turbomachine Initialisation

# Working Directory: The results are stored here
workdir=os.path.abspath('turboExample')   # '/path/to/workdir'

# 2 Bladerows (Rotor/Stator) with 3 / 5 blade sections
configuration=[3,5]

# If calculated once use this if you didn't change anything
useNurbsXml=0

# This initializes the turbomachine object
turbo=turboMachine.turboMachine(workdir,configuration)

# Debug mode (1 for progress output)
turbo.debug=1


# These Variables are used to define the geometry of hub and shroud
z1=0.
z2=80.
z3=140.
r1=125.104
r2=128.348
r3=132.
shroudr=192.468

#################################################    ROTOR     #############################################

## Rowdefinition

# The bladeRow-objects are stored in the list turbo.listRows

# Use the already calculated nurbs surface?
if useNurbsXml==1:
	turbo.listRows[0].useNurbsXml=True

# 0 for definition by stacking axis + stacking point + chord length 
turbo.listRows[0].definition = 0

# number of blades only relevant for output and further use
turbo.listRows[0].numberBlades = 23

# rotational speed, only relevant for future use
turbo.listRows[0].omega = 850

# The default degree for several interpolations during calculation
turbo.listRows[0].degree=3

# The degree of nurbs interpolation in radial direction
turbo.listRows[0].degreeU=2

# Hub and Shroud definition in z-r-space, at least degree+1 coordinates each
turbo.listRows[0].coordinatesHub = zeros((5,2))
turbo.listRows[0].coordinatesShroud = zeros((5,2))
turbo.listRows[0].coordinatesHub[:,0]=linspace(z1,z2,5)
turbo.listRows[0].coordinatesHub[:,1]=linspace(r1,r2,5)
turbo.listRows[0].coordinatesShroud[:,0]=linspace(z1,z2,5)
turbo.listRows[0].coordinatesShroud[:,1]=shroudr

# Stacking line definition in z-r-space. In this case with a backward sweep. First and last point should be outside of domain to make sure the stackingline cuts hub and shroud lines
turbo.listRows[0].coordinatesStackingLine = zeros((3,2))
turbo.listRows[0].coordinatesStackingLine[:,0]=linspace((z2-z1)/3+z1,(z2-z1)/2+z1/2,3)
turbo.listRows[0].coordinatesStackingLine[:,1]=linspace(r1,shroudr,3)

# User defined blade section distribution, defined by relative coordinate list
turbo.listRows[0].spanDistr=1
turbo.listRows[0].span=[0.0,0.5,1.0]

# This defines a positive bow and a slight backward lean in the blade: a spanwise offset for each blade section
turbo.listRows[0].periphalOffset=[0.0,radians(0.5),radians(-0.5)]

# Set the output to 1 for progress info
turbo.listRows[0].debug=1

# Use the angle preserving algorithm for calculation
turbo.listRows[0].angleFlag=1

# Interpolate the surface with a periodic (degree-1 derivable) trailing edge
turbo.listRows[0].teflag="round" # "sharp"

## Profile definition

# Set the angle of attack and outflow angle relative to 2D Profile coordinates
turbo.listRows[0].setIOAngles([122.49,129.57,134.42],[92.7,108.42,118.12])

# Set the stagger angles for each section
turbo.listRows[0].setStaggerAngles([13.5,26.6,36.4])

# Set the chord length for each section
turbo.listRows[0].setChordLengths([51.76,55.25,52.41])

# Set the maximum Thickness for each section
turbo.listRows[0].setMaxThickness([0.1,0.1,0.1])

# Coefficians for polynomial thickness distribution (see profile2D)
CoeffArray = zeros((9))
CoeffArray[0]= 0
CoeffArray[2]= 0.1305026
CoeffArray[3]= -0.55961045
CoeffArray[4]= 0.33630738
CoeffArray[1]= -(CoeffArray[2]+CoeffArray[3]+CoeffArray[4])
CoeffArray[5]= 4.7
CoeffArray[6]= 0.71143029
CoeffArray[7]= 0.71323412
CoeffArray[8]= 1.3

# Set for each profile the profile2D-Parameters
for p in turbo.listRows[0].listProfiles:
	#p.Ca0=1.2
	p.debug=0		#debug level 0-2
	p.thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
	p.weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
	p.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
	p.profileResolution=200
	p.flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
	p.flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
	p.polyThickRBs = CoeffArray



#####################################         STATOR          ####################################################################

if len(configuration)>1:

	## Rowdefinition
	if useNurbsXml==1:
		turbo.listRows[1].useNurbsXml=True

	turbo.listRows[1].definition = 0
	turbo.listRows[1].numberBlades = 37

	turbo.listRows[1].coordinatesHub = zeros((5,2))
	turbo.listRows[1].coordinatesShroud = zeros((5,2))
	turbo.listRows[1].coordinatesHub[:,0]=linspace(z2,z3,5)
	turbo.listRows[1].coordinatesHub[:,1]=linspace(r2,r3,5)
	turbo.listRows[1].coordinatesShroud[:,0]=linspace(z2,z3,5)
	turbo.listRows[1].coordinatesShroud[:,1]=shroudr

	turbo.listRows[1].coordinatesStackingLine = zeros((5,2))
	turbo.listRows[1].coordinatesStackingLine[:,0]=(z3-z2)/2+z2
	turbo.listRows[1].coordinatesStackingLine[:,1]=linspace(r2,shroudr,5)

	turbo.listRows[1].spanDistr=1
	turbo.listRows[1].span=[0.0,0.01,0.5,0.99,1.0]

	turbo.listRows[1].debug=1
	turbo.listRows[1].angleFlag=1
	turbo.listRows[1].teflag="round"
	turbo.listRows[1].degree=3
	turbo.listRows[1].degreeU=2

	## Profildefinition
	turbo.listRows[1].setIOAngles([60.89,60.89,66.4,69.63,69.63],[90.0,90.0,90.0,90.0,90.0])
	turbo.listRows[1].setStaggerAngles([-9.0,-9.0,-10.0,-11.0,-11.0])
	turbo.listRows[1].setChordLengths([30,30,30,30,30])
	turbo.listRows[1].setMaxThickness([0.1,0.1,0.1,0.1,0.1])
	
	#Polynome Coefficients
	CoeffArray = zeros([1,9],'d')
	a0 = 0
	a2 = 0.1305026
	a3 = -0.55961045
	a4 = 0.33630738
	a1 = -(a2+a3+a4)

	p0 = 4.7
	p1 = 0.71143029
	p2 = 0.71323412
	p3 = 1.3

	CoeffArray[0][0]= a0
	CoeffArray[0][1]= a1
	CoeffArray[0][2]= a2
	CoeffArray[0][3]= a3
	CoeffArray[0][4]= a4
	CoeffArray[0][5]= p0
	CoeffArray[0][6]= p1
	CoeffArray[0][7]= p2
	CoeffArray[0][8]= p3
	
	for p in turbo.listRows[1].listProfiles:
		#p.Ca0=1.2
		p.debug=0		#debug level 0-2
		p.thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
		p.weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
		p.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
		p.profileResolution=200
		p.flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
		p.flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
		p.polyThickRBs = CoeffArray

###################################################################################################################################


# Now start the calculation of the turbomachine
turbo.computeTurboMachine()


############## Exporting ######################

for n in range(turbo.numberRows):
	turbo.listRows[n].exportBladeToIGES()
	turbo.listRows[n].exportBladeToSTEP()
	turbo.listRows[n].exportBladeToSTL(deflection=1,coefficient=0.0001)
	turbo.listRows[n].writeVRML()
	
	turbo.listRows[n].exportHubShroudToSTEP()
	turbo.listRows[n].exportHubShroudToSTL()

############ opencascade test display ###############
#turbo.listRows[0].qtviewer() #Only select one display per run!
#turbo.displayViewer()
turbo.interactiveViewer()

